@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: api_string.cpp 2255 2014-09-25 16:12:38Z oconrad $
+ * Version $Id$
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -26,7 +26,8 @@
 // This library is free software; you can redistribute   //
 // it and/or modify it under the terms of the GNU Lesser //
 // General Public License as published by the Free       //
-// Software Foundation, version 2.1 of the License.      //
+// Software Foundation, either version 2.1 of the        //
+// License, or (at your option) any later version.       //
 //                                                       //
 // This library is distributed in the hope that it will  //
 // be useful, but WITHOUT ANY WARRANTY; without even the //
@@ -36,9 +37,7 @@
 //                                                       //
 // You should have received a copy of the GNU Lesser     //
 // General Public License along with this program; if    //
-// not, write to the Free Software Foundation, Inc.,     //
-// 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, //
-// USA.                                                  //
+// not, see <http://www.gnu.org/licenses/>.              //
 //                                                       //
 //-------------------------------------------------------//
 //                                                       //
@@ -68,6 +67,9 @@
 #include <wx/string.h>
 #include <wx/datetime.h>
 #include <wx/tokenzr.h>
+#include <wx/wxcrt.h>
+#include <wx/wxcrtvararg.h>
+#include <wx/convauto.h>
 
 #include "api_core.h"
 
@@ -109,6 +111,7 @@ CSG_String::CSG_String(wchar_t Character, size_t nRepeat)
 	m_pString	= new wxString(Character, nRepeat);
 }
 
+//---------------------------------------------------------
 CSG_String::CSG_String(const class wxString *pString)
 {
 	if( pString )
@@ -117,6 +120,16 @@ CSG_String::CSG_String(const class wxString *pString)
 		m_pString	= new wxString;
 }
 
+bool CSG_String::Create(const class wxString *pString)
+{
+	if( pString )
+		*m_pString	= *pString;
+	else
+		m_pString->Clear();
+
+	return( true );
+}
+
 //---------------------------------------------------------
 CSG_String::~CSG_String(void)
 {
@@ -126,8 +139,6 @@ CSG_String::~CSG_String(void)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -169,8 +180,6 @@ CSG_String & CSG_String::operator = (wchar_t Character)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -188,8 +197,6 @@ bool CSG_String::is_Empty(void)	const
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -237,8 +244,6 @@ void CSG_String::Set_Char(size_t i, wchar_t Character)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -262,8 +267,6 @@ const wchar_t * CSG_String::w_str(void) const
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -272,21 +275,22 @@ void CSG_String::Clear(void)
 	m_pString->Clear();
 }
 
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
 //---------------------------------------------------------
 CSG_String CSG_String::Format(const char *Format, ...)
 {
 	CSG_String	s;
 
-	va_list	argptr;
-	
 #ifdef _SAGA_LINUX
-	// workaround as we only use wide characters
-	// since wx 2.9.4 so interpret strings as multibyte
-	wxString	_Format(Format);	_Format.Replace("%s", "%ls");
-	va_start(argptr, _Format);
+	wxString _Format(Format); _Format.Replace("%s", "%ls");	// workaround as we only use wide characters since wx 2.9.4 so interpret strings as multibyte
+	va_list	argptr; va_start(argptr, _Format);
 	s.m_pString->PrintfV(_Format, argptr);
 #else
-	va_start(argptr, Format);
+	va_list	argptr; va_start(argptr, Format);
 	s.m_pString->PrintfV(Format, argptr);
 #endif
 
@@ -300,16 +304,12 @@ CSG_String CSG_String::Format(const wchar_t *Format, ...)
 {
 	CSG_String	s;
 
-	va_list	argptr;
-	
 #ifdef _SAGA_LINUX
-	// workaround as we only use wide characters
-	// since wx 2.9.4 so interpret strings as multibyte
-	wxString	_Format(Format);	_Format.Replace("%s", "%ls");
-	va_start(argptr, _Format);
+	wxString _Format(Format); _Format.Replace("%s", "%ls");	// workaround as we only use wide characters since wx 2.9.4 so interpret strings as multibyte
+	va_list	argptr; va_start(argptr, _Format);
 	s.m_pString->PrintfV(_Format, argptr);
 #else
-	va_start(argptr, Format);
+	va_list	argptr; va_start(argptr, Format);
 	s.m_pString->PrintfV(Format, argptr);
 #endif
 
@@ -321,50 +321,40 @@ CSG_String CSG_String::Format(const wchar_t *Format, ...)
 //---------------------------------------------------------
 int CSG_String::Printf(const char *Format, ...)
 {
-	va_list	argptr;
-
 #ifdef _SAGA_LINUX
-	// workaround as we only use wide characters
-	// since wx 2.9.4 so interpret strings as multibyte
-	wxString	_Format(Format);	_Format.Replace("%s", "%ls");
-	va_start(argptr, _Format);
-	m_pString->PrintfV(_Format, argptr);
+	wxString _Format(Format); _Format.Replace("%s", "%ls");	// workaround as we only use wide characters since wx 2.9.4 so interpret strings as multibyte
+	va_list	argptr; va_start(argptr, _Format);
+	int Result	= m_pString->PrintfV(_Format, argptr);
 #else
-	va_start(argptr, Format);
-	m_pString->PrintfV(Format, argptr);
+	va_list	argptr; va_start(argptr, Format);
+	int Result	= m_pString->PrintfV(Format, argptr);
 #endif
 
 	va_end(argptr);
 
-	return( (int)Length() );
+	return( Result );
 }
 
 //---------------------------------------------------------
 int CSG_String::Printf(const wchar_t *Format, ...)
 {
-	va_list	argptr;
-
 #ifdef _SAGA_LINUX
-	// workaround as we only use wide characters
-	// since wx 2.9.4 so interpret strings as multibyte
-	wxString	_Format(Format);	_Format.Replace("%s", "%ls");
-	va_start(argptr, _Format);
-	m_pString->PrintfV(_Format, argptr);
+	wxString _Format(Format); _Format.Replace("%s", "%ls");	// workaround as we only use wide characters since wx 2.9.4 so interpret strings as multibyte
+	va_list	argptr; va_start(argptr, _Format);
+	int Result	= m_pString->PrintfV(_Format, argptr);
 #else
-	va_start(argptr, Format);
-	m_pString->PrintfV(Format, argptr);
+	va_list	argptr; va_start(argptr, Format);
+	int Result	= m_pString->PrintfV(Format, argptr);
 #endif
 
 	va_end(argptr);
 
-	return( (int)Length() );
+	return( Result );
 }
 
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -444,8 +434,6 @@ void CSG_String::operator += (wchar_t Character)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -535,8 +523,6 @@ CSG_String		operator + (wchar_t A, const CSG_String &B)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -585,8 +571,6 @@ CSG_String & CSG_String::Make_Upper(void)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -614,8 +598,6 @@ CSG_String & CSG_String::Remove(size_t pos, size_t len)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -628,11 +610,20 @@ int CSG_String::Trim(bool fromRight)
 	return( (int)(n - m_pString->Length()) );
 }
 
+//---------------------------------------------------------
+int CSG_String::Trim_Both(void)
+{
+	size_t	n	= m_pString->Length();
+
+	m_pString->Trim( true);
+	m_pString->Trim(false);
+
+	return( (int)(n - m_pString->Length()) );
+}
+
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -661,8 +652,6 @@ bool CSG_String::Contains(const CSG_String &String) const
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -712,8 +701,6 @@ CSG_String CSG_String::BeforeLast(wchar_t Character) const
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -737,8 +724,6 @@ CSG_String CSG_String::Left(size_t count) const
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -788,6 +773,183 @@ bool CSG_String::asDouble(double &Value) const
 }
 
 
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+CSG_String CSG_String::from_UTF8(const char *String, size_t Length)
+{
+	CSG_String	s;
+
+	if( String )
+	{
+		if( !Length )
+		{
+			Length	= strlen(String);
+		}
+
+		*s.m_pString	= wxString::FromUTF8(String, Length);
+	}
+
+	return( s );
+}
+
+//---------------------------------------------------------
+/**
+* This function creates a single char array and fills it
+* with the UTF-8 representation of this string object.
+* If successful pString will point to the address of the
+* allocated memory. Freeing memory is in response of the
+* calling function.
+*/
+//---------------------------------------------------------
+size_t CSG_String::to_UTF8(char **pString) const
+{
+	if( !is_Empty() )
+	{
+		const wxScopedCharBuffer	Buffer	= m_pString->utf8_str();
+
+		if( (*pString = (char *)SG_Malloc(Buffer.length())) != NULL )
+		{
+			memcpy(*pString, Buffer.data(), Buffer.length());
+
+			return( Buffer.length() );
+		}
+	}
+
+	*pString	= NULL;
+
+	return( 0 );
+}
+
+//---------------------------------------------------------
+CSG_Buffer CSG_String::to_UTF8(void) const
+{
+	CSG_Buffer	String;
+
+	if( !is_Empty() )
+	{
+		const wxScopedCharBuffer	Buffer	= m_pString->utf8_str();
+
+		String.Set_Data(Buffer.data(), Buffer.length());
+	}
+
+	return( String );
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+/**
+* This function creates a single char array and fills it
+* with the multibyte representation of this string object.
+* If successful pString will point to the address of the
+* allocated memory. Freeing memory is in response of the
+* calling function.
+*/
+//---------------------------------------------------------
+size_t CSG_String::to_MBChar(char **pString, int Encoding) const
+{
+	CSG_Buffer	String(to_MBChar(Encoding));
+
+	if( String.Get_Size() && (*pString = (char *)SG_Malloc(String.Get_Size())) != NULL )
+	{
+		memcpy(*pString, String.Get_Data(), String.Get_Size());
+
+		return( String.Get_Size() );
+	}
+
+	*pString	= NULL;
+
+	return( 0 );
+}
+
+//---------------------------------------------------------
+CSG_Buffer CSG_String::to_MBChar(int Encoding) const
+{
+	CSG_Buffer	String;
+
+	if( !is_Empty() )
+	{
+		wxScopedCharBuffer	Buffer;
+
+		switch( Encoding )	// selecting the appropriate wxMBConv class
+		{
+		case SG_FILE_ENCODING_ANSI   : Buffer = m_pString->mb_str(wxConvLibc       ); break;
+		case SG_FILE_ENCODING_UTF7   : Buffer = m_pString->mb_str(wxConvUTF7       ); break;
+		case SG_FILE_ENCODING_UTF8   : Buffer = m_pString->mb_str(wxConvUTF8       ); break;
+		case SG_FILE_ENCODING_UTF16LE: Buffer = m_pString->mb_str(wxMBConvUTF16LE()); break;
+		case SG_FILE_ENCODING_UTF16BE: Buffer = m_pString->mb_str(wxMBConvUTF16BE()); break;
+		case SG_FILE_ENCODING_UTF32LE: Buffer = m_pString->mb_str(wxMBConvUTF32LE()); break;
+		case SG_FILE_ENCODING_UTF32BE: Buffer = m_pString->mb_str(wxMBConvUTF32BE()); break;
+		default                      : Buffer = m_pString->mb_str(wxConvAuto     ()); break;
+		}
+
+		String.Set_Data(Buffer.data(), Buffer.length());
+	}
+
+	return( String );
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+/**
+  * This function creates a single char array and fills it
+  * with the ASCII representation of this string object.
+  * If successful pString will point to the address of the
+  * allocated memory. Freeing memory is in response of the
+  * calling function.
+*/
+//---------------------------------------------------------
+bool CSG_String::to_ASCII(char **pString, char Replace)	const
+{
+	if( !is_Empty() )
+	{
+		#if wxCHECK_VERSION(3, 1, 0)
+			const wxScopedCharBuffer	Buffer	= m_pString->ToAscii(Replace);
+		#else
+			const wxScopedCharBuffer	Buffer	= m_pString->ToAscii();
+		#endif
+
+		if( (*pString = (char *)SG_Malloc(Buffer.length())) != NULL )
+		{
+			memcpy(*pString, Buffer.data(), Buffer.length());
+
+			return( true );
+		}
+	}
+
+	return( false );
+}
+
+//---------------------------------------------------------
+CSG_Buffer CSG_String::to_ASCII(char Replace) const
+{
+	CSG_Buffer	String;
+
+	if( !is_Empty() )
+	{
+		#if wxCHECK_VERSION(3, 1, 0)
+			const wxScopedCharBuffer	Buffer	= m_pString->ToAscii(Replace);
+		#else
+			const wxScopedCharBuffer	Buffer	= m_pString->ToAscii();
+		#endif
+
+		String.Set_Data(Buffer.data(), Buffer.length());
+	}
+
+	return( String );
+}
+
+
 ///////////////////////////////////////////////////////////
 //														 //
 //														 //
@@ -875,6 +1037,28 @@ bool CSG_Strings::Add(const CSG_String &String)
 	return( true );
 }
 
+//---------------------------------------------------------
+bool CSG_Strings::Del(int Index)
+{
+	if( Index >= 0 && Index < m_nStrings )
+	{
+		delete(m_Strings[Index]);
+
+		m_nStrings--;
+
+		for(int i=Index; i<m_nStrings; i++)
+		{
+			m_Strings[i]	= m_Strings[i + 1];
+		}
+
+		m_Strings	= (CSG_String **)SG_Realloc(m_Strings, m_nStrings * sizeof(CSG_String *));
+
+		return( true );
+	}
+
+	return( false );
+}
+
 //---------------------------------------------------------
 CSG_Strings & CSG_Strings::operator  += (const CSG_String &String)
 {
@@ -904,31 +1088,89 @@ bool CSG_Strings::Set_Count(int nStrings)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-int				SG_Printf(const SG_Char *Format, ...)
+bool SG_is_Character_Numeric(int Character)
 {
-	va_list	argptr;
+	switch( Character )
+	{
+	case '0':
+	case '1':
+	case '2':
+	case '3':
+	case '4':
+	case '5':
+	case '6':
+	case '7':
+	case '8':
+	case '9':
+	case '-':
+	case '+':
+	case '.':
+	case ',':
+	case 'e':
+	case 'E':
+		return( true );
+	}
 
-	va_start(argptr, Format);
+	return( false );
+}
 
-	int		ret	= wxVprintf(Format, argptr);
 
-	va_end(argptr);
+///////////////////////////////////////////////////////////
+//														 //
+//														 //
+//														 //
+///////////////////////////////////////////////////////////
 
-	return( ret );
+//---------------------------------------------------------
+int				SG_Printf(const CSG_String &Format)
+{
+	return( SG_Printf(Format.c_str()) );
+}
+
+int				SG_Printf(const char *Format, ...)
+{
+#ifdef _SAGA_LINUX
+	wxString _Format(Format); _Format.Replace("%s", "%ls");	// workaround as we only use wide characters since wx 2.9.4 so interpret strings as multibyte
+	va_list	argptr; va_start(argptr, _Format); int ret = wxVprintf(_Format, argptr); va_end(argptr); return( ret );
+#else
+	va_list	argptr; va_start(argptr,  Format); int ret = wxVprintf( Format, argptr); va_end(argptr); return( ret );
+#endif
+}
+
+int				SG_Printf(const wchar_t *Format, ...)
+{
+#ifdef _SAGA_LINUX
+	wxString _Format(Format); _Format.Replace("%s", "%ls");	// workaround as we only use wide characters since wx 2.9.4 so interpret strings as multibyte
+	va_list	argptr; va_start(argptr, _Format); int ret = wxVprintf(_Format, argptr); va_end(argptr); return( ret );
+#else
+	va_list	argptr; va_start(argptr,  Format); int ret = wxVprintf( Format, argptr); va_end(argptr); return( ret );
+#endif
 }
 
 //---------------------------------------------------------
-int				SG_FPrintf(FILE* stream, const SG_Char *Format, ...)
+int				SG_FPrintf(FILE *Stream, const CSG_String &String)
 {
-	va_list	argptr;
+	return( SG_FPrintf(Stream, String.c_str()) );
+}
 
-	va_start(argptr, Format);
+int				SG_FPrintf(FILE *Stream, const char *Format, ...)
+{
+#ifdef _SAGA_LINUX
+	wxString _Format(Format); _Format.Replace("%s", "%ls");	// workaround as we only use wide characters since wx 2.9.4 so interpret strings as multibyte
+	va_list	argptr; va_start(argptr, _Format); int ret = wxVfprintf(Stream, _Format, argptr); va_end(argptr); return( ret );
+#else
+	va_list	argptr; va_start(argptr,  Format); int ret = wxVfprintf(Stream,  Format, argptr); va_end(argptr); return( ret );
+#endif
+}
 
-	int		ret	= wxVfprintf(stream, Format, argptr);
-
-	va_end(argptr);
-
-	return( ret );
+int				SG_FPrintf(FILE *Stream, const wchar_t *Format, ...)
+{
+#ifdef _SAGA_LINUX
+	wxString _Format(Format); _Format.Replace("%s", "%ls");	// workaround as we only use wide characters since wx 2.9.4 so interpret strings as multibyte
+	va_list	argptr; va_start(argptr, _Format); int ret = wxVfprintf(Stream, _Format, argptr); va_end(argptr); return( ret );
+#else
+	va_list	argptr; va_start(argptr,  Format); int ret = wxVfprintf(Stream,  Format, argptr); va_end(argptr); return( ret );
+#endif
 }
 
 
@@ -1021,48 +1263,6 @@ double			SG_Degree_To_Double(const CSG_String &String)
 	return( sig * (d + h / 60.0 + s / (60.0 * 60.0)) );
 }
 
-//---------------------------------------------------------
-CSG_String		SG_Number_To_Date(int Value)
-{
-	return( SG_Number_To_Date((double)Value) );
-}
-
-CSG_String		SG_Number_To_Date(double Value)
-{
-	int		y, m, d;
-
-	y	= (int)(Value / 10000);	Value	-= y * 10000;
-	m	= (int)(Value / 100);	Value	-= m * 100;
-	d	= (int)(Value / 1);
-
-	return( CSG_String::Format(SG_T("%02d.%02d.%04d"), d, m, y) );
-}
-
-//---------------------------------------------------------
-int				SG_Date_To_Number(const CSG_String &String)
-{
-	if( String.Length() > 0 )
-	{
-		CSG_String	s(String), sValue;
-
-		sValue	= s.AfterLast	('.');
-		int	y	= sValue.asInt();
-		sValue	= s.BeforeLast	('.');	s		= sValue;
-
-		sValue	= s.AfterLast	('.');
-		int	m	= sValue.asInt();
-		sValue	= s.BeforeLast	('.');	s		= sValue;
-		int	d	= sValue.asInt();
-
-		if( d < 1 )	d	= 1;	else if( d > 31 )	d	= 31;
-		if( m < 1 )	m	= 1;	else if( m > 12 )	m	= 12;
-
-		return( 10000 * y + 100 * m + 1 * d );
-	}
-
-	return( 0.0 );
-}
-
 
 ///////////////////////////////////////////////////////////
 //														 //
@@ -1121,26 +1321,51 @@ void			SG_Flip_Decimal_Separators(CSG_String &String)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-CSG_String		SG_Get_String(double Value, int Precision, bool bScientific)
+/**
+  * Returns floating point number 'Value' as formatted string.
+  * The format depends on the 'Precision' value. 
+  * If Precision is -99 (the default) it will simply use %f format.
+  * If 'Precision' is -98 the string will always be formatted
+  * with the %e format specification (i.e. scientific: d.dddd e dd).
+  * If Precision is zero (= 0) no decimals will be printed
+  * If Precision is positive (> 0) it specifies the fix number
+  * of decimals ("%.*f", Precision, Value),
+  * if negative only significant decimals will be plotted up to a
+  * maximum of the absolute value of 'Precision' digits.
+*/
+//---------------------------------------------------------
+CSG_String		SG_Get_String(double Value, int Precision)
 {
 	CSG_String	s;
 
-	if( Precision >= 0 )
+	if     ( Precision == -99 )
 	{
-		s.Printf(SG_T("%.*f"), Precision, Value);
+		s.Printf("%f", Value);
 	}
-	else if( Precision == -1 )
+	else if( Precision == -98 )
 	{
-		s.Printf(SG_T("%f"), Value);
+		s.Printf("%e", Value);
 	}
-	else // if( Precision == -2 )
+	else if( Precision ==   0 )
+	{
+		s.Printf("%d", (int)Value);
+	}
+	else if( Precision  >   0 )
+	{
+		s.Printf("%.*f", Precision, Value);
+	}
+	else // if( Precision < 0 )
 	{
 		Precision	= SG_Get_Significant_Decimals(Value, abs(Precision));
 
-		s.Printf(SG_T("%.*f"), SG_Get_Significant_Decimals(Value, abs(Precision)), Value);
-
-		if( Precision > 0 )
+		if( Precision == 0 )
 		{
+			s.Printf("%.0f", Value);
+		}
+		else // if( Precision > 0 )
+		{
+			s.Printf("%.*f", Precision, Value);
+
 			while( s.Length() > 1 && s[s.Length() - 1] == '0' )
 			{
 				s	= s.Left(s.Length() - 1);
@@ -1158,6 +1383,22 @@ CSG_String		SG_Get_String(double Value, int Precision, bool bScientific)
 	return( s );
 }
 
+//---------------------------------------------------------
+CSG_String		SG_Get_String(int Value, int Precision)
+{
+	if( Precision > 0 )
+	{
+		return( SG_Get_String((double)Value, Precision) );
+	}
+
+	if( Precision < 0 )
+	{
+		return( CSG_String::Format("%0*d", Precision, Value) );
+	}
+
+	return( CSG_String::Format("%d", Value) );
+}
+
 
 ///////////////////////////////////////////////////////////
 //														 //
@@ -1244,6 +1485,26 @@ void CSG_String_Tokenizer::Set_String(const CSG_String &String, const CSG_String
 }
 
 
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+CSG_Strings SG_String_Tokenize(const CSG_String &String, const CSG_String &Delimiters, TSG_String_Tokenizer_Mode Mode)
+{
+	CSG_Strings	Strings;
+
+	CSG_String_Tokenizer	Tokenizer(String, Delimiters, Mode);
+
+	while( Tokenizer.Has_More_Tokens() )
+	{
+		Strings	+= Tokenizer.Get_Next_Token();
+	}
+
+	return( Strings );
+}
+
+
 ///////////////////////////////////////////////////////////
 //														 //
 //														 //
