@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: data_manager.cpp 1495 2012-10-19 14:03:30Z oconrad $
+ * Version $Id$
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -26,7 +26,8 @@
 // This library is free software; you can redistribute   //
 // it and/or modify it under the terms of the GNU Lesser //
 // General Public License as published by the Free       //
-// Software Foundation, version 2.1 of the License.      //
+// Software Foundation, either version 2.1 of the        //
+// License, or (at your option) any later version.       //
 //                                                       //
 // This library is distributed in the hope that it will  //
 // be useful, but WITHOUT ANY WARRANTY; without even the //
@@ -36,9 +37,7 @@
 //                                                       //
 // You should have received a copy of the GNU Lesser     //
 // General Public License along with this program; if    //
-// not, write to the Free Software Foundation, Inc.,     //
-// 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, //
-// USA.                                                  //
+// not, see <http://www.gnu.org/licenses/>.              //
 //                                                       //
 //-------------------------------------------------------//
 //                                                       //
@@ -62,7 +61,7 @@
 
 //---------------------------------------------------------
 #include "data_manager.h"
-#include "module_library.h"
+#include "tool_library.h"
 
 
 ///////////////////////////////////////////////////////////
@@ -92,8 +91,6 @@ CSG_Data_Collection::CSG_Data_Collection(CSG_Data_Manager *pManager, TSG_Data_Ob
 {
 	m_pManager	= pManager;
 	m_Type		= Type;
-
-	m_Objects.Create(sizeof(CSG_Data_Object *));
 }
 
 //---------------------------------------------------------
@@ -103,11 +100,11 @@ CSG_Data_Collection::~CSG_Data_Collection(void)
 }
 
 //---------------------------------------------------------
-CSG_Data_Object * CSG_Data_Collection::Get(const CSG_String &File) const
+CSG_Data_Object * CSG_Data_Collection::Get(const CSG_String &File, bool bNative) const
 {
 	for(size_t i=0; i<Count(); i++)
 	{
-		if( !File.Cmp(Get(i)->Get_File_Name()) )
+		if( !File.Cmp(Get(i)->Get_File_Name(bNative)) )
 		{
 			return( Get(i) );
 		}
@@ -133,7 +130,7 @@ bool CSG_Data_Collection::Exists(CSG_Data_Object *pObject) const
 //---------------------------------------------------------
 bool CSG_Data_Collection::Add(CSG_Data_Object *pObject)
 {
-	if( pObject != DATAOBJECT_NOTSET && pObject != DATAOBJECT_CREATE && pObject->Get_ObjectType() == m_Type )
+	if( pObject != DATAOBJECT_NOTSET && pObject != DATAOBJECT_CREATE )
 	{
 		if( Exists(pObject) )
 		{
@@ -142,7 +139,7 @@ bool CSG_Data_Collection::Add(CSG_Data_Object *pObject)
 
 		if( m_Objects.Inc_Array() )
 		{
-			((CSG_Data_Object **)m_Objects.Get_Array())[Count() - 1]	= pObject;
+			m_Objects[Count() - 1]	= pObject;
 
 			if( m_pManager == &g_Data_Manager )
 			{
@@ -235,15 +232,35 @@ bool CSG_Data_Collection::Delete_Unsaved(bool bDetachOnly)
 
 //---------------------------------------------------------
 CSG_Grid_Collection::CSG_Grid_Collection(CSG_Data_Manager *pManager)
-	: CSG_Data_Collection(pManager, DATAOBJECT_TYPE_Grid)
+	: CSG_Data_Collection(pManager, SG_DATAOBJECT_TYPE_Grid)
 {}
 
 //---------------------------------------------------------
 bool CSG_Grid_Collection::Exists(CSG_Data_Object *pObject) const
 {
-	if( pObject != DATAOBJECT_NOTSET && pObject != DATAOBJECT_CREATE )
+	if( pObject == DATAOBJECT_NOTSET || pObject == DATAOBJECT_CREATE )
 	{
-		return( CSG_Data_Collection::Exists(pObject) );
+		return( false );
+	}
+
+	for(size_t i=0; i<Count(); i++)
+	{
+		if( pObject == Get(i) )
+		{
+			return( true );
+		}
+		else if( Get(i)->Get_ObjectType() == SG_DATAOBJECT_TYPE_Grids )
+		{	// does object (CSG_Grid) belong to a grid collection (CSG_Grids)
+			CSG_Grids	*pGrids	= (CSG_Grids *)Get(i);
+
+			for(int j=0; j<pGrids->Get_NZ(); j++)
+			{
+				if( pObject == pGrids->Get_Grid_Ptr(j) )
+				{
+					return( true );
+				}
+			}
+		}
 	}
 
 	return( false );
@@ -252,9 +269,17 @@ bool CSG_Grid_Collection::Exists(CSG_Data_Object *pObject) const
 //---------------------------------------------------------
 bool CSG_Grid_Collection::Add(CSG_Data_Object *pObject)
 {
-	if( pObject != DATAOBJECT_NOTSET && pObject != DATAOBJECT_CREATE && pObject->Get_ObjectType() == DATAOBJECT_TYPE_Grid )
+	if( pObject != DATAOBJECT_NOTSET && pObject != DATAOBJECT_CREATE )
 	{
-		CSG_Grid_System	System	= ((CSG_Grid *)pObject)->Get_System();
+		CSG_Grid_System	System;
+
+		switch( pObject->Get_ObjectType() )
+		{
+		case SG_DATAOBJECT_TYPE_Grid :	System	= ((CSG_Grid  *)pObject)->Get_System(); break;
+		case SG_DATAOBJECT_TYPE_Grids:	System	= ((CSG_Grids *)pObject)->Get_System(); break;
+
+		default:	return( false );
+		}
 
 		if( System.is_Valid() )
 		{
@@ -283,12 +308,10 @@ bool CSG_Grid_Collection::Add(CSG_Data_Object *pObject)
 //---------------------------------------------------------
 CSG_Data_Manager::CSG_Data_Manager(void)
 {
-	m_pTable		= new CSG_Data_Collection(this, DATAOBJECT_TYPE_Table     );
-	m_pTIN			= new CSG_Data_Collection(this, DATAOBJECT_TYPE_TIN       );
-	m_pPoint_Cloud	= new CSG_Data_Collection(this, DATAOBJECT_TYPE_PointCloud);
-	m_pShapes		= new CSG_Data_Collection(this, DATAOBJECT_TYPE_Shapes    );
-
-	m_Grid_Systems.Create(sizeof(CSG_Grid_Collection *));
+	m_pTable		= new CSG_Data_Collection(this, SG_DATAOBJECT_TYPE_Table     );
+	m_pTIN			= new CSG_Data_Collection(this, SG_DATAOBJECT_TYPE_TIN       );
+	m_pPoint_Cloud	= new CSG_Data_Collection(this, SG_DATAOBJECT_TYPE_PointCloud);
+	m_pShapes		= new CSG_Data_Collection(this, SG_DATAOBJECT_TYPE_Shapes    );
 }
 
 //---------------------------------------------------------
@@ -314,14 +337,14 @@ CSG_Data_Collection * CSG_Data_Manager::_Get_Collection(CSG_Data_Object *pObject
 	{
 		switch( pObject->Get_ObjectType() )
 		{
+		case SG_DATAOBJECT_TYPE_Table     : return( m_pTable       );
+		case SG_DATAOBJECT_TYPE_TIN       : return( m_pTIN         );
+		case SG_DATAOBJECT_TYPE_PointCloud: return( m_pPoint_Cloud );
+		case SG_DATAOBJECT_TYPE_Shapes    : return( m_pShapes      );
+		case SG_DATAOBJECT_TYPE_Grid      : return( Get_Grid_System(((CSG_Grid  *)pObject)->Get_System()) );
+		case SG_DATAOBJECT_TYPE_Grids     : return( Get_Grid_System(((CSG_Grids *)pObject)->Get_System()) );
+
 		default: break;
-
-		case DATAOBJECT_TYPE_Table:      return( m_pTable       );
-		case DATAOBJECT_TYPE_TIN:        return( m_pTIN         );
-		case DATAOBJECT_TYPE_PointCloud: return( m_pPoint_Cloud );
-		case DATAOBJECT_TYPE_Shapes:     return( m_pShapes      );
-
-		case DATAOBJECT_TYPE_Grid:       return( Get_Grid_System(((CSG_Grid *)pObject)->Get_System()) );
 		}
 	}
 
@@ -372,18 +395,18 @@ bool CSG_Data_Manager::Exists(CSG_Data_Object *pObject) const
 }
 
 //---------------------------------------------------------
-CSG_Data_Object *  CSG_Data_Manager::Find(const CSG_String &File) const
+CSG_Data_Object *  CSG_Data_Manager::Find(const CSG_String &File, bool bNative) const
 {
 	CSG_Data_Object	*pObject;
 
-	if( (pObject = m_pTable      ->Get(File)) != NULL )	return( pObject );
-	if( (pObject = m_pTIN        ->Get(File)) != NULL )	return( pObject );
-	if( (pObject = m_pPoint_Cloud->Get(File)) != NULL )	return( pObject );
-	if( (pObject = m_pShapes     ->Get(File)) != NULL )	return( pObject );
+	if( (pObject = m_pTable      ->Get(File, bNative)) != NULL )	return( pObject );
+	if( (pObject = m_pTIN        ->Get(File, bNative)) != NULL )	return( pObject );
+	if( (pObject = m_pPoint_Cloud->Get(File, bNative)) != NULL )	return( pObject );
+	if( (pObject = m_pShapes     ->Get(File, bNative)) != NULL )	return( pObject );
 
 	for(size_t i=0; i<Grid_System_Count(); i++)
 	{
-		if( (pObject = Get_Grid_System(i)->Get(File)) != NULL )	return( pObject );
+		if( (pObject = Get_Grid_System(i)->Get(File, bNative)) != NULL )	return( pObject );
 	}
 
 	return(	NULL );
@@ -399,44 +422,54 @@ bool CSG_Data_Manager::Add(CSG_Data_Object *pObject)
 {
 	CSG_Data_Collection	*pCollection	= _Get_Collection(pObject);
 
-	if( pCollection == NULL && pObject != DATAOBJECT_NOTSET && pObject != DATAOBJECT_CREATE && pObject->Get_ObjectType() == DATAOBJECT_TYPE_Grid && m_Grid_Systems.Inc_Array() )
+	if( pCollection == NULL && pObject != DATAOBJECT_NOTSET && pObject != DATAOBJECT_CREATE && (pObject->Get_ObjectType() == SG_DATAOBJECT_TYPE_Grid || pObject->Get_ObjectType() == SG_DATAOBJECT_TYPE_Grids) && m_Grid_Systems.Inc_Array() )
 	{
 		pCollection	= new CSG_Grid_Collection(this);
 
-		((CSG_Data_Collection **)m_Grid_Systems.Get_Array())[m_Grid_Systems.Get_Size() - 1]	= pCollection;
+		m_Grid_Systems[m_Grid_Systems.Get_Size() - 1]	= pCollection;
 	}
 
 	return( pCollection && pCollection->Add(pObject) );
 }
 
 //---------------------------------------------------------
-bool CSG_Data_Manager::Add(const CSG_String &File, TSG_Data_Object_Type Type)
+CSG_Data_Object * CSG_Data_Manager::Add(const CSG_String &File, TSG_Data_Object_Type Type)
 {
 	//-----------------------------------------------------
-	if( Type == DATAOBJECT_TYPE_Undefined )
+	if( Type == SG_DATAOBJECT_TYPE_Undefined )
 	{
-		if( SG_File_Cmp_Extension(File, SG_T("txt" ))
-		||	SG_File_Cmp_Extension(File, SG_T("csv" ))
-		||	SG_File_Cmp_Extension(File, SG_T("dbf" )) )
+		if( SG_File_Cmp_Extension(File, "txt"     )
+		||	SG_File_Cmp_Extension(File, "csv"     )
+		||	SG_File_Cmp_Extension(File, "dbf"     ) )
 		{
-			Type	= DATAOBJECT_TYPE_Table;
+			Type	= SG_DATAOBJECT_TYPE_Table;
 		}
 
-		if( SG_File_Cmp_Extension(File, SG_T("shp" )) )
+		if( SG_File_Cmp_Extension(File, "shp"     ) )
 		{
-			Type	= DATAOBJECT_TYPE_Shapes;
+			Type	= SG_DATAOBJECT_TYPE_Shapes;
 		}
 
-		if( SG_File_Cmp_Extension(File, SG_T("spc" )) )
+		if( SG_File_Cmp_Extension(File, "sg-pts-z")
+		||  SG_File_Cmp_Extension(File, "sg-pts"  )
+		||  SG_File_Cmp_Extension(File, "spc"     ) )
 		{
-			Type	= DATAOBJECT_TYPE_PointCloud;
+			Type	= SG_DATAOBJECT_TYPE_PointCloud;
 		}
 
-		if(	SG_File_Cmp_Extension(File, SG_T("sgrd"))
-		||	SG_File_Cmp_Extension(File, SG_T("dgm" ))
-		||	SG_File_Cmp_Extension(File, SG_T("grd" )) )
+		if(	SG_File_Cmp_Extension(File, "sg-grd-z")
+		||	SG_File_Cmp_Extension(File, "sg-grd"  )
+		||	SG_File_Cmp_Extension(File, "sgrd"    )
+		||	SG_File_Cmp_Extension(File, "dgm"     )
+		||	SG_File_Cmp_Extension(File, "grd"     ) )
 		{
-			Type	= DATAOBJECT_TYPE_Grid;
+			Type	= SG_DATAOBJECT_TYPE_Grid;
+		}
+
+		if( SG_File_Cmp_Extension(File, "sg-gds-z")
+		||  SG_File_Cmp_Extension(File, "sg-gds"  ) )
+		{
+			Type	= SG_DATAOBJECT_TYPE_Grids;
 		}
 	}
 
@@ -445,19 +478,20 @@ bool CSG_Data_Manager::Add(const CSG_String &File, TSG_Data_Object_Type Type)
 
 	switch( Type )
 	{
-	default:							pObject	= NULL;							break;
-	case DATAOBJECT_TYPE_Table:			pObject	= new CSG_Table			(File);	break;
-	case DATAOBJECT_TYPE_Shapes:		pObject	= new CSG_Shapes		(File);	break;
-	case DATAOBJECT_TYPE_TIN:			pObject	= new CSG_TIN			(File);	break;
-	case DATAOBJECT_TYPE_PointCloud:	pObject	= new CSG_PointCloud	(File);	break;
-	case DATAOBJECT_TYPE_Grid:			pObject	= new CSG_Grid			(File);	break;
+	case SG_DATAOBJECT_TYPE_Table     : pObject = new CSG_Table     (File); break;
+	case SG_DATAOBJECT_TYPE_Shapes    : pObject = new CSG_Shapes    (File); break;
+	case SG_DATAOBJECT_TYPE_TIN       : pObject = new CSG_TIN       (File); break;
+	case SG_DATAOBJECT_TYPE_PointCloud: pObject = new CSG_PointCloud(File); break;
+	case SG_DATAOBJECT_TYPE_Grid      : pObject = new CSG_Grid      (File); break;
+	case SG_DATAOBJECT_TYPE_Grids     : pObject = new CSG_Grids     (File); break;
+	default                           : pObject = NULL                    ; break;
 	}
 
 	if( pObject )
 	{
-		if( pObject->is_Valid() )
+		if( pObject->is_Valid() && Add(pObject) )
 		{
-			return( Add(pObject) );
+			return( pObject );
 		}
 
 		delete(pObject);
@@ -468,62 +502,95 @@ bool CSG_Data_Manager::Add(const CSG_String &File, TSG_Data_Object_Type Type)
 }
 
 //---------------------------------------------------------
-bool CSG_Data_Manager::_Add_External(const CSG_String &File)
+CSG_Data_Object * CSG_Data_Manager::_Add_External(const CSG_String &File)
 {
-	//-----------------------------------------------------
-	bool		bResult	= false;
-
-	CSG_Module	*pImport;
+	CSG_Data_Object *pData	= NULL;
 
 	if( !SG_File_Exists(File) )
 	{
-		return( false );
+		return( pData );
 	}
 
+	CSG_Tool	*pImport	= NULL;
+
+	SG_UI_Msg_Lock(true);
+
 	//-----------------------------------------------------
 	// Image Import
 
-	if(	(	SG_File_Cmp_Extension(File, SG_T("bmp"))
-		||	SG_File_Cmp_Extension(File, SG_T("gif"))
-		||	SG_File_Cmp_Extension(File, SG_T("jpg"))
-		||	SG_File_Cmp_Extension(File, SG_T("png"))
-		||	SG_File_Cmp_Extension(File, SG_T("pcx")) )
-	&&  (pImport = SG_Get_Module_Library_Manager().Get_Module(SG_T("io_grid_image"), 1)) != NULL
-	&&   pImport->Get_Parameters()->Set_Parameter("FILE", File, PARAMETER_TYPE_FilePath) )
+	if(	(	SG_File_Cmp_Extension(File, "bmp")
+		||	SG_File_Cmp_Extension(File, "gif")
+		||	SG_File_Cmp_Extension(File, "jpg")
+		||	SG_File_Cmp_Extension(File, "png")
+		||	SG_File_Cmp_Extension(File, "pcx") )
+	&&  (pImport = SG_Get_Tool_Library_Manager().Create_Tool("io_grid_image", 1)) != NULL
+	&&   pImport->Set_Parameter("FILE", File, PARAMETER_TYPE_FilePath) )
 	{
 		pImport->Set_Manager(this);
-		bResult	= pImport->Execute();
-		pImport->Set_Manager(&g_Data_Manager);
+
+		if( pImport->Execute() )
+		{
+			pData	= pImport->Get_Parameter("OUT_GRID")->asDataObject();
+		}
 	}
 
+	SG_Get_Tool_Library_Manager().Delete_Tool(pImport);
+
 	//-----------------------------------------------------
 	// GDAL Import
 
-	if( !bResult
-	&&  (pImport = SG_Get_Module_Library_Manager().Get_Module(SG_T("io_gdal"), 0)) != NULL
-	&&   pImport->Get_Parameters()->Set_Parameter(SG_T("FILES"), File, PARAMETER_TYPE_FilePath) )
+	if( !pData
+	&&  (pImport = SG_Get_Tool_Library_Manager().Create_Tool("io_gdal", 0)) != NULL
+	&&   pImport->Set_Parameter("FILES", File, PARAMETER_TYPE_FilePath) )
 	{
 		pImport->Set_Manager(this);
-		bResult	= pImport->Execute();
-		if( this != &g_Data_Manager )	{	pImport->Set_Manager(&g_Data_Manager);	}
+
+		if( pImport->Execute() )
+		{
+			pData	= pImport->Get_Parameter("GRIDS")->asList()->Get_Item(0);
+		}
 	}
 
+	SG_Get_Tool_Library_Manager().Delete_Tool(pImport);
+
 	//-----------------------------------------------------
 	// OGR Import
 
-	if( !bResult
-	&&  (pImport = SG_Get_Module_Library_Manager().Get_Module(SG_T("io_gdal"), 3)) != NULL
-	&&   pImport->Get_Parameters()->Set_Parameter(SG_T("FILES"), File, PARAMETER_TYPE_FilePath) )
+	if( !pData
+	&&  (pImport = SG_Get_Tool_Library_Manager().Create_Tool("io_gdal", 3)) != NULL
+	&&   pImport->Set_Parameter("FILES", File, PARAMETER_TYPE_FilePath) )
 	{
 		pImport->Set_Manager(this);
-		bResult	= pImport->Execute();
-		pImport->Set_Manager(&g_Data_Manager);
+
+		if( pImport->Execute() )
+		{
+			pData	= pImport->Get_Parameter("SHAPES")->asList()->Get_Item(0);
+		}
 	}
 
+	SG_Get_Tool_Library_Manager().Delete_Tool(pImport);
+
 	//-----------------------------------------------------
+	// LAS Import
 
+	if( !pData && SG_File_Cmp_Extension(File, "las")
+	&&  (pImport = SG_Get_Tool_Library_Manager().Create_Tool("io_shapes_las", 1)) != NULL
+	&&   pImport->Set_Parameter("FILES", File, PARAMETER_TYPE_FilePath) )
+	{
+		pImport->Set_Manager(this);
 
-	return( bResult );
+		if( pImport->Execute() )
+		{
+			pData	= pImport->Get_Parameter("POINTS")->asDataObject();
+		}
+	}
+
+	SG_Get_Tool_Library_Manager().Delete_Tool(pImport);
+
+	//-----------------------------------------------------
+	SG_UI_Msg_Lock(false);
+
+	return( pData );
 }
 
 
@@ -622,7 +689,7 @@ bool CSG_Data_Manager::Delete(CSG_Data_Collection *pCollection, bool bDetachOnly
 	if( pCollection == m_pShapes      )	{	return( pCollection->Delete_All(bDetachOnly) );	}
 
 	//-----------------------------------------------------
-	if( pCollection->m_Type == DATAOBJECT_TYPE_Grid )
+	if( pCollection->m_Type == SG_DATAOBJECT_TYPE_Grid )
 	{
 		CSG_Grid_Collection	**pSystems	= (CSG_Grid_Collection **)m_Grid_Systems.Get_Array();
 
@@ -664,7 +731,7 @@ bool CSG_Data_Manager::Delete(CSG_Data_Object *pObject, bool bDetachOnly)
 
 	if( pCollection && pCollection->Delete(pObject, bDetachOnly) )
 	{
-		if( pCollection->m_Type == DATAOBJECT_TYPE_Grid && pCollection->Count() == 0 )
+		if( pCollection->m_Type == SG_DATAOBJECT_TYPE_Grid && pCollection->Count() == 0 )
 		{
 			Delete(pCollection, bDetachOnly);
 		}
@@ -731,47 +798,134 @@ bool CSG_Data_Manager::Delete_Unsaved(bool bDetachOnly)
 //														 //
 ///////////////////////////////////////////////////////////
 
-//---------------------------------------------------------
-#define SUMMARY_ADD_INT(label, value)	s += CSG_String::Format(SG_T("<tr><td valign=\"top\">%s</td><td valign=\"top\">%d</td></tr>"), label, value)
-
 //---------------------------------------------------------
 CSG_String CSG_Data_Manager::Get_Summary(void)	const
 {
-	//-----------------------------------------------------
 	CSG_String	s;
 
-/*	s	+= CSG_String::Format(SG_T("<b>%s</b>"), _TL("Tool Libraries"));
-
-	s	+= SG_T("<table border=\"0\">");
-
-	SUMMARY_ADD_INT(_TL("Available Libraries"), Get_Count());
-	SUMMARY_ADD_INT(_TL("Available Tools"    ), nModules);
-
-	s	+= SG_T("</table>");
-
 	//-----------------------------------------------------
-	s	+= CSG_String::Format(SG_T("<hr><b>%s:</b><table border=\"1\">"), _TL("Tool Libraries"));
-
-	s	+= CSG_String::Format(SG_T("<tr><th>%s</th><th>%s</th><th>%s</th><th>%s</th></tr>"),
-			_TL("Library"),
-			_TL("Tools"),
-			_TL("Name"),
-			_TL("Location")
-		);
-
-	for(i=0; i<Get_Count(); i++)
+	if( Get_Table()->Count() > 0 )
 	{
-		s	+= CSG_String::Format(SG_T("<tr><td>%s</td><td>%d</td><td>%s</td><td>%s</td></tr>"),
-				SG_File_Get_Name(Get_Library(i)->Get_File_Name(), false).c_str(),
-				Get_Library(i)->Get_Count(),
-				Get_Library(i)->Get_Name().c_str(),
-				SG_File_Get_Path(Get_Library(i)->Get_File_Name()).c_str()
+		s	+= CSG_String::Format("___\n%s [%d %s]\n", _TL("Table"), Get_Table()->Count(), _TL("objects"));
+
+		for(size_t i=0; i<Get_Table()->Count(); i++)
+		{
+			CSG_Table	*pObject	= (CSG_Table *)Get_Table()->Get(i);
+
+			s	+= CSG_String::Format("- [%d %s] %s\n",
+				pObject->Get_Count(), _TL("records"),
+				pObject->Get_Name()
 			);
+		}
 	}
 
-	s	+= SG_T("</table>");
+	//-----------------------------------------------------
+	if( Get_Shapes()->Count() > 0 )
+	{
+		s	+= CSG_String::Format("___\n%s [%d %s]\n", _TL("Shapes"), Get_Shapes()->Count(), _TL("objects"));
+
+		for(size_t i=0; i<Get_Shapes()->Count(); i++)
+		{
+			CSG_Shapes	*pObject	= (CSG_Shapes *)Get_Shapes()->Get(i);
+
+			s	+= CSG_String::Format("- [%s; %d %s] %s\n",
+				pObject->Get_Type() == SHAPE_TYPE_Point   ? _TL("point"  ) :
+				pObject->Get_Type() == SHAPE_TYPE_Points  ? _TL("points" ) :
+				pObject->Get_Type() == SHAPE_TYPE_Line    ? _TL("line"   ) :
+				pObject->Get_Type() == SHAPE_TYPE_Polygon ? _TL("polygon") : _TL("unknown"),
+				pObject->Get_Count(), _TL("records"),
+				pObject->Get_Name()
+			);
+		}
+	}
+
+	//-----------------------------------------------------
+	if( Get_Point_Cloud()->Count() > 0 )
+	{
+		s	+= CSG_String::Format("___\n%s [%d %s]\n", _TL("Point Cloud"), Get_Point_Cloud()->Count(), _TL("objects"));
+
+		for(size_t i=0; i<Get_Point_Cloud()->Count(); i++)
+		{
+			CSG_PointCloud	*pObject	= (CSG_PointCloud *)Get_Point_Cloud()->Get(i);
+
+			s	+= CSG_String::Format("- [%d %s] %s\n",
+				pObject->Get_Count(), _TL("records"),
+				pObject->Get_Name()
+			);
+		}
+	}
+
+	//-----------------------------------------------------
+//	if( Get_TIN()->Count() > 0 )
+//	{
+//		s	+= CSG_String::Format("___\n%s [%d %s]\n", _TL("TIN"), Get_TIN()->Count(), _TL("objects"));
+//
+//		for(size_t i=0; i<Get_TIN()->Count(); i++)
+//		{
+//			CSG_TIN	*pObject	= (CSG_TIN *)Get_TIN()->Get(i);
+//
+//			s	+= CSG_String::Format("- [%d %s] %s\n",
+//				pObject->Get_Count(), _TL("nodes"),
+//				pObject->Get_Name()
+//			);
+//		}
+//	}
+
+	//-----------------------------------------------------
+	if( Grid_System_Count() > 0 )
+	{
+		sLong	memory	= 0;
+
+		for(size_t i=0; i<Grid_System_Count(); i++)
+		{
+			CSG_Grid_Collection	*pSystem	= Get_Grid_System(i);
+
+			s	+= CSG_String::Format("___\n%s [%s; %d %s]\n", _TL("Grid System"), pSystem->m_System.Get_Name(), pSystem->Count(), _TL("objects"));
+
+			for(size_t j=0; j<pSystem->Count(); j++)
+			{
+				if( pSystem->Get(j)->Get_ObjectType() == SG_DATAOBJECT_TYPE_Grid )
+				{
+					CSG_Grid	*pObject	= (CSG_Grid *)pSystem->Get(j);
+
+					s	+= CSG_String::Format("- [%s; %.1fmb] %s\n",
+						SG_Data_Type_Get_Name(pObject->Get_Type()).c_str(),
+						pObject->Get_Memory_Size() / (double)N_MEGABYTE_BYTES,
+						pObject->Get_Name()
+					);
+
+					memory	+= pObject->Get_Memory_Size();
+				}
+
+				if( pSystem->Get(j)->Get_ObjectType() == SG_DATAOBJECT_TYPE_Grids )
+				{
+					CSG_Grids	*pObject	= (CSG_Grids *)pSystem->Get(j);
+
+					s	+= CSG_String::Format("- [%s; %d %s; %.1fmb] %s\n",
+						SG_Data_Type_Get_Name(pObject->Get_Type()).c_str(),
+						pObject->Get_NZ(), _TL("grids"),
+						pObject->Get_Memory_Size() / (double)N_MEGABYTE_BYTES,
+						pObject->Get_Name()
+					);
+
+					memory	+= pObject->Get_Memory_Size();
+				}
+			}
+		}
+
+		s	+= CSG_String::Format("_\n%s: %.1fmb\n", _TL("Total memory in use by grids"), memory / (double)N_MEGABYTE_BYTES);
+	}
+
+	//-----------------------------------------------------
+	if( s.is_Empty() )
+	{
+		s	+= CSG_String::Format("%s - %s\n--- %s ---\n", _TL("Data Manager"), _TL("Summary"), _TL("no data"));
+	}
+	else
+	{
+		s.Prepend(CSG_String::Format("%s - %s\n", _TL("Data Manager"), _TL("Summary")));
+	}
 
-*/
 	//-----------------------------------------------------
 	return( s );
 }
