@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: geo_tools.h 2380 2015-01-29 16:51:02Z oconrad $
+ * Version $Id$
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -26,7 +26,8 @@
 // This library is free software; you can redistribute   //
 // it and/or modify it under the terms of the GNU Lesser //
 // General Public License as published by the Free       //
-// Software Foundation, version 2.1 of the License.      //
+// Software Foundation, either version 2.1 of the        //
+// License, or (at your option) any later version.       //
 //                                                       //
 // This library is distributed in the hope that it will  //
 // be useful, but WITHOUT ANY WARRANTY; without even the //
@@ -36,9 +37,7 @@
 //                                                       //
 // You should have received a copy of the GNU Lesser     //
 // General Public License along with this program; if    //
-// not, write to the Free Software Foundation, Inc.,     //
-// 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, //
-// USA.                                                  //
+// not, see <http://www.gnu.org/licenses/>.              //
 //                                                       //
 //-------------------------------------------------------//
 //                                                       //
@@ -557,7 +556,7 @@ public:
 	virtual ~CSG_Distance_Weighting(void);
 
 	bool					Create_Parameters	(class CSG_Parameters *pParameters, bool bDialog = true);
-	int						Enable_Parameters	(class CSG_Parameters *pParameters);
+	static bool				Enable_Parameters	(class CSG_Parameters *pParameters);
 	bool					Set_Parameters		(class CSG_Parameters *pParameters);
 	class CSG_Parameters *	Get_Parameters		(void)	const		{	return( m_pParameters );	}
 
@@ -700,23 +699,33 @@ public:
 	bool							Assign					(const CSG_Projection &Projection);
 	CSG_Projection &				operator =				(const CSG_Projection &Projection)	{	Assign(Projection);	return( *this );	}
 
-									CSG_Projection			(int EPSG_ID);
-	bool							Create					(int EPSG_ID);
-	bool							Assign					(int EPSG_ID);
-	CSG_Projection &				operator =				(int EPSG_ID)						{	Assign(EPSG_ID);	return( *this );	}
+									CSG_Projection			(int Authority_ID, const SG_Char *Authority = NULL);
+	bool							Create					(int Authority_ID, const SG_Char *Authority = NULL);
+	bool							Assign					(int Authority_ID, const SG_Char *Authority = NULL);
+	CSG_Projection &				operator =				(int Authority_ID)					{	Assign(Authority_ID);	return( *this );	}
 
 									CSG_Projection			(const CSG_String &Projection, TSG_Projection_Format Format = SG_PROJ_FMT_WKT);
 	bool							Create					(const CSG_String &Projection, TSG_Projection_Format Format = SG_PROJ_FMT_WKT);
 	bool							Assign					(const CSG_String &Projection, TSG_Projection_Format Format = SG_PROJ_FMT_WKT);
 	CSG_Projection &				operator =				(const CSG_String &Projection)		{	Assign(Projection);	return( *this );	}
 
+									CSG_Projection			(const CSG_String &WKT, const CSG_String &Proj4);
+	bool							Create					(const CSG_String &WKT, const CSG_String &Proj4);
+	bool							Assign					(const CSG_String &WKT, const CSG_String &Proj4);
+
 	bool							is_Okay					(void)	const	{	return( m_Type != SG_PROJ_TYPE_CS_Undefined );	}
 	bool							is_Equal				(const CSG_Projection &Projection)	const;
 	bool							operator ==				(const CSG_Projection &Projection)	const	{	return( is_Equal(Projection) == true  );	}
 	bool							operator !=				(const CSG_Projection &Projection)	const	{	return( is_Equal(Projection) == false );	}
 
-	bool							Load					(const CSG_String &File_Name, TSG_Projection_Format Format = SG_PROJ_FMT_WKT);
-	bool							Save					(const CSG_String &File_Name, TSG_Projection_Format Format = SG_PROJ_FMT_WKT)	const;
+	bool							Set_GCS_WGS84			(void);
+	bool							Set_UTM_WGS84			(int Zone, bool bSouth = false);
+
+	bool							Load					(const CSG_String &FileName, TSG_Projection_Format Format = SG_PROJ_FMT_WKT);
+	bool							Save					(const CSG_String &FileName, TSG_Projection_Format Format = SG_PROJ_FMT_WKT)	const;
+
+	bool							Load					(CSG_File &Stream, TSG_Projection_Format Format = SG_PROJ_FMT_WKT);
+	bool							Save					(CSG_File &Stream, TSG_Projection_Format Format = SG_PROJ_FMT_WKT)	const;
 
 	bool							Load					(const CSG_MetaData &Projection);
 	bool							Save					(      CSG_MetaData &Projection)	const;
@@ -726,7 +735,7 @@ public:
 	const CSG_String &				Get_Proj4				(void)	const	{	return( m_Proj4         );	}
 	const CSG_String &				Get_Authority			(void)	const	{	return( m_Authority     );	}
 	int								Get_Authority_ID		(void)	const	{	return( m_Authority_ID  );	}
-	int								Get_EPSG				(void)	const	{	return( m_Authority.Cmp(SG_T("EPSG")) ? -1 : m_Authority_ID );	}
+	int								Get_EPSG				(void)	const	{	return( m_Authority.Cmp("EPSG") ? -1 : m_Authority_ID );	}
 
 	CSG_String						Get_Description			(void)	const;
 
@@ -777,11 +786,11 @@ public:
 	void							Destroy					(void);
 
 	bool							Reset_Dictionary		(void);
-	bool							Load_Dictionary			(const CSG_String &File_Name);
-	bool							Save_Dictionary			(const CSG_String &File_Name);
+	bool							Load_Dictionary			(const CSG_String &FileName);
+	bool							Save_Dictionary			(const CSG_String &FileName);
 
-	bool							Load_DB					(const CSG_String &File_Name, bool bAppend = false);
-	bool							Save_DB					(const CSG_String &File_Name);
+	bool							Load_DB					(const CSG_String &FileName, bool bAppend = false);
+	bool							Save_DB					(const CSG_String &FileName);
 
 	int								Get_Count				(void)	const;
 
@@ -812,9 +821,11 @@ private:
 	class CSG_Table					*m_pProjections;
 
 
-	void							_On_Construction		(void);
+	void							_On_Construction			(void);
 
-	static bool						_WKT_to_MetaData		(CSG_MetaData &MetaData, const CSG_String &WKT);
+	static bool						_WKT_to_MetaData			(CSG_MetaData &MetaData, const CSG_String &WKT);
+
+	bool							_WKT_to_Proj4_Set_Datum		(CSG_String &Proj4, const CSG_MetaData &WKT)	const;
 
 	bool							_Proj4_Read_Parameter		(CSG_String &Value, const CSG_String &Proj4, const CSG_String &Key)	const;
 	bool							_Proj4_Get_Ellipsoid		(CSG_String &Value, const CSG_String &Proj4)	const;
@@ -830,6 +841,12 @@ private:
 //---------------------------------------------------------
 SAGA_API_DLL_EXPORT CSG_Projections &	SG_Get_Projections	(void);
 
+//---------------------------------------------------------
+SAGA_API_DLL_EXPORT bool		SG_Get_Projected				(class CSG_Shapes *pSource, class CSG_Shapes *pTarget, const CSG_Projection &Target);
+
+SAGA_API_DLL_EXPORT bool		SG_Get_Projected				(const CSG_Projection &Source, const CSG_Projection &Target, TSG_Point &Point    );
+SAGA_API_DLL_EXPORT bool		SG_Get_Projected				(const CSG_Projection &Source, const CSG_Projection &Target, TSG_Rect  &Rectangle);
+
 
 ///////////////////////////////////////////////////////////
 //														 //
@@ -853,6 +870,7 @@ SAGA_API_DLL_EXPORT double		SG_Get_Angle_Of_Direction		(double dx, double dy);
 SAGA_API_DLL_EXPORT double		SG_Get_Angle_Of_Direction		(double ax, double ay, double bx, double by);
 SAGA_API_DLL_EXPORT double		SG_Get_Angle_Of_Direction		(const TSG_Point &A);
 SAGA_API_DLL_EXPORT double		SG_Get_Angle_Of_Direction		(const TSG_Point &A, const TSG_Point &B);
+SAGA_API_DLL_EXPORT double		SG_Get_Angle_Difference			(double a, double b);
 
 SAGA_API_DLL_EXPORT bool		SG_Get_Crossing					(TSG_Point &Crossing, const TSG_Point &a1, const TSG_Point &a2, const TSG_Point &b1, const TSG_Point &b2, bool bExactMatch = true);
 SAGA_API_DLL_EXPORT bool		SG_Get_Crossing_InRegion		(TSG_Point &Crossing, const TSG_Point &a , const TSG_Point & b, const TSG_Rect &Region);
