@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: grid_operation.cpp 2230 2014-09-11 16:23:09Z oconrad $
+ * Version $Id$
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -26,7 +26,8 @@
 // This library is free software; you can redistribute   //
 // it and/or modify it under the terms of the GNU Lesser //
 // General Public License as published by the Free       //
-// Software Foundation, version 2.1 of the License.      //
+// Software Foundation, either version 2.1 of the        //
+// License, or (at your option) any later version.       //
 //                                                       //
 // This library is distributed in the hope that it will  //
 // be useful, but WITHOUT ANY WARRANTY; without even the //
@@ -36,9 +37,7 @@
 //                                                       //
 // You should have received a copy of the GNU Lesser     //
 // General Public License along with this program; if    //
-// not, write to the Free Software Foundation, Inc.,     //
-// 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, //
-// USA.                                                  //
+// not, see <http://www.gnu.org/licenses/>.              //
 //                                                       //
 //-------------------------------------------------------//
 //                                                       //
@@ -77,42 +76,52 @@
 //---------------------------------------------------------
 void CSG_Grid::Assign_NoData(void)
 {
-	Assign(Get_NoData_Value());
+	double	Value	= Get_NoData_Value();
+
+	#pragma omp parallel for
+	for(sLong i=0; i<Get_NCells(); i++)
+	{
+		Set_Value(i, Value, false);
+	}
 }
 
 //---------------------------------------------------------
 bool CSG_Grid::Assign(double Value)
 {
-	if( is_Valid() )
+	if( !is_Valid() )
 	{
-		if( Value == 0.0 && m_Memory_Type == GRID_MEMORY_Normal )
-		{
-			for(int n=0, m=_Get_nLineBytes(); n<Get_NY(); n++)
-			{
-				memset(m_Values[n], 0, m);
-			}
-		}
-		else
-		{
-			for(sLong n=0; n<Get_NCells(); n++)
-			{
-				Set_Value(n, Value);
-			}
-		}
-
-		//-------------------------------------------------
-		Get_History().Destroy();
-		Get_History().Add_Child(SG_T("GRID_OPERATION"), Value)->Add_Property(SG_T("NAME"), _TL("Assign"));
-
-		//-------------------------------------------------
-		m_zStats.Invalidate();
-
-		Set_Update_Flag(false);
-
-		return( true );
+		return( false );
 	}
 
-	return( false );
+	if( Value == 0.0 && !is_Cached() )
+	{
+		#pragma omp parallel for
+		for(int y=0; y<Get_NY(); y++)
+		{
+			memset(m_Values[y], 0, Get_nLineBytes());
+		}
+	}
+	else
+	{
+		#pragma omp parallel for
+		for(int y=0; y<Get_NY(); y++)
+		{
+			for(int x=0; x<Get_NX(); x++)
+			{
+				Set_Value(x, y, Value);
+			}
+		}
+	}
+
+	//-----------------------------------------------------
+	Get_History().Destroy();
+	Get_History().Add_Child("GRID_OPERATION", Value)->Add_Property("NAME", _TL("Assign"));
+
+	m_Statistics.Invalidate();
+
+	Set_Update_Flag(false);
+
+	return( true );
 }
 
 
@@ -126,10 +135,10 @@ bool CSG_Grid::Assign(double Value)
 bool CSG_Grid::Assign(CSG_Data_Object *pObject)
 {
 	return( pObject && pObject->is_Valid() && pObject->Get_ObjectType() == Get_ObjectType()
-		&&  Assign((CSG_Grid *)pObject, GRID_INTERPOLATION_Undefined) );
+		&&  Assign((CSG_Grid *)pObject, GRID_RESAMPLING_Undefined) );
 }
 
-bool CSG_Grid::Assign(CSG_Grid *pGrid, TSG_Grid_Interpolation Interpolation)
+bool CSG_Grid::Assign(CSG_Grid *pGrid, TSG_Grid_Resampling Interpolation)
 {
 	//-----------------------------------------------------
 	if(	!is_Valid() || !pGrid || !pGrid->is_Valid() || is_Intersecting(pGrid->Get_Extent()) == INTERSECTION_None )
@@ -165,42 +174,41 @@ bool CSG_Grid::Assign(CSG_Grid *pGrid, TSG_Grid_Interpolation Interpolation)
 	&&	fmod(Get_XMin() - pGrid->Get_XMin(), Get_Cellsize()) == 0.0
 	&&	fmod(Get_YMin() - pGrid->Get_YMin(), Get_Cellsize()) == 0.0	)
 	{
-		bResult	= _Assign_Interpolated(pGrid, GRID_INTERPOLATION_NearestNeighbour);
+		bResult	= _Assign_Interpolated(pGrid, GRID_RESAMPLING_NearestNeighbour);
 	}
 
 	//---------------------------------------------------------
 	else switch( Interpolation )
 	{
-	case GRID_INTERPOLATION_NearestNeighbour:
-	case GRID_INTERPOLATION_Bilinear:
-	case GRID_INTERPOLATION_InverseDistance:
-	case GRID_INTERPOLATION_BicubicSpline:
-	case GRID_INTERPOLATION_BSpline:
+	case GRID_RESAMPLING_NearestNeighbour:
+	case GRID_RESAMPLING_Bilinear:
+	case GRID_RESAMPLING_BicubicSpline:
+	case GRID_RESAMPLING_BSpline:
 		bResult	= _Assign_Interpolated(pGrid, Interpolation);
 		break;
 
-	case GRID_INTERPOLATION_Mean_Nodes:
-	case GRID_INTERPOLATION_Mean_Cells:
-		bResult	= _Assign_MeanValue   (pGrid, Interpolation != GRID_INTERPOLATION_Mean_Nodes);
+	case GRID_RESAMPLING_Mean_Nodes:
+	case GRID_RESAMPLING_Mean_Cells:
+		bResult	= _Assign_MeanValue   (pGrid, Interpolation != GRID_RESAMPLING_Mean_Nodes);
 		break;
 
-	case GRID_INTERPOLATION_Minimum:
-	case GRID_INTERPOLATION_Maximum:
-		bResult	= _Assign_ExtremeValue(pGrid, Interpolation == GRID_INTERPOLATION_Maximum);
+	case GRID_RESAMPLING_Minimum:
+	case GRID_RESAMPLING_Maximum:
+		bResult	= _Assign_ExtremeValue(pGrid, Interpolation == GRID_RESAMPLING_Maximum);
 		break;
 
-	case GRID_INTERPOLATION_Majority:
+	case GRID_RESAMPLING_Majority:
 		bResult	= _Assign_Majority    (pGrid);
 		break;
 
 	default:
 		if( Get_Cellsize() < pGrid->Get_Cellsize() )	// Down-Scaling...
 		{
-			bResult	= _Assign_Interpolated(pGrid, GRID_INTERPOLATION_BSpline);
+			bResult	= _Assign_Interpolated(pGrid, GRID_RESAMPLING_BSpline);
 		}
 		else											// Up-Scaling...
 		{
-			bResult	= _Assign_MeanValue(pGrid, Interpolation != GRID_INTERPOLATION_Mean_Nodes);
+			bResult	= _Assign_MeanValue(pGrid, Interpolation != GRID_RESAMPLING_Mean_Nodes);
 		}
 		break;
 	}
@@ -225,7 +233,7 @@ bool CSG_Grid::Assign(CSG_Grid *pGrid, TSG_Grid_Interpolation Interpolation)
 }
 
 //---------------------------------------------------------
-bool CSG_Grid::_Assign_Interpolated(CSG_Grid *pGrid, TSG_Grid_Interpolation Interpolation)
+bool CSG_Grid::_Assign_Interpolated(CSG_Grid *pGrid, TSG_Grid_Resampling Interpolation)
 {
 	double	py	= Get_YMin();
 
@@ -308,120 +316,59 @@ bool CSG_Grid::_Assign_MeanValue(CSG_Grid *pGrid, bool bAreaProportional)
 	}
 
 	//-----------------------------------------------------
-	Assign_NoData();
+	double	d	= Get_Cellsize() / pGrid->Get_Cellsize();
 
-	CSG_Matrix	S(Get_NY(), Get_NX()), N(Get_NY(), Get_NX());
+	double	ox	= (Get_XMin(true) - pGrid->Get_XMin()) / pGrid->Get_Cellsize();
+	double	py	= (Get_YMin(true) - pGrid->Get_YMin()) / pGrid->Get_Cellsize();
 
-	double	d	= pGrid->Get_Cellsize() / Get_Cellsize();
-
-	//-----------------------------------------------------
-	if( bAreaProportional == false )
+	for(int y=0; y<Get_NY() && SG_UI_Process_Set_Progress(y, Get_NY()); y++, py+=d)
 	{
-		double	ax	= 0.5 + (pGrid->Get_XMin() - Get_XMin()) / Get_Cellsize();
-		double	py	= 0.5 + (pGrid->Get_YMin() - Get_YMin()) / Get_Cellsize();
+		int		ay	= (int)(bAreaProportional ? floor(py    ) : ceil (py    ));
+		int		by	= (int)(bAreaProportional ? ceil (py + d) : floor(py + d));
 
-		for(int y=0; y<pGrid->Get_NY() && SG_UI_Process_Set_Progress(y, pGrid->Get_NY()); y++, py+=d)
-		{
-			int	iy	= (int)floor(py);
-
-			if( iy >= 0 && iy < Get_NY() )
-			{
-				#pragma omp parallel for
-				for(int x=0; x<pGrid->Get_NX(); x++)
-				{
-					if( !pGrid->is_NoData(x, y) )
-					{
-						int	ix	= (int)floor(ax + x * d);
-						
-						if( ix >= 0 && ix < Get_NX() )
-						{
-							S[ix][iy]	+= pGrid->asDouble(x, y);
-							N[ix][iy]	++;
-						}
-					}
-				}
-			}
-		}
-	}
-
-	//-----------------------------------------------------
-	else // if( bAreaProportional == true )
-	{
-		double	ax	= ((pGrid->Get_XMin() - 0.5 * pGrid->Get_Cellsize()) - (Get_XMin() - 0.5 * Get_Cellsize())) / Get_Cellsize();
-		double	py	= ((pGrid->Get_YMin() - 0.5 * pGrid->Get_Cellsize()) - (Get_YMin() - 0.5 * Get_Cellsize())) / Get_Cellsize();
-
-		for(int y=0; y<pGrid->Get_NY() && SG_UI_Process_Set_Progress(y, pGrid->Get_NY()); y++, py+=d)
-		{
-			if( py > -d || py < Get_NY() )
-			{
-				int		iy	= (int)floor(py);
-
-				double	wy	= (py + d) - iy;	wy	= wy < 1.0 ? 1.0 : wy - 1.0; 
-
-				#pragma omp parallel for
-				for(int x=0; x<pGrid->Get_NX(); x++)
-				{
-					if( !pGrid->is_NoData(x, y) )
-					{
-						double	px	= ax + x * d;
-
-						if( px > -d && px < Get_NX() )
-						{
-							int		jy, jx, ix	= (int)floor(px);
-
-							double	wx	= (px + d) - ix;	wx	= wx < 1.0 ? 1.0 : wx - 1.0; 
-
-							double	z	= pGrid->asDouble(x, y);
-
-							if( iy >= 0 && iy < Get_NY() )		// wy > 0.0 is always true
-							{
-								if( ix >= 0 && ix < Get_NX() )	// wx > 0.0 is always true
-								{
-									double	w	= wx * wy;
-									S[ix][iy]	+= w * z;
-									N[ix][iy]	+= w;
-								}
-
-								if( wx < 1.0 && (jx = ix + 1) >= 0 && jx < Get_NX() )
-								{
-									double	w	= (1.0 - wx) * wy;
-									S[jx][iy]	+= w * z;
-									N[jx][iy]	+= w;
-								}
-							}
-
-							if( wy < 1.0 && (jy = iy + 1) >= 0 && jy < Get_NY() )
-							{
-								if( ix >= 0 && ix < Get_NX() )
-								{
-									double	w	= wx * (1.0 - wy);
-									S[ix][jy]	+= w * z;
-									N[ix][jy]	+= w;
-								}
-
-								if( wx < 1.0 && (jx = ix + 1) >= 0 && jx < Get_NX() )
-								{
-									double	w	= (1.0 - wx) * (1.0 - wy);
-									S[jx][jy]	+= w * z;
-									N[jx][jy]	+= w;
-								}
-							}
-						}
-					}
-				}
-			}
-		}
-	}
-
-	//-----------------------------------------------------
-	for(int y=0; y<Get_NY() && SG_UI_Process_Set_Progress(y, Get_NY()); y++)
-	{
+		//-------------------------------------------------
 		#pragma omp parallel for
 		for(int x=0; x<Get_NX(); x++)
 		{
-			if( N[x][y] > 0.0 )
+			double	px	= ox + x * d;
+
+			int		ax	= (int)(bAreaProportional ? floor(px    ) : ceil (px    ));
+			int		bx	= (int)(bAreaProportional ? ceil (px + d) : floor(px + d));
+
+			CSG_Rect	rMean(px, py, px + d, py + d);
+
+			CSG_Simple_Statistics	s;
+
+			for(int iy=ay; iy<=by; iy++)
 			{
-				Set_Value(x, y, S[x][y] / N[x][y]);
+				if( iy >= 0 && iy < pGrid->Get_NY() )
+				{
+					for(int ix=ax; ix<=bx; ix++)
+					{
+						if( ix >= 0 && ix < pGrid->Get_NX() && !pGrid->is_NoData(ix, iy) )
+						{
+							if( bAreaProportional )
+							{
+								CSG_Rect	r(ix - 0.5, iy - 0.5, ix + 0.5, iy + 0.5);
+
+								if( r.Intersect(rMean) )
+								{
+									s.Add_Value(pGrid->asDouble(ix, iy), r.Get_Area());
+								}
+							}
+							else
+							{
+								s.Add_Value(pGrid->asDouble(ix, iy));
+							}
+						}
+					}
+				}
+			}
+
+			//---------------------------------------------
+			if( s.Get_Count() > 0 )
+			{
+				Set_Value(x, y, s.Get_Mean());
 			}
 			else
 			{
@@ -448,8 +395,6 @@ bool CSG_Grid::_Assign_Majority(CSG_Grid *pGrid)
 	}
 
 	//-----------------------------------------------------
-	CSG_Class_Statistics	m;
-
 	Set_NoData_Value(pGrid->Get_NoData_Value());
 
 	Assign_NoData();
@@ -483,7 +428,7 @@ bool CSG_Grid::_Assign_Majority(CSG_Grid *pGrid)
 
 				if( ax < pGrid->Get_NX() && bx > 0 )
 				{
-					m.Reset();
+					CSG_Unique_Number_Statistics	s;
 
 					if( ax < 0 )
 					{
@@ -501,15 +446,14 @@ bool CSG_Grid::_Assign_Majority(CSG_Grid *pGrid)
 						{
 							if( !pGrid->is_NoData(ix, iy) )
 							{
-								m.Add_Value(pGrid->asDouble(ix, iy));
+								s	+= pGrid->asDouble(ix, iy);
 							}
 						}
 					}
 
-					int		n;
-					double	z;
+					int	n; double	z;
 
-					if( m.Get_Majority(z, n) )//&& n > 1 )
+					if( s.Get_Majority(z, n) )//&& n > 1 )
 					{
 						Set_Value(x, y, z);
 					}
@@ -532,7 +476,7 @@ bool CSG_Grid::_Assign_Majority(CSG_Grid *pGrid)
 //---------------------------------------------------------
 CSG_Grid & CSG_Grid::operator = (const CSG_Grid &Grid)
 {
-	Assign((CSG_Grid *)&Grid, GRID_INTERPOLATION_Undefined);
+	Assign((CSG_Grid *)&Grid, GRID_RESAMPLING_Undefined);
 
 	return( *this );
 }
@@ -696,47 +640,42 @@ CSG_Grid & CSG_Grid::_Operation_Arithmetic(const CSG_Grid &Grid, TSG_Grid_Operat
 {
 	if( is_Intersecting(Grid.Get_Extent()) )
 	{
-		int						x, y;
-		double					xWorld, yWorld, Value;
-		TSG_Grid_Interpolation	Interpolation;
+		TSG_Grid_Resampling	Interpolation	=
+			Get_Cellsize() == Grid.Get_Cellsize() && fmod(Get_XMin() - Grid.Get_XMin(), Get_Cellsize()) == 0.0
+		&&	Get_Cellsize() == Grid.Get_Cellsize() && fmod(Get_YMin() - Grid.Get_YMin(), Get_Cellsize()) == 0.0
+		?	GRID_RESAMPLING_NearestNeighbour
+		:	GRID_RESAMPLING_BSpline;
 
-		Interpolation	=	Get_Cellsize() == Grid.Get_Cellsize() && fmod(Get_XMin() - Grid.Get_XMin(), Get_Cellsize()) == 0.0
-						&&	Get_Cellsize() == Grid.Get_Cellsize() && fmod(Get_YMin() - Grid.Get_YMin(), Get_Cellsize()) == 0.0
-						?	GRID_INTERPOLATION_NearestNeighbour
-						:	GRID_INTERPOLATION_BSpline;
-
-		for(y=0, yWorld=Get_YMin(); y<Get_NY() && SG_UI_Process_Set_Progress(y, Get_NY()); y++, yWorld+=Get_Cellsize())
+		for(int y=0; y<Get_NY() && SG_UI_Process_Set_Progress(y, Get_NY()); y++)
 		{
-			for(x=0, xWorld=Get_XMin(); x<Get_NX(); x++, xWorld+=Get_Cellsize())
+			double	yWorld	= Get_YMin() + y * Get_Cellsize();
+
+			#pragma omp parallel for
+			for(int x=0; x<Get_NX(); x++)
 			{
-				if( !Grid.Get_Value(xWorld, yWorld, Value, Interpolation, true) )
+				if( !is_NoData(x, y) )
 				{
-					Set_NoData(x, y);
-				}
-				else switch( Operation )
-				{
-				case GRID_OPERATION_Addition:
-					Add_Value(x, y,  Value);
-					break;
+					double	xWorld	= Get_XMin() + x * Get_Cellsize(), Value;
 
-				case GRID_OPERATION_Subtraction:
-					Add_Value(x, y, -Value);
-					break;
-
-				case GRID_OPERATION_Multiplication:
-					Mul_Value(x, y,  Value);
-					break;
-
-				case GRID_OPERATION_Division:
-					if( Value != 0.0 )
+					if( Grid.Get_Value(xWorld, yWorld, Value, Interpolation) )
 					{
-						Mul_Value(x, y, 1.0 / Value);
+						switch( Operation )
+						{
+						case GRID_OPERATION_Addition      :	Add_Value(x, y,  Value);	break;
+						case GRID_OPERATION_Subtraction   :	Add_Value(x, y, -Value);	break;
+						case GRID_OPERATION_Multiplication:	Mul_Value(x, y,  Value);	break;
+						case GRID_OPERATION_Division      :
+							if( Value != 0.0 )
+							{
+								Mul_Value(x, y, 1.0 / Value);
+							}
+							else
+							{
+								Set_NoData(x, y);
+							}
+							break;
+						}
 					}
-					else
-					{
-						Set_NoData(x, y);
-					}
-					break;
 				}
 			}
 		}
@@ -744,25 +683,18 @@ CSG_Grid & CSG_Grid::_Operation_Arithmetic(const CSG_Grid &Grid, TSG_Grid_Operat
 		SG_UI_Process_Set_Ready();
 
 		//-------------------------------------------------
+		CSG_String	Name;
+
 		switch( Operation )
 		{
-		case GRID_OPERATION_Addition:
-			Get_History().Add_Child(SG_T("GRID_OPERATION"), Grid.Get_Name())->Add_Property(SG_T("NAME"), _TL("Addition"));
-			break;
-
-		case GRID_OPERATION_Subtraction:
-			Get_History().Add_Child(SG_T("GRID_OPERATION"), Grid.Get_Name())->Add_Property(SG_T("NAME"), _TL("Subtraction"));
-			break;
-
-		case GRID_OPERATION_Multiplication:
-			Get_History().Add_Child(SG_T("GRID_OPERATION"), Grid.Get_Name())->Add_Property(SG_T("NAME"), _TL("Multiplication"));
-			break;
-
-		case GRID_OPERATION_Division:
-			Get_History().Add_Child(SG_T("GRID_OPERATION"), Grid.Get_Name())->Add_Property(SG_T("NAME"), _TL("Division"));
-			break;
+		case GRID_OPERATION_Addition      :	Name	= _TL("Addition"      );	break;
+		case GRID_OPERATION_Subtraction   :	Name	= _TL("Subtraction"   );	break;
+		case GRID_OPERATION_Multiplication:	Name	= _TL("Multiplication");	break;
+		case GRID_OPERATION_Division      :	Name	= _TL("Division"      );	break;
 		}
 
+		Get_History().Add_Child("GRID_OPERATION", Grid.Get_Name())->Add_Property("NAME", Name);
+
 		Get_History().Add_Children(((CSG_Grid *)&Grid)->Get_History());
 	}
 
@@ -773,41 +705,42 @@ CSG_Grid & CSG_Grid::_Operation_Arithmetic(const CSG_Grid &Grid, TSG_Grid_Operat
 CSG_Grid & CSG_Grid::_Operation_Arithmetic(double Value, TSG_Grid_Operation Operation)
 {
 	//-----------------------------------------------------
+	CSG_String	Name;
+
 	switch( Operation )
 	{
-	case GRID_OPERATION_Addition:
+	case GRID_OPERATION_Addition      :
+		Name	=  _TL("Addition");
 		if( Value == 0.0 )
 			return( *this );
-
-		Get_History().Add_Child(SG_T("GRID_OPERATION"), Value)->Add_Property(SG_T("NAME"), _TL("Addition"));
 		break;
 
-	case GRID_OPERATION_Subtraction:
+	case GRID_OPERATION_Subtraction   :
+		Name	=  _TL("Subtraction");
 		if( Value == 0.0 )
 			return( *this );
-
-		Get_History().Add_Child(SG_T("GRID_OPERATION"), Value)->Add_Property(SG_T("NAME"), _TL("Subtraction"));
 		Value	= -Value;
 		break;
 
 	case GRID_OPERATION_Multiplication:
+		Name	=  _TL("Multiplication");
 		if( Value == 1.0 )
 			return( *this );
-
-		Get_History().Add_Child(SG_T("GRID_OPERATION"), Value)->Add_Property(SG_T("NAME"), _TL("Multiplication"));
 		break;
 
-	case GRID_OPERATION_Division:
+	case GRID_OPERATION_Division      :
+		Name	=  _TL("Division");
 		if( Value == 0.0 )
 			return( *this );
-
-		Get_History().Add_Child(SG_T("GRID_OPERATION"), Value)->Add_Property(SG_T("NAME"), _TL("Division"));
 		Value	= 1.0 / Value;
 		break;
 	}
 
+	Get_History().Add_Child("GRID_OPERATION", Value)->Add_Property("NAME", Name);
+
 	//-----------------------------------------------------
-	for(int y=0; y<Get_NY() && SG_UI_Process_Set_Progress(y, Get_NY()); y++)
+	#pragma omp parallel for
+	for(int y=0; y<Get_NY(); y++)
 	{
 		for(int x=0; x<Get_NX(); x++)
 		{
@@ -815,22 +748,17 @@ CSG_Grid & CSG_Grid::_Operation_Arithmetic(double Value, TSG_Grid_Operation Oper
 			{
 				switch( Operation )
 				{
-				case GRID_OPERATION_Addition:
-				case GRID_OPERATION_Subtraction:
-					Add_Value(x, y, Value);
-					break;
+				case GRID_OPERATION_Addition      :
+				case GRID_OPERATION_Subtraction   :	Add_Value(x, y, Value);	break;
 
 				case GRID_OPERATION_Multiplication:
-				case GRID_OPERATION_Division:
-					Mul_Value(x, y, Value);
-					break;
+				case GRID_OPERATION_Division      :	Mul_Value(x, y, Value);	break;
 				}
 			}
 		}
 	}
 
-	SG_UI_Process_Set_Ready();
-
+	//-----------------------------------------------------
 	return( *this );
 }
 
@@ -844,90 +772,64 @@ CSG_Grid & CSG_Grid::_Operation_Arithmetic(double Value, TSG_Grid_Operation Oper
 //---------------------------------------------------------
 void CSG_Grid::Invert(void)
 {
-	int		x, y;
-	double	zMin, zMax;
-
-	if( is_Valid() && Get_ZRange() > 0.0 )
+	if( is_Valid() && Get_Range() > 0.0 )
 	{
-		zMin	= Get_ZMin();
-		zMax	= Get_ZMax();
+		double	Min	= Get_Min();
+		double	Max	= Get_Max();
 
-		for(y=0; y<Get_NY() && SG_UI_Process_Set_Progress(y, Get_NY()); y++)
+		#pragma omp parallel for
+		for(int y=0; y<Get_NY(); y++)
 		{
-			for(x=0; x<Get_NX(); x++)
+			for(int x=0; x<Get_NX(); x++)
 			{
 				if( !is_NoData(x, y) )
 				{
-					Set_Value(x, y, zMax - (asDouble(x, y) - zMin));
+					Set_Value(x, y, Max - (asDouble(x, y) - Min));
 				}
 			}
 		}
 
-		SG_UI_Process_Set_Ready();
-
-		Get_History().Add_Child(SG_T("GRID_OPERATION"), _TL("Inversion"));
+		Get_History().Add_Child("GRID_OPERATION", _TL("Inverted"));
 	}
 }
 
 //---------------------------------------------------------
 void CSG_Grid::Flip(void)
 {
-	int		x, yA, yB;
-	double	*Line, d;
-
 	if( is_Valid() )
 	{
-		Line	= (double *)SG_Malloc(Get_NX() * sizeof(double));
-
-		for(yA=0, yB=Get_NY()-1; yA<yB && SG_UI_Process_Set_Progress(2 * yA, Get_NY()); yA++, yB--)
+		#pragma omp parallel for
+		for(int x=0; x<Get_NX(); x++)
 		{
-			for(x=0; x<Get_NX(); x++)
+			for(int yA=0, yB=Get_NY()-1; yA<yB; yA++, yB--)
 			{
-				Line[x]	= asDouble(x, yA);
-			}
-
-			for(x=0; x<Get_NX(); x++)
-			{
-				d		= Line[x];
-				Line[x]	= asDouble(x, yB);
+				double	d	   = asDouble(x, yA);
+				Set_Value(x, yA, asDouble(x, yB));
 				Set_Value(x, yB, d);
 			}
-
-			for(x=0; x<Get_NX(); x++)
-			{
-				Set_Value(x, yA, Line[x]);
-			}
 		}
 
-		SG_UI_Process_Set_Ready();
-
-		SG_Free(Line);
-
-		Get_History().Add_Child(SG_T("GRID_OPERATION"), _TL("Vertically mirrored"));
+		Get_History().Add_Child("GRID_OPERATION", _TL("Mirrored vertically"));
 	}
 }
 
 //---------------------------------------------------------
 void CSG_Grid::Mirror(void)
 {
-	int		xA, xB, y;
-	double	d;
-
 	if( is_Valid() )
 	{
-		for(y=0; y<Get_NY() && SG_UI_Process_Set_Progress(y, Get_NY()); y++)
+		#pragma omp parallel for
+		for(int y=0; y<Get_NY(); y++)
 		{
-			for(xA=0, xB=Get_NX()-1; xA<xB; xA++, xB--)
+			for(int xA=0, xB=Get_NX()-1; xA<xB; xA++, xB--)
 			{
-				d			=    asDouble(xA, y);
+				double	d	   = asDouble(xA, y);
 				Set_Value(xA, y, asDouble(xB, y));
 				Set_Value(xB, y, d);
 			}
 		}
 
-		SG_UI_Process_Set_Ready();
-
-		Get_History().Add_Child(SG_T("GRID_OPERATION"), _TL("Horizontally mirrored"));
+		Get_History().Add_Child("GRID_OPERATION", _TL("Mirrored horizontally"));
 	}
 }
 
@@ -941,123 +843,107 @@ void CSG_Grid::Mirror(void)
 //---------------------------------------------------------
 bool CSG_Grid::Normalise(void)
 {
-	if( !is_Valid() || Get_ZRange() <= 0.0 )
+	if( is_Valid() && Get_Range() > 0.0 )
 	{
-		return( false );
-	}
+		double	Min		= Get_Min  ();
+		double	Range	= Get_Range();
 
-	SG_UI_Process_Set_Text(_TL("Normalisation"));
-
-	double	zMin	= Get_ZMin();
-	double	zRange	= Get_ZRange();
-
-	for(int y=0; y<Get_NY() && SG_UI_Process_Set_Progress(y, Get_NY()); y++)
-	{
 		#pragma omp parallel for
-		for(int x=0; x<Get_NX(); x++)
+		for(int y=0; y<Get_NY(); y++)
 		{
-			if( !is_NoData(x, y) )
+			for(int x=0; x<Get_NX(); x++)
 			{
-				Set_Value(x, y, (asDouble(x, y) - zMin) / zRange);
+				if( !is_NoData(x, y) )
+				{
+					Set_Value(x, y, (asDouble(x, y) - Min) / Range);
+				}
 			}
 		}
+
+		Get_History().Add_Child("GRID_OPERATION", _TL("Normalisation"));
+
+		return( true );
 	}
 
-	SG_UI_Process_Set_Ready();
-
-	Get_History().Add_Child(SG_T("GRID_OPERATION"), _TL("Normalisation"));
-
-	return( true );
+	return( false );
 }
 
 //---------------------------------------------------------
 bool CSG_Grid::DeNormalise(double Minimum, double Maximum)
 {
-	if( !is_Valid() || Minimum > Maximum )
-	{
-		return( false );
-	}
-
-	SG_UI_Process_Set_Text(_TL("Denormalisation"));
-
-	for(int y=0; y<Get_NY() && SG_UI_Process_Set_Progress(y, Get_NY()); y++)
+	if( is_Valid() && Minimum < Maximum )
 	{
 		#pragma omp parallel for
-		for(int x=0; x<Get_NX(); x++)
+		for(int y=0; y<Get_NY(); y++)
 		{
-			if( !is_NoData(x, y) )
+			for(int x=0; x<Get_NX(); x++)
 			{
-				Set_Value(x, y, Minimum + asDouble(x, y) * (Maximum - Minimum));
+				if( !is_NoData(x, y) )
+				{
+					Set_Value(x, y, Minimum + asDouble(x, y) * (Maximum - Minimum));
+				}
 			}
 		}
+
+		Get_History().Add_Child("GRID_OPERATION", _TL("Denormalisation"));
+
+		return( true );
 	}
 
-	SG_UI_Process_Set_Ready();
-
-	Get_History().Add_Child(SG_T("GRID_OPERATION"), _TL("Denormalisation"));
-
-	return( true );
+	return( false );
 }
 
 //---------------------------------------------------------
 bool CSG_Grid::Standardise(void)
 {
-	if( !is_Valid() || Get_StdDev() <= 0.0 )
+	if( is_Valid() && Get_StdDev() > 0.0 )
 	{
-		return( false );
-	}
+		double	Mean	= Get_Mean  ();
+		double	StdDev	= Get_StdDev();
 
-	SG_UI_Process_Set_Text(_TL("Standardisation"));
-
-	double	Mean	= Get_Mean();
-	double	StdDev	= Get_StdDev();
-
-	for(int y=0; y<Get_NY() && SG_UI_Process_Set_Progress(y, Get_NY()); y++)
-	{
 		#pragma omp parallel for
-		for(int x=0; x<Get_NX(); x++)
+		for(int y=0; y<Get_NY(); y++)
 		{
-			if( !is_NoData(x, y) )
+			for(int x=0; x<Get_NX(); x++)
 			{
-				Set_Value(x, y, (asDouble(x, y) - Mean) / StdDev);
+				if( !is_NoData(x, y) )
+				{
+					Set_Value(x, y, (asDouble(x, y) - Mean) / StdDev);
+				}
 			}
 		}
+
+		Get_History().Add_Child("GRID_OPERATION", _TL("Standardisation"));
+
+		return( true );
 	}
 
-	SG_UI_Process_Set_Ready();
-
-	Get_History().Add_Child(SG_T("GRID_OPERATION"), _TL("Standardisation"));
-
 	return( false );
 }
 
 //---------------------------------------------------------
 bool CSG_Grid::DeStandardise(double Mean, double StdDev)
 {
-	if( !is_Valid() || StdDev <= 0.0 )
-	{
-		return( false );
-	}
-
-	SG_UI_Process_Set_Text(_TL("Destandardisation"));
-
-	for(int y=0; y<Get_NY() && SG_UI_Process_Set_Progress(y, Get_NY()); y++)
+	if( is_Valid() && StdDev > 0.0 )
 	{
 		#pragma omp parallel for
-		for(int x=0; x<Get_NX(); x++)
+		for(int y=0; y<Get_NY(); y++)
 		{
-			if( !is_NoData(x, y) )
+			for(int x=0; x<Get_NX(); x++)
 			{
-				Set_Value(x, y, Mean + asDouble(x, y) * StdDev);
+				if( !is_NoData(x, y) )
+				{
+					Set_Value(x, y, Mean + asDouble(x, y) * StdDev);
+				}
 			}
 		}
+
+		Get_History().Add_Child("GRID_OPERATION", _TL("Destandardisation"));
+
+		return( true );
 	}
 
-	SG_UI_Process_Set_Ready();
-
-	Get_History().Add_Child(SG_T("GRID_OPERATION"), _TL("Destandardisation"));
-
-	return( true );
+	return( false );
 }
 
 
@@ -1068,42 +954,44 @@ bool CSG_Grid::DeStandardise(double Mean, double StdDev)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-int CSG_Grid::Get_Gradient_NeighborDir(int x, int y, bool bMustBeLower)	const
+/**
+  * Returns the direction to which the downward gradient is
+  * steepest. This implements the Deterministic 8 (D8) algorithm
+  * for identifying a single flow direction based on elevation
+  * data. Direction is numbered clock-wise beginning with 0 for
+  * the North. If no direction can be identified result will be a -1.
+  * If 'bDown' is not true the cell that the direction is pointing
+  * to might have a higher value than the center cell. If 'bNoEdges'
+  * is true a -1 will be returned for all cells that are at the
+  * edge of the data area.
+*/
+//---------------------------------------------------------
+int CSG_Grid::Get_Gradient_NeighborDir(int x, int y, bool bDown, bool bNoEdges)	const
 {
-	int		i, ix, iy, Direction;
-	double	z, dz, dzMax;
-
-	Direction	= -1;
+	int	Direction	= -1;
 
 	if( is_InGrid(x, y) )
 	{
-		z		= asDouble(x, y);
-		dzMax	= 0.0;
+		double	z	= asDouble(x, y), dzMax	= 0.0;
 
-		for(i=0; i<8; i++)
+		for(int i=0; i<8; i++)
 		{
-			ix	= m_System.Get_xTo(i, x);
-			iy	= m_System.Get_yTo(i, y);
+			int	ix	= m_System.Get_xTo(i, x);
+			int	iy	= m_System.Get_yTo(i, y);
 
-			if( !is_InGrid(ix, iy) )
+			if( is_InGrid(ix, iy) )
 			{
-				if( 1 )	// flag 'bStopOnNoData'
+				double	dz	= (z - asDouble(ix, iy)) / m_System.Get_Length(i);
+
+				if( (!bDown || dz > 0.0) && (Direction < 0 || dzMax < dz) )
 				{
-					return( -1 );
+					dzMax		= dz;
+					Direction	= i;
 				}
 			}
-			else
+			else if( bNoEdges )
 			{
-				dz	= (z - asDouble(ix, iy)) / m_System.Get_Length(i);
-
-				if( (bMustBeLower && dz > 0.0) || !bMustBeLower )
-				{
-					if( Direction < 0 || (dz > dzMax) )
-					{
-						Direction	= i;
-						dzMax		= dz;
-					}
-				}
+				return( -1 );
 			}
 		}
 	}
@@ -1112,59 +1000,117 @@ int CSG_Grid::Get_Gradient_NeighborDir(int x, int y, bool bMustBeLower)	const
 }
 
 //---------------------------------------------------------
-bool CSG_Grid::Get_Gradient(int x, int y, double &Decline, double &Azimuth) const
+/**
+  * Calculates the gradient of a cell interpreting all grid cell values
+  * as elevation surface. Calculation uses the formulas proposed
+  * by Zevenbergen & Thorne (1986). Slope and aspect values are calculated
+  * in radians. Aspect is zero for the North direction and increases
+  * clockwise.
+*/
+//---------------------------------------------------------
+bool CSG_Grid::Get_Gradient(int x, int y, double &Slope, double &Aspect) const
 {
-	int		i, ix, iy, iDir;
-	double	z, zm[4], G, H;
-
 	if( is_InGrid(x, y) )
 	{
-		z		= asDouble(x, y);
+		double	z	= asDouble(x, y), dz[4];
 
-		for(i=0, iDir=0; i<4; i++, iDir+=2)
+		for(int i=0, iDir=0, ix, iy; i<4; i++, iDir+=2)
 		{
-			ix		= m_System.Get_xTo(iDir, x);
-			iy		= m_System.Get_yTo(iDir, y);
-
-			if( is_InGrid(ix, iy) )
+			if( is_InGrid(
+				ix = m_System.Get_xTo  (iDir, x),
+				iy = m_System.Get_yTo  (iDir, y)) )
 			{
-				zm[i]	= asDouble(ix, iy) - z;
+				dz[i]	= asDouble(ix, iy) - z;
+			}
+			else if( is_InGrid(
+				ix = m_System.Get_xFrom(iDir, x),
+				iy = m_System.Get_yFrom(iDir, y)) )
+			{
+				dz[i]	= z - asDouble(ix, iy);
 			}
 			else
 			{
-				ix		= m_System.Get_xFrom(iDir, x);
-				iy		= m_System.Get_yFrom(iDir, y);
-
-				if( is_InGrid(ix, iy) )
-				{
-					zm[i]	= z - asDouble(ix, iy);
-				}
-				else
-				{
-					zm[i]	= 0.0;
-				}
+				dz[i]	= 0.0;
 			}
 		}
 
-		G		= (zm[0] - zm[2]) / (2.0 * Get_Cellsize());
-        H		= (zm[1] - zm[3]) / (2.0 * Get_Cellsize());
+		double G	= (dz[0] - dz[2]) / (2.0 * Get_Cellsize());
+        double H	= (dz[1] - dz[3]) / (2.0 * Get_Cellsize());
 
-		Decline	= atan(sqrt(G*G + H*H));
-
-		if( G != 0.0 )
-			Azimuth	= M_PI_180 + atan2(H, G);
-		else
-			Azimuth	= H > 0.0 ? M_PI_270 : (H < 0.0 ? M_PI_090 : -1.0);
+		Slope	= atan(sqrt(G*G + H*H));
+		Aspect	= G != 0.0 ? M_PI_180 + atan2(H, G) : H > 0.0 ? M_PI_270 : H < 0.0 ? M_PI_090 : -1.0;
 
 		return( true );
 	}
 
-	Decline	= 0.0;
-	Azimuth	= -1.0;
+	Slope	=  0.0;
+	Aspect	= -1.0;
 
 	return( false );
 }
 
+//---------------------------------------------------------
+/**
+  * Calculates the gradient for the given world coordinate.
+  * Calculation uses the formulas proposed by Zevenbergen & Thorne (1986).
+  * Slope and aspect values are calulated in radians.
+  * Aspect is zero for the North direction and increases clockwise.
+*/
+//---------------------------------------------------------
+bool CSG_Grid::Get_Gradient(double x, double y, double &Slope, double &Aspect, TSG_Grid_Resampling Interpolation) const
+{
+	double	z, iz, dz[4];
+
+	if( Get_Value(x, y, z, Interpolation) )
+	{
+		for(int i=0, iDir=0; i<4; i++, iDir+=2)
+		{
+			if( Get_Value(
+				x + Get_Cellsize() * m_System.Get_xTo  (iDir),
+				y + Get_Cellsize() * m_System.Get_yTo  (iDir), iz, Interpolation) )
+			{
+				dz[i]	= iz - z;
+			}
+			else if( Get_Value(
+				x + Get_Cellsize() * m_System.Get_xFrom(iDir),
+				y + Get_Cellsize() * m_System.Get_yFrom(iDir), iz, Interpolation) )
+			{
+				dz[i]	= z - iz;
+			}
+			else
+			{
+				dz[i]	= 0.0;
+			}
+		}
+
+		double G	= (dz[0] - dz[2]) / (2.0 * Get_Cellsize());
+        double H	= (dz[1] - dz[3]) / (2.0 * Get_Cellsize());
+
+		Slope	= atan(sqrt(G*G + H*H));
+		Aspect	= G != 0.0 ? M_PI_180 + atan2(H, G) : H > 0.0 ? M_PI_270 : H < 0.0 ? M_PI_090 : -1.0;
+
+		return( true );
+	}
+
+	Slope	=  0.0;
+	Aspect	= -1.0;
+
+	return( false );
+}
+
+//---------------------------------------------------------
+/**
+  * Calculates the gradient for the given world coordinate.
+  * Calculation uses the formulas proposed by Zevenbergen & Thorne (1986).
+  * Slope and aspect values are calculated in radians.
+  * Aspect is zero for the North direction and increases clockwise.
+*/
+//---------------------------------------------------------
+bool CSG_Grid::Get_Gradient(const TSG_Point &p, double &Incline, double &Azimuth, TSG_Grid_Resampling Interpolation) const
+{
+	return( Get_Gradient(p.x, p.y, Incline, Azimuth, Interpolation) );
+}
+
 
 ///////////////////////////////////////////////////////////
 //														 //
