@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: grid_system.cpp 2494 2015-05-21 14:09:38Z oconrad $
+ * Version $Id$
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -26,7 +26,8 @@
 // This library is free software; you can redistribute   //
 // it and/or modify it under the terms of the GNU Lesser //
 // General Public License as published by the Free       //
-// Software Foundation, version 2.1 of the License.      //
+// Software Foundation, either version 2.1 of the        //
+// License, or (at your option) any later version.       //
 //                                                       //
 // This library is distributed in the hope that it will  //
 // be useful, but WITHOUT ANY WARRANTY; without even the //
@@ -36,9 +37,7 @@
 //                                                       //
 // You should have received a copy of the GNU Lesser     //
 // General Public License along with this program; if    //
-// not, write to the Free Software Foundation, Inc.,     //
-// 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, //
-// USA.                                                  //
+// not, see <http://www.gnu.org/licenses/>.              //
 //                                                       //
 //-------------------------------------------------------//
 //                                                       //
@@ -63,6 +62,7 @@
 //---------------------------------------------------------
 #include "grid.h"
 #include "shapes.h"
+#include "parameters.h"
 
 
 ///////////////////////////////////////////////////////////
@@ -72,115 +72,73 @@
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-#define GRID_SYSTEM_PRECISION 10000000000	// 10 decimal digits, precision used for storing cellsize and extent
+int		CSG_Grid_System::m_Precision	= 16;	// 16 decimal digits, default precision used for storing cellsize and extent
+
+//---------------------------------------------------------
+int CSG_Grid_System::Set_Precision(int Decimals)
+{
+	if( Decimals >= 0 )
+	{
+		m_Precision	= Decimals;
+	}
+
+	return( m_Precision );
+}
+
+//---------------------------------------------------------
+int CSG_Grid_System::Get_Precision(void)
+{
+	return( m_Precision );
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
 CSG_Grid_System::CSG_Grid_System(void)
 {
-	Assign(0.0, 0.0, 0.0, 0, 0);
+	Destroy();
 }
 
 //---------------------------------------------------------
 CSG_Grid_System::CSG_Grid_System(const CSG_Grid_System &System)
 {
-	Assign(System);
+	Create(System);
 }
 
 //---------------------------------------------------------
 CSG_Grid_System::CSG_Grid_System(double Cellsize, const CSG_Rect &Extent)
 {
-	Assign(Cellsize, Extent);
+	Create(Cellsize, Extent);
 }
 
 //---------------------------------------------------------
 CSG_Grid_System::CSG_Grid_System(double Cellsize, double xMin, double yMin, double xMax, double yMax)
 {
-	Assign(Cellsize, xMin, yMin, xMax, yMax);
+	Create(Cellsize, xMin, yMin, xMax, yMax);
 }
 
 //---------------------------------------------------------
 CSG_Grid_System::CSG_Grid_System(double Cellsize, double xMin, double yMin, int NX, int NY)
 {
-	Assign(Cellsize, xMin, yMin, NX, NY);
+	Create(Cellsize, xMin, yMin, NX, NY);
 }
 
 //---------------------------------------------------------
 CSG_Grid_System::~CSG_Grid_System(void)
 {
+	Destroy();
 }
 
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-bool CSG_Grid_System::is_Valid(void) const
-{
-	return( m_Cellsize > 0.0 );
-}
-
-//---------------------------------------------------------
-const SG_Char * CSG_Grid_System::Get_Name(bool bShort)
-{
-	if( is_Valid() )
-	{
-		if( bShort )
-		{
-			m_Name.Printf(SG_T("%.*f; %dx %dy; %.*fx %.*fy"),
-				SG_Get_Significant_Decimals(Get_Cellsize()), Get_Cellsize(),
-				Get_NX(), Get_NY(),
-				SG_Get_Significant_Decimals(Get_XMin    ()), Get_XMin    (),
-				SG_Get_Significant_Decimals(Get_YMin    ()), Get_YMin    ()
-			);
-		}
-		else
-		{
-			m_Name.Printf(SG_T("%s: %f, %s: %dx/%dy, %s: %fx/%fy"),
-				_TL("Cell size"        ), Get_Cellsize(),
-				_TL("Number of cells"  ), Get_NX(), Get_NY(),
-				_TL("Lower left corner"), Get_XMin(), Get_YMin()
-			);
-		}
-	}
-	else
-	{
-		m_Name	= _TL("<not set>");
-	}
-
-	return( m_Name );
-}
-
-
-///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
-///////////////////////////////////////////////////////////
-
-//---------------------------------------------------------
-bool CSG_Grid_System::operator == (const CSG_Grid_System &System) const
-{
-	return( is_Equal(System) );
-}
-
-//---------------------------------------------------------
-void CSG_Grid_System::operator = (const CSG_Grid_System &System)
-{
-	Assign(System);
-}
-
-
-///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
-///////////////////////////////////////////////////////////
-
-//---------------------------------------------------------
-bool CSG_Grid_System::Assign(const CSG_Grid_System &System)
+bool CSG_Grid_System::Create(const CSG_Grid_System &System)
 {
 	m_NX			= System.m_NX;
 	m_NY			= System.m_NY;
@@ -197,33 +155,38 @@ bool CSG_Grid_System::Assign(const CSG_Grid_System &System)
 }
 
 //---------------------------------------------------------
-bool CSG_Grid_System::Assign(double Cellsize, const CSG_Rect &Extent)
+bool CSG_Grid_System::Create(double Cellsize, const CSG_Rect &Extent)
 {
 	if( Cellsize > 0.0 && Extent.Get_XRange() >= 0.0 && Extent.Get_YRange() >= 0.0 )
 	{
-		return( Assign(Cellsize, Extent.Get_XMin(), Extent.Get_YMin(),
-			1 + (int)(0.5 + Extent.Get_XRange() / Cellsize),
-			1 + (int)(0.5 + Extent.Get_YRange() / Cellsize))
-		);
+		int		nx	= 1 + (int)(0.5 + Extent.Get_XRange() / Cellsize);
+		int		ny	= 1 + (int)(0.5 + Extent.Get_YRange() / Cellsize);
+
+		double	x	= !fmod(Extent.Get_XRange(), Cellsize) ? Extent.Get_XMin() : Extent.Get_Center().Get_X() - Cellsize * nx / 2.;
+		double	y	= !fmod(Extent.Get_YRange(), Cellsize) ? Extent.Get_YMin() : Extent.Get_Center().Get_Y() - Cellsize * ny / 2.;
+
+		return( Create(Cellsize, x, y, nx, ny) );
 	}
 
-	return( Assign(0.0, 0.0, 0.0, 0, 0) );
+	Destroy();
+
+	return( false );
 }
 
 //---------------------------------------------------------
-bool CSG_Grid_System::Assign(double Cellsize, double xMin, double yMin, double xMax, double yMax)
+bool CSG_Grid_System::Create(double Cellsize, double xMin, double yMin, double xMax, double yMax)
 {
-	return( Assign(Cellsize, CSG_Rect(xMin, yMin, xMax, yMax)) );
+	return( Create(Cellsize, CSG_Rect(xMin, yMin, xMax, yMax)) );
 }
 
 //---------------------------------------------------------
-bool CSG_Grid_System::Assign(double Cellsize, double xMin, double yMin, int NX, int NY)
+bool CSG_Grid_System::Create(double Cellsize, double xMin, double yMin, int NX, int NY)
 {
 	if( Cellsize > 0.0 && NX > 0 && NY > 0 )
 	{
-		Cellsize	= (sLong)(Cellsize * GRID_SYSTEM_PRECISION) / (double)GRID_SYSTEM_PRECISION;
-		xMin		= (sLong)(xMin     * GRID_SYSTEM_PRECISION) / (double)GRID_SYSTEM_PRECISION;
-		yMin		= (sLong)(yMin     * GRID_SYSTEM_PRECISION) / (double)GRID_SYSTEM_PRECISION;
+		Cellsize	= SG_Get_Rounded(Cellsize, m_Precision);
+		xMin		= SG_Get_Rounded(xMin    , m_Precision);
+		yMin		= SG_Get_Rounded(yMin    , m_Precision);
 
 		if( Cellsize > 0.0 )
 		{
@@ -262,6 +225,88 @@ bool CSG_Grid_System::Assign(double Cellsize, double xMin, double yMin, int NX,
 	return( false );
 }
 
+//---------------------------------------------------------
+bool CSG_Grid_System::Destroy(void)
+{
+	Create(0.0, 0.0, 0.0, 0, 0);
+
+	return( true );
+}
+
+//---------------------------------------------------------
+bool CSG_Grid_System::Assign(const CSG_Grid_System &System)
+{	return( Create(System) );	}
+
+bool CSG_Grid_System::Assign(double Cellsize, const CSG_Rect &Extent)
+{	return( Create(Cellsize, Extent) );	}
+
+bool CSG_Grid_System::Assign(double Cellsize, double xMin, double yMin, double xMax, double yMax)
+{	return( Create(Cellsize, xMin, yMin, xMax, yMax) );	}
+
+bool CSG_Grid_System::Assign(double Cellsize, double xMin, double yMin, int NX, int NY)
+{	return( Create(Cellsize, xMin, yMin, NX, NY) );	}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CSG_Grid_System::is_Valid(void) const
+{
+	return( m_Cellsize > 0.0 );
+}
+
+//---------------------------------------------------------
+const SG_Char * CSG_Grid_System::Get_Name(bool bShort)
+{
+	if( is_Valid() )
+	{
+		if( bShort )
+		{
+			m_Name.Printf("%.*f; %dx %dy; %.*fx %.*fy",
+				SG_Get_Significant_Decimals(Get_Cellsize()), Get_Cellsize(),
+				Get_NX(), Get_NY(),
+				SG_Get_Significant_Decimals(Get_XMin    ()), Get_XMin    (),
+				SG_Get_Significant_Decimals(Get_YMin    ()), Get_YMin    ()
+			);
+		}
+		else
+		{
+			m_Name.Printf("%s: %f, %s: %dx/%dy, %s: %fx/%fy",
+				_TL("Cell size"        ), Get_Cellsize(),
+				_TL("Number of cells"  ), Get_NX(), Get_NY(),
+				_TL("Lower left corner"), Get_XMin(), Get_YMin()
+			);
+		}
+	}
+	else
+	{
+		m_Name	= _TL("<not set>");
+	}
+
+	return( m_Name );
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+//														 //
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CSG_Grid_System::operator == (const CSG_Grid_System &System) const
+{
+	return( is_Equal(System) );
+}
+
+//---------------------------------------------------------
+void CSG_Grid_System::operator = (const CSG_Grid_System &System)
+{
+	Create(System);
+}
+
 
 ///////////////////////////////////////////////////////////
 //														 //
@@ -296,10 +341,10 @@ bool CSG_Grid_System::is_Equal(double Cellsize, const TSG_Rect &Extent) const
 //---------------------------------------------------------
 CSG_Grid_Cell_Addressor::CSG_Grid_Cell_Addressor(void)
 {
-	m_Cells.Add_Field(SG_T("X"), SG_DATATYPE_Int);
-	m_Cells.Add_Field(SG_T("Y"), SG_DATATYPE_Int);
-	m_Cells.Add_Field(SG_T("D"), SG_DATATYPE_Double);
-	m_Cells.Add_Field(SG_T("W"), SG_DATATYPE_Double);
+	m_Cells.Add_Field("X", SG_DATATYPE_Int   );
+	m_Cells.Add_Field("Y", SG_DATATYPE_Int   );
+	m_Cells.Add_Field("D", SG_DATATYPE_Double);
+	m_Cells.Add_Field("W", SG_DATATYPE_Double);
 }
 
 //---------------------------------------------------------
@@ -311,6 +356,139 @@ bool CSG_Grid_Cell_Addressor::Destroy(void)
 }
 
 
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CSG_Grid_Cell_Addressor::Add_Parameters(CSG_Parameters &Parameters, const SG_Char *Parent, int Style)
+{
+	Parameters.Add_Choice("", "KERNEL_TYPE", _TL("Kernel Type"),
+		_TL("The shape of the filter kernel."),
+		"0|1|", 1
+	);
+
+	CSG_String	Unit	= (Style & SG_GRIDCELLADDR_PARM_MAPUNIT) == 0
+		? _TL("The kernel radius in cells.")
+		: _TL("The kernel radius in map units.");
+
+	if( (Style & SG_GRIDCELLADDR_PARM_SIZEDBL) == 0 )
+	{
+		Parameters.Add_Int   ("", "KERNEL_RADIUS", _TL("Kernel Radius"), Unit, 2  , 1  , true);
+	}
+	else
+	{
+		Parameters.Add_Double("", "KERNEL_RADIUS", _TL("Kernel Radius"), Unit, 1.0, 0.0, true);
+	}
+
+	//-----------------------------------------------------
+	CSG_String	Types;
+
+	if( (Style & SG_GRIDCELLADDR_PARM_SQUARE) != 0 )
+	{
+		Types	+= CSG_String::Format("{%d}%s|", SG_GRIDCELLADDR_PARM_SQUARE , _TL("Square"));
+	}
+
+	if( (Style & SG_GRIDCELLADDR_PARM_CIRCLE) != 0 )
+	{
+		Types	+= CSG_String::Format("{%d}%s|", SG_GRIDCELLADDR_PARM_CIRCLE , _TL("Circle"));
+	}
+
+	if( (Style & SG_GRIDCELLADDR_PARM_ANNULUS) != 0 )
+	{
+		Types	+= CSG_String::Format("{%d}%s|", SG_GRIDCELLADDR_PARM_ANNULUS, _TL("Annulus"));
+
+		Parameters.Add_Double("", "KERNEL_INNER", _TL("Inner Kernel Radius"), _TL(""));
+	}
+
+	if( (Style & SG_GRIDCELLADDR_PARM_SECTOR) != 0 )
+	{
+		Types	+= CSG_String::Format("{%d}%s|", SG_GRIDCELLADDR_PARM_SECTOR , _TL("Sector"));
+
+		Parameters.Add_Double("", "KERNEL_DIRECTION", _TL("Kernel Direction"), _TL(""));
+		Parameters.Add_Double("", "KERNEL_TOLERANCE", _TL("Kernel Tolerance"), _TL(""));
+	}
+
+	Parameters("KERNEL_TYPE")->asChoice()->Set_Items(Types);
+
+	return( true );
+}
+
+//---------------------------------------------------------
+bool CSG_Grid_Cell_Addressor::Set_Parameters(CSG_Parameters &Parameters, int Type)
+{
+	if( Type == 0 && Parameters("KERNEL_TYPE") )
+	{
+		Type	= Parameters("KERNEL_TYPE")->asChoice()->Get_Item_Data(Parameters("KERNEL_TYPE")->asInt()).asInt();
+	}
+
+	switch( Type )
+	{
+	case SG_GRIDCELLADDR_PARM_SQUARE:
+		return( Set_Radius(
+			Parameters("KERNEL_RADIUS"   )->asDouble(),
+			true
+		));
+
+	case SG_GRIDCELLADDR_PARM_CIRCLE:
+		return( Set_Radius(
+			Parameters("KERNEL_RADIUS"   )->asDouble(),
+			false
+		));
+
+	case SG_GRIDCELLADDR_PARM_ANNULUS:
+		return( Set_Annulus(
+			Parameters("KERNEL_INNER"    )->asDouble(),
+			Parameters("KERNEL_RADIUS"   )->asDouble()
+		));
+
+	case SG_GRIDCELLADDR_PARM_SECTOR:
+		return( Set_Sector(
+			Parameters("KERNEL_RADIUS"   )->asDouble(),
+			Parameters("KERNEL_DIRECTION")->asDouble(),
+			Parameters("KERNEL_TOLERANCE")->asDouble()
+		));
+	}
+
+	return( false );
+}
+
+//---------------------------------------------------------
+bool CSG_Grid_Cell_Addressor::Set_Square (CSG_Parameters &Parameters)	{	return( Set_Parameters(Parameters, SG_GRIDCELLADDR_PARM_SQUARE ) );	}
+bool CSG_Grid_Cell_Addressor::Set_Circle (CSG_Parameters &Parameters)	{	return( Set_Parameters(Parameters, SG_GRIDCELLADDR_PARM_CIRCLE ) );	}
+bool CSG_Grid_Cell_Addressor::Set_Annulus(CSG_Parameters &Parameters)	{	return( Set_Parameters(Parameters, SG_GRIDCELLADDR_PARM_ANNULUS) );	}
+bool CSG_Grid_Cell_Addressor::Set_Sector (CSG_Parameters &Parameters)	{	return( Set_Parameters(Parameters, SG_GRIDCELLADDR_PARM_SECTOR ) );	}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CSG_Grid_Cell_Addressor::Enable_Parameters(CSG_Parameters &Parameters)
+{
+	if( Parameters("KERNEL_TYPE") )
+	{
+		int	Type	= Parameters("KERNEL_TYPE")->asChoice()->Get_Item_Data(Parameters("KERNEL_TYPE")->asInt()).asInt();
+
+		Parameters.Set_Enabled("KERNEL_INNER"    , Type == SG_GRIDCELLADDR_PARM_ANNULUS);
+		Parameters.Set_Enabled("KERNEL_DIRECTION", Type == SG_GRIDCELLADDR_PARM_SECTOR );
+		Parameters.Set_Enabled("KERNEL_TOLERANCE", Type == SG_GRIDCELLADDR_PARM_SECTOR );
+	}
+
+	if( Parameters("DW_WEIGHTING") )
+	{
+		int	Method	= Parameters("DW_WEIGHTING")->asInt();
+
+		Parameters.Set_Enabled("DW_IDW_OFFSET", Method == 1);
+		Parameters.Set_Enabled("DW_IDW_POWER" , Method == 1);
+		Parameters.Set_Enabled("DW_BANDWIDTH" , Method >= 2);
+	}
+
+	return( true );
+}
+
+
 ///////////////////////////////////////////////////////////
 //														 //
 ///////////////////////////////////////////////////////////
@@ -329,6 +507,9 @@ bool CSG_Grid_Cell_Addressor::Set_Radius(double Radius, bool bSquare)
 {
 	Destroy();
 
+	m_Type		= bSquare ? 1 : 0;
+	m_Parms[0]	= Radius;
+
 	//-----------------------------------------------------
 	if( Radius > 0.0 )
 	{
@@ -367,6 +548,10 @@ bool CSG_Grid_Cell_Addressor::Set_Annulus(double inner_Radius, double outer_Radi
 {
 	Destroy();
 
+	m_Type		= 2;
+	m_Parms[0]	= outer_Radius;
+	m_Parms[1]	= inner_Radius;
+
 	//-----------------------------------------------------
 	if( inner_Radius <= outer_Radius )
 	{
@@ -408,14 +593,17 @@ bool CSG_Grid_Cell_Addressor::Set_Sector(double Radius, double Direction, double
 {
 	Destroy();
 
+	m_Type		= 3;
+	m_Parms[0]	= Radius;
+	m_Parms[2]	= Direction;
+	m_Parms[3]	= Tolerance;
+
 	//-----------------------------------------------------
 	if( Radius > 0.0 )
 	{
-		TSG_Point			a, b;
-		CSG_Shapes			Polygons(SHAPE_TYPE_Polygon);	// Polygons.Add_Field(SG_T("ID"), SG_DATATYPE_Int);
-		CSG_Shape_Polygon	*pPolygon	= (CSG_Shape_Polygon *)Polygons.Add_Shape();
+		TSG_Point	a, b;
 
-		Direction	= fmod(Direction, M_PI_360);	if( Direction < 0.0 )	Direction	+= M_PI_360;
+		Direction	= fmod(Direction, M_PI_360); if( Direction < 0.0 ) Direction += M_PI_360;
 
 		if( Direction < M_PI_090 )
 		{
@@ -440,6 +628,9 @@ bool CSG_Grid_Cell_Addressor::Set_Sector(double Radius, double Direction, double
 
 		double	d	= 10.0 * SG_Get_Length(Radius, Radius);
 
+		CSG_Shapes	Polygons(SHAPE_TYPE_Polygon);
+		CSG_Shape_Polygon	*pPolygon	= (CSG_Shape_Polygon *)Polygons.Add_Shape();
+
 		pPolygon->Add_Point(b.x, b.y);
 		pPolygon->Add_Point(a.x, a.y);
 		pPolygon->Add_Point(a.x + d * sin(Direction - Tolerance), a.y + d * cos(Direction - Tolerance));
@@ -453,10 +644,10 @@ bool CSG_Grid_Cell_Addressor::Set_Sector(double Radius, double Direction, double
 			{
 				if( (d = SG_Get_Length(x, y)) <= Radius )
 				{
-					if( pPolygon->Contains( x,  y) )	ADD_CELL( x,  y, d);
-					if( pPolygon->Contains( y, -x) )	ADD_CELL( y, -x, d);
-					if( pPolygon->Contains(-x, -y) )	ADD_CELL(-x, -y, d);
-					if( pPolygon->Contains(-y,  x) )	ADD_CELL(-y,  x, d);
+					if( pPolygon->Contains( x,  y) ) ADD_CELL( x,  y, d);
+					if( pPolygon->Contains( y, -x) ) ADD_CELL( y, -x, d);
+					if( pPolygon->Contains(-x, -y) ) ADD_CELL(-x, -y, d);
+					if( pPolygon->Contains(-y,  x) ) ADD_CELL(-y,  x, d);
 				}
 			}
 		}
