@@ -1,6 +1,3 @@
-/**********************************************************
- * Version $Id: mat_tools.h 2452 2015-03-20 14:37:10Z oconrad $
- *********************************************************/
 
 ///////////////////////////////////////////////////////////
 //                                                       //
@@ -26,7 +23,8 @@
 // This library is free software; you can redistribute   //
 // it and/or modify it under the terms of the GNU Lesser //
 // General Public License as published by the Free       //
-// Software Foundation, version 2.1 of the License.      //
+// Software Foundation, either version 2.1 of the        //
+// License, or (at your option) any later version.       //
 //                                                       //
 // This library is distributed in the hope that it will  //
 // be useful, but WITHOUT ANY WARRANTY; without even the //
@@ -36,9 +34,7 @@
 //                                                       //
 // You should have received a copy of the GNU Lesser     //
 // General Public License along with this program; if    //
-// not, write to the Free Software Foundation, Inc.,     //
-// 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, //
-// USA.                                                  //
+// not, see <http://www.gnu.org/licenses/>.              //
 //                                                       //
 //-------------------------------------------------------//
 //                                                       //
@@ -53,15 +49,6 @@
 //                                                       //
 ///////////////////////////////////////////////////////////
 
-//---------------------------------------------------------
-
-
-///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
-///////////////////////////////////////////////////////////
-
 //---------------------------------------------------------
 #ifndef HEADER_INCLUDED__SAGA_API__mat_tools_H
 #define HEADER_INCLUDED__SAGA_API__mat_tools_H
@@ -129,6 +116,7 @@
 #define SG_ROUND_TO_DWORD(x)	((DWORD)(x < 0.0 ? x - 0.5 : x + 0.5))
 #define SG_ROUND_TO_INT(x)		((int  )(x < 0.0 ? x - 0.5 : x + 0.5))
 #define SG_ROUND_TO_LONG(x)		((long )(x < 0.0 ? x - 0.5 : x + 0.5))
+#define SG_ROUND_TO_ULONG(x)	((uLong)(x < 0.0 ? x - 0.5 : x + 0.5))
 #define SG_ROUND_TO_SLONG(x)	((sLong)(x < 0.0 ? x - 0.5 : x + 0.5))
 
 
@@ -155,6 +143,10 @@ SAGA_API_DLL_EXPORT	int			SG_Compare_Int			(const void *a, const void *b);
 SAGA_API_DLL_EXPORT	int			SG_Compare_Double		(const void *a, const void *b);
 SAGA_API_DLL_EXPORT	int			SG_Compare_Char_Ptr		(const void *a, const void *b);
 
+//---------------------------------------------------------
+SAGA_API_DLL_EXPORT	double		SG_Degree_To_Decimal	(              double  Deg, double  Min, double  Sec);
+SAGA_API_DLL_EXPORT	void		SG_Decimal_To_Degree	(double Value, double &Deg, double &Min, double &Sec);
+
 
 ///////////////////////////////////////////////////////////
 //														 //
@@ -278,7 +270,9 @@ public:
 	bool						Set_Zero			(void);
 	bool						Set_Unity			(void);
 
-	bool						Sort				(void);
+	bool						Flip_Values			(void);
+
+	bool						Sort				(bool bAscending = true);
 
 	double						Get_Length			(void)						const;
 	double						Get_Angle			(const CSG_Vector &Vector)	const;
@@ -296,12 +290,19 @@ private:
 
 	CSG_Array					m_Array;
 
-
 };
 
 //---------------------------------------------------------
 SAGA_API_DLL_EXPORT CSG_Vector	operator *			(double Scalar, const CSG_Vector &Vector);
 
+//---------------------------------------------------------
+SAGA_API_DLL_EXPORT bool	SG_VectorR2_Rotate		(double &x, double &y, double Angle);
+SAGA_API_DLL_EXPORT bool	SG_VectorR2_Rotate		(double     Vector[2], double Angle);
+SAGA_API_DLL_EXPORT bool	SG_VectorR2_Rotate		(CSG_Vector &Vector  , double Angle);
+
+SAGA_API_DLL_EXPORT bool	SG_VectorR3_Rotate		(double     Vector[3], size_t Axis, double Angle);
+SAGA_API_DLL_EXPORT bool	SG_VectorR3_Rotate		(CSG_Vector &Vector  , size_t Axis, double Angle);
+
 
 ///////////////////////////////////////////////////////////
 //														 //
@@ -500,7 +501,7 @@ private:
 
 	int							m_maxRadius, m_nPoints, *m_nPoints_R;
 
-	typedef struct
+	typedef struct SSG_Grid_Radius
 	{
 		int						x, y;
 
@@ -558,29 +559,45 @@ public:
 	bool						Create				(const CSG_Vector &Values, bool bHoldValues = false);
 
 	void						Invalidate			(void);
+	bool						Evaluate			(void);
 
-	bool						is_Evaluated		(void)	const	{	return( m_bEvaluated );	}
+	int							is_Evaluated		(void)	const	{	return( m_bEvaluated );	}
+
+	bool						Set_Count			(sLong Count);
 
 	sLong						Get_Count			(void)	const	{	return( m_nValues );	}
 	double						Get_Weights			(void)	const	{	return( m_Weights );	}
 
-	double						Get_Minimum			(void)		{	if( !m_bEvaluated )	_Evaluate(); return( m_Minimum	);	}
-	double						Get_Maximum			(void)		{	if( !m_bEvaluated )	_Evaluate(); return( m_Maximum	);	}
-	double						Get_Range			(void)		{	if( !m_bEvaluated )	_Evaluate(); return( m_Range	);	}
-	double						Get_Sum				(void)		{	if( !m_bEvaluated )	_Evaluate(); return( m_Sum		);	}
-	double						Get_Mean			(void)		{	if( !m_bEvaluated )	_Evaluate(); return( m_Mean		);	}
-	double						Get_Variance		(void)		{	if( !m_bEvaluated )	_Evaluate(); return( m_Variance	);	}
-	double						Get_StdDev			(void)		{	if( !m_bEvaluated )	_Evaluate(); return( m_StdDev	);	}
+	double						Get_Minimum			(void)		{	if( m_bEvaluated < 1 )	_Evaluate(1); return( m_Minimum  );	}
+	double						Get_Maximum			(void)		{	if( m_bEvaluated < 1 )	_Evaluate(1); return( m_Maximum  );	}
+	double						Get_Range			(void)		{	if( m_bEvaluated < 1 )	_Evaluate(1); return( m_Range    );	}
+	double						Get_Sum				(void)		{	if( m_bEvaluated < 1 )	_Evaluate(1); return( m_Sum      );	}
+	double						Get_Sum_Of_Squares	(void)		{	if( m_bEvaluated < 1 )	_Evaluate(1); return( m_Sum2     );	}
+	double						Get_Mean			(void)		{	if( m_bEvaluated < 1 )	_Evaluate(1); return( m_Mean     );	}
+	double						Get_Variance		(void)		{	if( m_bEvaluated < 1 )	_Evaluate(1); return( m_Variance );	}
+	double						Get_StdDev			(void)		{	if( m_bEvaluated < 1 )	_Evaluate(1); return( m_StdDev   );	}
+
+	double						Get_Kurtosis		(void)		{	if( m_bEvaluated < 2 )	_Evaluate(2); return( m_Kurtosis );	}
+	double						Get_Skewness		(void)		{	if( m_bEvaluated < 2 )	_Evaluate(2); return( m_Skewness );	}
+	double						Get_SkewnessPearson	(void);
 
 	double						Get_Median			(void)		{	return( Get_Quantile(50.0) );	}
 	double						Get_Quantile		(double Quantile);
+	double						Get_Gini			(void);
+
+	sLong						Get_IndexOfMinimum	(void);
+	sLong						Get_IndexOfMaximum	(void);
+
+	sLong						Get_nValues_Above	(double Threshold, bool bEquals = false);
+	sLong						Get_nValues_Below	(double Threshold, bool bEquals = false);
 
 	void						Add					(const CSG_Simple_Statistics &Statistics);
 
 	void						Add_Value			(double Value, double Weight = 1.0);
 
-	double						Get_Value			(sLong i)	const	{	return( i >= 0 && i < (sLong)m_Values.Get_Size() ? Get_Values()[i] : m_Mean );	}
 	double *					Get_Values			(void)		const	{	return( (double *)m_Values.Get_Array() );	}
+	double						Get_Value			(sLong i)	const	{	return( i >= 0 && i < (sLong)m_Values.Get_Size() ? Get_Values()[i] : m_Mean );	}
+	double						operator []			(sLong i)	const	{	return( i >= 0 && i < (sLong)m_Values.Get_Size() ? Get_Values()[i] : m_Mean );	}
 
 
 	CSG_Simple_Statistics &		operator  =			(const CSG_Simple_Statistics &Statistics)	{	Create(Statistics);	return( *this );	}
@@ -590,16 +607,18 @@ public:
 
 protected:
 
-	bool						m_bEvaluated, m_bSorted;
+	bool						m_bSorted;
+
+	int							m_bEvaluated;
 
 	sLong						m_nValues;
 
-	double						m_Weights, m_Sum, m_Sum2, m_Minimum, m_Maximum, m_Range, m_Mean, m_Variance, m_StdDev;
+	double						m_Weights, m_Sum, m_Sum2, m_Minimum, m_Maximum, m_Range, m_Mean, m_Variance, m_StdDev, m_Kurtosis, m_Skewness, m_Gini;
 
 	CSG_Array					m_Values;
 
 
-	void						_Evaluate			(void);
+	void						_Evaluate			(int Level = 1);
 
 };
 
@@ -611,75 +630,320 @@ protected:
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-class SAGA_API_DLL_EXPORT CSG_Class_Statistics
+class SAGA_API_DLL_EXPORT CSG_Unique_Value_Statistics
 {
+public:
+	CSG_Unique_Value_Statistics					(void)	{}
+	virtual ~CSG_Unique_Value_Statistics		(void)	{}
+
+	virtual void			Create				(bool bWeights = false)	{	m_bWeights	= bWeights;	}
+
+	int						Get_Count			(void)	const	{	return( (int)m_Count.Get_Size() );	}
+	int						Get_Count			(int i)	const	{	return( m_Count[i] );	}
+	double					Get_Weight			(int i)	const	{	return( m_bWeights ? m_Weight[i] : m_Count[i] );	}
+
+	virtual int				Get_Majority		(bool bWeighted = false)	const;
+	virtual int				Get_Minority		(bool bWeighted = false)	const;
+
+
+protected:
+
+	bool					m_bWeights;
+
+	CSG_Array_Int			m_Count;
+
+	CSG_Vector				m_Weight;
+
+};
+
+//---------------------------------------------------------
+class SAGA_API_DLL_EXPORT CSG_Unique_Number_Statistics : public CSG_Unique_Value_Statistics
+{
+public:
+	CSG_Unique_Number_Statistics				(bool bWeights = false)	{	Create(bWeights);	}
+	virtual ~CSG_Unique_Number_Statistics		(void)	{}
+
+	virtual void			Create				(bool bWeights = false);
+
+	void					operator +=			(double Value)	{	Add_Value(Value);	}
+	void					Add_Value			(double Value, double Weight = 1.0);
+	double					Get_Value			(int i)	const	{	return( m_Value[i] );	}
+	bool					Get_Class			(int i, double &Value, int &Count)	const
+	{
+		if( i < 0 || i >= Get_Count() )	return( false );
+
+		Count	= m_Count[i];
+		Value	= m_Value[i];
+
+		return( true );
+	}
+
+	bool					Get_Majority		(double &Value            )	const	{	int	Count; return( Get_Class(CSG_Unique_Value_Statistics::Get_Majority(), Value, Count) );	}
+	bool					Get_Majority		(double &Value, int &Count)	const	{	           return( Get_Class(CSG_Unique_Value_Statistics::Get_Majority(), Value, Count) && Count > 0 );	}
+	bool					Get_Minority		(double &Value            )	const	{	int	Count; return( Get_Class(CSG_Unique_Value_Statistics::Get_Minority(), Value, Count) );	}
+	bool					Get_Minority		(double &Value, int &Count)	const	{	           return( Get_Class(CSG_Unique_Value_Statistics::Get_Minority(), Value, Count) && Count > 0 );	}
+
+
 private:
 
-	typedef struct SClass
-	{
-		int		Count;
-
-		double	Value;
-	}
-	TClass;
+	CSG_Vector				m_Value;
 
+};
 
+//---------------------------------------------------------
+class SAGA_API_DLL_EXPORT CSG_Unique_String_Statistics : public CSG_Unique_Value_Statistics
+{
 public:
-	 CSG_Class_Statistics(void);
-	~CSG_Class_Statistics(void);
+	CSG_Unique_String_Statistics				(bool bWeights = false)	{	Create(bWeights);	}
+	virtual ~CSG_Unique_String_Statistics		(void)	{}
 
-	void			Create					(void);
+	virtual void			Create				(bool bWeights = false);
+
+	void					operator +=			(const CSG_String &Value)	{	Add_Value(Value);	}
+	void					Add_Value			(const CSG_String &Value, double Weight = 1.0);
+	const SG_Char *			Get_Value			(int i)	const	{	return( m_Value[i] );	}
+	bool					Get_Class			(int i, CSG_String &Value, int &Count)	const
+	{
+		if( i < 0 || i >= Get_Count() )	return( false );
+
+		Count	= m_Count[i];
+		Value	= m_Value[i];
+
+		return( true );
+	}
+
+	bool					Get_Majority		(CSG_String &Value            )	const	{	int	Count; return( Get_Class(CSG_Unique_Value_Statistics::Get_Majority(), Value, Count) );	}
+	bool					Get_Majority		(CSG_String &Value, int &Count)	const	{	           return( Get_Class(CSG_Unique_Value_Statistics::Get_Majority(), Value, Count) && Count > 0 );	}
+	bool					Get_Minority		(CSG_String &Value            )	const	{	int	Count; return( Get_Class(CSG_Unique_Value_Statistics::Get_Minority(), Value, Count) );	}
+	bool					Get_Minority		(CSG_String &Value, int &Count)	const	{	           return( Get_Class(CSG_Unique_Value_Statistics::Get_Minority(), Value, Count) && Count > 0 );	}
+
+
+private:
+
+	CSG_Strings				m_Value;
+
+};
+
+
+///////////////////////////////////////////////////////////
+//														 //
+//														 //
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+class SAGA_API_DLL_EXPORT CSG_Category_Statistics
+{
+public:
+	         CSG_Category_Statistics		(TSG_Data_Type Type = SG_DATATYPE_String);
+	virtual ~CSG_Category_Statistics		(void);
+
+	void			Create					(TSG_Data_Type Type = SG_DATATYPE_String);
 	void			Destroy					(void);
 
-	void			Reset					(void)	{	m_Array.Set_Array(0, (void **)&m_Classes, false);	}
+	TSG_Data_Type	Get_Category_Type		(void)	const;
 
-	int				Get_Count				(void)	{	return( (int)m_Array.Get_Size() );	}
+	int				Add_Value				(int               Value);
+	int				Add_Value				(double            Value);
+	int				Add_Value				(const CSG_String &Value);
 
-	int				Get_Class_Count			(int i)	{	return( i >= 0 && i < Get_Count() ? m_Classes[i].Count : 0 );	}
-	double			Get_Class_Value			(int i)	{	return( i >= 0 && i < Get_Count() ? m_Classes[i].Value : 0 );	}
+	void			operator +=				(int               Value)	{	Add_Value(Value);	}
+	void			operator +=				(double            Value)	{	Add_Value(Value);	}
+	void			operator +=				(const CSG_String &Value)	{	Add_Value(Value);	}
 
-	bool			Get_Class				(int i, double &Value, int &Count)
+	bool			Sort					(void);
+
+	int				Get_Count				(void         )	const;
+	int				Get_Count				(int iCategory)	const;
+
+	int				asInt					(int iCategory)	const;
+	double			asDouble				(int iCategory)	const;
+	CSG_String		asString				(int iCategory)	const;
+
+	int				Get_Category			(int               Value)	const;
+	int				Get_Category			(double            Value)	const;
+	int				Get_Category			(const CSG_String &Value)	const;
+
+	bool			Get_Category			(int iCategory, int        &Value, int &Count)	const
 	{
-		if( i >= 0 && i < Get_Count() )
-		{
-			Count	= m_Classes[i].Count;
-			Value	= m_Classes[i].Value;
+		Count	= Get_Count(iCategory);
+		Value	= asInt    (iCategory);
 
-			return( true );
-		}
-
-		return( false );
+		return( iCategory >= 0 && iCategory < Get_Count() );
 	}
 
-	bool			Get_Class				(int i, int &Value, int &Count)
+	bool			Get_Category			(int iCategory, double     &Value, int &Count)	const
 	{
-		if( i >= 0 && i < Get_Count() )
-		{
-			Count	=      m_Classes[i].Count;
-			Value	= (int)m_Classes[i].Value;
+		Count	= Get_Count(iCategory);
+		Value	= asDouble (iCategory);
 
-			return( true );
-		}
-
-		return( false );
+		return( iCategory >= 0 && iCategory < Get_Count() );
 	}
 
-	void			Add_Value				(double Value);
+	bool			Get_Category			(int iCategory, CSG_String &Value, int &Count)	const
+	{
+		Count	= Get_Count(iCategory);
+		Value	= asString (iCategory);
+
+		return( iCategory >= 0 && iCategory < Get_Count() );
+	}
 
 	int				Get_Majority			(void);
-	bool			Get_Majority			(double &Value);
-	bool			Get_Majority			(double &Value, int &Count);
+	bool			Get_Majority			(int        &Value            )	{	int	Count; return( Get_Category(Get_Majority(), Value, Count) );	}
+	bool			Get_Majority			(double     &Value            )	{	int	Count; return( Get_Category(Get_Majority(), Value, Count) );	}
+	bool			Get_Majority			(CSG_String &Value            )	{	int	Count; return( Get_Category(Get_Majority(), Value, Count) );	}
+	bool			Get_Majority			(int        &Value, int &Count)	{	           return( Get_Category(Get_Majority(), Value, Count) && Count > 0 );	}
+	bool			Get_Majority			(double     &Value, int &Count)	{	           return( Get_Category(Get_Majority(), Value, Count) && Count > 0 );	}
+	bool			Get_Majority			(CSG_String &Value, int &Count)	{	           return( Get_Category(Get_Majority(), Value, Count) && Count > 0 );	}
 
 	int				Get_Minority			(void);
-	bool			Get_Minority			(double &Value);
-	bool			Get_Minority			(double &Value, int &Count);
+	bool			Get_Minority			(int        &Value            )	{	int	Count; return( Get_Category(Get_Minority(), Value, Count) );	}
+	bool			Get_Minority			(double     &Value            )	{	int	Count; return( Get_Category(Get_Minority(), Value, Count) );	}
+	bool			Get_Minority			(CSG_String &Value            )	{	int	Count; return( Get_Category(Get_Minority(), Value, Count) );	}
+	bool			Get_Minority			(int        &Value, int &Count)	{	           return( Get_Category(Get_Minority(), Value, Count) && Count > 0 );	}
+	bool			Get_Minority			(double     &Value, int &Count)	{	           return( Get_Category(Get_Minority(), Value, Count) && Count > 0 );	}
+	bool			Get_Minority			(CSG_String &Value, int &Count)	{	           return( Get_Category(Get_Minority(), Value, Count) && Count > 0 );	}
 
 
 private:
 
-	CSG_Array		m_Array;
+	class CSG_Table	*m_pTable;
 
-	TClass			*m_Classes;
+};
+
+
+///////////////////////////////////////////////////////////
+//														 //
+//														 //
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+class SAGA_API_DLL_EXPORT CSG_Histogram
+{
+public:
+	CSG_Histogram(void);
+	virtual ~CSG_Histogram(void);
+
+	bool			Destroy				(void);
+
+	CSG_Histogram						(size_t nClasses, double Minimum, double Maximum);
+	bool			Create				(size_t nClasses, double Minimum, double Maximum);
+
+	CSG_Histogram						(size_t nClasses, double Minimum, double Maximum, const CSG_Vector &Values          , size_t maxSamples = 0);
+	bool			Create				(size_t nClasses, double Minimum, double Maximum, const CSG_Vector &Values          , size_t maxSamples = 0);
+
+	CSG_Histogram						(size_t nClasses, double Minimum, double Maximum, class CSG_Table *pTable, int Field, size_t maxSamples = 0);
+	bool			Create				(size_t nClasses, double Minimum, double Maximum, class CSG_Table *pTable, int Field, size_t maxSamples = 0);
+
+	CSG_Histogram						(size_t nClasses, double Minimum, double Maximum, class CSG_Grid  *pGrid            , size_t maxSamples = 0);
+	bool			Create				(size_t nClasses, double Minimum, double Maximum, class CSG_Grid  *pGrid            , size_t maxSamples = 0);
+
+	CSG_Histogram						(size_t nClasses, double Minimum, double Maximum, class CSG_Grids *pGrids           , size_t maxSamples = 0);
+	bool			Create				(size_t nClasses, double Minimum, double Maximum, class CSG_Grids *pGrids           , size_t maxSamples = 0);
+
+	//-----------------------------------------------------
+	void			Add_Value			(double Value);
+
+	bool			Scale_Element_Count	(double Scale);
+
+	bool			Update				(void);
+
+	size_t			Get_Class_Count		(void)		const	{	return( m_nClasses );	}
+
+	size_t			Get_Element_Count	(void)		const	{	return( m_nClasses > 0 ? m_Cumulative[m_nClasses - 1] : 0 );	}
+	size_t			Get_Element_Maximum	(void)		const	{	return( m_nMaximum );	}
+
+	size_t			Get_Elements		(int    i)	const	{	return( Get_Elements((size_t)i) );	}
+	size_t			Get_Elements		(size_t i)	const	{	return( i < m_nClasses ? m_Elements[i] : 0 );	}
+
+	size_t			Get_Cumulative		(int    i)	const	{	return( Get_Cumulative((size_t)i) );	}
+	size_t			Get_Cumulative		(size_t i)	const	{	return( i < m_nClasses ? m_Cumulative[i] : 0 );	}
+
+	double			Get_Value			(double i)	const	{	return( m_nClasses < 1 ? m_Minimum : m_Minimum + i * m_ClassWidth );	}
+
+	double			Get_Break			(int    i)	const	{	return( Get_Value((double)(i)) );	}
+	double			Get_Break			(size_t i)	const	{	return( Get_Value((double)(i)) );	}
+
+	double			Get_Center			(int    i)	const	{	return( Get_Value((double)(i + 0.5)) );	}
+	double			Get_Center			(size_t i)	const	{	return( Get_Value((double)(i + 0.5)) );	}
+
+	//-----------------------------------------------------
+	void			operator +=			(double Value)		{	Add_Value(Value);	}
+
+	size_t			operator []			(int    i)	const	{	return( Get_Elements(i) );	}
+	size_t			operator []			(size_t i)	const	{	return( Get_Elements(i) );	}
+
+	double			Get_Quantile		(double   Quantile)	const;
+	double			Get_Quantile_Value	(double      Value)	const;
+
+	double			Get_Percentile		(double Percentile)	const;
+	double			Get_Percentile_Value(double      Value)	const;
+
+	const CSG_Simple_Statistics &	Get_Statistics	(void)	const	{	return( m_Statistics );	}
+
+
+private:
+
+	size_t					m_nClasses, m_nMaximum, *m_Elements, *m_Cumulative;
+
+	double					m_Minimum, m_Maximum, m_ClassWidth;
+
+	CSG_Simple_Statistics	m_Statistics;
+
+
+	void					_On_Construction	(void);
+
+	bool					_Create				(size_t nClasses, double Minimum, double Maximum);
+
+	bool					_Update				(sLong nElements);
+
+};
+
+
+///////////////////////////////////////////////////////////
+//														 //
+//														 //
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+class SAGA_API_DLL_EXPORT CSG_Natural_Breaks
+{
+public:
+	CSG_Natural_Breaks(void);
+	virtual ~CSG_Natural_Breaks(void);
+
+	CSG_Natural_Breaks				(class CSG_Table *pTable, int Field, int nClasses, int Histogram = 0);
+	bool			Create			(class CSG_Table *pTable, int Field, int nClasses, int Histogram = 0);
+
+	CSG_Natural_Breaks				(class CSG_Grid  *pGrid            , int nClasses, int Histogram = 0);
+	bool			Create			(class CSG_Grid  *pGrid            , int nClasses, int Histogram = 0);
+
+	CSG_Natural_Breaks				(class CSG_Grids *pGrids           , int nClasses, int Histogram = 0);
+	bool			Create			(class CSG_Grids *pGrids           , int nClasses, int Histogram = 0);
+
+	CSG_Natural_Breaks				(const CSG_Vector &Values          , int nClasses, int Histogram = 0);
+	bool			Create			(const CSG_Vector &Values          , int nClasses, int Histogram = 0);
+
+	int				Get_Count		(void)	const	{	return( m_Breaks.Get_N() );	}
+	double			Get_Break		(int i)	const	{	return( m_Breaks[i] );	}
+
+	double			operator []		(int i)	const	{	return( m_Breaks[i] );	}
+
+
+private:
+
+	CSG_Histogram	m_Histogram;
+
+	CSG_Vector		m_Breaks, m_Values;
+
+
+	bool			_Histogram		(int nClasses);
+
+	double			_Get_Value		(int i);
+
+	bool			_Calculate		(int nClasses);
 
 };
 
@@ -703,21 +967,21 @@ class SAGA_API_DLL_EXPORT CSG_Cluster_Analysis
 {
 public:
 	CSG_Cluster_Analysis(void);
-	~CSG_Cluster_Analysis(void);
-	
+	virtual ~CSG_Cluster_Analysis(void);
+
 	bool					Create				(int nFeatures);
 	bool					Destroy				(void);
 
 	bool					Add_Element			(void);
 	bool					Set_Feature			(int iElement, int iFeature, double Value);
 
-	int						Get_Cluster			(int iElement)	const	{	return( iElement >= 0 && iElement < Get_nElements() ? m_Cluster[iElement] : -1 );	}
+	int						Get_Cluster			(int iElement)	const	{	return( iElement >= 0 && iElement < Get_nElements() ? m_Clusters[iElement] : -1 );	}
 
-	bool					Execute				(int Method, int nClusters, int nMaxIterations = 0);
+	bool					Execute				(int Method, int nClusters, int nMaxIterations = 0, int Initialization = 0);
 
 	int						Get_nElements		(void)	const	{	return( (int)m_Features.Get_Size() );	}
-	int						Get_nFeatures		(void)	const	{	return( m_nFeatures );	}
-	int						Get_nClusters		(void)	const	{	return( m_nClusters );	}
+	int						Get_nFeatures		(void)	const	{	return(      m_nFeatures           );	}
+	int						Get_nClusters		(void)	const	{	return( (int)m_nMembers.Get_Size() );	}
 
 	int						Get_Iteration		(void)	const	{	return( m_Iteration );	}
 
@@ -730,15 +994,22 @@ public:
 
 private:
 
-	int						*m_Cluster, m_Iteration, m_nFeatures, m_nClusters, *m_nMembers;
+	int						m_Iteration, m_nFeatures;
 
-	double					*m_Variance, **m_Centroid, m_SP;
+	double					m_SP;
+
+	CSG_Array_Int			m_Clusters, m_nMembers;
 
 	CSG_Array				m_Features;
 
+	CSG_Vector				m_Variance;
 
-	bool					Minimum_Distance	(bool bInitialize, int nMaxIterations);
-	bool					Hill_Climbing		(bool bInitialize, int nMaxIterations);
+	CSG_Matrix				m_Centroid;
+
+
+	bool					_Minimum_Distance	(bool bInitialize, int nMaxIterations);
+
+	bool					_Hill_Climbing		(bool bInitialize, int nMaxIterations);
 
 };
 
@@ -750,7 +1021,7 @@ private:
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-enum
+enum ESG_Classify_Supervised
 {
 	SG_CLASSIFY_SUPERVISED_BinaryEncoding	= 0,
 	SG_CLASSIFY_SUPERVISED_ParallelEpiped,
@@ -788,7 +1059,11 @@ public:
 	int							Get_Feature_Count			(void)			{	return( m_nFeatures );				}
 
 	int							Get_Class_Count				(void)			{	return( m_nClasses );				}
-	const CSG_String &			Get_Class_ID				(int iClass)	{	return( m_pClasses[iClass]->m_ID );	}
+
+	const CSG_String &			Get_Class_ID				(int iClass)				{	return( m_pClasses[iClass]->m_ID );	}
+	double						Get_Class_Mean				(int iClass, int iFeature)	{	return( m_pClasses[iClass]->m_Mean[iFeature] );	}
+	double						Get_Class_Minimum			(int iClass, int iFeature)	{	return( m_pClasses[iClass]->m_Min [iFeature] );	}
+	double						Get_Class_Maximum			(int iClass, int iFeature)	{	return( m_pClasses[iClass]->m_Max [iFeature] );	}
 
 	int							Get_Class					(const CSG_String &Class_ID);
 	bool						Get_Class					(const CSG_Vector &Features, int &Class, double &Quality, int Method);
@@ -871,15 +1146,6 @@ enum ESG_mRMR_Method
 	SG_mRMR_Method_MIQ			// Mutual Information Quotient (MIQ)
 };
 
-//---------------------------------------------------------
-enum
-{
-	SG_mRMR_SELFLD_RANK	= 0,
-	SG_mRMR_SELFLD_INDEX,
-	SG_mRMR_SELFLD_NAME,
-	SG_mRMR_SELFLD_SCORE
-};
-
 
 ///////////////////////////////////////////////////////////
 //                                                       //
@@ -1348,17 +1614,26 @@ public:
 	bool						Destroy				(void);
 
 	bool						Add_Sample			(double Weight, double Dependent, const CSG_Vector &Predictors);
-	int							Get_Sample_Count	(void)	const	{	return( m_Y.Get_NRows()     );	}
-	int							Get_Predictor_Count	(void)	const	{	return( m_Y.Get_NCols() - 1 );	}
+	int							Get_Sample_Count	(void)	const	{	return( m_X.Get_NRows()     );	}
+	int							Get_Predictor_Count	(void)	const	{	return( m_X.Get_NCols() - 1 );	}
 
-	bool						Calculate			(const CSG_Vector &Weights, const CSG_Vector &Dependents, const CSG_Matrix &Predictors);
-	bool						Calculate			(void);
+	bool						Calculate			(const CSG_Vector &Weights, const CSG_Vector &Dependents, const CSG_Matrix &Predictors, bool bLogistic = false);
+	bool						Calculate			(bool bLogistic = false);
 
 	double						Get_R2				(void)	const	{	return( m_r2   );	}
 	const CSG_Vector &			Get_RCoeff			(void)	const	{	return( m_b    );	}
 	double						Get_RCoeff			(int i)	const	{	return( m_b[i] );	}
 	double						operator []			(int i)	const	{	return( m_b[i] );	}
 
+	//-----------------------------------------------------
+	int							Get_Log_maxIter		(void)	const	{	return( m_Log_maxIter    );	}
+	double						Get_Log_Epsilon		(void)	const	{	return( m_Log_Epsilon    );	}
+	double						Get_Log_Difference	(void)	const	{	return( m_Log_Difference );	}
+
+	bool						Set_Log_maxIter		(int    maxIter   );
+	bool						Set_Log_Epsilon		(double Epsilon   );
+	bool						Set_Log_Difference	(double Difference);
+
 	//-----------------------------------------------------
 	bool						Get_CrossValidation	(int nSubSamples = 0);
 
@@ -1370,13 +1645,22 @@ public:
 
 private:
 
-	int							m_CV_nSamples;
+	int							m_Log_maxIter, m_CV_nSamples;
 
-	double						m_r2, m_CV_RMSE, m_CV_NRMSE, m_CV_R2;
+	double						m_r2, m_Log_Epsilon, m_Log_Difference, m_CV_RMSE, m_CV_NRMSE, m_CV_R2;
 
-	CSG_Vector					m_x, m_w, m_b;
+	CSG_Vector					m_y, m_w, m_b;
 
-	CSG_Matrix					m_Y;
+	CSG_Matrix					m_X;
+
+
+	CSG_Vector					_Log_Get_Beta		(const CSG_Matrix &X, const CSG_Vector &y, const CSG_Vector &w);
+	CSG_Vector					_Log_Get_Beta		(const CSG_Vector &b, const CSG_Matrix &X, const CSG_Vector &y, const CSG_Vector &w, const CSG_Vector &p);
+	CSG_Matrix					_Log_Get_Xwp		(const CSG_Vector &p, const CSG_Matrix &X, const CSG_Vector &w);
+	CSG_Vector					_Log_Get_Ywp		(const CSG_Vector &p, const CSG_Vector &y, const CSG_Vector &w);
+	CSG_Vector					_Log_Get_Props		(const CSG_Matrix &X, const CSG_Vector &b);
+	bool						_Log_NoChange		(const CSG_Vector &b_old, const CSG_Vector &b_new);
+	bool						_Log_OutOfControl	(const CSG_Vector &b_old, const CSG_Vector &b_new);
 
 };
 
@@ -1402,12 +1686,11 @@ public:
 
 	bool						Destroy				(void);
 
-	static CSG_String			Get_Help_Operators	(void);
+	static CSG_String			Get_Help_Operators	(bool bHTML = true, const CSG_String Additional[][2] = NULL);
 
 	bool						Get_Error			(CSG_String &Message);
 
 	int							Add_Function		(const SG_Char *Name, TSG_PFNC_Formula_1 f, int N_of_Pars, int Varying);
-	int							Del_Function		(SG_Char *Name);
 
 	bool						Set_Formula			(const CSG_String &Formula);
 	CSG_String					Get_Formula			(void)	const	{	return( m_sFormula );	}
@@ -1424,12 +1707,12 @@ public:
 
 
 	//-----------------------------------------------------
-	typedef struct 
+	typedef struct SSG_Formula_Item
 	{
-		SG_Char					*name;
-		TSG_PFNC_Formula_1		f;			
-		int						n_pars;		
-		int						varying;	// Does the result of the function vary even when the parameters stay the same? varying = 1 for e.g. random - number generators.
+		const SG_Char		*name;
+		TSG_PFNC_Formula_1	f;
+		int					n_pars;		
+		int					varying;	// Does the result of the function vary even when the parameters stay the same? varying = 1 for e.g. random - number generators.
 	}
 	TSG_Formula_Item;
 
@@ -1437,7 +1720,7 @@ public:
 private:
 
 	//-----------------------------------------------------
-	typedef struct 
+	typedef struct SMAT_Formula
 	{
 		SG_Char					*code;
 		double					*ctable;
@@ -1452,6 +1735,9 @@ private:
 
 	TMAT_Formula				m_Formula;
 
+	TSG_Formula_Item			*m_Functions;
+
+
 	CSG_String					m_sFormula, m_sError;
 
 	const SG_Char				*i_error;
@@ -1495,7 +1781,8 @@ enum ESG_Trend_String
 	SG_TREND_STRING_Formula	= 0,
 	SG_TREND_STRING_Function,
 	SG_TREND_STRING_Formula_Parameters,
-	SG_TREND_STRING_Complete
+	SG_TREND_STRING_Complete,
+	SG_TREND_STRING_Compact
 };
 
 //---------------------------------------------------------
@@ -1503,62 +1790,67 @@ class SAGA_API_DLL_EXPORT CSG_Trend
 {
 public:
 	CSG_Trend(void);
-	virtual ~CSG_Trend(void);
 
-	bool						Set_Formula			(const SG_Char *Formula = NULL);
+	bool						Set_Formula			(const CSG_String &Formula);
 	CSG_String					Get_Formula			(int Type = SG_TREND_STRING_Complete);
 
-	int							Get_Parameter_Count	(void) const	{	return( m_Params.m_Count );		}
-	double *					Get_Parameters		(void) const	{	return( m_Params.m_A );			}
+	int							Get_Parameter_Count	(void) const	{	return( m_Params   .Get_Count() );	}
+	double *					Get_Parameters		(void) const	{	return( m_Params.m_A.Get_Data() );	}
+	bool						Init_Parameter		(const SG_Char &Variable, double Value);
 
 	void						Clr_Data			(void);
-	void						Set_Data			(double *xData, double *yData, int nData, bool bAdd = false);
-	void						Set_Data			(const CSG_Points &Data, bool bAdd = false);
-	void						Add_Data			(double x, double y);
-	int							Get_Data_Count		(void) const	{	return( m_Data.Get_Count() );	}
-	double						Get_Data_X			(int Index)		{	return( m_Data.Get_X(Index) );	}
-	double						Get_Data_Y			(int Index)		{	return( m_Data.Get_Y(Index) );	}
-	double						Get_Data_XMin		(void)			{	return( m_xMin );	}
-	double						Get_Data_XMax		(void)			{	return( m_xMax );	}
-	double						Get_Data_YMin		(void)			{	return( m_yMin );	}
-	double						Get_Data_YMax		(void)			{	return( m_yMax );	}
+	bool						Add_Data			(double  x, double  y);
+	void						Set_Data			(double *x, double *y, int n, bool bAdd = false);
+	void						Set_Data			(const CSG_Points &Data     , bool bAdd = false);
+	int							Get_Data_Count		(void)	const	{	return( (int)m_Data[0].Get_Count() );	}
+	double						Get_Data_X			(int i)	const	{	return( m_Data[0][i] );	}
+	double						Get_Data_XMin		(void)			{	return( m_Data[0].Get_Minimum() );	}
+	double						Get_Data_XMax		(void)			{	return( m_Data[0].Get_Maximum() );	}
+	CSG_Simple_Statistics &		Get_Data_XStats		(void)			{	return( m_Data[0] );	}
+	double						Get_Data_Y			(int i)	const	{	return( m_Data[1][i] );	}
+	double						Get_Data_YMin		(void)			{	return( m_Data[1].Get_Minimum() );	}
+	double						Get_Data_YMax		(void)			{	return( m_Data[1].Get_Maximum() );	}
+	CSG_Simple_Statistics &		Get_Data_YStats		(void)			{	return( m_Data[1] );	}
 
 	bool						Set_Max_Iterations	(int Iterations);
-	int							Get_Max_Iterations	(void)			{	return( m_Iter_Max);	}
+	int							Get_Max_Iterations	(void)	const	{	return( m_Iter_Max);	}
 	bool						Set_Max_Lambda		(double Lambda);
-	double						Get_Max_Lambda		(void)			{	return( m_Lambda_Max);	}
+	double						Get_Max_Lambda		(void)	const	{	return( m_Lambda_Max);	}
 
-	bool						Get_Trend			(double *xData, double *yData, int nData, const SG_Char *Formula = NULL);
-	bool						Get_Trend			(const CSG_Points &Data, const SG_Char *Formula = NULL);
+	bool						Get_Trend			(double *x, double *y, int n, const CSG_String &Formula);
+	bool						Get_Trend			(const CSG_Points &Data     , const CSG_String &Formula);
+	bool						Get_Trend			(const CSG_String &Formula);
 	bool						Get_Trend			(void);
 
-	bool						is_Okay				(void)			{	return( m_bOkay );		}
+	bool						is_Okay				(void)	const	{	return( m_bOkay );	}
 
 	CSG_String					Get_Error			(void);
 
-	double						Get_ChiSquare		(void);
-	double						Get_R2				(void);
+	double						Get_ChiSquare		(void)	const	{	return( m_bOkay ? m_ChiSqr   : 0.0 );	}
+	double						Get_R2				(void)	const	{	return( m_bOkay ? m_ChiSqr_o : 0.0 );	}
 
-	double						Get_Value			(double x);
+	double						Get_Value			(double x)	const	{	return( m_bOkay ? m_Formula.Get_Value(x) : 0.0 );	}
 
 
 private:
 
 	//-----------------------------------------------------
-	class SAGA_API_DLL_EXPORT CFncParams
+	class SAGA_API_DLL_EXPORT CParams
 	{
 	public:
-		CFncParams(void);
-		virtual ~CFncParams(void);
+		CParams(void)	{}
 
-		bool					Create				(const SG_Char *Variables, int nVariables);
+		bool					Create				(const CSG_String &Variables);
 		bool					Destroy				(void);
 
-		int						m_Count;
+		int						Get_Count			(void)	const	{	return( (int)m_Variables.Length() );	}
 
-		SG_Char					*m_Variables;
 
-		double					*m_A, *m_Atry, *m_dA, *m_dA2, *m_Beta, **m_Alpha, **m_Covar;
+		CSG_String				m_Variables;
+
+		CSG_Vector				m_A, m_Atry, m_dA, m_dA2, m_Beta;
+
+		CSG_Matrix				m_Alpha, m_Covar;
 
 	};
 
@@ -1568,20 +1860,20 @@ private:
 
 	int							m_Iter_Max;
 
-	double						m_ChiSqr, m_ChiSqr_o, m_Lambda, m_Lambda_Max, m_xMin, m_xMax, m_yMin, m_yMax;
+	double						m_ChiSqr, m_ChiSqr_o, m_Lambda, m_Lambda_Max;
 
-	CSG_Points					m_Data;
+	CParams						m_Params;
 
-	CFncParams					m_Params;
+	CSG_Simple_Statistics		m_Data[2];
 
 	CSG_Formula					m_Formula;
 
 
 	bool						_Fit_Function		(void);
 	bool						_Get_Gaussj			(void);
-	bool						_Get_mrqcof			(double *Parameters, double **Alpha, double *Beta);
+	bool						_Get_mrqcof			(CSG_Vector &Parameters, CSG_Matrix &Alpha, CSG_Vector &Beta);
 
-	void						_Get_Function		(double x, double *Parameters, double &y, double *dy_da);
+	void						_Get_Function		(double &y, double *dy_da, double x, const double *Parameters);
 
 };
 
