@@ -1,6 +1,3 @@
-/**********************************************************
- * Version $Id: pointcloud.cpp 2436 2015-03-11 09:31:04Z reklov_w $
- *********************************************************/
 
 ///////////////////////////////////////////////////////////
 //                                                       //
@@ -26,7 +23,8 @@
 // This library is free software; you can redistribute   //
 // it and/or modify it under the terms of the GNU Lesser //
 // General Public License as published by the Free       //
-// Software Foundation, version 2.1 of the License.      //
+// Software Foundation, either version 2.1 of the        //
+// License, or (at your option) any later version.       //
 //                                                       //
 // This library is distributed in the hope that it will  //
 // be useful, but WITHOUT ANY WARRANTY; without even the //
@@ -36,9 +34,7 @@
 //                                                       //
 // You should have received a copy of the GNU Lesser     //
 // General Public License along with this program; if    //
-// not, write to the Free Software Foundation, Inc.,     //
-// 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, //
-// USA.                                                  //
+// not, see <http://www.gnu.org/licenses/>.              //
 //                                                       //
 //-------------------------------------------------------//
 //                                                       //
@@ -74,8 +70,9 @@
 #define PC_FILE_VERSION		"SGPC01"
 
 #define PC_STR_NBYTES		32
+#define PC_DAT_NBYTES		32
 
-#define PC_GET_NBYTES(type)	(type == SG_DATATYPE_String ? PC_STR_NBYTES : (int)SG_Data_Type_Get_Size(type))
+#define PC_GET_NBYTES(type)	(type == SG_DATATYPE_String ? PC_STR_NBYTES : type == SG_DATATYPE_Date ? PC_DAT_NBYTES : (int)SG_Data_Type_Get_Size(type))
 
 
 ///////////////////////////////////////////////////////////
@@ -97,9 +94,9 @@ CSG_PointCloud * SG_Create_PointCloud(const CSG_PointCloud &PointCloud)
 }
 
 //---------------------------------------------------------
-CSG_PointCloud * SG_Create_PointCloud(const CSG_String &File_Name)
+CSG_PointCloud * SG_Create_PointCloud(const CSG_String &FileName)
 {
-	return( new CSG_PointCloud(File_Name) );
+	return( new CSG_PointCloud(FileName) );
 }
 
 //---------------------------------------------------------
@@ -108,7 +105,6 @@ CSG_PointCloud * SG_Create_PointCloud(CSG_PointCloud *pStructure)
 	return( new CSG_PointCloud(pStructure) );
 }
 
-
 ///////////////////////////////////////////////////////////
 //														 //
 //														 //
@@ -128,7 +124,7 @@ bool CSG_PointCloud::Create(void)
 {
 	Destroy();
 
-	Add_Field(SG_T(""), SG_DATATYPE_Undefined);	// add x, y, z fields
+	Add_Field("", SG_DATATYPE_Undefined);	// add x, y, z fields
 
 	return( true );
 }
@@ -144,28 +140,21 @@ CSG_PointCloud::CSG_PointCloud(const CSG_PointCloud &PointCloud)
 
 bool CSG_PointCloud::Create(const CSG_PointCloud &PointCloud)
 {
-	if( Assign((CSG_Data_Object *)&PointCloud) )
-	{
-		Set_Name(PointCloud.Get_Name());
-
-		return( true );
-	}
-
-	return( false );
+	return( Assign((CSG_Data_Object *)&PointCloud) );
 }
 
 //---------------------------------------------------------
-CSG_PointCloud::CSG_PointCloud(const CSG_String &File_Name)
+CSG_PointCloud::CSG_PointCloud(const CSG_String &FileName)
 	: CSG_Shapes()
 {
 	_On_Construction();
 
-	Create(File_Name);
+	Create(FileName);
 }
 
-bool CSG_PointCloud::Create(const CSG_String &File_Name)
+bool CSG_PointCloud::Create(const CSG_String &FileName)
 {
-	return( _Load(File_Name) );
+	return( _Load(FileName) );
 }
 
 //---------------------------------------------------------
@@ -213,9 +202,6 @@ void CSG_PointCloud::_On_Construction(void)
 	m_Cursor		= NULL;
 	m_bXYZPrecDbl	= true;
 
-	m_Selected		= NULL;
-	m_nSelected		= 0;
-
 	Set_NoData_Value(-999999);
 
 	Set_Update_Flag();
@@ -224,8 +210,7 @@ void CSG_PointCloud::_On_Construction(void)
 	m_Shapes.Add_Shape();
 	m_Shapes_Index	= -1;
 
-	m_Array_Points  .Create(sizeof(char *), 0, SG_ARRAY_GROWTH_3);
-	m_Array_Selected.Create(sizeof(int   ), 0, SG_ARRAY_GROWTH_3);
+	m_Array_Points.Create(sizeof(char *), 0, SG_ARRAY_GROWTH_3);
 }
 
 //---------------------------------------------------------
@@ -246,9 +231,9 @@ bool CSG_PointCloud::Destroy(void)
 			delete(m_Field_Stats[i]);
 		}
 
-		SG_Free(m_Field_Name);
-		SG_Free(m_Field_Type);
-		SG_Free(m_Field_Stats);
+		SG_Free(m_Field_Name  );
+		SG_Free(m_Field_Type  );
+		SG_Free(m_Field_Stats );
 		SG_Free(m_Field_Offset);
 
 		_On_Construction();
@@ -267,62 +252,240 @@ bool CSG_PointCloud::Destroy(void)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-bool CSG_PointCloud::_Load(const CSG_String &File_Name)
+bool CSG_PointCloud::_Load(const CSG_String &FileName)
 {
-	TSG_Data_Type	Type;
+	SG_UI_Msg_Add(CSG_String::Format("%s: %s...", _TL("Loading point cloud"), FileName.c_str()), true);
 
-	char		ID[6];
-	int			i, iBuffer, nPointBytes, nFields;
-	char		Name[1024];
-	CSG_File	Stream;
+	bool	bResult	= false;
 
-	SG_UI_Msg_Add(CSG_String::Format(SG_T("%s: %s..."), _TL("Load point cloud"), File_Name.c_str()), true);
+	if( SG_File_Cmp_Extension(FileName, "sg-pts-z") ) // POINTCLOUD_FILE_FORMAT_Compressed
+	{
+		CSG_File_Zip	Stream(FileName, SG_FILE_R);
+
+		CSG_String	_FileName(SG_File_Get_Name(FileName, false) + ".");
+
+		if( (bResult = Stream.Get_File(_FileName + "sg-pts")) == false )
+		{
+			for(size_t i=0; i<Stream.Get_File_Count(); i++)
+			{
+				if( SG_File_Cmp_Extension(Stream.Get_File_Name(i), "sg-pts") )
+				{
+					_FileName	= SG_File_Get_Name(Stream.Get_File_Name(i), false) + ".";
+
+					break;
+				}
+			}
+
+			bResult	= Stream.Get_File(_FileName + "sg-pts");
+		}
+
+		if( bResult && _Load(Stream) )
+		{
+			if( Stream.Get_File(_FileName + "sg-info") )
+			{
+				Load_MetaData(Stream);
+			}
+
+			if( Stream.Get_File(_FileName + "sg-prj") )
+			{
+				Get_Projection().Load(Stream, SG_PROJ_FMT_WKT);
+			}
+		}
+	}
+	else // if( SG_File_Cmp_Extension(FileName, "sg-pts"/"spc") ) // POINTCLOUD_FILE_FORMAT_Normal
+	{
+		CSG_File	Stream(FileName, SG_FILE_R, true);
+
+		if( (bResult = _Load(Stream)) == true )
+		{
+			Load_MetaData(FileName);
+
+			Get_Projection().Load(SG_File_Make_Path("", FileName, "sg-prj"), SG_PROJ_FMT_WKT);
+		}
+	}
 
 	//-----------------------------------------------------
-	if( !Stream.Open(File_Name, SG_FILE_R, true) )
-	{
-		SG_UI_Msg_Add(_TL("failed"), false, SG_UI_MSG_STYLE_FAILURE);
-		SG_UI_Msg_Add_Error(_TL("file could not be opened."));
+	SG_UI_Process_Set_Ready();
 
+	if( bResult )
+	{
+		Set_Modified(false);
+
+		Set_File_Name(FileName, true);
+
+		SG_UI_Msg_Add(_TL("okay"), false, SG_UI_MSG_STYLE_SUCCESS);
+
+		return( true );
+	}
+
+	SG_UI_Msg_Add(_TL("failed"), false, SG_UI_MSG_STYLE_FAILURE);
+
+	return( false );
+}
+
+//---------------------------------------------------------
+bool CSG_PointCloud::Save(const CSG_String &_FileName, int Format)
+{
+	if( Format == POINTCLOUD_FILE_FORMAT_Undefined )
+	{
+		Format	= SG_File_Cmp_Extension(_FileName, "sg-pts-z")
+			? POINTCLOUD_FILE_FORMAT_Compressed
+			: POINTCLOUD_FILE_FORMAT_Normal;
+	}
+
+	bool	bResult	= false;
+
+	CSG_String	FileName(_FileName);
+
+	switch( Format )
+	{
+	//-----------------------------------------------------
+	case POINTCLOUD_FILE_FORMAT_Compressed: default:
+		{
+			SG_File_Set_Extension(FileName, "sg-pts-z");
+
+			SG_UI_Msg_Add(CSG_String::Format("%s: %s...", _TL("Saving point cloud"), FileName.c_str()), true);
+
+			CSG_File_Zip	Stream(FileName, SG_FILE_W);
+
+			CSG_String	Name	= SG_File_Get_Name(FileName, false) + ".";
+
+			if( Stream.Add_File(Name + "sg-pts") && _Save(Stream) )
+			{
+				if( Stream.Add_File(Name + "sg-pts-hdr") )
+				{
+					CSG_MetaData	Header = _Create_Header();		Header.Save(Stream);
+				}
+
+				if( Stream.Add_File(Name + "sg-info") )
+				{
+					Save_MetaData(Stream);
+				}
+
+				if( Get_Projection().is_Okay() && Stream.Add_File(Name + "sg-prj") )
+				{
+					Get_Projection().Save(Stream);
+				}
+
+				bResult	= true;
+			}
+		}
+		break;
+
+	//-----------------------------------------------------
+	case POINTCLOUD_FILE_FORMAT_Normal:
+		{
+			SG_File_Set_Extension(FileName, "sg-pts");
+
+			SG_UI_Msg_Add(CSG_String::Format("%s: %s...", _TL("Saving point cloud"), FileName.c_str()), true);
+
+			CSG_File	Stream(FileName, SG_FILE_W, true);
+
+			if( _Save(Stream) )
+			{
+				CSG_MetaData	Header = _Create_Header();		Header.Save(SG_File_Make_Path("", FileName, "sg-pts-hdr"));
+
+				Save_MetaData(FileName);
+
+				if( Get_Projection().is_Okay() )
+				{
+					Get_Projection().Save(SG_File_Make_Path("", FileName, "sg-prj"), SG_PROJ_FMT_WKT);
+				}
+
+				bResult	= true;
+			}
+		}
+		break;
+	}
+
+	//-----------------------------------------------------
+	SG_UI_Process_Set_Ready();
+
+	if( bResult )
+	{
+		Set_Modified(false);
+
+		Set_File_Name(FileName, true);
+
+		SG_UI_Msg_Add(_TL("okay"), false, SG_UI_MSG_STYLE_SUCCESS);
+
+		return( true );
+	}
+
+	SG_UI_Msg_Add(_TL("failed"), false, SG_UI_MSG_STYLE_FAILURE);
+
+	return( false );
+}
+
+//---------------------------------------------------------
+bool CSG_PointCloud::Get_Header_Content(const CSG_String &FileName, CSG_MetaData &Header)
+{
+	bool	bResult	= false;
+
+	if( SG_File_Cmp_Extension(FileName, "sg-pts-z") ) // POINTCLOUD_FILE_FORMAT_Compressed
+	{
+		CSG_File_Zip	Stream(FileName, SG_FILE_R);
+
+		CSG_String	_FileName(SG_File_Get_Name(FileName, false) + ".");
+
+		if( Stream.Get_File(_FileName + "sg-pts-hdr") )
+		{
+			bResult = Header.Load(Stream);
+		}
+	}
+	else // if( SG_File_Cmp_Extension(FileName, "sg-pts"/"spc") ) // POINTCLOUD_FILE_FORMAT_Normal
+	{
+		bResult = Header.Load(FileName, SG_T("sg-pts-hdr"));
+	}
+
+	return( bResult );
+}
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CSG_PointCloud::_Load(CSG_File &Stream)
+{
+	if( !Stream.is_Reading() )
+	{
 		return( false );
 	}
 
+	//-----------------------------------------------------
+	char	ID[6];
+
 	if( !Stream.Read(ID, 6) || strncmp(ID, PC_FILE_VERSION, 5) != 0 )
 	{
-		SG_UI_Msg_Add(_TL("failed"), false, SG_UI_MSG_STYLE_FAILURE);
-		SG_UI_Msg_Add_Error(_TL("incompatible file."));
-
 		return( false );
 	}
 
+	int		nPointBytes;
+
 	if( !Stream.Read(&nPointBytes, sizeof(int)) || nPointBytes < (int)(3 * sizeof(float)) )
 	{
-		SG_UI_Msg_Add(_TL("failed"), false, SG_UI_MSG_STYLE_FAILURE);
-		SG_UI_Msg_Add_Error(_TL("incompatible file."));
-
 		return( false );
 	}
 
+	int		nFields;
+
 	if( !Stream.Read(&nFields, sizeof(int)) || nFields < 3 )
 	{
-		SG_UI_Msg_Add(_TL("failed"), false, SG_UI_MSG_STYLE_FAILURE);
-		SG_UI_Msg_Add_Error(_TL("incompatible file."));
-
 		return( false );
 	}
 
 	//-----------------------------------------------------
 	Destroy();
 
-	for(i=0; i<nFields; i++)
+	for(int i=0; i<nFields; i++)
 	{
-		if( !Stream.Read(&Type		, sizeof(TSG_Data_Type))
-		||	!Stream.Read(&iBuffer	, sizeof(int)) || !(iBuffer > 0 && iBuffer < 1024)
-		||	!Stream.Read(Name		, iBuffer) )
-		{
-			SG_UI_Msg_Add(_TL("failed"), false, SG_UI_MSG_STYLE_FAILURE);
-			SG_UI_Msg_Add_Error(_TL("incompatible file."));
+		TSG_Data_Type Type; int iBuffer; char Name[1024];
 
+		if( !Stream.Read(&Type   , sizeof(TSG_Data_Type))
+		||  !Stream.Read(&iBuffer, sizeof(int)) || !(iBuffer > 0 && iBuffer < 1024)
+		||  !Stream.Read(Name    , iBuffer) )
+		{
 			return( false );
 		}
 
@@ -330,13 +493,13 @@ bool CSG_PointCloud::_Load(const CSG_String &File_Name)
 		{
 			switch( Type )
 			{
-			default:	Type	= SG_DATATYPE_Undefined;	break;
-			case 1:		Type	= SG_DATATYPE_Char;			break;
-			case 2:		Type	= SG_DATATYPE_Short;		break;
-			case 3:		Type	= SG_DATATYPE_Int;			break;
-			case 4:		Type	= SG_DATATYPE_Long;			break;
-			case 5:		Type	= SG_DATATYPE_Float;		break;
-			case 6:		Type	= SG_DATATYPE_Double;		break;
+			default: Type = SG_DATATYPE_Undefined; break;
+			case  1: Type = SG_DATATYPE_Char     ; break;
+			case  2: Type = SG_DATATYPE_Short    ; break;
+			case  3: Type = SG_DATATYPE_Int      ; break;
+			case  4: Type = SG_DATATYPE_Long     ; break;
+			case  5: Type = SG_DATATYPE_Float    ; break;
+			case  6: Type = SG_DATATYPE_Double   ; break;
 			}
 		}
 
@@ -344,73 +507,37 @@ bool CSG_PointCloud::_Load(const CSG_String &File_Name)
 
 		if( !_Add_Field(CSG_String((const char *)Name), Type) )
 		{
-			SG_UI_Msg_Add(_TL("failed"), false, SG_UI_MSG_STYLE_FAILURE);
-			SG_UI_Msg_Add_Error(_TL("incompatible file."));
-
 			return( false );
 		}
 	}
 
-	if( m_nPointBytes != nPointBytes + 1 )
-	{
-		SG_UI_Msg_Add(_TL("failed"), false, SG_UI_MSG_STYLE_FAILURE);
-		SG_UI_Msg_Add_Error(_TL("incompatible file."));
-
-		return( false );
-	}
-
 	//-----------------------------------------------------
-	long		fLength	= Stream.Length();
+	sLong	fLength	= Stream.Length();
 
-	while( _Inc_Array() && Stream.Read(m_Cursor + 1, nPointBytes) && SG_UI_Process_Set_Progress(Stream.Tell(), fLength) )
-	{}
+	while( _Inc_Array() && Stream.Read(m_Cursor + 1, nPointBytes) && SG_UI_Process_Set_Progress((double)Stream.Tell(), (double)fLength) )
+	{
+		// nop
+	}
 
 	_Dec_Array();
 
-	Set_File_Name(File_Name);
-
-	Load_MetaData(File_Name);
-
-	if( 0 > Get_Count() )
-	{
-		SG_UI_Msg_Add(_TL("failed"), false, SG_UI_MSG_STYLE_FAILURE);
-		SG_UI_Msg_Add_Error(_TL("no records in file."));
-
-		return( false );
-	}
-
-	//-----------------------------------------------------
-	SG_UI_Process_Set_Ready();
-
-	Get_Projection().Load(SG_File_Make_Path(NULL, File_Name, SG_T("prj")), SG_PROJ_FMT_WKT);
-
-	SG_UI_Msg_Add(_TL("okay"), false, SG_UI_MSG_STYLE_SUCCESS);
-
 	return( true );
 }
 
 //---------------------------------------------------------
-bool CSG_PointCloud::_Save(const CSG_String &File_Name)
+bool CSG_PointCloud::_Save(CSG_File &Stream)
 {
-	CSG_File	Stream;
-
-	SG_UI_Msg_Add(CSG_String::Format(SG_T("%s: %s..."), _TL("Save point cloud"), File_Name.c_str()), true);
-
-	CSG_String	sFile_Name = SG_File_Make_Path(NULL, File_Name, SG_T("spc"));
-
-	if( Stream.Open(sFile_Name, SG_FILE_W, true) == false )
+	if( !Stream.is_Writing() )
 	{
-		SG_UI_Msg_Add(_TL("failed"), false, SG_UI_MSG_STYLE_FAILURE);
-		SG_UI_Msg_Add_Error(_TL("unable to create file."));
-
 		return( false );
 	}
 
+	//-----------------------------------------------------
 	int		i, iBuffer, nPointBytes	= m_nPointBytes - 1;
 
 	Stream.Write((void *)PC_FILE_VERSION, 6);
-	Stream.Write(&nPointBytes	, sizeof(int));
-	Stream.Write(&m_nFields		, sizeof(int));
+	Stream.Write(&nPointBytes, sizeof(int));
+	Stream.Write(&m_nFields  , sizeof(int));
 
 	for(i=0; i<m_nFields; i++)
 	{
@@ -428,24 +555,42 @@ bool CSG_PointCloud::_Save(const CSG_String &File_Name)
 		Stream.Write(m_Points[i] + 1, nPointBytes);
 	}
 
-	Set_Modified(false);
-
-	Set_File_Name(sFile_Name);
-
-	Save_MetaData(File_Name);
-
-	Get_Projection().Save(SG_File_Make_Path(NULL, File_Name, SG_T("prj")), SG_PROJ_FMT_WKT);
-
-	SG_UI_Msg_Add(_TL("okay"), false, SG_UI_MSG_STYLE_SUCCESS);
-
 	return( true );
 }
 
-bool CSG_PointCloud::Save(const CSG_String &File_Name, int Format)
+//---------------------------------------------------------
+CSG_MetaData CSG_PointCloud::_Create_Header(void) const
 {
-	return( _Save(File_Name) );
-}
+	CSG_MetaData	Header;
 
+	Header.Set_Name("PointCloudHeaderFile");
+	Header.Add_Property("Version", "1.0");
+
+	CSG_MetaData	*pPoints	= Header.Add_Child("Points");
+	CSG_MetaData	*pBBox		= Header.Add_Child("BBox");
+	CSG_MetaData	*pNoData	= Header.Add_Child("NoData");
+	CSG_MetaData	*pAttr		= Header.Add_Child("Attributes");
+
+	pPoints	->Add_Property("Value"	, CSG_String::Format("%d", Get_Point_Count()));
+	pBBox	->Add_Property("XMin"	, Get_Minimum(0));
+	pBBox	->Add_Property("YMin"	, Get_Minimum(1));
+	pBBox	->Add_Property("ZMin"	, Get_Minimum(2));
+	pBBox	->Add_Property("XMax"	, Get_Maximum(0));
+	pBBox	->Add_Property("YMax"	, Get_Maximum(1));
+	pBBox	->Add_Property("ZMax"	, Get_Maximum(2));
+	pNoData	->Add_Property("Value"	, Get_NoData_Value());
+	pAttr	->Add_Property("Count"	, Get_Field_Count());
+
+	for(int iField=0; iField<Get_Field_Count(); iField++)
+	{
+		CSG_MetaData	*pField = pAttr->Add_Child(CSG_String::Format("Field_%d", iField + 1));
+
+		pField->Add_Property("Name", Get_Field_Name(iField));
+		pField->Add_Property("Type", gSG_Data_Type_Identifier[Get_Field_Type(iField)]);
+	}
+
+	return (Header);
+}
 
 ///////////////////////////////////////////////////////////
 //														 //
@@ -462,8 +607,12 @@ bool CSG_PointCloud::Assign(CSG_Data_Object *pObject)
 
 		CSG_PointCloud	*pPointCloud	= (CSG_PointCloud *)pObject;
 
+		Set_Name(pPointCloud->Get_Name());
+
 		Get_History()	= pPointCloud->Get_History();
 
+		Get_Projection().Create(pPointCloud->Get_Projection());
+
 		for(int iField=0; iField<pPointCloud->m_nFields; iField++)
 		{
 			_Add_Field(pPointCloud->m_Field_Name[iField]->c_str(), pPointCloud->m_Field_Type[iField]);
@@ -631,22 +780,29 @@ bool CSG_PointCloud::_Set_Field_Value(char *pPoint, int iField, double Value)
 
 		switch( m_Field_Type[iField] )
 		{
-		default:
-		case SG_DATATYPE_Undefined:	break;
-		case SG_DATATYPE_Byte  :	*((BYTE   *)pPoint)	= (BYTE  )Value;	break;
-		case SG_DATATYPE_Char  :	*((char   *)pPoint)	= (char  )Value;	break;
-		case SG_DATATYPE_Word  :	*((WORD   *)pPoint)	= (WORD  )Value;	break;
-		case SG_DATATYPE_Short :	*((short  *)pPoint)	= (short )Value;	break;
-		case SG_DATATYPE_DWord :	*((DWORD  *)pPoint)	= (DWORD )Value;	break;
-		case SG_DATATYPE_Int   :	*((int    *)pPoint)	= (int   )Value;	break;
-		case SG_DATATYPE_Long  :	*((long   *)pPoint)	= (long  )Value;	break;
-		case SG_DATATYPE_Float :	*((float  *)pPoint)	= (float )Value;	break;
-		case SG_DATATYPE_Double:	*((double *)pPoint)	= (double)Value;	break;
-		case SG_DATATYPE_String:	sprintf(    pPoint, "%f"    , Value);	break;
+		case SG_DATATYPE_Byte  : *((BYTE   *)pPoint) = (BYTE  )Value ; break;
+		case SG_DATATYPE_Char  : *((char   *)pPoint) = (char  )Value ; break;
+		case SG_DATATYPE_Word  : *((WORD   *)pPoint) = (WORD  )Value ; break;
+		case SG_DATATYPE_Short : *((short  *)pPoint) = (short )Value ; break;
+		case SG_DATATYPE_DWord : *((DWORD  *)pPoint) = (DWORD )Value ; break;
+		case SG_DATATYPE_Int   : *((int    *)pPoint) = (int   )Value ; break;
+		case SG_DATATYPE_Long  : *((sLong  *)pPoint) = (sLong )Value ; break;
+		case SG_DATATYPE_ULong : *((uLong  *)pPoint) = (uLong )Value ; break;
+		case SG_DATATYPE_Float : *((float  *)pPoint) = (float )Value ; break;
+		case SG_DATATYPE_Double: *((double *)pPoint) = (double)Value ; break;
+		case SG_DATATYPE_String: sprintf(    pPoint, "%f"    , Value); break;
+		default                :                                       break;
 		}
 
 		m_Field_Stats[iField]->Invalidate();
 
+		Set_Modified();
+
+		if( iField < 3 )
+		{
+			Set_Update_Flag();	// extent might have changed
+		}
+
 		return( true );
 	}
 
@@ -662,17 +818,18 @@ double CSG_PointCloud::_Get_Field_Value(char *pPoint, int iField) const
 
 		switch( m_Field_Type[iField] )
 		{
-		case SG_DATATYPE_Undefined:		default:	break;
-		case SG_DATATYPE_Byte  :	return( *((BYTE   *)pPoint) );
-		case SG_DATATYPE_Char  :	return( *((char   *)pPoint) );
-		case SG_DATATYPE_Word  :	return( *((WORD   *)pPoint) );
-		case SG_DATATYPE_Short :	return( *((short  *)pPoint) );
-		case SG_DATATYPE_DWord :	return( *((DWORD  *)pPoint) );
-		case SG_DATATYPE_Int   :	return( *((int    *)pPoint) );
-		case SG_DATATYPE_Long  :	return( *((long   *)pPoint) );
-		case SG_DATATYPE_Float :	return( *((float  *)pPoint) );
-		case SG_DATATYPE_Double:	return( *((double *)pPoint) );
-		case SG_DATATYPE_String:	return( atof(       pPoint) );
+		case SG_DATATYPE_Byte  : return( (double)*((BYTE   *)pPoint) );
+		case SG_DATATYPE_Char  : return( (double)*((char   *)pPoint) );
+		case SG_DATATYPE_Word  : return( (double)*((WORD   *)pPoint) );
+		case SG_DATATYPE_Short : return( (double)*((short  *)pPoint) );
+		case SG_DATATYPE_DWord : return( (double)*((DWORD  *)pPoint) );
+		case SG_DATATYPE_Int   : return( (double)*((int    *)pPoint) );
+		case SG_DATATYPE_Long  : return( (double)*((sLong  *)pPoint) );
+		case SG_DATATYPE_ULong : return( (double)*((uLong  *)pPoint) );
+		case SG_DATATYPE_Float : return( (double)*((float  *)pPoint) );
+		case SG_DATATYPE_Double: return( (double)*((double *)pPoint) );
+		case SG_DATATYPE_String: return( (double)atof(       pPoint) );
+		default                : break;
 		}
 	}
 
@@ -680,49 +837,59 @@ double CSG_PointCloud::_Get_Field_Value(char *pPoint, int iField) const
 }
 
 //---------------------------------------------------------
-bool CSG_PointCloud::Set_Value(int iPoint, int iField, const SG_Char *Value)
+bool CSG_PointCloud::_Set_Field_Value(char *pPoint, int iField, const SG_Char *Value)
 {
-	if( iPoint >= 0 && iPoint < m_nRecords && iField >= 0 && iField < m_nFields && Value && *Value )
+	if( pPoint && iField >= 0 && iField < m_nFields && Value )
 	{
 		CSG_String	s(Value);
 
-		if( m_Field_Type[iField] == SG_DATATYPE_String )
+		switch( m_Field_Type[iField] )
 		{
-			memset(m_Points[iPoint] + m_Field_Offset[iField], 0, PC_STR_NBYTES);
-			memcpy(m_Points[iPoint] + m_Field_Offset[iField], s.b_str(), s.Length() > PC_STR_NBYTES ? PC_STR_NBYTES : s.Length());
-		}
-		else
-		{
-			double	d;
-
-			if( s.asDouble(d) )
+		default:
 			{
-				return( _Set_Field_Value(m_Points[iPoint], iField, d) );
+				double	d;
+
+				return( s.asDouble(d) && _Set_Field_Value(pPoint, iField, d) );
 			}
+			break;
+
+		case SG_DATATYPE_Date:
+		case SG_DATATYPE_String:
+			pPoint	+= m_Field_Offset[iField];
+			memset(pPoint, 0, PC_STR_NBYTES);
+			memcpy(pPoint, s.b_str(), s.Length() > PC_STR_NBYTES ? PC_STR_NBYTES : s.Length());
+			break;
 		}
+
+		return( true );
 	}
 
 	return( false );
 }
 
 //---------------------------------------------------------
-bool CSG_PointCloud::Get_Value(int iPoint, int iField, CSG_String &Value)	const
+bool CSG_PointCloud::_Get_Field_Value(char *pPoint, int iField, CSG_String &Value)	const
 {
-	if( iPoint >= 0 && iPoint < m_nRecords && iField >= 0 && iField < m_nFields )
+	if( pPoint && iField >= 0 && iField < m_nFields )
 	{
-		if( m_Field_Type[iField] == SG_DATATYPE_String )
+		switch( m_Field_Type[iField] )
 		{
-			char	s[PC_STR_NBYTES + 1];
+		default:
+			Value.Printf("%f", _Get_Field_Value(pPoint, iField));
+			break;
 
-			memcpy(s, m_Points[iPoint] + m_Field_Offset[iField], PC_STR_NBYTES);
+		case SG_DATATYPE_Date:
+		case SG_DATATYPE_String:
+			{
+				char	s[PC_STR_NBYTES + 1];
 
-			s[PC_STR_NBYTES]	= '\0';
+				memcpy(s, pPoint + m_Field_Offset[iField], PC_STR_NBYTES);
 
-			Value	= s;
-		}
-		else
-		{
-			Value.Printf("%f", _Get_Field_Value(m_Points[iPoint], iField));
+				s[PC_STR_NBYTES]	= '\0';
+
+				Value	= s;
+			}
+			break;
 		}
 
 		return( true );
@@ -772,21 +939,33 @@ TSG_Point_Z CSG_PointCloud::Get_Point(int iPoint)	const
 	}
 	else
 	{
-		p.x	= p.y	= p.z	= 0.0;
+		p.x = p.y = p.z = 0.0;
 	}
 
 	return( p );
 }
 
 //---------------------------------------------------------
-bool CSG_PointCloud::On_NoData_Changed(void)
+bool CSG_PointCloud::Set_Point(const TSG_Point_Z &Point)
 {
-	for(int i=3; i<m_nFields; i++)
+	return( _Set_Field_Value(m_Cursor, 0, Point.x)
+		&&  _Set_Field_Value(m_Cursor, 1, Point.y)
+		&&  _Set_Field_Value(m_Cursor, 2, Point.z)
+	);
+}
+
+//---------------------------------------------------------
+bool CSG_PointCloud::Set_Point(int iPoint, const TSG_Point_Z &Point)
+{
+	if( iPoint >= 0 && iPoint < m_nRecords )
 	{
-		m_Field_Stats[i]->Invalidate();
+		return( _Set_Field_Value(m_Points[iPoint], 0, Point.x)
+			&&  _Set_Field_Value(m_Points[iPoint], 1, Point.y)
+			&&  _Set_Field_Value(m_Points[iPoint], 2, Point.z)
+		);
 	}
 
-	return( true );
+	return( false );
 }
 
 
@@ -856,15 +1035,13 @@ bool CSG_PointCloud::Del_Points(void)
 		SG_Free(m_Points[iPoint]);
 	}
 
-	m_Array_Points  .Destroy();
-	m_Array_Selected.Destroy();
+	m_Array_Points.Destroy();
 
 	m_nRecords	= 0;
 	m_Points	= NULL;
 	m_Cursor	= NULL;
 
-	m_nSelected	= 0;
-	m_Selected	= NULL;
+	m_Selection.Set_Array(0);
 
 	return( true );
 }
@@ -941,7 +1118,9 @@ bool CSG_PointCloud::_Stats_Update(int iField) const
 {
 	if( iField >= 0 && iField < m_nFields && Get_Count() > 0 )
 	{
-		if( !m_Field_Stats[iField]->is_Evaluated() )
+		CSG_Simple_Statistics	*pStatistics	= m_Field_Stats[iField];
+
+		if( !pStatistics->is_Evaluated() )
 		{
 			char	**pPoint	= m_Points;
 
@@ -951,9 +1130,11 @@ bool CSG_PointCloud::_Stats_Update(int iField) const
 
 				if( iField < 3 || is_NoData_Value(Value) == false )
 				{
-					m_Field_Stats[iField]->Add_Value(Value);
+					pStatistics->Add_Value(Value);
 				}
 			}
+
+			pStatistics->Get_Mean();	// evaluate! prevent values to be added more than once!
 		}
 
 		return( true );
@@ -963,6 +1144,25 @@ bool CSG_PointCloud::_Stats_Update(int iField) const
 }
 
 
+///////////////////////////////////////////////////////////
+//														 //
+//														 //
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CSG_PointCloud::On_Reload(void)
+{
+	return( Create(Get_File_Name(false)) );
+}
+
+//---------------------------------------------------------
+bool CSG_PointCloud::On_Delete(void)
+{
+	return( SG_File_Delete(Get_File_Name(false)) );
+}
+
+
 ///////////////////////////////////////////////////////////
 //														 //
 //														 //
@@ -982,17 +1182,22 @@ CSG_Shape * CSG_PointCloud::_Set_Shape(int iPoint)
 
 		for(int i=0; i<Get_Field_Count(); i++)
 		{
-			Set_Value(i, pShape->asDouble(i));
+			switch( Get_Field_Type(i) )
+			{
+			default                : _Set_Field_Value(m_Cursor, i, pShape->asDouble(i)); break;
+			case SG_DATATYPE_Date  :
+			case SG_DATATYPE_String: _Set_Field_Value(m_Cursor, i, pShape->asString(i)); break;
+			}
 		}
 
 		Set_Value(0, pShape->Get_Point(0).x);
 		Set_Value(1, pShape->Get_Point(0).y);
-		Set_Value(2, pShape->Get_Z(0));
+		Set_Value(2, pShape->Get_Z    (0)  );
 	}
 
 	if( iPoint >= 0 && iPoint < Get_Count() )
 	{
-		if( iPoint != m_Shapes_Index )
+		if(1|| iPoint != m_Shapes_Index )
 		{
 			m_Cursor	= m_Points[iPoint];
 
@@ -1001,10 +1206,22 @@ CSG_Shape * CSG_PointCloud::_Set_Shape(int iPoint)
 
 			for(int i=0; i<Get_Field_Count(); i++)
 			{
-				pShape->Set_Value(i, Get_Value(i));
+				switch( Get_Field_Type(i) )
+				{
+				default: pShape->Set_Value(i, _Get_Field_Value(m_Cursor, i)); break;
+
+				case SG_DATATYPE_Date  :
+				case SG_DATATYPE_String:
+					{
+						CSG_String	s; _Get_Field_Value(m_Cursor, i, s); pShape->Set_Value(i, s);
+					}
+					break;
+				}
 			}
 
 			m_Shapes_Index	= iPoint;
+			pShape->m_Index	= iPoint;
+			pShape->Set_Selected(is_Selected(iPoint));
 		}
 
 		m_Shapes.Set_Modified(false);
@@ -1076,7 +1293,7 @@ CSG_Table_Record * CSG_PointCloud::Ins_Record(int iRecord, CSG_Table_Record *pCo
 //---------------------------------------------------------
 CSG_Table_Record * CSG_PointCloud::Add_Record(CSG_Table_Record *pCopy)
 {
-	return( NULL );
+	return( Add_Shape(pCopy, SHAPE_COPY) );
 }
 
 //---------------------------------------------------------
@@ -1090,7 +1307,14 @@ CSG_Shape * CSG_PointCloud::Add_Shape(CSG_Table_Record *pCopy, TSG_ADD_Shape_Cop
 		{
 			if( Get_Field_Type(iField) == pCopy->Get_Table()->Get_Field_Type(iField) )
 			{
-				Set_Value(iField, pCopy->asDouble(iField));
+				if( SG_Data_Type_is_Numeric(Get_Field_Type(iField)) )
+				{
+					Set_Value(iField, pCopy->asDouble(iField));
+				}
+				else
+				{
+					Set_Value(iField, pCopy->asString(iField));
+				}
 			}
 		}
 	}
@@ -1108,51 +1332,35 @@ CSG_Shape * CSG_PointCloud::Add_Shape(CSG_Table_Record *pCopy, TSG_ADD_Shape_Cop
 //---------------------------------------------------------
 bool CSG_PointCloud::Select(int iRecord, bool bInvert)
 {
-	if( !bInvert )
+	if( !bInvert && Get_Selection_Count() > 0 )
 	{
-		for(int i=0; i<m_nSelected; i++)
+		for(size_t i=0; i<Get_Selection_Count(); i++)
 		{
-			m_Points[m_Selected[i]][0]	&= ~SG_TABLE_REC_FLAG_Selected;
+			m_Points[Get_Selection_Index(i)][0]	&= ~SG_TABLE_REC_FLAG_Selected;
 		}
 
-		m_Array_Selected.Destroy();
-		m_Selected	= NULL;
-		m_nSelected	= 0;
+		m_Selection.Destroy();
 	}
 
 	if( Set_Cursor(iRecord) )
 	{
 		if( (m_Cursor[0] & SG_TABLE_REC_FLAG_Selected) == 0 )	// select
 		{
-			if( m_Array_Selected.Set_Array(m_nSelected + 1, (void **)&m_Selected) )
+			if( _Add_Selection(iRecord) )
 			{
 				m_Cursor[0]	|= SG_TABLE_REC_FLAG_Selected;
 
-				m_Selected[m_nSelected++]	= iRecord;
-
 				return( true );
 			}
 		}
 		else													// deselect
 		{
-			m_Cursor[0]	&= ~SG_TABLE_REC_FLAG_Selected;
-
-			m_nSelected--;
-
-			for(int i=0; i<m_nSelected; i++)
+			if( _Del_Selection(iRecord) )
 			{
-				if( m_Points[iRecord] == m_Points[m_Selected[i]] )
-				{
-					for(; i<m_nSelected; i++)
-					{
-						m_Selected[i]	= m_Selected[i + 1];
-					}
-				}
+				m_Cursor[0]	&= ~SG_TABLE_REC_FLAG_Selected;
+
+				return( true );
 			}
-
-			m_Array_Selected.Set_Array(m_nSelected, (void **)&m_Selected);
-
-			return( true );
 		}
 	}
 
@@ -1187,7 +1395,7 @@ bool CSG_PointCloud::Select(TSG_Rect Extent, bool bInvert)
 		}
 	}
 
-	return( m_nSelected > 0 );
+	return( Get_Selection_Count() > 0 );
 }
 
 //---------------------------------------------------------
@@ -1215,24 +1423,24 @@ bool CSG_PointCloud::is_Selected(int iRecord)	const
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-CSG_Shape * CSG_PointCloud::Get_Selection(int Index)
+CSG_Shape * CSG_PointCloud::Get_Selection(size_t Index)
 {
-	return( Index >= 0 && Index < m_nSelected ? _Set_Shape(m_Selected[Index]) : NULL );
+	return( Index < Get_Selection_Count() ? _Set_Shape((int)Get_Selection_Index(Index)) : NULL );
 }
 
 //---------------------------------------------------------
 const CSG_Rect & CSG_PointCloud::Get_Selection_Extent(void)
 {
-	if( m_nSelected > 0 && Set_Cursor(m_Selected[0]) )
+	if( Get_Selection_Count() > 0 && Set_Cursor((int)Get_Selection_Index(0)) )
 	{
 		TSG_Rect	r;
 
 		r.xMin	= r.xMax	= Get_X();
 		r.yMin	= r.yMax	= Get_Y();
 
-		for(int i=1; i<m_nSelected; i++)
+		for(size_t i=1; i<Get_Selection_Count(); i++)
 		{
-			if( Set_Cursor(m_Selected[i]) )
+			if( Set_Cursor((int)Get_Selection_Index(i)) )
 			{
 				if( Get_X() < r.xMin )	r.xMin	= Get_X();	else if( Get_X() > r.xMax )	r.xMax	= Get_X();
 				if( Get_Y() < r.yMin )	r.yMin	= Get_Y();	else if( Get_Y() > r.yMax )	r.yMax	= Get_Y();
@@ -1257,36 +1465,33 @@ const CSG_Rect & CSG_PointCloud::Get_Selection_Extent(void)
 //---------------------------------------------------------
 int CSG_PointCloud::Del_Selection(void)
 {
-	if( m_nSelected <= 0 )
+	int	n	= 0;
+
+	if( Get_Selection_Count() > 0 )
 	{
-		return( 0 );
-	}
+		m_Selection.Set_Array(0);
 
-	//-----------------------------------------------------
-	int		i, n;
+		m_Cursor	= NULL;
 
-	m_Array_Selected.Set_Array(0, (void **)&m_Selected);
-	m_nSelected	= 0;
-	m_Cursor	= NULL;
-
-	for(i=0, n=0; i<m_nRecords; i++)
-	{
-		if( (m_Points[i][0] & SG_TABLE_REC_FLAG_Selected) != 0 )
+		for(int i=0; i<m_nRecords; i++)
 		{
-			SG_Free(m_Points[i]);
-		}
-		else
-		{
-			if( n < i )
+			if( (m_Points[i][0] & SG_TABLE_REC_FLAG_Selected) != 0 )
 			{
-				m_Points[n]	= m_Points[i];
+				SG_Free(m_Points[i]);
 			}
+			else
+			{
+				if( n < i )
+				{
+					m_Points[n]	= m_Points[i];
+				}
 
-			n++;
+				n++;
+			}
 		}
-	}
 
-	m_Array_Points.Set_Array(m_nRecords = n, (void **)&m_Points);
+		m_Array_Points.Set_Array(m_nRecords = n, (void **)&m_Points);
+	}
 
 	return( n );
 }
@@ -1294,29 +1499,26 @@ int CSG_PointCloud::Del_Selection(void)
 //---------------------------------------------------------
 int CSG_PointCloud::Inv_Selection(void)
 {
-	char	**pPoint;
-	int		i, n;
-
-	n	= m_nRecords - m_nSelected;
-
-	if( m_Array_Selected.Set_Array(n, (void **)&m_Selected) )
+	if( m_Selection.Set_Array((size_t)m_nRecords - Get_Selection_Count()) )
 	{
-		for(i=0, m_nSelected=0, pPoint=m_Points; i<m_nRecords; i++, pPoint++)
-		{
-			if( ((*pPoint)[0] & SG_TABLE_REC_FLAG_Selected) == 0 && m_nSelected < n )
-			{
-				m_Selected[m_nSelected++]	= i;
+		char	**pPoint	= m_Points;
 
-				(*pPoint)[0]	|= SG_TABLE_REC_FLAG_Selected;
+		for(size_t i=0, n=0; i<(size_t)m_nRecords && n<Get_Selection_Count(); i++, pPoint++)
+		{
+			if( ((*pPoint)[0] & SG_TABLE_REC_FLAG_Selected) != 0 )
+			{
+				(*pPoint)[0]	&= ~SG_TABLE_REC_FLAG_Selected;
 			}
 			else
 			{
-				(*pPoint)[0]	&= ~SG_TABLE_REC_FLAG_Selected;
+				(*pPoint)[0]	|= SG_TABLE_REC_FLAG_Selected;
+
+				_Set_Selection(i, n++);
 			}
 		}
 	}
 
-	return( m_nSelected );
+	return( (int)Get_Selection_Count() );
 }
 
 
