@@ -1,6 +1,3 @@
-/**********************************************************
- * Version $Id: quadtree.cpp 2431 2015-03-03 17:21:59Z oconrad $
- *********************************************************/
 
 ///////////////////////////////////////////////////////////
 //                                                       //
@@ -16,8 +13,7 @@
 //                                                       //
 //                     quadtree.cpp                      //
 //                                                       //
-//                 Copyright (C) 2009 by                 //
-//                      Olaf Conrad                      //
+//          Copyright (C) 2009 by Olaf Conrad            //
 //                                                       //
 //-------------------------------------------------------//
 //                                                       //
@@ -27,7 +23,8 @@
 // This library is free software; you can redistribute   //
 // it and/or modify it under the terms of the GNU Lesser //
 // General Public License as published by the Free       //
-// Software Foundation, version 2.1 of the License.      //
+// Software Foundation, either version 2.1 of the        //
+// License, or (at your option) any later version.       //
 //                                                       //
 // This library is distributed in the hope that it will  //
 // be useful, but WITHOUT ANY WARRANTY; without even the //
@@ -37,9 +34,7 @@
 //                                                       //
 // You should have received a copy of the GNU Lesser     //
 // General Public License along with this program; if    //
-// not, write to the Free Software Foundation, Inc.,     //
-// 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, //
-// USA.                                                  //
+// not, see <http://www.gnu.org/licenses/>.              //
 //                                                       //
 //-------------------------------------------------------//
 //                                                       //
@@ -52,15 +47,6 @@
 //                                                       //
 ///////////////////////////////////////////////////////////
 
-//---------------------------------------------------------
-
-
-///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
-///////////////////////////////////////////////////////////
-
 //---------------------------------------------------------
 #include "shapes.h"
 
@@ -72,37 +58,23 @@
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-CSG_PRQuadTree_Node::CSG_PRQuadTree_Node(double xCenter, double yCenter, double Size)
-	: CSG_PRQuadTree_Item(xCenter, yCenter, Size)
+CSG_PRQuadTree_Node::CSG_PRQuadTree_Node(const CSG_Rect &Extent, int Quadrant)
+	: CSG_PRQuadTree_Item(Extent, Quadrant)
 {
-	m_pChildren[0]	=
-	m_pChildren[1]	=
-	m_pChildren[2]	=
-	m_pChildren[3]	= NULL;
+	m_pChildren[0] = m_pChildren[1] = m_pChildren[2] = m_pChildren[3] = NULL;
 }
 
 //---------------------------------------------------------
 CSG_PRQuadTree_Node::CSG_PRQuadTree_Node(CSG_PRQuadTree_Leaf *pLeaf)
-	: CSG_PRQuadTree_Item(pLeaf->m_xCenter, pLeaf->m_yCenter, pLeaf->m_Size)
+	: CSG_PRQuadTree_Item(pLeaf->m_Extent)
 {
-	m_pChildren[0]	=
-	m_pChildren[1]	=
-	m_pChildren[2]	=
-	m_pChildren[3]	= NULL;
+	m_pChildren[0] = m_pChildren[1] = m_pChildren[2] = m_pChildren[3] = NULL;
 
-	int		i		=  pLeaf->Get_Y() < m_yCenter
-					? (pLeaf->Get_X() < m_xCenter ? 0 : 3)
-					: (pLeaf->Get_X() < m_xCenter ? 1 : 2);
+	int	Quadrant	=  Get_Quadrant(pLeaf->Get_Point());
 
-	pLeaf->m_Size	= 0.5 * m_Size;
+	pLeaf->Set_Extent(m_Extent, Quadrant);
 
-	switch( i )
-	{
-	case 0:	m_pChildren[i]	= pLeaf; pLeaf->m_xCenter -= pLeaf->m_Size; pLeaf->m_yCenter -= pLeaf->m_Size;	break;
-	case 1:	m_pChildren[i]	= pLeaf; pLeaf->m_xCenter -= pLeaf->m_Size; pLeaf->m_yCenter += pLeaf->m_Size;	break;
-	case 2:	m_pChildren[i]	= pLeaf; pLeaf->m_xCenter += pLeaf->m_Size; pLeaf->m_yCenter += pLeaf->m_Size;	break;
-	case 3:	m_pChildren[i]	= pLeaf; pLeaf->m_xCenter += pLeaf->m_Size; pLeaf->m_yCenter -= pLeaf->m_Size;	break;
-	}
+	m_pChildren[Quadrant] = pLeaf;
 }
 
 //---------------------------------------------------------
@@ -141,74 +113,63 @@ CSG_PRQuadTree_Item * CSG_PRQuadTree_Node::Get_Child(double x, double y)
 //---------------------------------------------------------
 bool CSG_PRQuadTree_Node::Add_Point(double x, double y, double z)
 {
-	if( Contains(x, y) )
+	if( !Contains(x, y) )
 	{
-		if( has_Statistics() )
+		return( false );
+	}
+
+	if( has_Statistics() )
+	{
+		Get_X()->Add_Value(x);
+		Get_Y()->Add_Value(y);
+		Get_Z()->Add_Value(z);
+	}
+
+	int	Quadrant	= Get_Quadrant(x, y);
+
+	//-----------------------------------------------------
+	if( m_pChildren[Quadrant] == NULL )
+	{
+		m_pChildren[Quadrant]	= new CSG_PRQuadTree_Leaf(m_Extent, Quadrant, x, y, z);
+
+		return( true );
+	}
+
+	//-----------------------------------------------------
+	if( m_pChildren[Quadrant]->is_Leaf() )
+	{
+		CSG_PRQuadTree_Leaf	*pLeaf	= m_pChildren[Quadrant]->asLeaf();
+
+		if( x != pLeaf->Get_X() || y != pLeaf->Get_Y() )
 		{
-			Get_X()->Add_Value(x);
-			Get_Y()->Add_Value(y);
-			Get_Z()->Add_Value(z);
-		}
-
-		int		i	=  y < m_yCenter ? (x < m_xCenter ? 0 : 3) : (x < m_xCenter ? 1 : 2);
-
-		//-------------------------------------------------
-		if( m_pChildren[i] == NULL )
-		{
-			double	Size	= 0.5 * m_Size;
-
-			switch( i )
+			if( has_Statistics() )
 			{
-			case 0:	m_pChildren[i]	= new CSG_PRQuadTree_Leaf(m_xCenter - Size, m_yCenter - Size, Size, x, y, z);	break;
-			case 1:	m_pChildren[i]	= new CSG_PRQuadTree_Leaf(m_xCenter - Size, m_yCenter + Size, Size, x, y, z);	break;
-			case 2:	m_pChildren[i]	= new CSG_PRQuadTree_Leaf(m_xCenter + Size, m_yCenter + Size, Size, x, y, z);	break;
-			case 3:	m_pChildren[i]	= new CSG_PRQuadTree_Leaf(m_xCenter + Size, m_yCenter - Size, Size, x, y, z);	break;
-			}
-
-			return( true );
-		}
-
-		//-----------------------------------------------------
-		if( m_pChildren[i]->is_Leaf() )
-		{
-			CSG_PRQuadTree_Leaf	*pLeaf	= m_pChildren[i]->asLeaf();
-
-			if( x != pLeaf->Get_X() || y != pLeaf->Get_Y() )
-			{
-				if( has_Statistics() )
-				{
-					m_pChildren[i]	= new CSG_PRQuadTree_Node_Statistics	(pLeaf);
-				}
-				else
-				{
-					m_pChildren[i]	= new CSG_PRQuadTree_Node				(pLeaf);
-				}
-
-				((CSG_PRQuadTree_Node *)m_pChildren[i])->Add_Point(x, y, z);
+				m_pChildren[Quadrant]	= new CSG_PRQuadTree_Node_Statistics(pLeaf);
 			}
 			else
 			{
-				if( !pLeaf->has_Statistics() )
-				{
-					m_pChildren[i]	= new CSG_PRQuadTree_Leaf_List(pLeaf->m_xCenter, pLeaf->m_yCenter, pLeaf->m_Size, x, y, pLeaf->Get_Z());
-
-					delete(pLeaf);
-				}
-
-				((CSG_PRQuadTree_Leaf_List *)m_pChildren[i])->Add_Value(z);
+				m_pChildren[Quadrant]	= new CSG_PRQuadTree_Node(pLeaf);
 			}
 
-			return( true );
+			((CSG_PRQuadTree_Node *)m_pChildren[Quadrant])->Add_Point(x, y, z);
+		}
+		else // if( x == pLeaf->Get_X() || y == pLeaf->Get_Y() ) // duplicate !!
+		{
+			if( !pLeaf->has_Statistics() )
+			{
+				m_pChildren[Quadrant]	= new CSG_PRQuadTree_Leaf_List(pLeaf->m_Extent, -1, x, y, pLeaf->m_z);
+
+				delete(pLeaf);
+			}
+
+			((CSG_PRQuadTree_Leaf_List *)m_pChildren[Quadrant])->Add_Value(z);
 		}
 
-		//-------------------------------------------------
-	//	if( m_pChildren[i]->is_Node() )
-		{
-			return( ((CSG_PRQuadTree_Node *)m_pChildren[i])->Add_Point(x, y, z) );
-		}
+		return( true );
 	}
 
-	return( false );
+	//-----------------------------------------------------
+	return( ((CSG_PRQuadTree_Node *)m_pChildren[Quadrant])->Add_Point(x, y, z) );	//	if( m_pChildren[i]->is_Node() )
 }
 
 
@@ -221,17 +182,17 @@ bool CSG_PRQuadTree_Node::Add_Point(double x, double y, double z)
 //---------------------------------------------------------
 CSG_PRQuadTree::CSG_PRQuadTree(void)
 {
-	m_pRoot			= NULL;
-	m_nPoints		= 0;
-	m_bPolar		= false;
+	m_pRoot		= NULL;
+	m_nPoints	= 0;
+	m_bPolar	= false;
 }
 
 //---------------------------------------------------------
 CSG_PRQuadTree::CSG_PRQuadTree(const TSG_Rect &Extent, bool bStatistics)
 {
-	m_pRoot			= NULL;
-	m_nPoints		= 0;
-	m_bPolar		= false;
+	m_pRoot		= NULL;
+	m_nPoints	= 0;
+	m_bPolar	= false;
 
 	Create(Extent, bStatistics);
 }
@@ -239,9 +200,9 @@ CSG_PRQuadTree::CSG_PRQuadTree(const TSG_Rect &Extent, bool bStatistics)
 //---------------------------------------------------------
 CSG_PRQuadTree::CSG_PRQuadTree(CSG_Shapes *pShapes, int Attribute, bool bStatistics)
 {
-	m_pRoot			= NULL;
-	m_nPoints		= 0;
-	m_bPolar		= false;
+	m_pRoot		= NULL;
+	m_nPoints	= 0;
+	m_bPolar	= false;
 
 	Create(pShapes, Attribute, bStatistics);
 }
@@ -259,21 +220,20 @@ bool CSG_PRQuadTree::Create(const CSG_Rect &Extent, bool bStatistics)
 
 	if( Extent.Get_XRange() > 0.0 && Extent.Get_YRange() > 0.0 )
 	{
+		double	Size	= (0.5 + 0.01) * (Extent.Get_XRange() > Extent.Get_YRange() ? Extent.Get_XRange() : Extent.Get_YRange());
+
+		CSG_Rect	r(
+			Extent.Get_XCenter() - Size, Extent.Get_YCenter() - Size,
+			Extent.Get_XCenter() + Size, Extent.Get_YCenter() + Size
+		);
+
 		if( bStatistics )
 		{
-			m_pRoot	= new CSG_PRQuadTree_Node_Statistics(
-				Extent.Get_XCenter(),
-				Extent.Get_YCenter(),
-				(0.5 + 0.01) * (Extent.Get_XRange() > Extent.Get_YRange() ? Extent.Get_XRange() : Extent.Get_YRange())
-			);
+			m_pRoot	= new CSG_PRQuadTree_Node_Statistics(r);
 		}
 		else
 		{
-			m_pRoot	= new CSG_PRQuadTree_Node(
-				Extent.Get_XCenter(),
-				Extent.Get_YCenter(),
-				(0.5 + 0.01) * (Extent.Get_XRange() > Extent.Get_YRange() ? Extent.Get_XRange() : Extent.Get_YRange())
-			);
+			m_pRoot	= new CSG_PRQuadTree_Node(r);
 		}
 
 		return( true );
@@ -331,8 +291,6 @@ void CSG_PRQuadTree::Destroy(void)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -357,8 +315,6 @@ bool CSG_PRQuadTree::Add_Point(const TSG_Point &p, double z)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -375,15 +331,25 @@ bool CSG_PRQuadTree::_Check_Root(double x, double y)
 	}
 
 	//-----------------------------------------------------
+	int	Quadrant	= y < m_pRoot->Get_yMin()	// bottom / top ?
+		? (x < m_pRoot->Get_xMin() ? 0 : 3) 	// bottom: left / right ?
+		: (x < m_pRoot->Get_xMin() ? 1 : 2);	// top   : left / right ?
+		
+	TSG_Rect	Extent	= m_pRoot->Get_Extent();
+
+	switch( Quadrant )
+	{
+	case 0: Extent.xMin -= m_pRoot->Get_Size(); Extent.yMin -= m_pRoot->Get_Size(); break; // bottom left
+	case 1: Extent.xMin -= m_pRoot->Get_Size(); Extent.yMax += m_pRoot->Get_Size(); break; // top left
+	case 2: Extent.xMax += m_pRoot->Get_Size(); Extent.yMin -= m_pRoot->Get_Size(); break; // bottom right
+	case 3: Extent.xMax += m_pRoot->Get_Size(); Extent.yMax += m_pRoot->Get_Size(); break; // top right
+	}
+
 	CSG_PRQuadTree_Node	*pRoot;
 
 	if( m_pRoot->has_Statistics() )
 	{
-		CSG_PRQuadTree_Node_Statistics	*pNode	= new CSG_PRQuadTree_Node_Statistics(
-			x < m_pRoot->Get_xMin() ? m_pRoot->Get_xMin() : m_pRoot->Get_xMax(),
-			y < m_pRoot->Get_yMin() ? m_pRoot->Get_yMin() : m_pRoot->Get_yMax(),
-			2.0 * m_pRoot->Get_Size()
-		);
+		CSG_PRQuadTree_Node_Statistics	*pNode	= new CSG_PRQuadTree_Node_Statistics(Extent);
 
 		pNode->m_x	= ((CSG_PRQuadTree_Node_Statistics *)m_pRoot)->m_x;
 		pNode->m_y	= ((CSG_PRQuadTree_Node_Statistics *)m_pRoot)->m_y;
@@ -393,18 +359,10 @@ bool CSG_PRQuadTree::_Check_Root(double x, double y)
 	}
 	else
 	{
-		pRoot	= new CSG_PRQuadTree_Node(
-			x < m_pRoot->Get_xMin() ? m_pRoot->Get_xMin() : m_pRoot->Get_xMax(),
-			y < m_pRoot->Get_yMin() ? m_pRoot->Get_yMin() : m_pRoot->Get_yMax(),
-			2.0 * m_pRoot->Get_Size()
-		);
+		pRoot	= new CSG_PRQuadTree_Node(Extent);
 	}
 
-	int	i	=  m_pRoot->Get_yCenter() < pRoot->Get_yCenter()
-			? (m_pRoot->Get_xCenter() < pRoot->Get_xCenter() ? 0 : 3)
-			: (m_pRoot->Get_xCenter() < pRoot->Get_xCenter() ? 1 : 2);
-
-	pRoot->m_pChildren[i]	= m_pRoot;
+	pRoot->m_pChildren[Quadrant]	= m_pRoot;
 
 	m_pRoot	= pRoot;
 
@@ -414,8 +372,6 @@ bool CSG_PRQuadTree::_Check_Root(double x, double y)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -510,8 +466,6 @@ inline bool CSG_PRQuadTree::_Radius_Intersects(double x, double y, double r, int
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -604,8 +558,6 @@ CSG_PRQuadTree_Leaf	* CSG_PRQuadTree::_Get_Nearest_Point(CSG_PRQuadTree_Item *pI
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -790,8 +742,6 @@ void CSG_PRQuadTree::_Select_Nearest_Points(CSG_Array &Selection, CSG_PRQuadTree
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -830,20 +780,17 @@ size_t CSG_PRQuadTree::Get_Nearest_Points(CSG_Points_Z &Points, double x, double
 #include "parameters.h"
 
 //---------------------------------------------------------
-CSG_Parameters_Search_Points::CSG_Parameters_Search_Points(void)
+CSG_Parameters_PointSearch::CSG_Parameters_PointSearch(void)
 {
 	m_pParameters	= NULL;
 
-	Finalize();
+	m_minPoints		= 1;
+	m_maxPoints		= 0;
+	m_Radius		= 0.;
 }
 
-
-///////////////////////////////////////////////////////////
-//														 //
-///////////////////////////////////////////////////////////
-
 //---------------------------------------------------------
-bool CSG_Parameters_Search_Points::Create(CSG_Parameters *pParameters, CSG_Parameter *pNode, int nPoints_Min)
+bool CSG_Parameters_PointSearch::Create(CSG_Parameters *pParameters, const CSG_String &Parent, size_t minPoints)
 {
 	if( pParameters == NULL || m_pParameters != NULL )
 	{
@@ -852,55 +799,49 @@ bool CSG_Parameters_Search_Points::Create(CSG_Parameters *pParameters, CSG_Param
 
 	m_pParameters	= pParameters;
 
-	//-----------------------------------------------------
-	CSG_Parameter	*pSub;
+	if( !Parent.is_Empty() && !(*m_pParameters)(Parent) )
+	{
+		m_pParameters->Add_Node("", Parent, _TL("Search Options"), _TL(""));
+	}
 
-	pSub	= m_pParameters->Add_Choice(
-		pNode	, "SEARCH_RANGE"		, _TL("Search Range"),
+	//-----------------------------------------------------
+	m_pParameters->Add_Choice(Parent,
+		"SEARCH_RANGE"		, _TL("Search Range"),
 		_TL(""),
-		CSG_String::Format(SG_T("%s|%s|"),
+		CSG_String::Format("%s|%s",
 			_TL("local"),
 			_TL("global")
-		)
+		), 1
 	);
 
-	m_pParameters->Add_Value(
-		pSub	, "SEARCH_RADIUS"		, _TL("Maximum Search Distance"),
+	m_pParameters->Add_Double("SEARCH_RANGE",
+		"SEARCH_RADIUS"		, _TL("Maximum Search Distance"),
 		_TL("local maximum search distance given in map units"),
-		PARAMETER_TYPE_Double	, 1000.0, 0, true
+		1000., 0., true
 	);
 
-	pSub	= m_pParameters->Add_Choice(
-		pNode	, "SEARCH_POINTS_ALL"	, _TL("Number of Points"),
+	m_pParameters->Add_Choice(Parent,
+		"SEARCH_POINTS_ALL"	, _TL("Number of Points"),
 		_TL(""),
-		CSG_String::Format(SG_T("%s|%s|"),
+		CSG_String::Format("%s|%s",
 			_TL("maximum number of nearest points"),
 			_TL("all points within search distance")
-		)
+		), 1
 	);
 
-	if( m_nPoints_Min >= 0 )
+	if( minPoints > 0 )
 	{
-		m_pParameters->Add_Value(
-			pSub	, "SEARCH_POINTS_MIN"	, _TL("Minimum"),
+		m_pParameters->Add_Int("SEARCH_POINTS_ALL",
+			"SEARCH_POINTS_MIN"	, _TL("Minimum"),
 			_TL("minimum number of points to use"),
-			PARAMETER_TYPE_Int, nPoints_Min, 1, true
+			(int)minPoints, 1, true
 		);
 	}
 
-	m_pParameters->Add_Value(
-		pSub	, "SEARCH_POINTS_MAX"	, _TL("Maximum"),
+	m_pParameters->Add_Int("SEARCH_POINTS_ALL",
+		"SEARCH_POINTS_MAX"	, _TL("Maximum"),
 		_TL("maximum number of nearest points"),
-		PARAMETER_TYPE_Int, 20, 1, true
-	);
-
-	m_pParameters->Add_Choice(
-		pSub	, "SEARCH_DIRECTION"	, _TL("Direction"),
-		_TL(""),
-		CSG_String::Format(SG_T("%s|%s|"),
-			_TL("all directions"),
-			_TL("quadrants")
-		)
+		20, 1, true
 	);
 
 	return( true );
@@ -912,10 +853,9 @@ bool CSG_Parameters_Search_Points::Create(CSG_Parameters *pParameters, CSG_Param
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-bool CSG_Parameters_Search_Points::On_Parameter_Changed(CSG_Parameters *pParameters, CSG_Parameter *pParameter)
+bool CSG_Parameters_PointSearch::On_Parameter_Changed(CSG_Parameters *pParameters, CSG_Parameter *pParameter)
 {
-	if( !m_pParameters || !pParameters || m_pParameters->Get_Identifier().Cmp(pParameters->Get_Identifier()) || !pParameter
-		|| !pParameter->asShapes() )
+	if( !m_pParameters || !pParameters || m_pParameters->Get_Identifier().Cmp(pParameters->Get_Identifier()) || !pParameter || !pParameter->asShapes() )
 	{
 		return( false );
 	}
@@ -928,20 +868,20 @@ bool CSG_Parameters_Search_Points::On_Parameter_Changed(CSG_Parameters *pParamet
 }
 
 //---------------------------------------------------------
-bool CSG_Parameters_Search_Points::On_Parameters_Enable(CSG_Parameters *pParameters, CSG_Parameter *pParameter)
+bool CSG_Parameters_PointSearch::On_Parameters_Enable(CSG_Parameters *pParameters, CSG_Parameter *pParameter)
 {
 	if( !m_pParameters || !pParameters || m_pParameters->Get_Identifier().Cmp(pParameters->Get_Identifier()) || !pParameter )
 	{
 		return( false );
 	}
 
-	if(	!SG_STR_CMP(pParameter->Get_Identifier(), "SEARCH_RANGE") )
+	if(	pParameter->Cmp_Identifier("SEARCH_RANGE") )
 	{
 		pParameters->Set_Enabled("SEARCH_RADIUS"    , pParameter->asInt() == 0);	// local
 		pParameters->Set_Enabled("SEARCH_POINTS_MIN", pParameter->asInt() == 0);	// when global, no minimum number of points
 	}
 
-	if(	!SG_STR_CMP(pParameter->Get_Identifier(), "SEARCH_POINTS_ALL") )
+	if(	pParameter->Cmp_Identifier("SEARCH_POINTS_ALL") )
 	{
 		pParameters->Set_Enabled("SEARCH_POINTS_MAX", pParameter->asInt() == 0);	// maximum number of points
 		pParameters->Set_Enabled("SEARCH_DIRECTION" , pParameter->asInt() == 0);	// maximum number of points per quadrant
@@ -956,21 +896,96 @@ bool CSG_Parameters_Search_Points::On_Parameters_Enable(CSG_Parameters *pParamet
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-bool CSG_Parameters_Search_Points::Do_Use_All(bool bUpdate)
+bool CSG_Parameters_PointSearch::Update(void)
 {
-	if( bUpdate && m_pParameters )
+	if( m_pParameters )
 	{
-		m_nPoints_Min	= m_pParameters->Get_Parameter("SEARCH_POINTS_MIN")
-						? m_pParameters->Get_Parameter("SEARCH_POINTS_MIN")->asInt   () : 0;
-		m_nPoints_Max	= m_pParameters->Get_Parameter("SEARCH_POINTS_ALL")->asInt   () == 0
-						? m_pParameters->Get_Parameter("SEARCH_POINTS_MAX")->asInt   () : 0;
-		m_Radius		= m_pParameters->Get_Parameter("SEARCH_RANGE"     )->asInt   () == 0
-						? m_pParameters->Get_Parameter("SEARCH_RADIUS"    )->asDouble() : 0.0;
-		m_Quadrant		= m_pParameters->Get_Parameter("SEARCH_DIRECTION" )->asInt   () == 0 ? -1 : 4;
+		m_minPoints	= (*m_pParameters)("SEARCH_POINTS_MIN")
+					? (*m_pParameters)("SEARCH_POINTS_MIN")->asInt   () : 0;
+		m_maxPoints	= (*m_pParameters)("SEARCH_POINTS_ALL")->asInt   () == 0
+					? (*m_pParameters)("SEARCH_POINTS_MAX")->asInt   () : 0;
+		m_Radius	= (*m_pParameters)("SEARCH_RANGE"     )->asInt   () == 0
+					? (*m_pParameters)("SEARCH_RADIUS"    )->asDouble() : 0.;
+
+		return( true );
 	}
 
-	return( m_nPoints_Max <= 0 && m_Radius <= 0.0 );
+	return( false );
 }
+
+//---------------------------------------------------------
+bool CSG_Parameters_PointSearch::Do_Use_All(bool bUpdate)
+{
+	if( bUpdate )
+	{
+		Update();
+	}
+
+	return( m_maxPoints == 0 && m_Radius <= 0. );
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+//														 //
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+CSG_Parameters_Search_Points::CSG_Parameters_Search_Points(void)
+{
+	Finalize();
+}
+
+//---------------------------------------------------------
+bool CSG_Parameters_Search_Points::Create(CSG_Parameters *pParameters, CSG_Parameter *pParent, int minPoints)
+{
+	return( pParent ? Create(pParameters, pParent->Get_Identifier(), minPoints) : Create(pParameters, "", minPoints) );
+}
+
+//---------------------------------------------------------
+bool CSG_Parameters_Search_Points::Create(CSG_Parameters *pParameters, const CSG_String &Parent, int minPoints)
+{
+	if( !CSG_Parameters_PointSearch::Create(pParameters, Parent, minPoints) )
+	{
+		return( false );
+	}
+
+	m_pParameters->Add_Choice("SEARCH_POINTS_ALL",
+		"SEARCH_DIRECTION"	, _TL("Direction"),
+		_TL(""),
+		CSG_String::Format("%s|%s",
+			_TL("all directions"),
+			_TL("quadrants")
+		)
+	);
+
+	return( true );
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CSG_Parameters_Search_Points::Update(void)
+{
+	if( m_pParameters )
+	{
+		m_Quadrant	= (*m_pParameters)("SEARCH_DIRECTION" )->asInt() == 0 ? -1 : 4;
+
+		return( CSG_Parameters_PointSearch::Update() );
+	}
+
+	return( false );
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
 //---------------------------------------------------------
 bool CSG_Parameters_Search_Points::Initialize(CSG_Shapes *pPoints, int zField)
 {
@@ -999,7 +1014,8 @@ bool CSG_Parameters_Search_Points::Finalize(void)
 	m_zField	= -1;
 
 	m_Radius	= 0.0;
-	m_nPoints	= m_nPoints_Min	= m_nPoints_Max = 0;
+	m_minPoints	= m_maxPoints = 0;
+	m_nPoints	= 0;
 	m_Quadrant	= -1;
 
 	m_Search.Destroy();
@@ -1015,18 +1031,19 @@ bool CSG_Parameters_Search_Points::Finalize(void)
 //---------------------------------------------------------
 int CSG_Parameters_Search_Points::Set_Location(double x, double y)
 {
-	if( m_nPoints_Max > 0 || m_Radius > 0.0 )	// using search engine
-	{
-		m_nPoints	= m_Search.Select_Nearest_Points(x, y, m_nPoints_Max, m_Radius, m_Quadrant);
-	}
-	else										// without search engine
+	if( Do_Use_All() )	// without search engine
 	{
 		m_nPoints	= m_pPoints->Get_Count();
 	}
+	else				// using search engine
+	{
+		m_nPoints	= (int)m_Search.Select_Nearest_Points(x, y, m_maxPoints, m_Radius, m_Quadrant);
+	}
 
 	return( m_nPoints );
 }
 
+//---------------------------------------------------------
 int CSG_Parameters_Search_Points::Set_Location(const TSG_Point &p)
 {
 	return( Set_Location(p.x, p.y) );
@@ -1072,15 +1089,13 @@ bool CSG_Parameters_Search_Points::Get_Point(int Index, double &x, double &y, do
 //---------------------------------------------------------
 bool CSG_Parameters_Search_Points::Get_Points(double x, double y, CSG_Points_Z &Points)
 {
-	CSG_Point	p(x, y);
-
-	return( Get_Points(p, Points) );
+	return( Get_Points(CSG_Point(x, y), Points) );
 }
 
 //---------------------------------------------------------
 bool CSG_Parameters_Search_Points::Get_Points(const TSG_Point &p, CSG_Points_Z &Points)
 {
-	return( m_Search.Get_Nearest_Points(Points, p, m_nPoints_Max, m_Radius, m_Quadrant) >= m_nPoints_Min );
+	return( m_Search.Get_Nearest_Points(Points, p, m_maxPoints, m_Radius, m_Quadrant) >= (size_t)m_minPoints );
 }
 
 
