@@ -1,6 +1,4 @@
-/**********************************************************
- * Version $Id: shapes.h 2181 2014-07-09 16:56:37Z oconrad $
- *********************************************************/
+
 ///////////////////////////////////////////////////////////
 //                                                       //
 //                         SAGA                          //
@@ -25,7 +23,8 @@
 // This library is free software; you can redistribute   //
 // it and/or modify it under the terms of the GNU Lesser //
 // General Public License as published by the Free       //
-// Software Foundation, version 2.1 of the License.      //
+// Software Foundation, either version 2.1 of the        //
+// License, or (at your option) any later version.       //
 //                                                       //
 // This library is distributed in the hope that it will  //
 // be useful, but WITHOUT ANY WARRANTY; without even the //
@@ -35,9 +34,7 @@
 //                                                       //
 // You should have received a copy of the GNU Lesser     //
 // General Public License along with this program; if    //
-// not, write to the Free Software Foundation, Inc.,     //
-// 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, //
-// USA.                                                  //
+// not, see <http://www.gnu.org/licenses/>.              //
 //                                                       //
 //-------------------------------------------------------//
 //                                                       //
@@ -52,15 +49,6 @@
 //                                                       //
 ///////////////////////////////////////////////////////////
 
-//---------------------------------------------------------
-
-
-///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
-///////////////////////////////////////////////////////////
-
 //---------------------------------------------------------
 #ifndef HEADER_INCLUDED__SAGA_API__shapes_H
 #define HEADER_INCLUDED__SAGA_API__shapes_H
@@ -105,6 +93,16 @@ TSG_Shape_Type;
 //---------------------------------------------------------
 SAGA_API_DLL_EXPORT CSG_String	SG_Get_ShapeType_Name	(TSG_Shape_Type Type);
 
+//---------------------------------------------------------
+typedef enum ESG_Shape_File_Format
+{
+	SHAPE_FILE_FORMAT_Undefined	= 0,
+	SHAPE_FILE_FORMAT_ESRI,
+	SHAPE_FILE_FORMAT_GeoPackage,
+	SHAPE_FILE_FORMAT_GeoJSON
+}
+TSG_Shape_File_Format;
+
 
 ///////////////////////////////////////////////////////////
 //														 //
@@ -140,13 +138,15 @@ public:
 	virtual int					Ins_Point			(TSG_Point Point,    int iPoint, int iPart = 0);
 	virtual int					Set_Point			(TSG_Point Point,    int iPoint, int iPart = 0);
 
+	virtual int					Add_Part			(class CSG_Shape_Part *pPart)					= 0;
 	virtual int					Del_Part			(int iPart)										= 0;
 	virtual int					Del_Parts			(void)											= 0;
 
 	virtual int					Get_Part_Count		(void)											= 0;
-	virtual int					Get_Point_Count		(void);
+	virtual int					Get_Point_Count		(void)											= 0;
 	virtual int					Get_Point_Count		(int iPart)										= 0;
-	virtual TSG_Point			Get_Point			(int iPoint, int iPart = 0, bool bAscending = true)	= 0;
+	virtual TSG_Point			Get_Point			(int iPoint)									= 0;
+	virtual TSG_Point			Get_Point			(int iPoint, int iPart, bool bAscending = true)	= 0;
 
 	virtual void				Set_Z				(double z, int iPoint, int iPart = 0)				{		}
 	virtual double				Get_Z				(int iPoint, int iPart = 0, bool bAscending = true)	{	return( 0.0 );	}
@@ -209,12 +209,16 @@ public:
 	virtual int					Set_Point			(double x, double y, int iPoint, int iPart = 0)		{	return( Add_Point(x, y) );	}
 	virtual int					Del_Point			(                    int iPoint, int iPart = 0)		{	return( -1 );				}
 
+	virtual int					Add_Part			(class CSG_Shape_Part *pPart)						{	return( -1 );				}
 	virtual int					Del_Part			(int iPart)											{	return( -1 );				}
 	virtual int					Del_Parts			(void)												{	return( -1 );				}
 
 	virtual int					Get_Part_Count		(void)												{	return( 1 );				}
+	virtual int					Get_Point_Count		(void)												{	return( 1 );				}
 	virtual int					Get_Point_Count		(int iPart)											{	return( 1 );				}
-	virtual TSG_Point			Get_Point			(int iPoint, int iPart = 0, bool bAscending = true)	{	return( m_Point );			}
+	virtual TSG_Point			Get_Point			(void)												{	return( m_Point );			}
+	virtual TSG_Point			Get_Point			(int iPoint)										{	return( m_Point );			}
+	virtual TSG_Point			Get_Point			(int iPoint, int iPart, bool bAscending = true)		{	return( m_Point );			}
 
 	virtual const CSG_Rect &	Get_Extent			(void);
 
@@ -393,14 +397,18 @@ public:
 	int							Ins_Point			(const TSG_Point &p, int iPoint, int iPart = 0)	{	return( Ins_Point(p.x, p.y, iPoint, iPart) );	}
 	int							Set_Point			(const TSG_Point &p, int iPoint, int iPart = 0)	{	return( Set_Point(p.x, p.y, iPoint, iPart) );	}
 
+	virtual int					Add_Part			(class CSG_Shape_Part *pPart);
 	virtual int					Del_Part			(int iPart);
 	virtual int					Del_Parts			(void);
 
 	virtual int					Get_Part_Count		(void)		{	return( m_nParts );		}
 	virtual CSG_Shape_Part *	Get_Part			(int iPart)	{	return( iPart >= 0 && iPart < m_nParts ? m_pParts[iPart] : NULL );	}
+
+	virtual int					Get_Point_Count		(void)		{	_Update_Extent();	return( m_nPoints );	}
 	virtual int					Get_Point_Count		(int iPart)	{	return( iPart >= 0 && iPart < m_nParts ? m_pParts[iPart]->Get_Count() : 0 );	}
 
-	virtual TSG_Point			Get_Point			(int iPoint, int iPart = 0, bool bAscending = true)
+	virtual TSG_Point			Get_Point			(int iPoint);
+	virtual TSG_Point			Get_Point			(int iPoint, int iPart, bool bAscending = true)
 	{
 		if( iPart >= 0 && iPart < m_nParts )
 		{
@@ -440,7 +448,7 @@ protected:
 
 	bool						m_bUpdate;
 
-	int							m_nParts;
+	int							m_nParts, m_nPoints;
 
 	double						m_ZMin, m_ZMax, m_MMin, m_MMax;
 
@@ -512,6 +520,16 @@ protected:
 //														 //
 ///////////////////////////////////////////////////////////
 
+//---------------------------------------------------------
+typedef enum ESG_Polygon_Point_Relation
+{
+	SG_POLYGON_POINT_Outside,
+	SG_POLYGON_POINT_Vertex,
+	SG_POLYGON_POINT_Edge,
+	SG_POLYGON_POINT_Interior
+}
+TSG_Polygon_Point_Relation;
+
 //---------------------------------------------------------
 class SAGA_API_DLL_EXPORT CSG_Shape_Polygon_Part : public CSG_Shape_Part
 {
@@ -527,9 +545,17 @@ public:
 
 	const TSG_Point &			Get_Centroid		(void)	{	_Update_Area();	return( m_Centroid );	}
 
-	bool						Contains			(const TSG_Point &Point);
+	TSG_Polygon_Point_Relation	Get_Point_Relation	(const TSG_Point &p);
+	TSG_Polygon_Point_Relation	Get_Point_Relation	(double x, double y);
+
+	bool						is_OnEdge			(const TSG_Point &p);
+	bool						is_OnEdge			(double x, double y);
+
+	bool						Contains			(const TSG_Point &p);
 	bool						Contains			(double x, double y);
 
+	bool						is_Neighbour		(CSG_Shape_Polygon_Part *pPart, bool bSimpleCheck = true);
+
 	double						Get_Distance		(TSG_Point Point, TSG_Point &Next);
 
 
@@ -577,11 +603,24 @@ public:
 	TSG_Point					Get_Centroid		(int iPart);
 	TSG_Point					Get_Centroid		(void);
 
-	bool						Contains			(const TSG_Point &Point, int iPart);
-	bool						Contains			(const TSG_Point &Point);
+	TSG_Polygon_Point_Relation	Get_Point_Relation	(const TSG_Point &p, int iPart);
+	TSG_Polygon_Point_Relation	Get_Point_Relation	(double x, double y, int iPart);
+	TSG_Polygon_Point_Relation	Get_Point_Relation	(const TSG_Point &p);
+	TSG_Polygon_Point_Relation	Get_Point_Relation	(double x, double y);
+
+
+	bool						is_OnEdge			(const TSG_Point &p, int iPart);
+	bool						is_OnEdge			(double x, double y, int iPart);
+	bool						is_OnEdge			(const TSG_Point &p);
+	bool						is_OnEdge			(double x, double y);
+
+	bool						Contains			(const TSG_Point &p, int iPart);
 	bool						Contains			(double x, double y, int iPart);
+	bool						Contains			(const TSG_Point &p);
 	bool						Contains			(double x, double y);
 
+	bool						is_Neighbour		(CSG_Shape_Polygon *pPolygon, bool bSimpleCheck = true);
+
 	virtual double				Get_Distance		(TSG_Point Point, TSG_Point &Next, int iPart);
 
 
@@ -642,11 +681,13 @@ public:
 
 	virtual bool					Destroy					(void);
 
-	virtual TSG_Data_Object_Type	Get_ObjectType			(void)	const			{	return( DATAOBJECT_TYPE_Shapes );	}
+	virtual TSG_Data_Object_Type	Get_ObjectType			(void)	const			{	return( SG_DATAOBJECT_TYPE_Shapes );	}
 
 	virtual bool					Assign					(CSG_Data_Object *pObject);
 
-	virtual bool					Save					(const CSG_String &File_Name, int Format = 0);
+	virtual bool					Save					(const CSG_String &File, int Format = 0);
+	virtual bool					Save					(const char       *File, int Format = 0)	{	return( Save(CSG_String(File), Format) );	}
+	virtual bool					Save					(const wchar_t    *File, int Format = 0)	{	return( Save(CSG_String(File), Format) );	}
 
 	virtual bool					is_Valid				(void)	const			{	return( m_Type != SHAPE_TYPE_Undefined && Get_Count() >= 0 );		}
 
@@ -696,6 +737,8 @@ protected:
 
 
 	virtual bool					On_Update				(void);
+	virtual bool					On_Reload				(void);
+	virtual bool					On_Delete				(void);
 
 	virtual void					_On_Construction		(void);
 
@@ -704,6 +747,9 @@ protected:
 
 private:
 
+	bool							_Load_GDAL				(const CSG_String &File_Name);
+	bool							_Save_GDAL				(const CSG_String &File_Name, const CSG_String &Driver);
+
 	bool							_Load_ESRI				(const CSG_String &File_Name);
 	bool							_Save_ESRI				(const CSG_String &File_Name);
 
@@ -733,6 +779,12 @@ SAGA_API_DLL_EXPORT CSG_Shapes *	SG_Create_Shapes	(TSG_Shape_Type Type, const SG
 SAGA_API_DLL_EXPORT CSG_Shapes *	SG_Create_Shapes	(CSG_Shapes *pTemplate);
 
 
+//---------------------------------------------------------
+SAGA_API_DLL_EXPORT bool					SG_Shapes_Set_File_Format_Default		(int Format);
+SAGA_API_DLL_EXPORT TSG_Shape_File_Format	SG_Shapes_Get_File_Format_Default		(void);
+SAGA_API_DLL_EXPORT CSG_String				SG_Shapes_Get_File_Extension_Default	(void);
+
+
 ///////////////////////////////////////////////////////////
 //														 //
 //				Point Region QuadTree					 //
@@ -748,20 +800,17 @@ public:
 	virtual bool			is_Node			(void)	const	{	return( false );	}
 	virtual bool			has_Statistics	(void)	const	{	return( false );	}
 
-	CSG_Rect				Get_Extent		(void)	const	{	return( CSG_Rect(m_xCenter - m_Size, m_yCenter - m_Size, m_xCenter + m_Size, m_yCenter + m_Size) );	}
-	double					Get_xMin		(void)	const	{	return( m_xCenter - m_Size );	}
-	double					Get_yMin		(void)	const	{	return( m_yCenter - m_Size );	}
-	double					Get_xCenter		(void)	const	{	return( m_xCenter          );	}
-	double					Get_yCenter		(void)	const	{	return( m_yCenter          );	}
-	double					Get_xMax		(void)	const	{	return( m_xCenter + m_Size );	}
-	double					Get_yMax		(void)	const	{	return( m_yCenter + m_Size );	}
-	double					Get_Size		(void)	const	{	return( m_Size * 2.0       );	}
+	const CSG_Rect &		Get_Extent		(void)	const	{	return( m_Extent );	}
+	double					Get_xMin		(void)	const	{	return( m_Extent.m_rect.xMin   );	}
+	double					Get_xCenter		(void)	const	{	return( m_Extent.Get_XCenter() );	}
+	double					Get_xMax		(void)	const	{	return( m_Extent.m_rect.xMax   );	}
+	double					Get_yMin		(void)	const	{	return( m_Extent.m_rect.yMin   );	}
+	double					Get_yCenter		(void)	const	{	return( m_Extent.Get_YCenter() );	}
+	double					Get_yMax		(void)	const	{	return( m_Extent.m_rect.yMax   );	}
+	double					Get_Size		(void)	const	{	return( m_Extent.Get_XRange()  );	}
 
-	bool					Contains		(double x, double y)	const
-	{
-		return(	m_xCenter - m_Size <= x && x < m_xCenter + m_Size
-			&&	m_yCenter - m_Size <= y && y < m_yCenter + m_Size );
-	}
+	bool					Contains		(const TSG_Point &p)	const	{	return(	m_Extent.Contains(p   ) );	}
+	bool					Contains		(double x, double y)	const	{	return(	m_Extent.Contains(x, y) );	}
 
 	class CSG_PRQuadTree_Leaf *	asLeaf		(void)	const	{	return( (class CSG_PRQuadTree_Leaf *)this );	}
 	class CSG_PRQuadTree_Node *	asNode		(void)	const	{	return( (class CSG_PRQuadTree_Node *)this );	}
@@ -769,15 +818,31 @@ public:
 
 protected:
 
-	CSG_PRQuadTree_Item(double xCenter, double yCenter, double Size)
+	CSG_PRQuadTree_Item(const CSG_Rect &Extent, int Quadrant = -1)	{	Set_Extent(Extent, Quadrant);	}
+
+	virtual ~CSG_PRQuadTree_Item(void)	{}
+
+	void					Set_Extent		(const CSG_Rect &Extent, int Quadrant = -1)
 	{
-		m_xCenter	= xCenter;
-		m_yCenter	= yCenter;
-		m_Size		= Size;
+		switch( Quadrant )
+		{
+		case  0: m_Extent.Assign(Extent.Get_XMin   (), Extent.Get_YMin   (), Extent.Get_XCenter(), Extent.Get_YCenter()); break; // bottom left
+		case  1: m_Extent.Assign(Extent.Get_XMin   (), Extent.Get_YCenter(), Extent.Get_XCenter(), Extent.Get_YMax   ()); break; // top left
+		case  2: m_Extent.Assign(Extent.Get_XCenter(), Extent.Get_YCenter(), Extent.Get_XMax   (), Extent.Get_YMax   ()); break; // top right
+		case  3: m_Extent.Assign(Extent.Get_XCenter(), Extent.Get_YMin   (), Extent.Get_XMax   (), Extent.Get_YCenter()); break; // bottom right
+		default: m_Extent.Assign(Extent); break;
+		}
+	}
+
+	int						Get_Quadrant	(const TSG_Point &p)	const	{	return( Get_Quadrant(p.x, p.y) );	}
+
+	int						Get_Quadrant	(double x, double y)	const
+	{
+		return( y < Get_yCenter() ? (x < Get_xCenter() ? 0 : 3) : (x < Get_xCenter() ? 1 : 2) );
 	}
 
 
-	double					m_xCenter, m_yCenter, m_Size;
+	CSG_Rect				m_Extent;
 
 };
 
@@ -798,14 +863,16 @@ public:
 
 protected:
 
-	CSG_PRQuadTree_Leaf(double xCenter, double yCenter, double Size, double x, double y, double z)
-		: CSG_PRQuadTree_Item(xCenter, yCenter, Size)
+	CSG_PRQuadTree_Leaf(const CSG_Rect &Extent, int Quadrant, double x, double y, double z)
+		: CSG_PRQuadTree_Item(Extent, Quadrant)
 	{
 		m_Point.x	= x;
 		m_Point.y	= y;
 		m_z			= z;
 	}
 
+	virtual ~CSG_PRQuadTree_Leaf(void)	{}
+
 
 	double					m_z;
 
@@ -835,14 +902,16 @@ public:
 
 protected:
 
-	CSG_PRQuadTree_Leaf_List(double xCenter, double yCenter, double Size, double x, double y, double z)
-		: CSG_PRQuadTree_Leaf(xCenter, yCenter, Size, x, y, z)
+	CSG_PRQuadTree_Leaf_List(const CSG_Rect &Extent, int Quadrant, double x, double y, double z)
+		: CSG_PRQuadTree_Leaf(Extent, Quadrant, x, y, z)
 	{
 		s_z.Create(true);
 
 		s_z.Add_Value(z);
 	}
 
+	virtual ~CSG_PRQuadTree_Leaf_List(void)	{}
+
 	void					Add_Value		(double z)
 	{
 		s_z.Add_Value(z);
@@ -875,7 +944,7 @@ public:
 
 protected:
 
-	CSG_PRQuadTree_Node(double xCenter, double yCenter, double Size);
+	CSG_PRQuadTree_Node(const CSG_Rect &Extent, int Quadrant = -1);
 	CSG_PRQuadTree_Node(CSG_PRQuadTree_Leaf *pLeaf);
 	virtual ~CSG_PRQuadTree_Node(void);
 
@@ -901,14 +970,16 @@ public:
 
 protected:
 
-	CSG_PRQuadTree_Node_Statistics(double xCenter, double yCenter, double Size)
-		: CSG_PRQuadTree_Node(xCenter, yCenter, Size)
+	CSG_PRQuadTree_Node_Statistics(const CSG_Rect &Extent, int Quadrant = -1)
+		: CSG_PRQuadTree_Node(Extent, Quadrant)
 	{}
 
 	CSG_PRQuadTree_Node_Statistics(CSG_PRQuadTree_Leaf *pLeaf)
 		: CSG_PRQuadTree_Node(pLeaf)
 	{}
 
+	virtual ~CSG_PRQuadTree_Node_Statistics(void)	{}
+
 
 	CSG_Simple_Statistics	m_x, m_y, m_z;
 
@@ -1015,6 +1086,186 @@ private:
 };
 
 
+///////////////////////////////////////////////////////////
+//														 //
+//														 //
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+class SAGA_API_DLL_EXPORT CSG_KDTree
+{
+public:
+	CSG_KDTree(void);
+	virtual ~CSG_KDTree(void);
+
+	virtual bool				Destroy				(void);
+
+	bool						is_Okay				(void)		const	{	return( m_pKDTree != NULL );	}
+
+	size_t						Get_Match_Count		(void    )	const	{	return( m_Indices.Get_Size() );	}
+	size_t						Get_Match_Index		(size_t i)	const	{	return( m_Indices  [i]       );	}
+	double						Get_Match_Distance	(size_t i)	const	{	return( m_Distances[i]       );	}
+	CSG_Shape *					Get_Match_Shape		(size_t i)	const;
+
+
+protected:
+
+	class CSG_KDTree_Adaptor	*m_pAdaptor;
+
+	void						*m_pKDTree;
+
+	CSG_Array_Int				m_Indices;
+
+	CSG_Vector					m_Distances;
+
+
+	void						_On_Construction	(void);
+
+};
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+class SAGA_API_DLL_EXPORT CSG_KDTree_2D : public CSG_KDTree
+{
+public:
+	CSG_KDTree_2D(void);
+	virtual ~CSG_KDTree_2D(void);
+
+								CSG_KDTree_2D		(CSG_Shapes *pPoints);
+	bool						Create				(CSG_Shapes *pPoints);
+
+	virtual bool				Destroy				(void);
+
+	virtual size_t				Get_Nearest_Points	(double Coordinate[2], size_t Count, double Radius);
+	virtual size_t				Get_Nearest_Points	(double Coordinate[2], size_t Count, double Radius, CSG_Array_Int &Indices, CSG_Vector &Distances);
+	virtual size_t				Get_Nearest_Points	(double Coordinate[2], size_t Count, size_t *Indices, double *Distances);
+	virtual bool				Get_Nearest_Point	(double Coordinate[2], size_t &Index, double &Distance);
+	virtual bool				Get_Nearest_Point	(double Coordinate[2], size_t &Index);
+	virtual CSG_Shape *			Get_Nearest_Shape	(double Coordinate[2]);
+
+	virtual size_t				Get_Nearest_Points	(double x, double y, size_t Count, double Radius);
+	virtual size_t				Get_Nearest_Points	(double x, double y, size_t Count, double Radius, CSG_Array_Int &Indices, CSG_Vector &Distances);
+	virtual size_t				Get_Nearest_Points	(double x, double y, size_t Count, size_t *Indices, double *Distances);
+	virtual bool				Get_Nearest_Point	(double x, double y, size_t &Index, double &Distance);
+	virtual bool				Get_Nearest_Point	(double x, double y, size_t &Index);
+	virtual CSG_Shape *			Get_Nearest_Shape	(double x, double y);
+
+};
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+class SAGA_API_DLL_EXPORT CSG_KDTree_3D : public CSG_KDTree
+{
+public:
+	CSG_KDTree_3D(void);
+	virtual ~CSG_KDTree_3D(void);
+
+								CSG_KDTree_3D		(CSG_Shapes     *pPoints, int zField = -1, double zScale = 1.);
+	bool						Create				(CSG_Shapes     *pPoints, int zField = -1, double zScale = 1.);
+
+								CSG_KDTree_3D		(CSG_PointCloud *pPoints);
+	bool						Create				(CSG_PointCloud *pPoints);
+
+	virtual bool				Destroy				(void);
+
+	virtual size_t				Get_Nearest_Points	(double Coordinate[3], size_t Count, double Radius);
+
+	virtual size_t				Get_Nearest_Points	(double Coordinate[3], size_t Count, size_t *Indices, double *Distances);
+	virtual size_t				Get_Nearest_Points	(double Coordinate[3], size_t Count, double Radius, CSG_Array_Int &Indices, CSG_Vector &Distances);
+	virtual bool				Get_Nearest_Point	(double Coordinate[3], size_t &Index, double &Distance);
+	virtual bool				Get_Nearest_Point	(double Coordinate[3], size_t &Index);
+	virtual CSG_Shape *			Get_Nearest_Shape	(double Coordinate[3]);
+
+	virtual size_t				Get_Nearest_Points	(double x, double y, double z, size_t Count, double Radius);
+	virtual size_t				Get_Nearest_Points	(double x, double y, double z, size_t Count, double Radius, CSG_Array_Int &Indices, CSG_Vector &Distances);
+	virtual size_t				Get_Nearest_Points	(double x, double y, double z, size_t Count, size_t *Indices, double *Distances);
+	virtual bool				Get_Nearest_Point	(double x, double y, double z, size_t &Index, double &Distance);
+	virtual bool				Get_Nearest_Point	(double x, double y, double z, size_t &Index);
+	virtual CSG_Shape *			Get_Nearest_Shape	(double x, double y, double z);
+
+};
+
+
+///////////////////////////////////////////////////////////
+//														 //
+//					Search Engine						 //
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+class SAGA_API_DLL_EXPORT CSG_Parameters_PointSearch
+{
+public:
+	CSG_Parameters_PointSearch(void);
+
+	virtual bool				Create					(CSG_Parameters *pParameters, const CSG_String &Parent = "", size_t minPoints = 0);
+
+	virtual bool				On_Parameter_Changed	(CSG_Parameters *pParameters, CSG_Parameter *pParameter);
+	virtual bool				On_Parameters_Enable	(CSG_Parameters *pParameters, CSG_Parameter *pParameter);
+
+	virtual bool				Update					(void);
+
+	bool						Do_Use_All				(bool bUpdate = false);
+
+	size_t						Get_Min_Points			(void)	const	{	return( m_minPoints );	}
+	size_t						Get_Max_Points			(void)	const	{	return( m_maxPoints );	}
+	double						Get_Radius				(void)	const	{	return( m_Radius    );	}
+
+
+protected:
+
+	size_t						m_minPoints, m_maxPoints;
+
+	double						m_Radius;
+
+	CSG_Parameters				*m_pParameters;
+
+};
+
+//---------------------------------------------------------
+class SAGA_API_DLL_EXPORT CSG_Parameters_Search_Points : public CSG_Parameters_PointSearch
+{
+public:
+	CSG_Parameters_Search_Points(void);
+
+	virtual bool				Create					(CSG_Parameters *pParameters, class CSG_Parameter *pParent       , int nPoints_Min = -1);
+	virtual bool				Create					(CSG_Parameters *pParameters, const CSG_String     &Parent = ""  , int nPoints_Min = -1);
+
+	virtual bool				Update					(void);
+
+	bool						Initialize				(CSG_Shapes *pPoints, int zField);
+	bool						Finalize				(void);
+
+	int							Set_Location			(double x, double y);
+	int							Set_Location			(const TSG_Point &p);
+
+	int							Get_Count				(void)	const	{	return( m_nPoints );	}
+	bool						Get_Point				(int Index, double &x, double &y, double &z);
+
+	bool						Get_Points				(double x, double y, CSG_Points_Z &Points);
+	bool						Get_Points				(const TSG_Point &p, CSG_Points_Z &Points);
+
+
+private:
+
+	int							m_zField, m_nPoints, m_Quadrant;
+
+	CSG_Shapes					*m_pPoints;
+
+	CSG_PRQuadTree				m_Search;
+
+};
+
+
 ///////////////////////////////////////////////////////////
 //														 //
 //					Search Engine						 //
