@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: table.cpp 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id$
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -26,7 +26,8 @@
 // This library is free software; you can redistribute   //
 // it and/or modify it under the terms of the GNU Lesser //
 // General Public License as published by the Free       //
-// Software Foundation, version 2.1 of the License.      //
+// Software Foundation, either version 2.1 of the        //
+// License, or (at your option) any later version.       //
 //                                                       //
 // This library is distributed in the hope that it will  //
 // be useful, but WITHOUT ANY WARRANTY; without even the //
@@ -36,9 +37,7 @@
 //                                                       //
 // You should have received a copy of the GNU Lesser     //
 // General Public License along with this program; if    //
-// not, write to the Free Software Foundation, Inc.,     //
-// 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, //
-// USA.                                                  //
+// not, see <http://www.gnu.org/licenses/>.              //
 //                                                       //
 //-------------------------------------------------------//
 //                                                       //
@@ -65,6 +64,7 @@
 //---------------------------------------------------------
 #include "table.h"
 #include "shapes.h"
+#include "tool_library.h"
 
 
 ///////////////////////////////////////////////////////////
@@ -84,11 +84,11 @@ CSG_Table * SG_Create_Table(const CSG_Table &Table)
 {
 	switch( Table.Get_ObjectType() )
 	{
-	case DATAOBJECT_TYPE_Table:
+	case SG_DATAOBJECT_TYPE_Table:
 		return( new CSG_Table(Table) );
 
-	case DATAOBJECT_TYPE_Shapes:
-	case DATAOBJECT_TYPE_PointCloud:
+	case SG_DATAOBJECT_TYPE_Shapes:
+	case SG_DATAOBJECT_TYPE_PointCloud:
 		return( SG_Create_Shapes(*((CSG_Shapes *)&Table)) );
 
 	default:
@@ -96,12 +96,6 @@ CSG_Table * SG_Create_Table(const CSG_Table &Table)
 	}
 }
 
-//---------------------------------------------------------
-CSG_Table * SG_Create_Table(const CSG_String &File_Name)
-{
-	return( new CSG_Table(File_Name) );
-}
-
 //---------------------------------------------------------
 CSG_Table * SG_Create_Table(CSG_Table *pTemplate)
 {
@@ -109,11 +103,11 @@ CSG_Table * SG_Create_Table(CSG_Table *pTemplate)
 	{
 		switch( pTemplate->Get_ObjectType() )
 		{
-		case DATAOBJECT_TYPE_Table:
+		case SG_DATAOBJECT_TYPE_Table:
 			return( new CSG_Table(pTemplate) );
 
-		case DATAOBJECT_TYPE_Shapes:
-		case DATAOBJECT_TYPE_PointCloud:
+		case SG_DATAOBJECT_TYPE_Shapes:
+		case SG_DATAOBJECT_TYPE_PointCloud:
 			return( SG_Create_Shapes((CSG_Shapes *)pTemplate) );
 
 		default:
@@ -124,6 +118,18 @@ CSG_Table * SG_Create_Table(CSG_Table *pTemplate)
 	return( new CSG_Table() );
 }
 
+//---------------------------------------------------------
+CSG_Table * SG_Create_Table(const CSG_String &File_Name, TSG_Table_File_Type Format, int Encoding)
+{
+	return( new CSG_Table(File_Name, Format, Encoding) );
+}
+
+//---------------------------------------------------------
+CSG_Table * SG_Create_Table(const CSG_String &File_Name, TSG_Table_File_Type Format, const SG_Char Separator, int Encoding)
+{
+	return( new CSG_Table(File_Name, Format, Encoding) );
+}
+
 
 ///////////////////////////////////////////////////////////
 //														 //
@@ -149,45 +155,119 @@ CSG_Table::CSG_Table(const CSG_Table &Table)
 
 bool CSG_Table::Create(const CSG_Table &Table)
 {
-	if( Assign((CSG_Data_Object *)&Table) )
+	return( Assign((CSG_Data_Object *)&Table) );
+}
+
+//---------------------------------------------------------
+CSG_Table::CSG_Table(const CSG_String &File_Name, TSG_Table_File_Type Format, int Encoding)
+{
+	_On_Construction();
+
+	Create(File_Name, Format, Encoding);
+}
+
+bool CSG_Table::Create(const CSG_String &File_Name, TSG_Table_File_Type Format, int Encoding)
+{
+	Destroy();
+
+	SG_UI_Msg_Add(CSG_String::Format("%s: %s...", _TL("Load table"), File_Name.c_str()), true);
+
+	//-----------------------------------------------------
+	bool	bResult	= File_Name.BeforeFirst(':').Cmp("PGSQL") && SG_File_Exists(File_Name) && Load(File_Name, (int)Format, '\0', Encoding);
+
+	if( bResult )
 	{
-		Set_Name(Table.Get_Name());
+		Set_File_Name(File_Name, true);
+	}
+
+	//-----------------------------------------------------
+	else if( File_Name.BeforeFirst(':').Cmp("PGSQL") == 0 )	// database source
+	{
+		CSG_String	s(File_Name);
+
+		s	= s.AfterFirst(':');	CSG_String	Host  (s.BeforeFirst(':'));
+		s	= s.AfterFirst(':');	CSG_String	Port  (s.BeforeFirst(':'));
+		s	= s.AfterFirst(':');	CSG_String	DBName(s.BeforeFirst(':'));
+		s	= s.AfterFirst(':');	CSG_String	Table (s.BeforeFirst(':'));
+
+		CSG_Tool	*pTool	= SG_Get_Tool_Library_Manager().Create_Tool("db_pgsql", 0);	// CGet_Connections
+
+		if(	pTool != NULL )
+		{
+			SG_UI_ProgressAndMsg_Lock(true);
+
+			//---------------------------------------------
+			CSG_Table	Connections;
+			CSG_String	Connection	= DBName + " [" + Host + ":" + Port + "]";
+
+			pTool->Set_Manager(NULL);
+			pTool->On_Before_Execution();
+
+			if( SG_TOOL_PARAMETER_SET("CONNECTIONS", &Connections) && pTool->Execute() )	// CGet_Connections
+			{
+				for(int i=0; !bResult && i<Connections.Get_Count(); i++)
+				{
+					if( !Connection.Cmp(Connections[i].asString(0)) )
+					{
+						bResult	= true;
+					}
+				}
+			}
+
+			SG_Get_Tool_Library_Manager().Delete_Tool(pTool);
+
+			//---------------------------------------------
+			if( bResult && (bResult = (pTool = SG_Get_Tool_Library_Manager().Create_Tool("db_pgsql", 12)) != NULL) == true )	// CPGIS_Table_Load
+			{
+				pTool->Set_Manager(NULL);
+				pTool->On_Before_Execution();
+
+				bResult	=  SG_TOOL_PARAMETER_SET("CONNECTION", Connection)
+						&& SG_TOOL_PARAMETER_SET("TABLES"    , Table)
+						&& SG_TOOL_PARAMETER_SET("TABLE"     , this)
+						&& pTool->Execute();
+
+				SG_Get_Tool_Library_Manager().Delete_Tool(pTool);
+			}
+
+			SG_UI_ProgressAndMsg_Lock(false);
+		}
+	}
+
+	//-----------------------------------------------------
+	if( bResult )
+	{
+		Set_Modified(false);
+		Set_Update_Flag();
+
+		SG_UI_Msg_Add(_TL("okay"), false, SG_UI_MSG_STYLE_SUCCESS);
 
 		return( true );
 	}
 
+	Destroy();
+
+	SG_UI_Msg_Add(_TL("failed"), false, SG_UI_MSG_STYLE_FAILURE);
+
 	return( false );
 }
 
 //---------------------------------------------------------
-CSG_Table::CSG_Table(const CSG_String &File_Name, TSG_Table_File_Type Format)
-{
-	_On_Construction();
-
-	Create(File_Name, Format);
-}
-
-bool CSG_Table::Create(const CSG_String &File_Name, TSG_Table_File_Type Format)
-{
-	return( _Load(File_Name, Format, NULL) );
-}
-
-//---------------------------------------------------------
-CSG_Table::CSG_Table(const CSG_String &File_Name, TSG_Table_File_Type Format, const SG_Char *Separator)
+CSG_Table::CSG_Table(const CSG_String &File_Name, TSG_Table_File_Type Format, const SG_Char Separator, int Encoding)
 	: CSG_Data_Object()
 {
 	_On_Construction();
 
-	Create(File_Name, Format, Separator);
+	Create(File_Name, Format, Separator, Encoding);
 }
 
-bool CSG_Table::Create(const CSG_String &File_Name, TSG_Table_File_Type Format, const SG_Char *Separator)
+bool CSG_Table::Create(const CSG_String &File_Name, TSG_Table_File_Type Format, const SG_Char Separator, int Encoding)
 {
-	return( _Load(File_Name, Format, Separator) );
+	return( Load(File_Name, (int)Format, Separator, Encoding) );
 }
 
 //---------------------------------------------------------
-CSG_Table::CSG_Table(CSG_Table *pTemplate)
+CSG_Table::CSG_Table(const CSG_Table *pTemplate)
 	: CSG_Data_Object()
 {
 	_On_Construction();
@@ -195,28 +275,32 @@ CSG_Table::CSG_Table(CSG_Table *pTemplate)
 	Create(pTemplate);
 }
 
-bool CSG_Table::Create(CSG_Table *pTemplate)
+bool CSG_Table::Create(const CSG_Table *pTemplate)
 {
-	Destroy();
-
-	if( pTemplate && pTemplate->Get_Field_Count() > 0 )
+	if( !pTemplate || pTemplate->Get_Field_Count() < 1 )
 	{
-		for(int i=0; i<pTemplate->Get_Field_Count(); i++)
-		{
-			Add_Field(pTemplate->Get_Field_Name(i), pTemplate->Get_Field_Type(i));
-		}
-
-		return( true );
+		return( false );
 	}
 
-	return( false );
+	Destroy();
+
+	Set_Name              (pTemplate->Get_Name       ());
+	Set_Description       (pTemplate->Get_Description());
+	Set_NoData_Value_Range(pTemplate->Get_NoData_Value(), pTemplate->Get_NoData_hiValue());
+
+	m_Encoding	= pTemplate->m_Encoding;
+
+	for(int i=0; i<pTemplate->Get_Field_Count(); i++)
+	{
+		Add_Field(pTemplate->Get_Field_Name(i), pTemplate->Get_Field_Type(i));
+	}
+
+	return( true );
 }
 
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -231,19 +315,18 @@ void CSG_Table::_On_Construction(void)
 	m_nRecords		= 0;
 	m_nBuffer		= 0;
 
-	m_nSelected		= 0;
-	m_Selected		= NULL;
+	m_Encoding		= SG_FILE_ENCODING_UTF8;
 
 	m_Index			= NULL;
 
+	m_Selection.Create(sizeof(size_t), 0, SG_ARRAY_GROWTH_3);
+
 	Set_Update_Flag();
 }
 
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -302,27 +385,23 @@ CSG_Table & CSG_Table::operator = (const CSG_Table &Table)
 bool CSG_Table::Assign(CSG_Data_Object *pObject)
 {
 	if( !pObject || !pObject->is_Valid()
-	||	(	pObject->Get_ObjectType() != DATAOBJECT_TYPE_Table
-		&&	pObject->Get_ObjectType() != DATAOBJECT_TYPE_Shapes
-		&&	pObject->Get_ObjectType() != DATAOBJECT_TYPE_PointCloud	) )
+	||	(	pObject->Get_ObjectType() != SG_DATAOBJECT_TYPE_Table
+		&&	pObject->Get_ObjectType() != SG_DATAOBJECT_TYPE_Shapes
+		&&	pObject->Get_ObjectType() != SG_DATAOBJECT_TYPE_PointCloud	) )
 	{
 		return( false );
 	}
 
-	Destroy();
-
 	CSG_Table	*pTable	= (CSG_Table *)pObject;
 
-	int		i;
-
-	for(i=0; i<pTable->m_nFields; i++)
+	if( !Create(pTable) )
 	{
-		Add_Field(pTable->m_Field_Name[i]->c_str(), pTable->m_Field_Type[i]);
+		return( false );
 	}
 
-	for(i=0; i<pTable->m_nRecords; i++)
+	for(int i=0; i<pTable->Get_Count(); i++)
 	{
-		Add_Record(pTable->m_Records[i]);
+		Add_Record(pTable->Get_Record(i));
 	}
 
 	Get_History()	= pTable->Get_History();
@@ -356,7 +435,7 @@ bool CSG_Table::Assign_Values(CSG_Table *pTable)
 //---------------------------------------------------------
 bool CSG_Table::is_Compatible(CSG_Table *pTable, bool bExactMatch) const
 {
-	if( Get_Field_Count() == pTable->Get_Field_Count() )
+	if( pTable && Get_Field_Count() == pTable->Get_Field_Count() )
 	{
 		for(int i=0; i<Get_Field_Count(); i++)
 		{
@@ -425,7 +504,7 @@ bool CSG_Table::Add_Field(const CSG_String &Name, TSG_Data_Type Type, int add_Fi
 	}
 
 	//-----------------------------------------------------
-	m_Field_Name [add_Field]	= new CSG_String(!Name.is_Empty() ? Name : CSG_String::Format(SG_T("%s_%d"), SG_T("FIELD"), m_nFields));
+	m_Field_Name [add_Field]	= new CSG_String(!Name.is_Empty() ? Name : CSG_String::Format("FIELD_%d", m_nFields));
 	m_Field_Type [add_Field]	= Type;
 	m_Field_Stats[add_Field]	= new CSG_Simple_Statistics();
 
@@ -500,41 +579,23 @@ bool CSG_Table::Set_Field_Type(int iField, TSG_Data_Type Type)
 {
 	if( iField >= 0 && iField < m_nFields )
 	{
-		if( Type != Get_Field_Type(iField) )
+		if( m_Field_Type[iField] != Type )
 		{
 			m_Field_Type[iField]	= Type;
 
 			for(int i=0; i<m_nRecords; i++)
 			{
-				CSG_Table_Value	*pOld	= m_Records[i]->m_Values[iField];
+				CSG_Table_Record	*pRecord	= m_Records[i];
+
 				CSG_Table_Value	*pNew	= CSG_Table_Record::_Create_Value(Type);
 
-				switch( Type )
-				{
-				default:
-				case SG_DATATYPE_String:
-				case SG_DATATYPE_Date:		pNew->Set_Value(pOld->asString());	break;
+				(*pNew)	= *pRecord->m_Values[iField];
 
-				case SG_DATATYPE_Color:
-				case SG_DATATYPE_Byte:
-				case SG_DATATYPE_Char:
-				case SG_DATATYPE_Word:
-				case SG_DATATYPE_Short:
-				case SG_DATATYPE_DWord:
-				case SG_DATATYPE_Int:
-				case SG_DATATYPE_ULong:
-				case SG_DATATYPE_Long:		pNew->Set_Value(pOld->asInt   ());	break;
+				delete(pRecord->m_Values[iField]);
 
-				case SG_DATATYPE_Float:
-				case SG_DATATYPE_Double:	pNew->Set_Value(pOld->asDouble());	break;
+				pRecord->m_Values[iField]	= pNew;
 
-				case SG_DATATYPE_Binary:	pNew->Set_Value(pOld->asBinary());	break;
-				}
-
-				m_Records[i]->m_Values[iField]	= pNew;
-				m_Records[i]->Set_Modified();
-
-				delete(pOld);
+				pRecord->Set_Modified();
 			}
 
 			Set_Modified();
@@ -547,29 +608,37 @@ bool CSG_Table::Set_Field_Type(int iField, TSG_Data_Type Type)
 }
 
 //---------------------------------------------------------
-int CSG_Table::Get_Field_Length(int iField)	const
+int CSG_Table::Get_Field_Length(int iField, int Encoding)	const
 {
-	int		Length	= 0;
+	size_t	Length	= 0;
 
 	if( iField >= 0 && iField < m_nFields && m_Field_Type[iField] == SG_DATATYPE_String )
 	{
 		for(int i=0; i<m_nRecords; i++)
 		{
-			const SG_Char	*s	= m_Records[i]->asString(iField);
+			CSG_String	s(m_Records[i]->asString(iField));
 
-			if( s && s[0] )
+			size_t	n;
+
+			switch( Encoding )
 			{
-				int		n	= (int)SG_STR_LEN(s);
+			default                      :
+			case SG_FILE_ENCODING_UTF7   : n = s.Length()            ; break;
+			case SG_FILE_ENCODING_UTF8   : n = s.to_UTF8().Get_Size(); break;
+			case SG_FILE_ENCODING_UTF16LE:
+			case SG_FILE_ENCODING_UTF16BE: n = s.Length() * 2        ; break;
+			case SG_FILE_ENCODING_UTF32LE:
+			case SG_FILE_ENCODING_UTF32BE: n = s.Length() * 4        ; break;
+			}
 
-				if( Length < n )
-				{
-					Length	= n;
-				}
+			if( Length < n )
+			{
+				Length	= n;
 			}
 		}
 	}
 
-	return( Length );
+	return( (int)Length );
 }
 
 //---------------------------------------------------------
@@ -681,7 +750,7 @@ CSG_Table_Record * CSG_Table::Add_Record(CSG_Table_Record *pCopy)
 	{
 		if( pCopy )
 		{
-			if( Get_ObjectType() == DATAOBJECT_TYPE_Shapes && pCopy->Get_Table()->Get_ObjectType() == DATAOBJECT_TYPE_Shapes )
+			if( Get_ObjectType() == SG_DATAOBJECT_TYPE_Shapes && pCopy->Get_Table()->Get_ObjectType() == SG_DATAOBJECT_TYPE_Shapes )
 			{
 				((CSG_Shape *)pRecord)->Assign((CSG_Shape *)pCopy, true);
 			}
@@ -737,7 +806,7 @@ CSG_Table_Record * CSG_Table::Ins_Record(int iRecord, CSG_Table_Record *pCopy)
 		{
 			if( is_Indexed() )
 			{
-				m_Index[i]		= m_Index[i - 1];
+				m_Index[i]	= m_Index[i - 1];
 			}
 
 			m_Records[i]			= m_Records[i - 1];
@@ -749,7 +818,8 @@ CSG_Table_Record * CSG_Table::Ins_Record(int iRecord, CSG_Table_Record *pCopy)
 			m_Index[iRecord]	= iRecord;
 		}
 
-		m_Records[iRecord]		= pRecord;
+		pRecord->m_Index	= iRecord;
+		m_Records[iRecord]	= pRecord;
 		m_nRecords++;
 
 		Set_Modified();
@@ -764,6 +834,17 @@ CSG_Table_Record * CSG_Table::Ins_Record(int iRecord, CSG_Table_Record *pCopy)
 	return( NULL );
 }
 
+//---------------------------------------------------------
+bool CSG_Table::Set_Record(int iRecord, CSG_Table_Record *pCopy)
+{
+	if( iRecord >= 0 && iRecord < m_nRecords && pCopy )
+	{
+		return( m_Records[iRecord]->Assign(pCopy) );
+	}
+
+	return( false );
+}
+
 //---------------------------------------------------------
 bool CSG_Table::Del_Record(int iRecord)
 {
@@ -820,24 +901,19 @@ bool CSG_Table::Del_Record(int iRecord)
 //---------------------------------------------------------
 bool CSG_Table::Del_Records(void)
 {
-	if( m_Records > 0 )
+	_Index_Destroy();
+
+	for(int iRecord=0; iRecord<m_nRecords; iRecord++)
 	{
-		_Index_Destroy();
-
-		for(int iRecord=0; iRecord<m_nRecords; iRecord++)
-		{
-			delete(m_Records[iRecord]);
-		}
-
-		SG_Free(m_Records);
-		m_Records	= NULL;
-		m_nRecords	= 0;
-		m_nBuffer	= 0;
-
-		return( true );
+		delete(m_Records[iRecord]);
 	}
 
-	return( false );
+	SG_FREE_SAFE(m_Records);
+
+	m_nRecords	= 0;
+	m_nBuffer	= 0;
+
+	return( true );
 }
 
 //---------------------------------------------------------
@@ -856,6 +932,177 @@ bool CSG_Table::Set_Record_Count(int nRecords)
 }
 
 
+///////////////////////////////////////////////////////////
+//														 //
+//														 //
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CSG_Table::Find_Record(int &iRecord, int iField, const CSG_String &Value, bool bCreateIndex)
+{
+	if( iField < 0 || iField >= m_nFields || m_nRecords < 1 )
+	{
+		return( false );
+	}
+
+	if( m_nRecords == 1 )
+	{
+		return( Value.Cmp(m_Records[iRecord = 0]->asString(iField)) == 0 );
+	}
+
+	if( bCreateIndex && iField != Get_Index_Field(0) )
+	{
+		Set_Index(iField, TABLE_INDEX_Ascending);
+	}
+
+	//-----------------------------------------------------
+	if( iField != Get_Index_Field(0) )
+	{
+		for(iRecord=0; iRecord<m_nRecords; iRecord++)
+		{
+			if( Value.Cmp(m_Records[iRecord]->asString(iField)) == 0 )
+			{
+				return( true );
+			}
+		}
+	}
+
+	//-----------------------------------------------------
+	else	// indexed search
+	{
+		#define GET_RECORD(i)	Get_Record_byIndex(bAscending ? (iRecord = i) : m_nRecords - 1 - (iRecord = i))
+
+		bool	bAscending	= Get_Index_Order(0) == TABLE_INDEX_Ascending;
+
+		double	d;
+
+		if( (d = Value.Cmp(GET_RECORD(0             )->asString(iField))) < 0 ) { return( false ); } else if( d == 0 ) { return( true ); }
+		if( (d = Value.Cmp(GET_RECORD(m_nRecords - 1)->asString(iField))) > 0 ) { return( false ); } else if( d == 0 ) { return( true ); }
+
+		for(int a=0, b=m_nRecords-1; b-a > 1; )
+		{
+			d	= Value.Cmp(GET_RECORD(a + (b - a) / 2)->asString(iField));
+
+			if( d > 0.0 )
+			{
+				a	= iRecord;
+			}
+			else if( d < 0.0 )
+			{
+				b	= iRecord;
+			}
+			else
+			{
+				iRecord	= Get_Record_byIndex(bAscending ? iRecord : m_nRecords - 1 - iRecord)->Get_Index();
+
+				return( true );
+			}
+		}
+
+		iRecord	= Get_Record_byIndex(bAscending ? iRecord : m_nRecords - 1 - iRecord)->Get_Index();
+	}
+
+	//-----------------------------------------------------
+	return( false );
+}
+
+//---------------------------------------------------------
+CSG_Table_Record * CSG_Table::Find_Record(int iField, const CSG_String &Value, bool bCreateIndex)
+{
+	int	iRecord;
+
+	if( Find_Record(iRecord, iField, Value, bCreateIndex) )
+	{
+		return( Get_Record(iRecord) );
+	}
+
+	return( NULL );
+}
+
+//---------------------------------------------------------
+bool CSG_Table::Find_Record(int &iRecord, int iField, double Value, bool bCreateIndex)
+{
+	if( iField < 0 || iField >= m_nFields || m_nRecords < 1 )
+	{
+		return( false );
+	}
+
+	if( m_nRecords == 1 )
+	{
+		return( Value == m_Records[iRecord = 0]->asDouble(iField) );
+	}
+
+	if( bCreateIndex && iField != Get_Index_Field(0) )
+	{
+		Set_Index(iField, TABLE_INDEX_Ascending);
+	}
+
+	//-----------------------------------------------------
+	if( iField != Get_Index_Field(0) )
+	{
+		for(iRecord=0; iRecord<m_nRecords; iRecord++)
+		{
+			if( Value == m_Records[iRecord]->asDouble(iField) )
+			{
+				return( true );
+			}
+		}
+	}
+
+	//-----------------------------------------------------
+	else	// indexed search
+	{
+		#define GET_RECORD(i)	Get_Record_byIndex(bAscending ? (iRecord = i) : m_nRecords - 1 - (iRecord = i))
+
+		bool	bAscending	= Get_Index_Order(0) == TABLE_INDEX_Ascending;
+
+		double	d;
+
+		if( (d = Value - GET_RECORD(0             )->asDouble(iField)) < 0.0 ) { return( false ); } else if( d == 0.0 ) { return( true ); }
+		if( (d = Value - GET_RECORD(m_nRecords - 1)->asDouble(iField)) > 0.0 ) { return( false ); } else if( d == 0.0 ) { return( true ); }
+
+		for(int a=0, b=m_nRecords-1; b-a > 1; )
+		{
+			d	= Value - GET_RECORD(a + (b - a) / 2)->asDouble(iField);
+
+			if( d > 0.0 )
+			{
+				a	= iRecord;
+			}
+			else if( d < 0.0 )
+			{
+				b	= iRecord;
+			}
+			else
+			{
+				iRecord	= Get_Record_byIndex(bAscending ? iRecord : m_nRecords - 1 - iRecord)->Get_Index();
+
+				return( true );
+			}
+		}
+
+		iRecord	= Get_Record_byIndex(bAscending ? iRecord : m_nRecords - 1 - iRecord)->Get_Index();
+	}
+
+	//-----------------------------------------------------
+	return( false );
+}
+
+//---------------------------------------------------------
+CSG_Table_Record * CSG_Table::Find_Record(int iField, double Value, bool bCreateIndex)
+{
+	int	iRecord;
+
+	if( Find_Record(iRecord, iField, Value, bCreateIndex) )
+	{
+		return( Get_Record(iRecord) );
+	}
+
+	return( NULL );
+}
+
+
 ///////////////////////////////////////////////////////////
 //														 //
 //						Value Access					 //
@@ -972,17 +1219,21 @@ bool CSG_Table::_Stats_Update(int iField) const
 {
 	if( iField >= 0 && iField < m_nFields && m_nRecords > 0 )
 	{
-		if( !m_Field_Stats[iField]->is_Evaluated() )
-		{
-			CSG_Table_Record	**ppRecord	= m_Records;
+		CSG_Simple_Statistics	*pStatistics	= m_Field_Stats[iField];
 
-			for(int iRecord=0; iRecord<m_nRecords; iRecord++, ppRecord++)
+		if( !pStatistics->is_Evaluated() )
+		{
+			for(int iRecord=0; iRecord<m_nRecords; iRecord++)
 			{
-				if( !(*ppRecord)->is_NoData(iField) )
+				CSG_Table_Record	*pRecord	= m_Records[iRecord];
+
+				if( !pRecord->is_NoData(iField) )
 				{
-					m_Field_Stats[iField]->Add_Value((*ppRecord)->asDouble(iField));
+					pStatistics->Add_Value(pRecord->asDouble(iField));
 				}
 			}
+
+			pStatistics->Evaluate();	// evaluate! prevent values to be added more than once!
 		}
 
 		return( true );
@@ -992,6 +1243,21 @@ bool CSG_Table::_Stats_Update(int iField) const
 }
 
 
+///////////////////////////////////////////////////////////
+//														 //
+//														 //
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CSG_Table::On_NoData_Changed(void)
+{
+	_Stats_Invalidate();
+
+	return( CSG_Data_Object::On_NoData_Changed() );
+}
+
+
 ///////////////////////////////////////////////////////////
 //														 //
 //						Index							 //
@@ -1028,6 +1294,22 @@ bool CSG_Table::Set_Index(int Field_1, TSG_Table_Index_Order Order_1, int Field_
 		_Index_Destroy();
 	}
 
+	//-----------------------------------------------------
+	if( Get_Selection_Count() > 0 )
+	{
+		size_t	n	= 0;
+
+		for(int i=0; i<m_nRecords && n<Get_Selection_Count(); i++)
+		{
+			CSG_Table_Record	*pRecord	= Get_Record_byIndex(i);
+
+			if( pRecord && pRecord->is_Selected() )
+			{
+				_Set_Selection(pRecord->Get_Index(), n++);
+			}
+		}
+	}
+
 	return( is_Indexed() );
 }
 
