@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: table.h 2431 2015-03-03 17:21:59Z oconrad $
+ * Version $Id$
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -26,7 +26,8 @@
 // This library is free software; you can redistribute   //
 // it and/or modify it under the terms of the GNU Lesser //
 // General Public License as published by the Free       //
-// Software Foundation, version 2.1 of the License.      //
+// Software Foundation, either version 2.1 of the        //
+// License, or (at your option) any later version.       //
 //                                                       //
 // This library is distributed in the hope that it will  //
 // be useful, but WITHOUT ANY WARRANTY; without even the //
@@ -36,9 +37,7 @@
 //                                                       //
 // You should have received a copy of the GNU Lesser     //
 // General Public License along with this program; if    //
-// not, write to the Free Software Foundation, Inc.,     //
-// 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, //
-// USA.                                                  //
+// not, see <http://www.gnu.org/licenses/>.              //
 //                                                       //
 //-------------------------------------------------------//
 //                                                       //
@@ -125,6 +124,7 @@ TSG_Table_Index_Order;
 class SAGA_API_DLL_EXPORT CSG_Table_Record
 {
 	friend class CSG_Table;
+	friend class CSG_PointCloud;
 
 public:
 
@@ -147,8 +147,8 @@ public:
 	bool						is_NoData		(int              iField)	const;
 	bool						is_NoData		(const CSG_String &Field)	const;
 
-	const SG_Char *				asString		(int              iField, int Decimals = -1)	const;
-	const SG_Char *				asString		(const CSG_String &Field, int Decimals = -1)	const;
+	const SG_Char *				asString		(int              iField, int Decimals = -99)	const;
+	const SG_Char *				asString		(const CSG_String &Field, int Decimals = -99)	const;
 
 	SG_Char						asChar			(int              iField)	const	{	return( (SG_Char)asInt   (iField) );	}
 	SG_Char						asChar			(const CSG_String &Field)	const	{	return( (SG_Char)asInt   ( Field) );	}
@@ -220,27 +220,38 @@ public:
 									CSG_Table			(const CSG_Table &Table);
 	bool							Create				(const CSG_Table &Table);
 
-									CSG_Table			(const CSG_String &File_Name, TSG_Table_File_Type Format = TABLE_FILETYPE_Undefined);
-	bool							Create				(const CSG_String &File_Name, TSG_Table_File_Type Format = TABLE_FILETYPE_Undefined);
+									CSG_Table			(const CSG_Table *pTemplate);
+	bool							Create				(const CSG_Table *pTemplate);
 
-									CSG_Table			(const CSG_String &File_Name, TSG_Table_File_Type Format, const SG_Char *Separator);
-	bool							Create				(const CSG_String &File_Name, TSG_Table_File_Type Format, const SG_Char *Separator);
+									CSG_Table			(const CSG_String &FileName, TSG_Table_File_Type Format = TABLE_FILETYPE_Undefined, int Encoding = SG_FILE_ENCODING_UNDEFINED);
+	bool							Create				(const CSG_String &FileName, TSG_Table_File_Type Format = TABLE_FILETYPE_Undefined, int Encoding = SG_FILE_ENCODING_UNDEFINED);
 
-									CSG_Table			(CSG_Table *pTemplate);
-	bool							Create				(CSG_Table *pTemplate);
+									CSG_Table			(const CSG_String &FileName, TSG_Table_File_Type Format, const SG_Char Separator  , int Encoding = SG_FILE_ENCODING_UNDEFINED);
+	bool							Create				(const CSG_String &FileName, TSG_Table_File_Type Format, const SG_Char Separator  , int Encoding = SG_FILE_ENCODING_UNDEFINED);
 
 	virtual ~CSG_Table(void);
 
 	virtual bool					Destroy				(void);
 
-	virtual TSG_Data_Object_Type	Get_ObjectType		(void)	const			{	return( DATAOBJECT_TYPE_Table );	}
+	virtual TSG_Data_Object_Type	Get_ObjectType		(void)	const			{	return( SG_DATAOBJECT_TYPE_Table );	}
 
 	CSG_Table &						operator =			(const CSG_Table &Table);
 	virtual bool					Assign				(CSG_Data_Object *pSource);
 	bool							Assign_Values		(CSG_Table *pTable);
 
-	virtual bool					Save				(const CSG_String &File_Name, int Format = 0);
-	virtual bool					Save				(const CSG_String &File_Name, int Format, const SG_Char *Separator);
+	bool							Load				(const CSG_String &File, int Format, SG_Char Separator, int Encoding = SG_FILE_ENCODING_UNDEFINED);
+
+	virtual bool					Save				(const CSG_String &File, int Format, SG_Char Separator, int Encoding = SG_FILE_ENCODING_UNDEFINED);
+	virtual bool					Save				(const char       *File, int Format, SG_Char Separator, int Encoding = SG_FILE_ENCODING_UNDEFINED)	{ return( Save(CSG_String(File), Format, Separator, Encoding) ); }
+	virtual bool					Save				(const wchar_t    *File, int Format, SG_Char Separator, int Encoding = SG_FILE_ENCODING_UNDEFINED)	{ return( Save(CSG_String(File), Format, Separator, Encoding) ); }
+
+	virtual bool					Save				(const CSG_String &File, int Format = 0);
+	virtual bool					Save				(const char       *File, int Format = 0)	{	return( Save(CSG_String(File), Format) );	}
+	virtual bool					Save				(const wchar_t    *File, int Format = 0)	{	return( Save(CSG_String(File), Format) );	}
+
+	bool							Set_File_Encoding	(int Encoding);
+	int								Get_File_Encoding	(void)	const	{	return( m_Encoding );	}
+
 	bool							Serialize			(CSG_File &Stream, bool bSave);
 
 	//-----------------------------------------------------
@@ -254,24 +265,26 @@ public:
 	int								Get_Field_Count		(void)			const	{	return( m_nFields );	}
 	const SG_Char *					Get_Field_Name		(int iField)	const	{	return( iField >= 0 && iField < m_nFields ? m_Field_Name[iField]->c_str() : NULL );			}
 	TSG_Data_Type					Get_Field_Type		(int iField)	const	{	return( iField >= 0 && iField < m_nFields ? m_Field_Type[iField] : SG_DATATYPE_Undefined );	}
-	int								Get_Field_Length	(int iField)	const;				// returns the maximum number of characters for data type string and zero for all other data types.
+	int								Get_Field_Length	(int iField, int Encoding = SG_FILE_ENCODING_UNDEFINED)	const;	// returns the maximum number of characters for data type string and zero for all other data types.
 	int								Get_Field			(const CSG_String &Name)	const;	// returns the zero based position of the field named 'Name' or '-1' if there is no field with such name.
 
 	bool							Set_Field_Name		(int iField, const SG_Char *Name);
 	bool							Set_Field_Type		(int iField, TSG_Data_Type  Type);
 
-	sLong							Get_N				(int iField)	const	{	return( _Stats_Update(iField) ? m_Field_Stats[iField]->Get_Count()    : 0   );	}
-	double							Get_Minimum			(int iField)	const	{	return( _Stats_Update(iField) ? m_Field_Stats[iField]->Get_Minimum()  : 0.0 );	}
-	double							Get_Maximum			(int iField)	const	{	return( _Stats_Update(iField) ? m_Field_Stats[iField]->Get_Maximum()  : 0.0 );	}
-	double							Get_Range			(int iField)	const	{	return( _Stats_Update(iField) ? m_Field_Stats[iField]->Get_Range()    : 0.0 );	}
-	double							Get_Sum				(int iField)	const	{	return( _Stats_Update(iField) ? m_Field_Stats[iField]->Get_Sum()      : 0.0 );	}
-	double							Get_Mean			(int iField)	const	{	return( _Stats_Update(iField) ? m_Field_Stats[iField]->Get_Mean()     : 0.0 );	}
-	double							Get_StdDev			(int iField)	const	{	return( _Stats_Update(iField) ? m_Field_Stats[iField]->Get_StdDev()   : 0.0 );	}
+	sLong							Get_N				(int iField)	const	{	return( _Stats_Update(iField) ? m_Field_Stats[iField]->Get_Count   () : 0   );	}
+	double							Get_Minimum			(int iField)	const	{	return( _Stats_Update(iField) ? m_Field_Stats[iField]->Get_Minimum () : 0.0 );	}
+	double							Get_Maximum			(int iField)	const	{	return( _Stats_Update(iField) ? m_Field_Stats[iField]->Get_Maximum () : 0.0 );	}
+	double							Get_Range			(int iField)	const	{	return( _Stats_Update(iField) ? m_Field_Stats[iField]->Get_Range   () : 0.0 );	}
+	double							Get_Sum				(int iField)	const	{	return( _Stats_Update(iField) ? m_Field_Stats[iField]->Get_Sum     () : 0.0 );	}
+	double							Get_Mean			(int iField)	const	{	return( _Stats_Update(iField) ? m_Field_Stats[iField]->Get_Mean    () : 0.0 );	}
+	double							Get_StdDev			(int iField)	const	{	return( _Stats_Update(iField) ? m_Field_Stats[iField]->Get_StdDev  () : 0.0 );	}
 	double							Get_Variance		(int iField)	const	{	return( _Stats_Update(iField) ? m_Field_Stats[iField]->Get_Variance() : 0.0 );	}
+	const CSG_Simple_Statistics &	Get_Statistics		(int iField)	const	{	_Stats_Update(iField); return( *m_Field_Stats[iField] );	}
 
 	//-----------------------------------------------------
 	virtual CSG_Table_Record *		Add_Record			(             CSG_Table_Record *pCopy = NULL);
 	virtual CSG_Table_Record *		Ins_Record			(int iRecord, CSG_Table_Record *pCopy = NULL);
+	virtual bool					Set_Record			(int iRecord, CSG_Table_Record *pCopy);
 	virtual bool					Del_Record			(int iRecord);
 	virtual bool					Del_Records			(void);
 	virtual bool					Set_Record_Count	(int nRecords);
@@ -298,6 +311,12 @@ public:
 		return( NULL );
 	}
 
+	//-----------------------------------------------------
+	virtual bool					Find_Record			(int &iRecord, int iField, const CSG_String &Value, bool bCreateIndex = false);
+	virtual CSG_Table_Record *		Find_Record			(              int iField, const CSG_String &Value, bool bCreateIndex = false);
+	virtual bool					Find_Record			(int &iRecord, int iField, double            Value, bool bCreateIndex = false);
+	virtual CSG_Table_Record *		Find_Record			(              int iField, double            Value, bool bCreateIndex = false);
+
 	//-----------------------------------------------------
 	virtual bool					Set_Value			(int iRecord, int iField, const SG_Char  *Value);
 	virtual bool					Set_Value			(int iRecord, int iField, double          Value);
@@ -308,8 +327,9 @@ public:
 	virtual void					Set_Modified		(bool bModified = true);
 
 	//-----------------------------------------------------
-	int								Get_Selection_Count	(void)			const	{	return( m_nSelected );	}
-	virtual CSG_Table_Record *		Get_Selection		(int Index = 0)	const	{	return( Index >= 0 && Index < m_nSelected ? Get_Record(m_Selected[Index]) : NULL );	}
+	size_t							Get_Selection_Count	(void)				const	{	return( m_Selection.Get_Size() );	}
+	size_t							Get_Selection_Index	(size_t Index = 0)	const	{	return( Index < m_Selection.Get_Size() ? *((size_t *)m_Selection.Get_Entry(Index)) : (size_t)Get_Count() );	}
+	virtual CSG_Table_Record *		Get_Selection		(size_t Index = 0)	const	{	return( Index < m_Selection.Get_Size() ? Get_Record((int)Get_Selection_Index(Index)) : NULL );	}
 
 	virtual bool					is_Selected			(int iRecord)	const	{	return( iRecord >= 0 && iRecord < m_nRecords ? m_Records[iRecord]->is_Selected() : false );	}
 
@@ -332,7 +352,7 @@ public:
 
 protected:
 
-	int								m_nFields, m_nRecords, m_nBuffer, m_nSelected;
+	int								m_nFields, m_nRecords, m_nBuffer, m_Encoding;
 
 	TSG_Data_Type					*m_Field_Type;
 
@@ -340,19 +360,30 @@ protected:
 
 	CSG_Simple_Statistics			**m_Field_Stats;
 
+	CSG_Array						m_Selection;
+
 
 	virtual void					_On_Construction	(void);
 
 	virtual CSG_Table_Record *		_Get_New_Record		(int Index);
 
+	bool							_Add_Selection		(size_t iRecord);
+	bool							_Set_Selection		(size_t iRecord, size_t Index);
+	bool							_Del_Selection		(size_t iRecord);
+
 	bool							_Stats_Invalidate	(void)			const;
 	bool							_Stats_Invalidate	(int iField)	const;
 	virtual bool					_Stats_Update		(int iField)	const;
 
+	virtual bool					On_NoData_Changed	(void);
+
+	virtual bool					On_Reload			(void);
+	virtual bool					On_Delete			(void);
+
 
 private:
 
-	int								*m_Index, m_Index_Field[3], *m_Selected;
+	int								*m_Index, m_Index_Field[3];
 
 	TSG_Table_Index_Order			m_Index_Order[3];
 
@@ -364,11 +395,14 @@ private:
 	bool							_Inc_Array			(void);
 	bool							_Dec_Array			(void);
 
-	bool							_Load				(const CSG_String &File_Name, TSG_Table_File_Type Format, const SG_Char *Separator);
-	bool							_Load_Text			(const CSG_String &File_Name, bool bHeadline, const SG_Char *Separator);
-	bool							_Save_Text			(const CSG_String &File_Name, bool bHeadline, const SG_Char *Separator);
-	bool							_Load_DBase			(const CSG_String &File_Name);
-	bool							_Save_DBase			(const CSG_String &File_Name);
+	size_t							_Load_Text_Trim		(      CSG_String &Text, const SG_Char Separator);
+	size_t							_Load_Text_EndQuote	(const CSG_String &Text, const SG_Char Separator);
+
+	bool							_Load_Text			(const CSG_String &FileName, bool bHeadline, const SG_Char Separator);
+	bool							_Save_Text			(const CSG_String &FileName, bool bHeadline, const SG_Char Separator);
+
+	bool							_Load_DBase			(const CSG_String &FileName);
+	bool							_Save_DBase			(const CSG_String &FileName);
 
 	void							_Index_Create		(void);
 	void							_Index_Destroy		(void);
@@ -392,10 +426,13 @@ SAGA_API_DLL_EXPORT CSG_Table *	SG_Create_Table	(void);
 SAGA_API_DLL_EXPORT CSG_Table *	SG_Create_Table	(const CSG_Table &Table);
 
 /** Safe table construction */
-SAGA_API_DLL_EXPORT CSG_Table *	SG_Create_Table	(const CSG_String &File_Name);
+SAGA_API_DLL_EXPORT CSG_Table *	SG_Create_Table	(CSG_Table *pTemplate);
 
 /** Safe table construction */
-SAGA_API_DLL_EXPORT CSG_Table *	SG_Create_Table	(CSG_Table *pTemplate);
+SAGA_API_DLL_EXPORT CSG_Table *	SG_Create_Table	(const CSG_String &FileName, TSG_Table_File_Type Format = TABLE_FILETYPE_Undefined, int Encoding = SG_FILE_ENCODING_UNDEFINED);
+
+/** Safe table construction */
+SAGA_API_DLL_EXPORT CSG_Table *	SG_Create_Table	(const CSG_String &FileName, TSG_Table_File_Type Format, const SG_Char Separator  , int Encoding = SG_FILE_ENCODING_UNDEFINED);
 
 
 ///////////////////////////////////////////////////////////
