@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: saga_cmd.cpp 2418 2015-02-19 20:58:24Z oconrad $
+ * Version $Id$
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -24,7 +24,8 @@
 // Geoscientific Analyses'. SAGA is free software; you   //
 // can redistribute it and/or modify it under the terms  //
 // of the GNU General Public License as published by the //
-// Free Software Foundation; version 2 of the License.   //
+// Free Software Foundation, either version 2 of the     //
+// License, or (at your option) any later version.       //
 //                                                       //
 // SAGA is distributed in the hope that it will be       //
 // useful, but WITHOUT ANY WARRANTY; without even the    //
@@ -32,11 +33,9 @@
 // PARTICULAR PURPOSE. See the GNU General Public        //
 // License for more details.                             //
 //                                                       //
-// You should have received a copy of the GNU General    //
-// Public License along with this program; if not,       //
-// write to the Free Software Foundation, Inc.,          //
-// 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, //
-// USA.                                                  //
+// You should have received a copy of the GNU Lesser     //
+// General Public License along with this program; if    //
+// not, see <http://www.gnu.org/licenses/>.              //
 //                                                       //
 //-------------------------------------------------------//
 //                                                       //
@@ -66,9 +65,9 @@
 #include <wx/app.h>
 #include <wx/utils.h>
 
+#include "config.h"
 #include "callback.h"
-
-#include "module_library.h"
+#include "tool.h"
 
 
 ///////////////////////////////////////////////////////////
@@ -85,20 +84,22 @@ bool		Execute_Script	(const CSG_String &Script);
 
 bool		Load_Libraries	(void);
 
-bool		Check_First		(const CSG_String &Argument);
+bool		Check_First		(const CSG_String &Argument, int argc, char *argv[]);
 bool		Check_Flags		(const CSG_String &Argument);
 
 void		Print_Libraries	(void);
-void		Print_Modules	(CSG_Module_Library *pLibrary);
-void		Print_Execution	(CSG_Module_Library *pLibrary, CSG_Module *pModule);
+void		Print_Tools		(const CSG_String &Library);
+void		Print_Execution	(CSG_Tool *pTool);
 
 void		Print_Logo		(void);
 void		Print_Get_Help	(void);
 void		Print_Help		(void);
+void		Print_Help		(const CSG_String &Library);
+void		Print_Help		(const CSG_String &Library, const CSG_String &Tool);
 void		Print_Version	(void);
 
-void		Create_Example	(void);
-void		Create_Docs		(void);
+void		Create_Batch	(const CSG_String &File);
+void		Create_Docs		(const CSG_String &Directory);
 
 
 ///////////////////////////////////////////////////////////
@@ -112,38 +113,50 @@ int		main	(int argc, char *argv[])
 {
 	if( !wxInitialize() )
 	{
-		CMD_Print_Error(SG_T("initialisation failed"));
+		CMD_Print_Error("initialisation failed");
 
 		return( 1 );
 	}
 
-#if !defined(_DEBUG)
-	wxSetAssertHandler(NULL);		// disable all wx asserts in SAGA release builds
-#endif
+	wxTheApp->SetVendorName("www.saga-gis.org");
+	wxTheApp->SetAppName   ("saga_cmd");
 
 //---------------------------------------------------------
-#if !defined(_DEBUG) && defined(_SAGA_VC)
-#define _MODULE_EXCEPTION
+#if !defined(_DEBUG)
+	wxSetAssertHandler(NULL);		// disable all wx asserts in SAGA release builds
+
+	#if !defined(_OPENMP) && defined(_SAGA_MSW)
+		#define _TOOL_EXCEPTION
+	#endif
+#endif
+//---------------------------------------------------------
+
+	bool bResult	= false;
+
+///////////////////////////////////////////////////////////
+#ifdef _TOOL_EXCEPTION
 _try
 {
 #endif
-//---------------------------------------------------------
+///////////////////////////////////////////////////////////
 
-	bool bResult	= Run(argc, argv);
+	bResult	= Run(argc, argv);
+
+///////////////////////////////////////////////////////////
+#ifdef _TOOL_EXCEPTION
+}
+_except(1)
+{
+	CMD_Print_Error(_TL("access violation"));
+}
+#endif
+///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
 #ifdef _DEBUG
 	CMD_Set_Interactive(true);
 	CMD_Get_Pause();
 #endif
-
-#ifdef _MODULE_EXCEPTION
-}
-_except(1)
-{
-	Print_Error(_TL("access violation"));
-}
-#endif
 //---------------------------------------------------------
 
 	fflush(stdout);
@@ -168,8 +181,10 @@ bool		Run(int argc, char *argv[])
 
 	SG_Set_UI_Callback(CMD_Get_Callback());
 
+	Config_Load();	// first load default configuration (if available). can be modified subsequently by flags
+
 	//-----------------------------------------------------
-	if( Check_First(argv[1]) )
+	if( Check_First(argv[1], argc - 2, argv + 2) )
 	{
 		return( true );
 	}
@@ -190,25 +205,13 @@ bool		Run(int argc, char *argv[])
 		return( false );
 	}
 
-	//-----------------------------------------------------
-	if( argc <= 1 )
-	{
-		CMD_Print_Error(_TL("no arguments for saga call"));
-
-		Print_Libraries();
-
-		return( false );
-	}
-
 	//-----------------------------------------------------
 	if( argc == 2 && SG_File_Exists(CSG_String(argv[1])) )
 	{
 		return( Execute_Script(argv[1]) );
 	}
-	else
-	{
-		return( Execute(argc, argv) );
-	}
+
+	return( Execute(argc, argv) );
 }
 
 
@@ -221,43 +224,48 @@ bool		Run(int argc, char *argv[])
 //---------------------------------------------------------
 bool		Execute(int argc, char *argv[])
 {
-	CSG_Module_Library	*pLibrary;
-	CSG_Module			*pModule;
+	//-----------------------------------------------------
+	CSG_String	Library	= argc <= 1 ? "" : argv[1];
 
-	if( argc == 1 || (pLibrary = SG_Get_Module_Library_Manager().Get_Library(CSG_String(argv[1]), true)) == NULL )
+	if( SG_Get_Tool_Library_Manager().Get_Library(Library, true) == NULL )
 	{
 		Print_Libraries();
 
 		return( false );
 	}
 
-	if( argc == 2 || (pModule = pLibrary->Get_Module(argv[2])) == NULL )
+	//-----------------------------------------------------
+	CSG_Tool	*pTool	= argc <= 2 ? NULL : SG_Get_Tool_Library_Manager().Get_Tool(Library, CSG_String(argv[2]));
+
+	if( pTool == NULL )
 	{
-		Print_Modules(pLibrary);
-
-		return( false );
-	}
-
-	if( argc == 3 && CMD_Get_XML() )
-	{	// Just output tool synopsis as XML-tagged text, then return.
-		SG_PRINTF(pModule->Get_Summary(true, "", "", true).c_str());
-
-		return( true );
-	}
-
-	if( pModule->needs_GUI() )
-	{
-		CMD_Print_Error(_TL("tool needs graphical user interface"), pModule->Get_Name());
+		Print_Tools(Library);
 
 		return( false );
 	}
 
 	//-----------------------------------------------------
-	Print_Execution(pLibrary, pModule);
+	if( argc == 3 && CMD_Get_XML() )
+	{	// Just output tool synopsis as XML-tagged text, then return.
+		SG_Printf(pTool->Get_Summary(true, "", "", SG_SUMMARY_FMT_XML).c_str());
 
-	CCMD_Module	CMD_Module(pLibrary, pModule);
+		return( true );
+	}
 
-	return( CMD_Module.Execute(argc - 2, argv + 2) );
+	//-----------------------------------------------------
+	if( pTool->needs_GUI() )
+	{
+		CMD_Print_Error(_TL("tool needs graphical user interface"), pTool->Get_Name());
+
+		return( false );
+	}
+
+	//-----------------------------------------------------
+	Print_Execution(pTool);
+
+	CCMD_Tool	CMD_Tool(pTool);
+
+	return( CMD_Tool.Execute(argc - 2, argv + 2) );
 }
 
 
@@ -341,7 +349,7 @@ bool		Execute_Script(const CSG_String &Script)
 {
 	if( CMD_Get_Show_Messages() )
 	{
-		CMD_Print(CSG_String::Format(SG_T("%s: %s"), _TL("Running Script"), Script.c_str()));
+		CMD_Print(CSG_String::Format("%s: %s", _TL("Running Script"), Script.c_str()));
 	}
 
 	CSG_File	Stream;
@@ -383,7 +391,7 @@ bool		Load_Libraries(const CSG_String &Directory)
 	bool	bShow	= CMD_Get_Show_Messages();
 
 	CMD_Set_Show_Messages(false);
-	int	n	= SG_Get_Module_Library_Manager().Add_Directory(Directory, false);
+	int	n	= SG_Get_Tool_Library_Manager().Add_Directory(Directory, false);
 	CMD_Set_Show_Messages(bShow);
 
 	return( n > 0 );
@@ -395,11 +403,12 @@ bool		Load_Libraries(void)
 	wxString	Path, CMD_Path	= SG_File_Get_Path(SG_UI_Get_Application_Path()).c_str();
 
     #if defined(_SAGA_LINUX)
-		Load_Libraries(wxT(MODULE_LIBRARY_PATH));
+		Load_Libraries(MODULE_LIBRARY_PATH);
+		Load_Libraries(SG_File_Make_Path(CSG_String(SHARE_PATH), SG_T("toolchains")));	// look for tool chains
 	#else
-		wxString	DLL_Path	= SG_File_Make_Path(CMD_Path, SG_T("dll")).c_str();
+		wxString	DLL_Path	= SG_File_Make_Path(&CMD_Path, "dll").c_str();
 
-		if( wxGetEnv(wxT("PATH"), &Path) && Path.Length() > 0 )
+		if( wxGetEnv("PATH", &Path) && !Path.IsEmpty() )
 		{
 			wxSetEnv("PATH", DLL_Path + wxT(";") + Path);
 		}
@@ -409,23 +418,28 @@ bool		Load_Libraries(void)
 		}
 
 		wxSetEnv("GDAL_DRIVER_PATH", DLL_Path);
+		wxSetEnv("GDAL_DATA"       , DLL_Path + "\\gdal-data");
 
-		Load_Libraries(SG_File_Make_Path(CMD_Path, SG_T("modules")));
+		Load_Libraries(SG_File_Make_Path(&CMD_Path, "tools"));
     #endif
 
-	if( wxGetEnv(SG_T("SAGA_MLB"), &Path) )
+	if( (wxGetEnv("SAGA_TLB", &Path) || wxGetEnv("SAGA_MLB", &Path)) && !Path.IsEmpty() )
 	{
-		while( Path.Length() > 0 )
-		{
-			Load_Libraries(CSG_String(&Path).BeforeFirst(';'));
+		#if defined(_SAGA_MSW)
+			CSG_String_Tokenizer	Paths(&Path, ";");	// colon (':') would split drive from paths!
+		#else
+			CSG_String_Tokenizer	Paths(&Path, ";:");	// colon (':') is more native to non-windows os than semi-colon (';'), we support both...
+		#endif
 
-			Path	= Path.AfterFirst(';');
+		while( Paths.Has_More_Tokens() )
+		{
+			Load_Libraries(Paths.Get_Next_Token());
 		}
 	}
 
-	if( SG_Get_Module_Library_Manager().Get_Count() <= 0 )
+	if( SG_Get_Tool_Library_Manager().Get_Count() <= 0 )
 	{
-		CMD_Print_Error(SG_T("could not load any tool library"));
+		CMD_Print_Error("could not load any tool library");
 
 		return( false );
 	}
@@ -441,32 +455,44 @@ bool		Load_Libraries(void)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-bool		Check_First		(const CSG_String &Argument)
+bool		Check_First		(const CSG_String &Argument, int argc, char *argv[])
 {
-	if( !Argument.CmpNoCase("-h") || !Argument.CmpNoCase("--help") )
+	if( !Argument.Cmp("-h") || !Argument.Cmp("--help") )
 	{
-		Print_Help();
+		switch( argc )
+		{
+		default:	Print_Help(                );	break;
+		case  1:	Print_Help(argv[0]         );	break;
+		case  2:	Print_Help(argv[0], argv[1]);	break;
+		}
 
 		return( true );
 	}
 
-	if( !Argument.CmpNoCase("-v") || !Argument.CmpNoCase("--version") )
+	if( !Argument.Cmp("-v") || !Argument.Cmp("--version") )
 	{
 		Print_Version();
 
 		return( true );
 	}
 
-	if( !Argument.CmpNoCase("-b") || !Argument.CmpNoCase("--batch") )
+	if( !Argument.Find("--create-config") )
 	{
-		Create_Example();
+		Config_Create(Argument.AfterFirst('='));
 
 		return( true );
 	}
 
-	if( !Argument.CmpNoCase("-d") || !Argument.CmpNoCase("--docs") )
+	if( !Argument.Find("--create-batch") )
 	{
-		Create_Docs();
+		Create_Batch(Argument.AfterFirst('='));
+
+		return( true );
+	}
+
+	if( !Argument.Find("--create-docs") )
+	{
+		Create_Docs(Argument.AfterFirst('='));
 
 		return( true );
 	}
@@ -477,11 +503,18 @@ bool		Check_First		(const CSG_String &Argument)
 //---------------------------------------------------------
 bool		Check_Flags		(const CSG_String &Argument)
 {
-	CSG_String	s(Argument.BeforeFirst(SG_T('=')));
+#if   defined(_SAGA_LINUX)
+	CSG_String	Path_Shared	= SHARE_PATH;
+#elif defined(_SAGA_MSW)
+	CSG_String	Path_Shared	= SG_File_Get_Path(SG_UI_Get_Application_Path());
+#endif
 
-	if( !s.CmpNoCase("-f") || !s.CmpNoCase("--flags") )
+	//-----------------------------------------------------
+	CSG_String	s(Argument.BeforeFirst('='));
+
+	if( !s.Cmp("-f") || !s.Cmp("--flags") )
 	{
-		s	= CSG_String(Argument).AfterFirst(SG_T('='));
+		s	= CSG_String(Argument).AfterFirst('=');
 
 		CMD_Set_Show_Progress(s.Find('q') < 0 && s.Find('s') < 0);	// q, s: no progress report
 		CMD_Set_Show_Messages(s.Find('r') < 0 && s.Find('s') < 0);	// r, s: no messages report
@@ -490,20 +523,20 @@ bool		Check_Flags		(const CSG_String &Argument)
 
 		if( s.Find('l') >= 0 )	// l: load translation dictionary
 		{
-			SG_Get_Translator() .Create(SG_File_Make_Path(SG_File_Get_Path(SG_UI_Get_Application_Path()),
-				SG_T("saga"    ), SG_T("lng")), false);
+			SG_Printf(CSG_String::Format("\n%s:", _TL("loading translation dictionary")));
+			SG_Printf(CSG_String::Format("\n%s.\n",
+				SG_Get_Translator().Create(SG_File_Make_Path(Path_Shared, SG_T("saga"), SG_T("lng")), false)
+				? _TL("success") : _TL("failed")
+			));
 		}
 
 		if( s.Find('p') >= 0 )	// p: load projections dictionary
 		{
-#if defined(_SAGA_LINUX)
-		SG_Get_Projections().Create(SG_File_Make_Path(SG_File_Get_Path( CSG_String(SHARE_PATH)),
-			SG_T("saga_prj"), SG_T("srs")));
-#endif
-#if defined(_SAGA_MSW)
-		SG_Get_Projections().Create(SG_File_Make_Path(SG_File_Get_Path(SG_UI_Get_Application_Path()),
-			SG_T("saga_prj"), SG_T("srs")));
-#endif
+			SG_Printf(CSG_String::Format("\n%s:", _TL("loading spatial reference system database")));
+			SG_Printf(CSG_String::Format("\n%s.\n",
+				SG_Get_Projections().Create(SG_File_Make_Path(Path_Shared, SG_T("saga_prj"), SG_T("srs")))
+				? _TL("success") : _TL("failed")
+			));
 		}
 
 		if( s.Find('o') >= 0 )	// o: load old style naming, has no effect if l-flag is set.
@@ -514,23 +547,21 @@ bool		Check_Flags		(const CSG_String &Argument)
 		return( true );
 	}
 
-	else if( !s.CmpNoCase("-c") || !s.CmpNoCase("--cores") )
+	//-----------------------------------------------------
+	else if( !s.Cmp("-c") || !s.Cmp("--cores") )
 	{
-		#ifdef _OPENMP
 		int	nCores	= 1;
 
-		if( !CSG_String(Argument).AfterFirst('=').asInt(nCores) || nCores > SG_Get_Max_Num_Procs_Omp() )
+		if( CSG_String(Argument).AfterFirst('=').asInt(nCores) )
 		{
-			nCores	= SG_Get_Max_Num_Procs_Omp();
+			SG_OMP_Set_Max_Num_Threads(nCores);
 		}
 
-		SG_Set_Max_Num_Threads_Omp(nCores);
-		#endif // _OPENMP
-
 		return( true );
 	}
 
-	else if( !s.CmpNoCase("-s") || !s.CmpNoCase("--story") )
+	//-----------------------------------------------------
+	else if( !s.Cmp("-s") || !s.Cmp("--story") )
 	{
 		int	Depth;
 
@@ -542,6 +573,15 @@ bool		Check_Flags		(const CSG_String &Argument)
 		return( true );
 	}
 
+	//-----------------------------------------------------
+	else if( !s.Cmp("-C") || !s.Cmp("--config") )
+	{
+		Config_Load(CSG_String(Argument).AfterFirst('='));
+
+		return( true );
+	}
+
+	//-----------------------------------------------------
 	return( false );
 }
 
@@ -555,17 +595,17 @@ bool		Check_Flags		(const CSG_String &Argument)
 //---------------------------------------------------------
 void		Print_Libraries	(void)
 {
-	CMD_Print_Error(_TL("select a library"));
-
 	if( CMD_Get_Show_Messages() )
 	{
 		if( CMD_Get_XML() )
 		{
-			SG_PRINTF(SG_Get_Module_Library_Manager().Get_Summary(SG_SUMMARY_FMT_XML_NO_INTERACTIVE).c_str());
+			SG_Printf(SG_Get_Tool_Library_Manager().Get_Summary(SG_SUMMARY_FMT_XML).c_str());
 		}
 		else
 		{
-			CMD_Print(SG_Get_Module_Library_Manager().Get_Summary(SG_SUMMARY_FMT_FLAT_NO_INTERACTIVE));
+			CMD_Print(SG_Get_Tool_Library_Manager().Get_Summary(SG_SUMMARY_FMT_FLAT));
+
+			CMD_Print_Error(_TL("select a library"));
 
 			Print_Get_Help();
 		}
@@ -573,19 +613,35 @@ void		Print_Libraries	(void)
 }
 
 //---------------------------------------------------------
-void		Print_Modules	(CSG_Module_Library *pLibrary)
+void		Print_Tools		(const CSG_String &Library)
 {
-	CMD_Print_Error(_TL("select a tool"));
-
 	if( CMD_Get_Show_Messages() )
 	{
 		if( CMD_Get_XML() )
 		{
-			SG_PRINTF(pLibrary->Get_Summary(SG_SUMMARY_FMT_XML_NO_INTERACTIVE).c_str());
+			for(int i=0; i<SG_Get_Tool_Library_Manager().Get_Count(); i++)
+			{
+				CSG_Tool_Library	*pLibrary	= SG_Get_Tool_Library_Manager().Get_Library(i);
+
+				if( !pLibrary->Get_Library_Name().Cmp(Library) )
+				{
+					SG_Printf(pLibrary->Get_Summary(SG_SUMMARY_FMT_XML , false).c_str());
+				}
+			}
 		}
 		else
 		{
-			CMD_Print(pLibrary->Get_Summary(SG_SUMMARY_FMT_FLAT_NO_INTERACTIVE));
+			for(int i=0; i<SG_Get_Tool_Library_Manager().Get_Count(); i++)
+			{
+				CSG_Tool_Library	*pLibrary	= SG_Get_Tool_Library_Manager().Get_Library(i);
+
+				if( !pLibrary->Get_Library_Name().Cmp(Library) )
+				{
+					CMD_Print(pLibrary->Get_Summary(SG_SUMMARY_FMT_FLAT, false));
+				}
+			}
+
+			CMD_Print_Error(_TL("select a tool"));
 
 			Print_Get_Help();
 		}
@@ -593,30 +649,35 @@ void		Print_Modules	(CSG_Module_Library *pLibrary)
 }
 
 //---------------------------------------------------------
-void		Print_Execution	(CSG_Module_Library *pLibrary, CSG_Module *pModule)
+void		Print_Execution	(CSG_Tool *pTool)
 {
 	if( CMD_Get_Show_Messages() )
 	{
 		if( CMD_Get_XML() )
 		{
-			SG_PRINTF(SG_T("<?xml version='1.0' encoding='UTF-8' standalone='yes' ?>\n"));
-			SG_PRINTF(SG_T("<%s>\n"), SG_XML_LIBRARY);
-			SG_PRINTF(SG_T("\t<%s>%s</%s>\n"), SG_XML_LIBRARY_PATH, pLibrary->Get_File_Name().c_str(), SG_XML_LIBRARY_PATH);
-			SG_PRINTF(SG_T("\t<%s>%s</%s>\n"), SG_XML_LIBRARY_NAME, pLibrary->Get_Name()     .c_str(), SG_XML_LIBRARY_NAME);
-			SG_PRINTF(SG_T("</%s>\n"), SG_XML_LIBRARY);
+			#define SG_XML_LIBRARY		SG_T("library")
+			#define SG_XML_LIBRARY_PATH	SG_T("path")
+			#define SG_XML_LIBRARY_NAME	SG_T("name")
+
+			SG_Printf("<?xml version='1.0' encoding='UTF-8' standalone='yes' ?>\n");
+			SG_Printf("<%s>\n", SG_XML_LIBRARY);
+			SG_Printf("\t<%s>%s</%s>\n", SG_XML_LIBRARY_PATH, pTool->Get_File_Name().c_str(), SG_XML_LIBRARY_PATH);
+			SG_Printf("\t<%s>%s</%s>\n", SG_XML_LIBRARY_NAME, pTool->Get_Library  ().c_str(), SG_XML_LIBRARY_NAME);
+			SG_Printf("</%s>\n", SG_XML_LIBRARY);
 		}
 		else
 		{
-			SG_PRINTF(SG_T("____________________________\n"));
-			SG_PRINTF(SG_T("%s:\t%s\n"     ), _TL("library path"), SG_File_Get_Path(pLibrary->Get_File_Name()       ).c_str());
-			SG_PRINTF(SG_T("%s:\t%s\n"     ), _TL("library name"), SG_File_Get_Name(pLibrary->Get_File_Name(), false).c_str());
-			SG_PRINTF(SG_T("%s:\t%s\n"     ), _TL("library     "), pLibrary->Get_Name     ().c_str());
-			SG_PRINTF(SG_T("%s:\t%s\n"     ), _TL("tool        "), pModule ->Get_Name     ().c_str());
-			SG_PRINTF(SG_T("%s:\t%s\n"     ), _TL("author      "), pModule ->Get_Author   ().c_str());
+			SG_Printf("____________________________\n");
+			SG_Printf("%s: %s\n", _TL("library path"), SG_File_Get_Path(pTool->Get_File_Name()       ).c_str());
+			SG_Printf("%s: %s\n", _TL("library name"), SG_File_Get_Name(pTool->Get_File_Name(), false).c_str());
+			SG_Printf("%s: %s\n", _TL("library     "), pTool->Get_Library().c_str());
+			SG_Printf("%s: %s\n", _TL("tool        "), pTool->Get_Name   ().c_str());
+			SG_Printf("%s: %s\n", _TL("identifier  "), pTool->Get_ID     ().c_str());
+			SG_Printf("%s: %s\n", _TL("author      "), pTool->Get_Author ().c_str());
 		#ifdef _OPENMP
-			SG_PRINTF(SG_T("%s:\t%d [%d]\n"), _TL("processors  "), SG_Get_Max_Num_Threads_Omp(), SG_Get_Max_Num_Procs_Omp());
+			SG_Printf("%s: %d [%d]\n", _TL("processors  "), SG_OMP_Get_Max_Num_Threads(), SG_OMP_Get_Max_Num_Procs());
 		#endif // _OPENMP
-			SG_PRINTF(SG_T("____________________________\n\n"));
+			SG_Printf("____________________________\n\n");
 		}
 	}
 }
@@ -678,26 +739,28 @@ void		Print_Help		(void)
 	CMD_Print(
 		"under GNU General Public License (GPL)\n"
 		"\n"
-		"____________________________\n"
+		"_____________________________________________________________________________\n"
+		"\n"
 		"Usage:\n"
 		"\n"
 		"saga_cmd [-h, --help]\n"
 		"saga_cmd [-v, --version]\n"
-		"saga_cmd [-b, --batch]\n"
-		"saga_cmd [-d, --docs]\n"
 #ifdef _OPENMP
-		"saga_cmd [-f, --flags][=qrsilpxo][-s, --story][=#][-c, --cores][=#] <LIBRARY> <MODULE> <OPTIONS>\n"
-		"saga_cmd [-f, --flags][=qrsilpxo][-s, --story][=#][-c, --cores][=#] <SCRIPT>\n"
+		"saga_cmd [-C, --config][=#][-s, --story][=#][-c, --cores][=#][-f, --flags][=#]\n"
+		"  <LIBRARY> <TOOL> <OPTIONS>\n"
+		"saga_cmd [-C, --config][=#][-s, --story][=#][-c, --cores][=#][-f, --flags][=#]\n"
+		"  <SCRIPT>\n"
 #else
-		"saga_cmd [-f, --flags][=qrsilpxo][-s, --story][=#] <LIBRARY> <MODULE> <module specific options...>\n"
-		"saga_cmd [-f, --flags][=qrsilpxo][-s, --story][=#] <SCRIPT>\n"
+		"saga_cmd [-C, --config][=#][-s, --story][=#][-f, --flags][=#]\n"
+		"  <LIBRARY> <TOOL> <OPTIONS>\n"
+		"saga_cmd [-C, --config][=#][-s, --story][=#][-f, --flags][=#]\n"
+		"  <SCRIPT>\n"
 #endif
 		"\n"
 		"[-h], [--help]   : help on usage\n"
-		"[-v], [--version]: print version information\n"
-		"[-b], [--batch]  : create a batch file example\n"
-		"[-d], [--docs]   : create tool documentation in current working directory\n"
+		"[-v], [--version]: version information\n"
 		"[-s], [--story]  : maximum data history depth (default is unlimited)\n"
+		"[-C], [--config] : configuration file (default is 'saga_cmd.ini')\n"
 #ifdef _OPENMP
 		"[-c], [--cores]  : number of physical processors to use for computation\n"
 #endif
@@ -710,60 +773,138 @@ void		Print_Help		(void)
 		"  p              : load projections dictionary\n"
 		"  x              : use XML markups for synopses and messages\n"
 		"  o              : load old style naming\n"
-		"\n"
 		"<LIBRARY>        : name of the library\n"
-		"<MODULE>         : either name or index of the tool\n"
+		"<TOOL>           : either name or index of the tool\n"
 		"<OPTIONS>        : tool specific options\n"
 		"<SCRIPT>         : saga cmd script file with one or more tool calls\n"
 		"\n"
-		"____________________________\n"
+		"saga_cmd --create-config[=file]\n"
+		"   creates a default configuration file. If no file name is specified\n"
+		"   it will use \'saga_cmd.ini\'.\n"
+		"\n"
+		"saga_cmd --create-batch[=file]\n"
+		"   creates a batch script file example.\n"
+		"\n"
+		"saga_cmd --create-docs[=directory]\n"
+		"   creates tool documentation in current working directory, if no other\n"
+		"   directory is given.\n"
+		"\n"
+		"_____________________________________________________________________________\n"
+		"\n"
 		"Example:\n"
 		"\n"
-		"  saga_cmd -f=s ta_lighting 0 -ELEVATION=c:\\dem.sgrd -SHADE=c:\\shade.sgrd\n"
+		"  saga_cmd ta_lighting 0 -ELEVATION=c:\\dem.sgrd -SHADE=c:\\shade.sgrd\n"
 		"\n"
-		"____________________________\n"
-		"Tool libraries in the \'modules\' subdirectory of the SAGA installation\n"
+		"_____________________________________________________________________________\n"
+		"\n"
+		"Tool libraries in the \'tools\' subdirectory of the SAGA installation\n"
 		"will be loaded automatically. Additional directories can be specified\n"
-		"by adding the environment variable \'SAGA_MLB\' and let it point to one\n"
+		"by adding the environment variable \'SAGA_TLB\' and let it point to one\n"
 		"or more directories, just the way it is done with the DOS \'PATH\' variable.\n"
 		"\n"
+		"A more convenient way to set various saga_cmd options is to edit a\n"
+		"configuration file. The default configuration file \'saga_cmd.ini\' will be\n"
+		"loaded automatically, if present. You can specify a different configuration "
+		"file with the \'-C\' or \'--config\' option. Use \'--create-config\' to\n"
+		"generate such a file with default options and edit it to fit your purposes.\n"
+		"\n"
 		"The SAGA command line interpreter is particularly useful for the processing\n"
 		"of complex work flows by defining a series of subsequent tool calls in a\n"
-		"script file. Calling saga_cmd with the option \'-b\' or \'--batch\' will\n"
+		"script file. Calling saga_cmd with the option \'--create-batch\' will\n"
 		"create an example of a DOS batch script file, which might be a good starting\n"
 		"point for the implementation of your own specific work flows.\n"
-		"____________________________\n"
+		"\n"
+		"_____________________________________________________________________________\n"
+		"\n"
+		"Please provide the following reference in your work if you are using SAGA:\n"
+		"\n"
+		"Conrad, O., Bechtel, B., Bock, M., Dietrich, H., Fischer, E., Gerlitz, L.,\n"
+		"  Wehberg, J., Wichmann, V., and Boehner, J. (2015):\n"
+		"  System for Automated Geoscientific Analyses (SAGA) v. 2.1.4.\n"
+		"  Geosci. Model Dev., 8, 1991-2007, doi:10.5194/gmd-8-1991-2015.\n"
+		"\n"
+		"  http://www.geosci-model-dev.net/8/1991/2015/gmd-8-1991-2015.html\n"
+		"\n"
+		"_____________________________________________________________________________\n"
 	);
 }
 
+//---------------------------------------------------------
+void		Print_Help		(const CSG_String &Library)
+{
+	Print_Logo();
+
+	CSG_Tool_Library	*pLibrary;
+
+	if( !Load_Libraries() || !(pLibrary = SG_Get_Tool_Library_Manager().Get_Library(Library, true)) )
+	{
+		Print_Libraries();
+	}
+	else
+	{
+		CMD_Print(pLibrary->Get_Description());
+		CMD_Print(pLibrary->Get_Summary(SG_SUMMARY_FMT_FLAT, false));
+	}
+}
+
+//---------------------------------------------------------
+void		Print_Help		(const CSG_String &Library, const CSG_String &Tool)
+{
+	Print_Logo();
+
+	if( !Load_Libraries() || !SG_Get_Tool_Library_Manager().Get_Library(Library, true) )
+	{
+		Print_Libraries();
+	}
+	else
+	{
+		CSG_Tool	*pTool	= SG_Get_Tool_Library_Manager().Get_Tool(Library, Tool);
+
+		if( !pTool )
+		{
+			Print_Tools(Library);
+		}
+		else
+		{
+			Print_Execution(pTool);
+
+			CMD_Print(pTool->Get_Summary(true, "", "", SG_SUMMARY_FMT_FLAT));
+		}
+	}
+}
+
 
 ///////////////////////////////////////////////////////////
 //														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-void		Create_Example	(void)
+void		Create_Batch	(const CSG_String &File)
 {
 	Print_Logo();
 
 	CSG_File	Stream;
 
-	CMD_Print(_TL("creating batch file example"));
+	CSG_String	_File(File.is_Empty() ? SG_File_Make_Path(SG_Dir_Get_Current(), SG_T("saga_cmd"), SG_T("bat")) : File);
 
-	//-----------------------------------------------------
-	if( !Stream.Open(SG_File_Make_Path(SG_Dir_Get_Current(), SG_T("saga_cmd_example"), SG_T("bat")), SG_FILE_W, false) )
+	CMD_Print(CSG_String::Format("%s [%s]", _TL("creating batch file example"), _File.c_str()));
+
+	if( !Stream.Open(_File, SG_FILE_W, false) )
 	{
 		CMD_Print(_TL("failed"));
 
 		return;
 	}
 
+	SG_File_Set_Extension(_File, SG_T("txt"));
+
+	//-----------------------------------------------------
 	Stream.Printf(
 		"@ECHO OFF\n"
 		"\n"
 		"SET FLAGS=-f=s\n"
 		"REM SET SAGA=.\n"
-		"REM SET SAGA_MLB=%%SAGA%%\\_private\n"
+		"REM SET SAGA_TLB=%%SAGA%%\\_private\n"
 		"REM PATH=PATH;%%SAGA%%\n"
 		"\n"
 		"IF EXIST dem.sgrd GOTO :GO\n"
@@ -792,18 +933,20 @@ void		Create_Example	(void)
 		"saga_cmd %%FLAGS%% ta_preprocessor      2 -DEM=dem.sgrd -DEM_PREPROC=dem.sgrd\n"
 		"saga_cmd %%FLAGS%% ta_lighting          0 -ELEVATION=dem.sgrd -SHADE=shade.sgrd -METHOD=0 -AZIMUTH=-45 -DECLINATION=45\n"
 		"saga_cmd %%FLAGS%% ta_morphometry       0 -ELEVATION=dem.sgrd -SLOPE=slope.sgrd -ASPECT=aspect.sgrd -C_CROS=hcurv.sgrd -C_LONG=vcurv.sgrd\n"
-		"saga_cmd %%FLAGS%% ta_hydrology         0 -ELEVATION=dem.sgrd -CAREA=carea.sgrd\n"
+		"saga_cmd %%FLAGS%% ta_hydrology         0 -ELEVATION=dem.sgrd -FLOW=flow.sgrd\n"
 		"\n"
 		"ECHO ____________________________\n"
 		"ECHO run saga cmd script\n"
 		"SET INPUT=dem.sgrd;slope.sgrd;hcurv.sgrd;vcurv.sgrd\n"
-		"saga_cmd %%FLAGS%% saga_cmd_example.txt\n"
+		"saga_cmd %%FLAGS%% %s\n"
 		"\n"
-		"PAUSE\n"
+		"PAUSE\n", _File.c_str()
 	);
 
 	//-----------------------------------------------------
-	if( !Stream.Open(SG_File_Make_Path(SG_Dir_Get_Current(), SG_T("saga_cmd_example"), SG_T("txt")), SG_FILE_W, false) )
+	CMD_Print(CSG_String::Format("%s [%s]", _TL("creating sub script example"), _File.c_str()));
+
+	if( !Stream.Open(_File, SG_FILE_W, false) )
 	{
 		CMD_Print(_TL("failed"));
 
@@ -846,17 +989,26 @@ void		Create_Example	(void)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-void		Create_Docs		(void)
+void		Create_Docs		(const CSG_String &Directory)
 {
 	Print_Logo();
 
+	CMD_Print(_TL("creating tool documentation files"));
+
+	CSG_String	_Directory(Directory.is_Empty() ? SG_Dir_Get_Current() : Directory);
+
+	if( !SG_Dir_Exists(_Directory) )
+	{
+		CMD_Print(_TL("directory does not exist"));
+
+		return;
+	}
+
 	if( Load_Libraries() )
 	{
-		CMD_Print(_TL("creating tool documentation files"));
-
 		CMD_Set_Show_Messages(false);
 
-		SG_Get_Module_Library_Manager().Get_Summary(SG_Dir_Get_Current());
+		SG_Get_Tool_Library_Manager().Get_Summary(_Directory);
 
 		CMD_Print(_TL("okay"));
 	}
