@@ -25,7 +25,8 @@
 // Geoscientific Analyses'. SAGA is free software; you   //
 // can redistribute it and/or modify it under the terms  //
 // of the GNU General Public License as published by the //
-// Free Software Foundation; version 2 of the License.   //
+// Free Software Foundation, either version 2 of the     //
+// License, or (at your option) any later version.       //
 //                                                       //
 // SAGA is distributed in the hope that it will be       //
 // useful, but WITHOUT ANY WARRANTY; without even the    //
@@ -34,10 +35,8 @@
 // License for more details.                             //
 //                                                       //
 // You should have received a copy of the GNU General    //
-// Public License along with this program; if not,       //
-// write to the Free Software Foundation, Inc.,          //
-// 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, //
-// USA.                                                  //
+// Public License along with this program; if not, see   //
+// <http://www.gnu.org/licenses/>.                       //
 //                                                       //
 //-------------------------------------------------------//
 //                                                       //
@@ -61,6 +60,7 @@
 
 //---------------------------------------------------------
 #include <wx/dcclient.h>
+#include <wx/clipbrd.h>
 
 #include "3d_view.h"
 
@@ -114,103 +114,98 @@ END_EVENT_TABLE()
 CSG_3DView_Panel::CSG_3DView_Panel(wxWindow *pParent, CSG_Grid *pDrape)
 	: wxPanel(pParent, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL|wxSUNKEN_BORDER|wxNO_FULL_REPAINT_ON_RESIZE)
 {
-	CSG_Parameter	*pNode, *pNode_1;
-
 	m_Parameters.Create(this, _TL("Properties"), _TL(""));
 
 	m_Parameters.Set_Callback_On_Parameter_Changed(_On_Parameter_Changed);
 
 	//-----------------------------------------------------
-	pNode	= m_Parameters.Add_Node(
-		NULL	, "NODE_GENERAL"	, _TL("General"),
+	m_Parameters.Add_Node("",
+		"NODE_GENERAL"	, _TL("General"),
 		_TL("")
 	);
 
-	pNode_1	= m_Parameters.Add_Choice(
-		pNode	, "CENTRAL"			, _TL("Projection"),
+	m_Parameters.Add_Choice("NODE_GENERAL",
+		"CENTRAL"		, _TL("Projection"),
 		_TL(""),
-		CSG_String::Format(SG_T("%s|%s|"),
+		CSG_String::Format("%s|%s|",
 			_TL("parallel"),
 			_TL("central")
 		), 1
 	);
 
-	m_Parameters.Add_Value(
-		pNode_1	, "CENTRAL_DIST"	, _TL("Central Projection Distance"),
+	m_Parameters.Add_Double("CENTRAL",
+		"CENTRAL_DIST"	, _TL("Central Projection Distance"),
 		_TL(""),
-		PARAMETER_TYPE_Double, m_Projector.Get_Central_Distance(), 1.0, true
+		m_Projector.Get_Central_Distance(), 1.0, true
 	);
 
-	m_Parameters.Add_Value(
-		pNode	, "BGCOLOR"			, _TL("Background Color"),
+	m_Parameters.Add_Color("NODE_GENERAL",
+		"BGCOLOR"		, _TL("Background Color"),
 		_TL(""),
-		PARAMETER_TYPE_Color, SG_COLOR_WHITE
+		SG_COLOR_WHITE
 	);
 
-	m_Parameters.Add_Value(
-		pNode	, "DRAW_BOX"		, _TL("Draw Box"),
+	m_Parameters.Add_Bool("NODE_GENERAL",
+		"DRAW_BOX"		, _TL("Draw Box"),
 		_TL(""),
-		PARAMETER_TYPE_Bool, true
+		true
 	);
 
-	pNode_1	= m_Parameters.Add_Value(
-		pNode	, "STEREO"			, _TL("Stereo Anaglyph"),
+	m_Parameters.Add_Bool("NODE_GENERAL",
+		"STEREO"		, _TL("Stereo Anaglyph"),
 		_TL(""),
-		PARAMETER_TYPE_Bool, false
+		false
 	);
 
-	m_Parameters.Add_Value(
-		pNode_1	, "STEREO_DIST"		, _TL("Stereo Eye Distance [Degree]"),
+	m_Parameters.Add_Double("STEREO",
+		"STEREO_DIST"	, _TL("Stereo Eye Distance [Degree]"),
 		_TL(""),
-		PARAMETER_TYPE_Double, 1.0, 0.0, true
+		1.0, 0.0, true
 	);
 
 	//-----------------------------------------------------
-	m_pDrape	= pDrape;
-
-	if( m_pDrape )
+	if( (m_pDrape = pDrape) != NULL )
 	{
-		pNode_1	= m_Parameters.Add_Value(
-			pNode	, "DO_DRAPE"	, _TL("Map Draping"),
+		m_Parameters.Add_Bool("NODE_GENERAL",
+			"DO_DRAPE"	, _TL("Map Draping"),
 			_TL(""),
-			PARAMETER_TYPE_Bool, true
+			true
 		);
 
-		m_Parameters.Add_Choice(
-			pNode_1	, "DRAPE_MODE"	, _TL("Map Draping Interpolation"),
+		m_Parameters.Add_Choice("DO_DRAPE",
+			"DRAPE_MODE", _TL("Map Drape Resampling"),
 			_TL(""),
-			CSG_String::Format(SG_T("%s|%s|%s|%s|%s|"),
-				_TL("None"),
-				_TL("Bilinear"),
-				_TL("Inverse Distance"),
-				_TL("Bicubic Spline"),
-				_TL("B-Spline")
+			CSG_String::Format("%s|%s|%s|%s|",
+				_TL("Nearest Neighbour"),
+				_TL("Bilinear Interpolation"),
+				_TL("Bicubic Spline Interpolation"),
+				_TL("B-Spline Interpolation")
 			), 0
 		);
 	}
 
 	//-----------------------------------------------------
-	pNode	= m_Parameters.Add_Node(
-		NULL	, "NODE_PLAYER"		, _TL("Sequencer"),
+	m_Parameters.Add_Node("",
+		"NODE_PLAYER"	, _TL("Sequencer"),
 		_TL("")
 	);
 
-	m_Parameters.Add_FilePath(
-		pNode	, "PLAY_FILE"		, _TL("Image File"),
+	m_Parameters.Add_FilePath("NODE_PLAYER",
+		"PLAY_FILE"		, _TL("Image File"),
 		_TL("file path, name and type used to save frames to image files"),
-		CSG_String::Format(SG_T("%s|%s|%s|%s|%s|%s|%s|%s|%s|%s"),
-			_TL("Windows or OS/2 Bitmap (*.bmp)")				, SG_T("*.bmp"),
-			_TL("JPEG - JFIF Compliant (*.jpg, *.jif, *.jpeg)")	, SG_T("*.jpg;*.jif;*.jpeg"),
-			_TL("Zsoft Paintbrush (*.pcx)")						, SG_T("*.pcx"),
-			_TL("Portable Network Graphics (*.png)")			, SG_T("*.png"),
-			_TL("Tagged Image File Format (*.tif, *.tiff)")		, SG_T("*.tif;*.tiff")
+		CSG_String::Format("%s|*.bmp|%s|*.jpg;*.jif;*.jpeg|%s|*.pcx|%s|*.png|%s|*.tif;*.tiff",
+			_TL("Windows or OS/2 Bitmap (*.bmp)"),
+			_TL("JPEG - JFIF Compliant (*.jpg, *.jif, *.jpeg)"),
+			_TL("Zsoft Paintbrush (*.pcx)"),
+			_TL("Portable Network Graphics (*.png)"),
+			_TL("Tagged Image File Format (*.tif, *.tiff)")
 		), NULL, true
 	);
 
-	m_pPlay	= m_Parameters.Add_FixedTable(
-		pNode	, "PLAY"			, _TL("Sequencer Positions"),
+	m_pPlay	= m_Parameters.Add_FixedTable("NODE_PLAYER",
+		"PLAY"			, _TL("Sequencer Positions"),
 		_TL("")
-		)->asTable();
+	)->asTable();
 
 	m_pPlay->Add_Field(_TL("Rotate X"        ), SG_DATATYPE_Double);
 	m_pPlay->Add_Field(_TL("Rotate Y"        ), SG_DATATYPE_Double);
@@ -230,6 +225,23 @@ CSG_3DView_Panel::CSG_3DView_Panel(wxWindow *pParent, CSG_Grid *pDrape)
 //														 //
 ///////////////////////////////////////////////////////////
 
+//---------------------------------------------------------
+bool CSG_3DView_Panel::Update_Parameters(bool bSave)
+{
+	if( bSave )
+	{
+		m_Parameters("CENTRAL"     )->Set_Value(m_Projector.is_Central());
+		m_Parameters("CENTRAL_DIST")->Set_Value(m_Projector.Get_Central_Distance());
+	}
+	else
+	{
+		m_Projector.do_Central          (m_Parameters("CENTRAL"     )->asBool());
+		m_Projector.Set_Central_Distance(m_Parameters("CENTRAL_DIST")->asDouble());
+	}
+
+	return( true );
+}
+
 //---------------------------------------------------------
 int CSG_3DView_Panel::_On_Parameter_Changed(CSG_Parameter *pParameter, int Flags)
 {
@@ -262,19 +274,19 @@ int CSG_3DView_Panel::On_Parameter_Changed(CSG_Parameters *pParameters, CSG_Para
 //---------------------------------------------------------
 int CSG_3DView_Panel::On_Parameters_Enable(CSG_Parameters *pParameters, CSG_Parameter *pParameter)
 {
-	if( !SG_STR_CMP(pParameter->Get_Identifier(), "CENTRAL") )
+	if( pParameter->Cmp_Identifier("CENTRAL") )
 	{
-		pParameters->Get_Parameter("CENTRAL_DIST")->Set_Enabled(pParameter->asInt() == 1);
+		pParameters->Set_Enabled("CENTRAL_DIST", pParameter->asInt() == 1);
 	}
 
-	if( !SG_STR_CMP(pParameter->Get_Identifier(), "DO_DRAPE") )
+	if( pParameter->Cmp_Identifier("DO_DRAPE") )
 	{
-		pParameters->Get_Parameter("DRAPE_MODE")->Set_Enabled(pParameter->asBool());
+		pParameters->Set_Enabled("DRAPE_MODE"  , pParameter->asBool());
 	}
 
-	if( !SG_STR_CMP(pParameter->Get_Identifier(), "STEREO") )
+	if( pParameter->Cmp_Identifier("STEREO") )
 	{
-		pParameters->Get_Parameter("STEREO_DIST")->Set_Enabled(pParameter->asBool());
+		pParameters->Set_Enabled("STEREO_DIST" , pParameter->asBool());
 	}
 
 	return( 1 );
@@ -301,42 +313,23 @@ void CSG_3DView_Panel::Update_Parent(void)
 //---------------------------------------------------------
 void CSG_3DView_Panel::On_Key_Down(wxKeyEvent &event)
 {
-	if( !event.ControlDown() )
+	//-----------------------------------------------------
+	if( event.ControlDown() )
 	{
 		switch( event.GetKeyCode() )
 		{
-		default:	event.Skip();	return;
+		case 'A': Play_Pos_Add();	return;
+		case 'D': Play_Pos_Del();	return;
+		case 'X': Play_Pos_Clr();	return;
 
-		case WXK_NUMPAD_ADD:
-		case WXK_ADD:		m_Projector.Set_xRotation(m_Projector.Get_xRotation() - 4.0 * M_DEG_TO_RAD);	break;
-		case WXK_NUMPAD_SUBTRACT:
-		case WXK_SUBTRACT:	m_Projector.Set_xRotation(m_Projector.Get_xRotation() + 4.0 * M_DEG_TO_RAD);	break;
+		case 'P': Play_Once   ();	return;
+		case 'L': Play_Loop   ();	return;
+		case 'S': Play_Save   ();	return;
 
-		case WXK_F3:		m_Projector.Set_yRotation(m_Projector.Get_yRotation() - 4.0 * M_DEG_TO_RAD);	break;
-		case WXK_F4:		m_Projector.Set_yRotation(m_Projector.Get_yRotation() + 4.0 * M_DEG_TO_RAD);	break;
-
-		case WXK_NUMPAD_MULTIPLY:
-		case WXK_MULTIPLY:	m_Projector.Set_zRotation(m_Projector.Get_zRotation() - 4.0 * M_DEG_TO_RAD);	break;
-		case WXK_NUMPAD_DIVIDE:
-		case WXK_DIVIDE:	m_Projector.Set_zRotation(m_Projector.Get_zRotation() + 4.0 * M_DEG_TO_RAD);	break;
-
-		case WXK_INSERT:	m_Projector.Set_xShift   (m_Projector.Get_xShift   () - 10.0);	break;
-		case WXK_DELETE:	m_Projector.Set_xShift   (m_Projector.Get_xShift   () + 10.0);	break;
-
-		case WXK_HOME:		m_Projector.Set_yShift   (m_Projector.Get_yShift   () - 10.0);	break;
-		case WXK_END:		m_Projector.Set_yShift   (m_Projector.Get_yShift   () + 10.0);	break;
-
-		case WXK_PAGEUP:	m_Projector.Set_zShift   (m_Projector.Get_zShift   () - 10.0);	break;
-		case WXK_PAGEDOWN:	m_Projector.Set_zShift   (m_Projector.Get_zShift   () + 10.0);	break;
-
-		case 'B':			m_Parameters("DRAW_BOX")->Set_Value(m_Parameters("DRAW_BOX")->asBool() == false);	break;
-		case 'S':			m_Parameters("STEREO"  )->Set_Value(m_Parameters("STEREO"  )->asBool() == false);	break;
-
-		case WXK_ESCAPE:	if( m_Play_State )	{ Play_Stop(); break; } else return;
+		case 'C': Save_toClipboard();	return;
 		}
 
-		Update_Parent();
-		Update_View();
+		event.Skip();
 	}
 
 	//-----------------------------------------------------
@@ -344,16 +337,61 @@ void CSG_3DView_Panel::On_Key_Down(wxKeyEvent &event)
 	{
 		switch( event.GetKeyCode() )
 		{
-		default:	event.Skip();	return;
+		default:
+			event.Skip();
+			return;
 
-		case 'A':	Play_Pos_Add ();	break;
-		case 'D':	Play_Pos_Del ();	break;
-		case 'X':	Play_Pos_Clr ();	break;
+		case WXK_ESCAPE:
+			if( !m_Play_State )
+			{
+				return;
+			}
 
-		case 'P':	Play_Once    ();	break;
-		case 'L':	Play_Loop    ();	break;
-		case 'S':	Play_Save    ();	break;
+			Play_Stop();
+			break;
+
+		case WXK_DOWN: case WXK_NUMPAD_DOWN: case WXK_ADD: case WXK_NUMPAD_ADD:
+			m_Projector.Inc_xRotation(-4.0 * M_DEG_TO_RAD);
+			break;
+
+		case WXK_UP: case WXK_NUMPAD_UP: case WXK_SUBTRACT: case WXK_NUMPAD_SUBTRACT:
+			m_Projector.Inc_xRotation( 4.0 * M_DEG_TO_RAD);
+			break;
+
+		case WXK_F3:
+			m_Projector.Inc_yRotation(-4.0 * M_DEG_TO_RAD);
+			break;
+
+		case WXK_F4:
+			m_Projector.Inc_yRotation( 4.0 * M_DEG_TO_RAD);
+			break;
+
+		case WXK_RIGHT: case WXK_NUMPAD_RIGHT: case WXK_MULTIPLY: case WXK_NUMPAD_MULTIPLY:
+			m_Projector.Inc_zRotation(-4.0 * M_DEG_TO_RAD);
+			break;
+
+		case WXK_LEFT: case WXK_NUMPAD_LEFT: case WXK_DIVIDE: case WXK_NUMPAD_DIVIDE:
+			m_Projector.Inc_zRotation( 4.0 * M_DEG_TO_RAD);
+			break;
+
+		case WXK_INSERT  : m_Projector.Inc_xShift(-10.0); break;
+		case WXK_DELETE  : m_Projector.Inc_xShift( 10.0); break;
+		case WXK_HOME    : m_Projector.Inc_yShift(-10.0); break;
+		case WXK_END     : m_Projector.Inc_yShift( 10.0); break;
+		case WXK_PAGEUP  : m_Projector.Inc_zShift(-10.0); break;
+		case WXK_PAGEDOWN: m_Projector.Inc_zShift( 10.0); break;
+
+		case 'B':
+			m_Parameters("DRAW_BOX")->Set_Value(m_Parameters("DRAW_BOX")->asBool() == false);
+			break;
+
+		case 'S':
+			m_Parameters("STEREO"  )->Set_Value(m_Parameters("STEREO"  )->asBool() == false);
+			break;
 		}
+
+		Update_Parent();
+		Update_View();
 	}
 }
 
@@ -416,8 +454,8 @@ void CSG_3DView_Panel::On_Mouse_RUp(wxMouseEvent &event)
 
 	if( m_Down_Screen.x != event.GetX() || m_Down_Screen.y != event.GetY() )
 	{
-		m_Projector.Set_xShift(m_Down_Value.x - GET_MOUSE_X_RELDIFF * 1000.0);
-		m_Projector.Set_yShift(m_Down_Value.y - GET_MOUSE_Y_RELDIFF * 1000.0);
+		m_Projector.Set_xShift(m_Down_Value.x - (m_Down_Screen.x - event.GetX()));
+		m_Projector.Set_yShift(m_Down_Value.y - (m_Down_Screen.y - event.GetY()));
 
 		Update_View();
 		Update_Parent();
@@ -446,7 +484,7 @@ void CSG_3DView_Panel::On_Mouse_MUp(wxMouseEvent &event)
 	if( m_Down_Screen.x != event.GetX() || m_Down_Screen.y != event.GetY() )
 	{
 		m_Projector.Set_yRotation(m_Down_Value.x + GET_MOUSE_X_RELDIFF * M_PI_180);
-		m_Projector.Set_zShift   (m_Down_Value.y + GET_MOUSE_Y_RELDIFF * 1000.0);
+		m_Projector.Set_zShift   (m_Down_Value.y + (m_Down_Screen.y - event.GetY()));
 
 		Update_View();
 		Update_Parent();
@@ -465,13 +503,13 @@ void CSG_3DView_Panel::On_Mouse_Motion(wxMouseEvent &event)
 		}
 		else if( event.RightIsDown() )
 		{
-			m_Projector.Set_xShift   (m_Down_Value.x - GET_MOUSE_X_RELDIFF * 1000.0);
-			m_Projector.Set_yShift   (m_Down_Value.y - GET_MOUSE_Y_RELDIFF * 1000.0);
+			m_Projector.Set_xShift   (m_Down_Value.x - (m_Down_Screen.x - event.GetX()));
+			m_Projector.Set_yShift   (m_Down_Value.y - (m_Down_Screen.y - event.GetY()));
 		}
 		else if( event.MiddleIsDown() )
 		{
 			m_Projector.Set_yRotation(m_Down_Value.x + GET_MOUSE_X_RELDIFF * M_PI_180);
-			m_Projector.Set_zShift   (m_Down_Value.y + GET_MOUSE_Y_RELDIFF * 1000.0);
+			m_Projector.Set_zShift   (m_Down_Value.y + (m_Down_Screen.y - event.GetY()));
 		}
 		else
 		{
@@ -488,7 +526,7 @@ void CSG_3DView_Panel::On_Mouse_Wheel(wxMouseEvent &event)
 {
 	if( event.GetWheelRotation() )
 	{
-		m_Projector.Set_zShift(m_Projector.Get_zShift()	+ 0.5 * event.GetWheelRotation());
+		m_Projector.Set_zShift(m_Projector.Get_zShift()	+ event.GetWheelRotation());
 
 		Update_View();
 	}
@@ -529,14 +567,18 @@ bool CSG_3DView_Panel::Update_View(bool bStatistics)
 	//-----------------------------------------------------
 	if( m_Play_State == SG_3DVIEW_PLAY_STOP )
 	{
-		m_bgColor		= m_Parameters("BGCOLOR"    )->asColor ();
-		m_bBox			= m_Parameters("DRAW_BOX"   )->asBool  ();
-		m_bStereo		= m_Parameters("STEREO"     )->asBool  ();
-		m_dStereo		= m_Parameters("STEREO_DIST")->asDouble();
-		m_Drape_Mode	= m_pDrape ? m_Parameters("DRAPE_MODE" )->asInt() : 0;
+		m_bgColor	= m_Parameters("BGCOLOR"    )->asColor ();
+		m_bBox		= m_Parameters("DRAW_BOX"   )->asBool  ();
+		m_bStereo	= m_Parameters("STEREO"     )->asBool  ();
+		m_dStereo	= m_Parameters("STEREO_DIST")->asDouble();
 
-		m_Projector.do_Central          (m_Parameters("CENTRAL")->asInt() == 1);
-		m_Projector.Set_Central_Distance(m_Parameters("CENTRAL_DIST")->asDouble());
+		switch( m_Parameters("DRAPE_MODE") ? m_Parameters("DRAPE_MODE")->asInt() : 0 )
+		{
+		default:	m_Drape_Mode	= GRID_RESAMPLING_NearestNeighbour;	break;
+		case  1:	m_Drape_Mode	= GRID_RESAMPLING_Bilinear;			break;
+		case  2:	m_Drape_Mode	= GRID_RESAMPLING_BicubicSpline;	break;
+		case  3:	m_Drape_Mode	= GRID_RESAMPLING_BSpline;			break;
+		}
 	}
 
 	//-----------------------------------------------------
@@ -585,6 +627,22 @@ bool CSG_3DView_Panel::Save_asImage(const CSG_String &FileName)
 	return( m_Image.SaveFile(FileName.c_str()) );
 }
 
+//---------------------------------------------------------
+bool CSG_3DView_Panel::Save_toClipboard(void)
+{
+	if( m_Image.IsOk() && m_Image.GetWidth() > 0 && m_Image.GetHeight() > 0 && wxTheClipboard->Open() )
+	{
+		wxBitmapDataObject	*pBMP	= new wxBitmapDataObject;
+		pBMP->SetBitmap(m_Image);
+		wxTheClipboard->SetData(pBMP);
+		wxTheClipboard->Close();
+
+		return( true );
+	}
+
+	return( false );
+}
+
 
 ///////////////////////////////////////////////////////////
 //														 //
