@@ -1,6 +1,3 @@
-/**********************************************************
- * Version $Id: dlg_about.cpp 2471 2015-04-19 09:32:08Z reklov_w $
- *********************************************************/
 
 ///////////////////////////////////////////////////////////
 //                                                       //
@@ -24,7 +21,8 @@
 // Geoscientific Analyses'. SAGA is free software; you   //
 // can redistribute it and/or modify it under the terms  //
 // of the GNU General Public License as published by the //
-// Free Software Foundation; version 2 of the License.   //
+// Free Software Foundation, either version 2 of the     //
+// License, or (at your option) any later version.       //
 //                                                       //
 // SAGA is distributed in the hope that it will be       //
 // useful, but WITHOUT ANY WARRANTY; without even the    //
@@ -33,10 +31,8 @@
 // License for more details.                             //
 //                                                       //
 // You should have received a copy of the GNU General    //
-// Public License along with this program; if not,       //
-// write to the Free Software Foundation, Inc.,          //
-// 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, //
-// USA.                                                  //
+// Public License along with this program; if not, see   //
+// <http://www.gnu.org/licenses/>.                       //
 //                                                       //
 //-------------------------------------------------------//
 //                                                       //
@@ -51,18 +47,10 @@
 //                                                       //
 ///////////////////////////////////////////////////////////
 
-//---------------------------------------------------------
-
-
-///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
-///////////////////////////////////////////////////////////
-
 //---------------------------------------------------------
 #include <saga_api/saga_api.h>
 #include <saga_api/clipper.hpp>
+#include <saga_api/nanoflann.hpp>
 
 #include <wx/notebook.h>
 
@@ -93,8 +81,6 @@ END_EVENT_TABLE()
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -106,29 +92,39 @@ CDLG_About::CDLG_About(void)
 	m_pControl	= new wxNotebook(this, -1, wxDefaultPosition, wxDefaultSize, wxNB_TOP|wxNB_MULTILINE);
 
 	//-----------------------------------------------------
-	m_pVersion	=
 	pText		= new wxTextCtrl(m_pControl, -1, wxT(""), wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE|wxTE_READONLY|wxTE_CENTRE|wxTE_RICH|wxTE_AUTO_URL|wxBORDER_SUNKEN);
 	pText->AppendText(_Get_Version());
 	m_pControl->AddPage(pText, _TL("Version"), false, -1);
 	pText->ShowPosition(0);
 
 	//-----------------------------------------------------
-	m_pControl->AddPage(new CDLG_About_Logo(m_pControl), _TL("Logo"), false, -1);
-
-	//-----------------------------------------------------
-	pText		= new wxTextCtrl(m_pControl, -1, wxT(""), wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE|wxTE_READONLY|wxTE_CENTRE);
-	pText->AppendText(_Get_Acknowledgements());
-	m_pControl->AddPage(pText, _TL("Acknowledgements"), false, -1);
+	pText		= new wxTextCtrl(m_pControl, -1, wxT(""), wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE|wxTE_READONLY|wxTE_CENTRE|wxTE_RICH|wxTE_AUTO_URL|wxBORDER_SUNKEN);
+	pText->AppendText(_Get_Citation());
+	m_pControl->AddPage(pText, _TL("Citation"), false, -1);
 	pText->ShowPosition(0);
 
 	//-----------------------------------------------------
-	pText		= new wxTextCtrl(m_pControl, -1, wxT(""), wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE|wxTE_READONLY|wxTE_CENTRE);
+	m_pControl->AddPage(new CDLG_About_Logo(m_pControl), _TL("Logo"), false, -1);
+
+	//-----------------------------------------------------
+//	pText		= new wxTextCtrl(m_pControl, -1, wxT(""), wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE|wxTE_READONLY|wxTE_CENTRE|wxTE_RICH|wxTE_AUTO_URL|wxBORDER_SUNKEN);
+//	pText->AppendText(_Get_Acknowledgements());
+//	m_pControl->AddPage(pText, _TL("Acknowledgements"), false, -1);
+//	pText->ShowPosition(0);
+
+//	acknowledgements tab has not been updated since ages.
+//	we remove it now, because we think it is not possible
+//	to pay adequate tribute to all contributors and supporters
+//	through this list. thanks a lot...
+
+	//-----------------------------------------------------
+	pText		= new wxTextCtrl(m_pControl, -1, wxT(""), wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE|wxTE_READONLY|wxTE_CENTRE|wxTE_RICH|wxTE_AUTO_URL|wxBORDER_SUNKEN);
 	pText->AppendText(_Get_GPL());
 	m_pControl->AddPage(pText, SG_T("GPL"), false, -1);
 	pText->ShowPosition(0);
 
 	//-----------------------------------------------------
-	pText		= new wxTextCtrl(m_pControl, -1, wxT(""), wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE|wxTE_READONLY|wxTE_CENTRE);
+	pText		= new wxTextCtrl(m_pControl, -1, wxT(""), wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE|wxTE_READONLY|wxTE_CENTRE|wxTE_RICH|wxTE_AUTO_URL|wxBORDER_SUNKEN);
 	pText->AppendText(_Get_LGPL());
 	m_pControl->AddPage(pText, SG_T("LGPL"), false, -1);
 	pText->ShowPosition(0);
@@ -137,15 +133,9 @@ CDLG_About::CDLG_About(void)
 	Set_Positions();
 }
 
-//---------------------------------------------------------
-CDLG_About::~CDLG_About(void)
-{}
-
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -157,24 +147,22 @@ void CDLG_About::Set_Position(wxRect r)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
 void CDLG_About::On_URL_Version(wxTextUrlEvent &event)
 {
-	if( !event.GetMouseEvent().Moving() )
+	if( event.GetMouseEvent().LeftDown() && !event.GetMouseEvent().Moving() )
 	{
-		Open_WebBrowser(m_pVersion->GetValue().Mid(event.GetURLStart(), event.GetURLEnd() - event.GetURLStart()));
+		wxString	Text	= ((wxTextCtrl *)event.GetEventObject())->GetValue();
+
+		Open_WebBrowser(Text.Mid(event.GetURLStart(), event.GetURLEnd() - event.GetURLStart()));
 	}
 }
 
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -205,8 +193,8 @@ wxString CDLG_About::_Get_Version(void)
 	s	+= "SAGA GUI\n";
 	s	+= "SAGA Graphical User Interface\n";
 	s	+= "\n";
-	s	+= "Copyrights (c) 2005-2015 by Olaf Conrad\n";
-	s	+= "Portions (c) 2008-2015 by Volker Wichmann\n";
+	s	+= "Copyrights (c) 2005-2019 by Olaf Conrad\n";
+	s	+= "Portions (c) 2008-2019 by Volker Wichmann\n";
 	s	+= "\n";
 	s	+= "GNU General Public License (GPL)\n";
 
@@ -215,8 +203,8 @@ wxString CDLG_About::_Get_Version(void)
 	s	+= "SAGA API\n";
 	s	+= "SAGA Application Programming Interface\n";
 	s	+= "\n";
-	s	+= "Copyrights (c) 2002-2015 by Olaf Conrad\n";
-	s	+= "Portions (c) 2008-2015 by Volker Wichmann\n";
+	s	+= "Copyrights (c) 2002-2019 by Olaf Conrad\n";
+	s	+= "Portions (c) 2008-2019 by Volker Wichmann\n";
 	s	+= "Portions (c) 2005-2006 by Victor Olaya\n";
 	s	+= "Portions (c) 2002 by Andre Ringeler\n";	// " (mat_formula.cpp)\n";
 	s	+= "\n";
@@ -228,9 +216,17 @@ wxString CDLG_About::_Get_Version(void)
 	s	+= wxVERSION_STRING	+ wxString("\n");
 	s	+= "http://www.wxwidgets.org/\n";
 	s	+= "\n";
-	s	+= "SAGA API includes the polygon clipping and offsetting library\n";
+	s	+= "SAGA API includes:\n";
+	s	+= "The polygon clipping and offsetting library\n";
 	s	+= "Clipper " + wxString(CLIPPER_VERSION) + "\n";
 	s	+= "http://sourceforge.net/projects/polyclipping/\n";
+	s	+= "\n";
+	s	+= "The Nearest Neighbor (NN) search with KD-trees library\n";
+	s	+= "nanoflann " + wxString::Format("%d.%d.%d",
+			(NANOFLANN_VERSION&0xf00)/0x100,
+			(NANOFLANN_VERSION&0x0f0)/0x010,
+			(NANOFLANN_VERSION&0x00f)/0x001) + "\n";
+	s	+= "http://github.com/jlblancoc/nanoflann/\n";
 
 	s	+= "_______________________\n\n";
 
@@ -241,20 +237,30 @@ wxString CDLG_About::_Get_Version(void)
 			"DJGPP"
 		#elif	defined(__DIGITALMARS__)
 			"Digital Mars"
+		#elif	defined(__MWERKS__)
+			"CodeWarrior MetroWerks compiler"
+		#elif	defined(__SUNCC__)
+			"Sun CC"
+		#elif	defined(__SUNPRO_CC)
+			"Oracle Solaris Studio"
+		#elif	defined(__SYMANTECC__)
+			"Symantec C++"
+		#elif	defined(__VISAGECPP__)
+			"IBM Visual Age (OS/2)"
+		#elif	defined(__IBMCPP__)
+			"IBM XL C/C++"
+		#elif	defined(__INTEL_COMPILER) || defined(__ICC)
+			"Intel ICC/ICPC"
+		#elif	defined(__PGI)
+			"Portland PGCC/PGCPP"
+		#elif	defined(__clang__)
+			"Clang/LLVM"
 		#elif	defined(__MINGW32__)
 			"MinGW"
 		#elif	defined(__GNUWIN32__)
 			"Gnu-Win32 compiler"
 		#elif	defined(__GNUG__)
 			"Gnu C++"
-		#elif	defined(__MWERKS__)
-			"CodeWarrior MetroWerks compiler"
-		#elif	defined(__SUNCC__)
-			"Sun CC"
-		#elif	defined(__SYMANTECC__)
-			"Symantec C++"
-		#elif	defined(__VISAGECPP__)
-			"IBM Visual Age (OS/2)"
 		#elif	defined(__VISUALC__)	// The value of this macro corresponds to the compiler version: 1020 for 4.2 (the first supported version), 1100 for 5.0, 1200 for 6.0 and so on
 			#if   __VISUALC__ == 1020
 				"Microsoft Visual C++ 4.2"
@@ -274,6 +280,10 @@ wxString CDLG_About::_Get_Version(void)
 				"Microsoft Visual C++ 2012"
 			#elif __VISUALC__ >= 1800 && __VISUALC__ < 1900
 				"Microsoft Visual C++ 2013"
+			#elif __VISUALC__ >= 1900 && __VISUALC__ < 1910
+				"Microsoft Visual C++ 2015"
+			#elif __VISUALC__ >= 1910 && __VISUALC__ < 2000
+				"Microsoft Visual C++ 2017"
 			#else
 				"Microsoft Visual C++"
 			#endif
@@ -291,6 +301,31 @@ wxString CDLG_About::_Get_Version(void)
 	return( s );
 }
 
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+wxString CDLG_About::_Get_Citation(void)
+{
+	return(
+		"\n"
+		"Please provide the following reference in your work if you are using SAGA:\n"
+		"\n_______________________\n\n"
+		"Conrad, O., Bechtel, B., Bock, M., Dietrich, H., Fischer, E., Gerlitz, L., Wehberg, J., Wichmann, V., and Boehner, J. (2015):\n"
+		"System for Automated Geoscientific Analyses (SAGA) v. 2.1.4.\n"
+		"Geosci. Model Dev., 8, 1991-2007, doi:10.5194/gmd-8-1991-2015.\n"
+		"\n_______________________\n\n"
+		"http://www.geosci-model-dev.net/8/1991/2015/gmd-8-1991-2015.html"
+	);
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
 //---------------------------------------------------------
 wxString CDLG_About::_Get_Acknowledgements(void)
 {
