@@ -1,6 +1,3 @@
-/**********************************************************
- * Version $Id: project.cpp 2514 2015-06-19 16:03:47Z oconrad $
- *********************************************************/
 
 ///////////////////////////////////////////////////////////
 //                                                       //
@@ -24,7 +21,8 @@
 // Geoscientific Analyses'. SAGA is free software; you   //
 // can redistribute it and/or modify it under the terms  //
 // of the GNU General Public License as published by the //
-// Free Software Foundation; version 2 of the License.   //
+// Free Software Foundation, either version 2 of the     //
+// License, or (at your option) any later version.       //
 //                                                       //
 // SAGA is distributed in the hope that it will be       //
 // useful, but WITHOUT ANY WARRANTY; without even the    //
@@ -33,10 +31,8 @@
 // License for more details.                             //
 //                                                       //
 // You should have received a copy of the GNU General    //
-// Public License along with this program; if not,       //
-// write to the Free Software Foundation, Inc.,          //
-// 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, //
-// USA.                                                  //
+// Public License along with this program; if not, see   //
+// <http://www.gnu.org/licenses/>.                       //
 //                                                       //
 //-------------------------------------------------------//
 //                                                       //
@@ -51,14 +47,6 @@
 //                                                       //
 ///////////////////////////////////////////////////////////
 
-//---------------------------------------------------------
-
-///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
-///////////////////////////////////////////////////////////
-
 //---------------------------------------------------------
 #include <wx/string.h>
 #include <wx/filename.h>
@@ -95,8 +83,11 @@
 #include "wksp_map.h"
 #include "wksp_map_layer.h"
 #include "wksp_map_graticule.h"
+#include "wksp_map_basemap.h"
 #include "wksp_layer.h"
 
+#include "data_source_pgsql.h"
+
 #include "project.h"
 
 
@@ -109,18 +100,18 @@
 //---------------------------------------------------------
 CWKSP_Project::CWKSP_Project(void)
 {
+	// nop
 }
 
 //---------------------------------------------------------
 CWKSP_Project::~CWKSP_Project(void)
 {
+	// nop
 }
 
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -146,7 +137,7 @@ bool CWKSP_Project::_Set_Project_Name(void)
 	{
 		wxFileName	fn(m_File_Name);
 
-		if( fn.GetFullName().CmpNoCase(wxT("saga_gui.cfg")) )
+		if( fn.GetFullName().CmpNoCase("saga_gui.cfg") )
 		{
 			g_pSAGA_Frame->Set_Project_Name(m_File_Name);
 
@@ -162,8 +153,6 @@ bool CWKSP_Project::_Set_Project_Name(void)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -208,7 +197,113 @@ bool CWKSP_Project::Save(const wxString &FileName, bool bSaveModified)
 
 ///////////////////////////////////////////////////////////
 //														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CWKSP_Project::Copy(void)
+{
+	wxString	FileName;
+
+	if( !DLG_Save(FileName, ID_DLG_PROJECT_SAVE) )
+	{
+		return( false );
+	}
+
+	wxFileName	Directory(FileName);
+
+	Directory.AppendDir(Directory.GetFullName());
+
+	if( !Directory.DirExists() )
+	{
+		Directory.Mkdir(wxS_DIR_DEFAULT, wxPATH_MKDIR_FULL);
+	}
+
+	size_t	i, j;
+
+	for(j=0; j<SG_Get_Data_Manager().Grid_System_Count(); j++)
+	{
+		for(i=0; i<SG_Get_Data_Manager().Get_Grid_System(j)->Count(); i++)
+		{
+			_Copy_To_File(g_pData->Get(SG_Get_Data_Manager().Get_Grid_System(j)->Get(i)), Directory.GetFullPath());
+		}
+	}
+
+	for(i=0; i<SG_Get_Data_Manager().Get_Point_Cloud()->Count(); i++)
+	{
+		_Copy_To_File(g_pData->Get(SG_Get_Data_Manager().Get_Point_Cloud()->Get(i)), Directory.GetFullPath());
+	}
+
+	for(i=0; i<SG_Get_Data_Manager().Get_Shapes()->Count(); i++)
+	{
+		_Copy_To_File(g_pData->Get(SG_Get_Data_Manager().Get_Shapes()->Get(i)), Directory.GetFullPath());
+	}
+
+	for(i=0; i<SG_Get_Data_Manager().Get_Table()->Count(); i++)
+	{
+		_Copy_To_File(g_pData->Get(SG_Get_Data_Manager().Get_Table()->Get(i)), Directory.GetFullPath());
+	}
+
+	for(i=0; i<SG_Get_Data_Manager().asTIN()->Count(); i++)
+	{
+		_Copy_To_File(g_pData->Get(SG_Get_Data_Manager().asTIN()->Get(i)), Directory.GetFullPath());
+	}
+
+	return( _Save(Directory.GetFullPath(), false, true) );
+}
+
+//---------------------------------------------------------
+bool CWKSP_Project::_Copy_To_File(CWKSP_Data_Item *pItem, const wxString &Directory)
+{
+	wxFileName	Path(Directory);
+
+	switch( pItem->Get_Type() )	// default extensions
+	{
+	case WKSP_ITEM_Table     : Path.SetExt("txt"     ); break;
+	case WKSP_ITEM_Shapes    : Path.SetExt(SG_Shapes_Get_File_Extension_Default().c_str()); break;
+	case WKSP_ITEM_TIN       : Path.SetExt(SG_Shapes_Get_File_Extension_Default().c_str()); break;
+	case WKSP_ITEM_PointCloud: Path.SetExt("sg-pts-z"); break;
+	case WKSP_ITEM_Grid      : Path.SetExt(SG_Grid_Get_File_Extension_Default().c_str()); break;
+	case WKSP_ITEM_Grids     : Path.SetExt("sg-gds-z"); break;
+	default:	return( false );
+	}
+
+	//-----------------------------------------------------
+	wxString	Name(pItem->Get_Object()->Get_Name());
+
+	Name.Replace(".", "-");
+	Name.Replace(":", "-");
+
+	Path.SetName(Name);
+
+	for(int i=1; Path.FileExists(); i++)
+	{
+		Path.SetName(Name + wxString::Format("_%d", i));
+	}
+
+	Name	= Path.GetFullPath();
+
+	return( pItem->Get_Object()->Save(&Name) );
+}
+
+
+///////////////////////////////////////////////////////////
 //														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CWKSP_Project::CopyToDB(void)
+{
+	return( true );
+}
+
+//---------------------------------------------------------
+bool CWKSP_Project::_Copy_To_Database(CWKSP_Data_Item *pItem, const wxString &Connection)
+{
+	return( true );
+}
+
+
+///////////////////////////////////////////////////////////
 //														 //
 ///////////////////////////////////////////////////////////
 
@@ -231,7 +326,7 @@ bool CWKSP_Project::_Load(const wxString &FileName, bool bAdd, bool bUpdateMenu)
 
 	//-------------------------------------------------
 	MSG_General_Add_Line();
-	MSG_General_Add(wxString::Format(wxT("%s: %s"), _TL("Load project"), FileName), true, true);
+	MSG_General_Add(wxString::Format("%s: %s", _TL("Load project"), FileName), true, true);
 
 	//-------------------------------------------------
 	bool			bSuccess	= false;
@@ -244,23 +339,27 @@ bool CWKSP_Project::_Load(const wxString &FileName, bool bAdd, bool bUpdateMenu)
 	}
 	else if( !wxFileExists(FileName) )
 	{
-		MSG_Error_Add(_TL("file does not exist.")            , true, true, SG_UI_MSG_STYLE_FAILURE);
+		MSG_Error_Add(_TL("file does not exist."            ), true, true, SG_UI_MSG_STYLE_FAILURE);
 	}
 	else if( !Project.Load(&FileName) )
 	{
-		MSG_Error_Add(_TL("could not read project file.")    , true, true, SG_UI_MSG_STYLE_FAILURE);
+		MSG_Error_Add(_TL("could not read project file."    ), true, true, SG_UI_MSG_STYLE_FAILURE);
 	}
-	else if( Project.Get_Name().Cmp(SG_T("SAGA_PROJECT")) )
+	else if( Project.Get_Name().CmpNoCase("SAGA_PROJECT") )
 	{
-		MSG_Error_Add(_TL("invalid project file.")           , true, true, SG_UI_MSG_STYLE_FAILURE);
+		MSG_Error_Add(_TL("invalid project file."           ), true, true, SG_UI_MSG_STYLE_FAILURE);
 	}
-	else if( (pNode = Project.Get_Child(SG_T("DATA"))) == NULL || pNode->Get_Children_Count() <= 0 )
+	else if( (pNode = Project.Get_Child("DATA")) == NULL || pNode->Get_Children_Count() <= 0 )
 	{
 		MSG_Error_Add(_TL("no data entries in project file."), true, true, SG_UI_MSG_STYLE_FAILURE);
 	}
+	else if( !_Load_DBConnections(*pNode) )
+	{
+		MSG_Error_Add(_TL("could not connect to database."  ), true, true, SG_UI_MSG_STYLE_FAILURE);
+	}
 	else
 	{
-		int			i;
+		int		i;
 
 		CSG_String	Version(Project.Get_Property("VERSION"));
 
@@ -271,12 +370,12 @@ bool CWKSP_Project::_Load(const wxString &FileName, bool bAdd, bool bUpdateMenu)
 
 		for(i=0; i<pNode->Get_Children_Count(); i++)
 		{
-			_Load_Data(*pNode->Get_Child(i), SG_File_Get_Path(FileName).w_str(), true , Version);
+			_Load_Data(*pNode->Get_Child(i), SG_File_Get_Path(&FileName).w_str(), true , Version);
 		}
 
 		for(i=0; i<pNode->Get_Children_Count(); i++)
 		{
-			_Load_Data(*pNode->Get_Child(i), SG_File_Get_Path(FileName).w_str(), false, Version);
+			_Load_Data(*pNode->Get_Child(i), SG_File_Get_Path(&FileName).w_str(), false, Version);
 		}
 
 		g_pData->Get_Menu_Files()->Set_Update(true);
@@ -284,11 +383,11 @@ bool CWKSP_Project::_Load(const wxString &FileName, bool bAdd, bool bUpdateMenu)
 		//-------------------------------------------------
 		g_pSAGA_Frame->Freeze();
 
-		if( (pNode = Project.Get_Child(SG_T("MAPS"))) != NULL && pNode->Get_Children_Count() > 0 )
+		if( (pNode = Project.Get_Child("MAPS")) != NULL && pNode->Get_Children_Count() > 0 )
 		{
 			for(int j=0; j<pNode->Get_Children_Count(); j++)
 			{
-				_Load_Map(*pNode->Get_Child(j), SG_File_Get_Path(FileName).w_str());
+				_Load_Map(*pNode->Get_Child(j), SG_File_Get_Path(&FileName).w_str());
 			}
 		}
 
@@ -306,7 +405,7 @@ bool CWKSP_Project::_Load(const wxString &FileName, bool bAdd, bool bUpdateMenu)
 	{
 		if( bUpdateMenu )
 		{
-			g_pData->Get_Menu_Files()->Recent_Add(DATAOBJECT_TYPE_Undefined, FileName);
+			g_pData->Get_Menu_Files()->Recent_Add(SG_DATAOBJECT_TYPE_Undefined, FileName);
 		}
 
 		MSG_General_Add(_TL("Project has been successfully loaded."), true, true, SG_UI_MSG_STYLE_SUCCESS);
@@ -320,7 +419,7 @@ bool CWKSP_Project::_Load(const wxString &FileName, bool bAdd, bool bUpdateMenu)
 
 	if( bUpdateMenu )
 	{
-		g_pData->Get_Menu_Files()->Recent_Del(DATAOBJECT_TYPE_Undefined, FileName);
+		g_pData->Get_Menu_Files()->Recent_Del(SG_DATAOBJECT_TYPE_Undefined, FileName);
 	}
 
 	MSG_General_Add(_TL("Could not load project."), true, true, SG_UI_MSG_STYLE_FAILURE);
@@ -351,7 +450,7 @@ bool CWKSP_Project::_Save(const wxString &FileName, bool bSaveModified, bool bUp
 	}
 
 	//-----------------------------------------------------
-	ProjectDir	= SG_File_Get_Path(FileName).w_str();
+	ProjectDir	= SG_File_Get_Path(&FileName).w_str();
 
 	Project.Set_Name    ("SAGA_PROJECT");
 	Project.Add_Property("VERSION", SAGA_VERSION);
@@ -413,8 +512,8 @@ bool CWKSP_Project::_Save(const wxString &FileName, bool bSaveModified, bool bUp
 			for(i=0; i<pGrids->Get_System(j)->Get_Count(); i++)
 			{
 				_Save_Data(*pNode, ProjectDir,
-					pGrids->Get_System(j)->Get_Data(i)->Get_Object(),
-					pGrids->Get_System(j)->Get_Data(i)->Get_Parameters()
+					((CWKSP_Data_Item *)pGrids->Get_System(j)->Get_Item(i))->Get_Object(),
+					((CWKSP_Data_Item *)pGrids->Get_System(j)->Get_Item(i))->Get_Parameters()
 				);
 			}
 		}
@@ -437,7 +536,7 @@ bool CWKSP_Project::_Save(const wxString &FileName, bool bSaveModified, bool bUp
 		m_File_Name	= FileName;
 
 		if( bUpdateMenu )
-			g_pData->Get_Menu_Files()->Recent_Add(DATAOBJECT_TYPE_Undefined, FileName);
+			g_pData->Get_Menu_Files()->Recent_Add(SG_DATAOBJECT_TYPE_Undefined, FileName);
 
 		MSG_General_Add(_TL("Project has been saved."), true, true, SG_UI_MSG_STYLE_SUCCESS);
 
@@ -451,7 +550,7 @@ bool CWKSP_Project::_Save(const wxString &FileName, bool bSaveModified, bool bUp
 	m_File_Name.Clear();
 
 	if( bUpdateMenu )
-		g_pData->Get_Menu_Files()->Recent_Del(DATAOBJECT_TYPE_Undefined, FileName);
+		g_pData->Get_Menu_Files()->Recent_Del(SG_DATAOBJECT_TYPE_Undefined, FileName);
 
 	MSG_General_Add(_TL("Could not save project."), true, true, SG_UI_MSG_STYLE_FAILURE);
 
@@ -463,47 +562,125 @@ bool CWKSP_Project::_Save(const wxString &FileName, bool bSaveModified, bool bUp
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CWKSP_Project::_Load_DBConnections(CSG_MetaData &Data)
+{
+	int			i;
+	CSG_Strings	Connections;
+
+	for(i=0; i<Data.Get_Children_Count(); i++)
+	{
+		CSG_String	Connection(Data[i].Get_Child("FILE") ? Data[i].Get_Child("FILE")->Get_Content() : "");
+
+		if( !Connection.BeforeFirst(':').CmpNoCase("PGSQL") )
+		{
+			Connection	= Connection.AfterFirst(':');	CSG_String	Host  (Connection.BeforeFirst(':'));
+			Connection	= Connection.AfterFirst(':');	CSG_String	Port  (Connection.BeforeFirst(':'));
+			Connection	= Connection.AfterFirst(':');	CSG_String	DBName(Connection.BeforeFirst(':'));
+
+			Connection	= Host + ":" + Port + ":" + DBName;
+
+			bool	bAdd	= true;
+
+			for(int j=0; j<Connections.Get_Count() && bAdd; j++)
+			{
+				if( !Connection.Cmp(Connections[j]) )
+				{
+					bAdd	= false;
+				}
+			}
+
+			if( bAdd )
+			{
+				Connections	+= Connection;
+			}
+		}
+	}
+
+	for(i=0; i<Connections.Get_Count(); i++)
+	{
+		CSG_String	Host  (Connections[i].BeforeFirst(':'));
+		CSG_String	Port  (Connections[i].AfterFirst (':').BeforeLast(':'));
+		CSG_String	DBName(Connections[i].AfterLast  (':'));
+
+		if( !PGSQL_Connect(Host, Port, DBName) )
+		{
+			return( false );
+		}
+	}
+
+	return( true );
+}
+
+
+///////////////////////////////////////////////////////////
 //														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
 bool CWKSP_Project::_Load_Data(CSG_MetaData &Entry, const wxString &ProjectDir, bool bLoad, const CSG_String &Version)
 {
-	if( Entry.Get_Name().Cmp(SG_T("DATASET")) )
+	if( !Entry.Cmp_Name("DATASET") || !Entry("FILE") || Entry["FILE"].Get_Content().is_Empty() )
 	{
 		return( false );
 	}
 
 	TSG_Data_Object_Type	Type	=
-		Entry.Cmp_Property("type", "GRID"  ) ? DATAOBJECT_TYPE_Grid
-	:	Entry.Cmp_Property("type", "TABLE" ) ? DATAOBJECT_TYPE_Table
-	:	Entry.Cmp_Property("type", "SHAPES") ? DATAOBJECT_TYPE_Shapes
-	:	Entry.Cmp_Property("type", "TIN"   ) ? DATAOBJECT_TYPE_TIN
-	:	Entry.Cmp_Property("type", "POINTS") ? DATAOBJECT_TYPE_PointCloud
-	:	DATAOBJECT_TYPE_Undefined;
+		Entry.Cmp_Property("type", "GRID"  ) ? SG_DATAOBJECT_TYPE_Grid
+	:	Entry.Cmp_Property("type", "GRIDS" ) ? SG_DATAOBJECT_TYPE_Grids
+	:	Entry.Cmp_Property("type", "TABLE" ) ? SG_DATAOBJECT_TYPE_Table
+	:	Entry.Cmp_Property("type", "SHAPES") ? SG_DATAOBJECT_TYPE_Shapes
+	:	Entry.Cmp_Property("type", "TIN"   ) ? SG_DATAOBJECT_TYPE_TIN
+	:	Entry.Cmp_Property("type", "POINTS") ? SG_DATAOBJECT_TYPE_PointCloud
+	:	SG_DATAOBJECT_TYPE_Undefined;
 
-	if( Type == DATAOBJECT_TYPE_Undefined )
+	if( Type == SG_DATAOBJECT_TYPE_Undefined )
 	{
 		return( false );
 	}
 
-	if( !Entry.Get_Child(SG_T("FILE")) )
+	//-----------------------------------------------------
+	wxString	File	= Entry("FILE")->Get_Content().c_str();
+
+	if( File.Find("PGSQL") != 0 && wxFileExists(Get_FilePath_Absolute(ProjectDir, File)) )
 	{
-		return( false );
+		File	= Get_FilePath_Absolute(ProjectDir, File);
 	}
 
-	wxString	File(Get_FilePath_Absolute(ProjectDir, Entry.Get_Child(SG_T("FILE"))->Get_Content().c_str()));
+	//-----------------------------------------------------
+	CWKSP_Base_Item	*pItem	= NULL;
 
-	if( !wxFileExists(File) )
+	if( bLoad )
 	{
-		return( false );
+		if( Type == SG_DATAOBJECT_TYPE_Grid && Entry("PARAMETERS"))
+		{
+			for(int i=0; i<Entry["PARAMETERS"].Get_Children_Count() && !pItem; i++)
+			{
+				if( Entry["PARAMETERS"][i].Cmp_Property("id", "FILE_CACHE") )
+				{
+					bool	bCached	= Entry["PARAMETERS"][i].Cmp_Content("TRUE", true);
+
+					pItem	= g_pData->Add(SG_Create_Grid(&File, SG_DATATYPE_Undefined, bCached));
+				}
+			}
+		}
 	}
 
-	CWKSP_Base_Item	*pItem	= bLoad ? g_pData->Open(File, Type) : _Get_byFileName(File);
+	if( !pItem )
+	{
+		pItem	= bLoad ? g_pData->Open(File, Type) : _Get_byFileName(File);
+	}
 
+	//-----------------------------------------------------
 	if( !pItem || !pItem->Get_Parameters() || !Entry.Get_Child("PARAMETERS") )
 	{
+		if( bLoad )
+		{
+			MSG_Error_Add(wxString::Format("%s [%s]", _TL("failed to load data"), File.c_str()));
+		}
+
 		return( false );
 	}
 
@@ -512,13 +689,13 @@ bool CWKSP_Project::_Load_Data(CSG_MetaData &Entry, const wxString &ProjectDir,
 
 	for(int i=0; i<pEntry->Get_Children_Count(); i++)
 	{
-		if( !pEntry->Get_Child(i)->Get_Name().CmpNoCase("DATA") && !pEntry->Get_Child(i)->Get_Content().is_Empty() )
+		if( !pEntry->Get_Child(i)->Get_Name().CmpNoCase("DATA") && !pEntry->Get_Child(i)->Get_Content().is_Empty() && pEntry->Get_Child(i)->Get_Content().BeforeFirst(':').CmpNoCase("PGSQL") )
 		{
-			wxString	s(Get_FilePath_Absolute(ProjectDir, pEntry->Get_Child(i)->Get_Content().w_str()));
+			wxString	File(Get_FilePath_Absolute(ProjectDir, pEntry->Get_Child(i)->Get_Content().w_str()));
 
-			pEntry->Get_Child(i)->Set_Content(&s);
+			pEntry->Get_Child(i)->Set_Content(&File);
 
-		// if( SG_Compare_SAGA_Version(Version) < 0 )
+		//	if( SG_Compare_SAGA_Version(Version) < 0 )
 			{
 				if( pEntry->Get_Child(i)->Cmp_Property("id", "OVERLAY_1") )
 				{
@@ -535,6 +712,25 @@ bool CWKSP_Project::_Load_Data(CSG_MetaData &Entry, const wxString &ProjectDir,
 
 	pItem->Get_Parameters()->Serialize(*Entry.Get_Child("PARAMETERS"), false);
 
+	//-----------------------------------------------------
+	if( SG_Compare_Version(Version, "7.0.0") < 0 )	// inter-version-compatibility
+	{
+		CSG_Parameter	*pParameter	= pItem->Get_Parameter("COLORS_TYPE");
+
+		if( pParameter && Type == SG_DATAOBJECT_TYPE_Grid )
+		{
+			if( pParameter->asInt() == 4 ) { pParameter->Set_Value(6); }	// Shade
+			if( pParameter->asInt() == 5 ) { pParameter->Set_Value(4); }	// RGB Overlay
+			if( pParameter->asInt() == 6 ) { pParameter->Set_Value(5); }	// RGB Composite
+		}
+	}
+
+	//-----------------------------------------------------
+	if( Type == SG_DATAOBJECT_TYPE_Grid )
+	{
+		pItem->Get_Parameter("FILE_CACHE")->Set_Value(((CWKSP_Grid *)pItem)->Get_Grid()->is_Cached());
+	}
+
 	pItem->Parameters_Changed();
 
 	return( true );
@@ -543,7 +739,7 @@ bool CWKSP_Project::_Load_Data(CSG_MetaData &Entry, const wxString &ProjectDir,
 //---------------------------------------------------------
 bool CWKSP_Project::_Save_Data(CSG_MetaData &Entry, const wxString &ProjectDir, CSG_Data_Object *pDataObject, CSG_Parameters *pParameters)
 {
-	if( !pDataObject || !pDataObject->Get_File_Name() || !wxFileExists(pDataObject->Get_File_Name()) )
+	if( !pDataObject || !pDataObject->Get_File_Name(false) || SG_STR_LEN(pDataObject->Get_File_Name(false)) == 0 )
 	{
 		return( false );
 	}
@@ -553,16 +749,26 @@ bool CWKSP_Project::_Save_Data(CSG_MetaData &Entry, const wxString &ProjectDir,
 	switch( pDataObject->Get_ObjectType() )
 	{
 	default:	return( false );
-	case DATAOBJECT_TYPE_Grid:			pEntry->Add_Property("type", "GRID"  );	break;
-	case DATAOBJECT_TYPE_Table:			pEntry->Add_Property("type", "TABLE" );	break;
-	case DATAOBJECT_TYPE_Shapes:		pEntry->Add_Property("type", "SHAPES");	break;
-	case DATAOBJECT_TYPE_TIN:			pEntry->Add_Property("type", "TIN"   );	break;
-	case DATAOBJECT_TYPE_PointCloud:	pEntry->Add_Property("type", "POINTS");	break;
+	case SG_DATAOBJECT_TYPE_Grid      :	pEntry->Add_Property("type", "GRID"  );	break;
+	case SG_DATAOBJECT_TYPE_Grids     :	pEntry->Add_Property("type", "GRIDS" );	break;
+	case SG_DATAOBJECT_TYPE_Table     :	pEntry->Add_Property("type", "TABLE" );	break;
+	case SG_DATAOBJECT_TYPE_Shapes    :	pEntry->Add_Property("type", "SHAPES");	break;
+	case SG_DATAOBJECT_TYPE_TIN       :	pEntry->Add_Property("type", "TIN"   );	break;
+	case SG_DATAOBJECT_TYPE_PointCloud:	pEntry->Add_Property("type", "POINTS");	break;
 	}
 
-	wxString	s(Get_FilePath_Relative(ProjectDir, pDataObject->Get_File_Name()));
-
-	pEntry->Add_Child("FILE", &s);
+	if( wxFileExists(pDataObject->Get_File_Name(false)) )
+	{
+		pEntry->Add_Child("FILE", SG_File_Get_Path_Relative(&ProjectDir, pDataObject->Get_File_Name(false)));
+	}
+	else if( pDataObject->Get_MetaData_DB().Get_Children_Count() > 0 )
+	{
+		pEntry->Add_Child("FILE", pDataObject->Get_File_Name(false));
+	}
+	else
+	{
+		return( false );
+	}
 
 	if( pParameters )
 	{
@@ -575,11 +781,14 @@ bool CWKSP_Project::_Save_Data(CSG_MetaData &Entry, const wxString &ProjectDir,
 
 		for(int i=0; i<pEntry->Get_Children_Count(); i++)
 		{
-			if( !pEntry->Get_Child(i)->Get_Name().CmpNoCase("DATA") && SG_File_Exists(pEntry->Get_Child(i)->Get_Content()) )
+			if( !pEntry->Get_Child(i)->Get_Name().CmpNoCase("DATA") )
 			{
-				wxString	s(Get_FilePath_Relative(ProjectDir, pEntry->Get_Child(i)->Get_Content().w_str()));
+				CSG_String	File	= pEntry->Get_Child(i)->Get_Content();
 
-				pEntry->Get_Child(i)->Set_Content(&s);
+				if( File.BeforeFirst(':').CmpNoCase("PGSQL") && SG_File_Exists(File) )
+				{
+					pEntry->Get_Child(i)->Set_Content(SG_File_Get_Path_Relative(&ProjectDir, File.w_str()));
+				}
 			}
 		}
 	}
@@ -590,8 +799,6 @@ bool CWKSP_Project::_Save_Data(CSG_MetaData &Entry, const wxString &ProjectDir,
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -599,11 +806,11 @@ bool CWKSP_Project::_Load_Map(CSG_MetaData &Entry, const wxString &ProjectDir)
 {
 	TSG_Rect	Extent;
 
-	if(  Entry.Get_Name().Cmp("MAP")
-	||	!Entry.Get_Child("XMIN") || !Entry.Get_Child("XMIN")->Get_Content().asDouble(Extent.xMin)
-	||	!Entry.Get_Child("XMAX") || !Entry.Get_Child("XMAX")->Get_Content().asDouble(Extent.xMax)
-	||	!Entry.Get_Child("YMIN") || !Entry.Get_Child("YMIN")->Get_Content().asDouble(Extent.yMin)
-	||	!Entry.Get_Child("YMAX") || !Entry.Get_Child("YMAX")->Get_Content().asDouble(Extent.yMax) )
+	if( !Entry.Cmp_Name("MAP")
+	||	!Entry("XMIN") || !Entry("XMIN")->Get_Content().asDouble(Extent.xMin)
+	||	!Entry("XMAX") || !Entry("XMAX")->Get_Content().asDouble(Extent.xMax)
+	||	!Entry("YMIN") || !Entry("YMIN")->Get_Content().asDouble(Extent.yMin)
+	||	!Entry("YMAX") || !Entry("YMAX")->Get_Content().asDouble(Extent.yMax) )
 	{
 		return( false );
 	}
@@ -621,12 +828,22 @@ bool CWKSP_Project::_Load_Map(CSG_MetaData &Entry, const wxString &ProjectDir)
 
 	for(i=0, n=0; i<pNode->Get_Children_Count(); i++)
 	{
-		if( !pNode->Get_Child(i)->Get_Name().Cmp("FILE") )
+		CSG_MetaData	&Layer	= *pNode->Get_Child(i);
+
+		if( Layer.Cmp_Name("FILE") )
 		{
-			CWKSP_Base_Item	*pItem	= _Get_byFileName(Get_FilePath_Absolute(ProjectDir, pNode->Get_Child(i)->Get_Content().w_str()));
+			wxString	FileName(Layer.Get_Content().w_str());
+
+			if( FileName.Find("PGSQL") != 0 )
+			{
+				FileName	= Get_FilePath_Absolute(ProjectDir, FileName);
+			}
+
+			CWKSP_Base_Item	*pItem	= _Get_byFileName(FileName);
 
 			if(	pItem &&
 			(   pItem->Get_Type() == WKSP_ITEM_Grid
+			||  pItem->Get_Type() == WKSP_ITEM_Grids
 			||  pItem->Get_Type() == WKSP_ITEM_TIN
 			||  pItem->Get_Type() == WKSP_ITEM_PointCloud
 			||  pItem->Get_Type() == WKSP_ITEM_Shapes) )
@@ -644,27 +861,58 @@ bool CWKSP_Project::_Load_Map(CSG_MetaData &Entry, const wxString &ProjectDir)
 	//-----------------------------------------------------
 	CWKSP_Map	*pMap	= new CWKSP_Map;
 
+	g_pMaps->Add(pMap);
+
+	if( Entry("PROJECTION") )
+	{
+		pMap->Get_Projection().Load(Entry["PROJECTION"]);
+	}
+
+	pMap->Get_Parameter("CRS_CHECK")->Set_Value(false);
+
 	for(int i=0; i<pNode->Get_Children_Count(); i++)
 	{
-		if( !pNode->Get_Child(i)->Get_Name().Cmp("FILE") )
+		CSG_MetaData	&Layer	= *pNode->Get_Child(i);
+
+		if( Layer.Cmp_Name("FILE") )
 		{
-			CWKSP_Base_Item	*pItem	= _Get_byFileName(Get_FilePath_Absolute(ProjectDir, pNode->Get_Child(i)->Get_Content().w_str()));
+			wxString	FileName(Layer.Get_Content().w_str());
+
+			if( FileName.Find("PGSQL") != 0 )
+			{
+				FileName	= Get_FilePath_Absolute(ProjectDir, FileName);
+			}
+
+			CWKSP_Base_Item	*pItem	= _Get_byFileName(FileName);
 
 			if(	pItem &&
 			(   pItem->Get_Type() == WKSP_ITEM_Grid
+			||  pItem->Get_Type() == WKSP_ITEM_Grids
 			||  pItem->Get_Type() == WKSP_ITEM_TIN
 			||  pItem->Get_Type() == WKSP_ITEM_PointCloud
 			||  pItem->Get_Type() == WKSP_ITEM_Shapes) )
 			{
-				g_pMaps->Add((CWKSP_Layer *)pItem, pMap);
+				CWKSP_Map_Layer	*pLayer	= pMap->Add_Layer((CWKSP_Layer *)pItem);
+
+				pLayer->Load_Settings(&Layer);
 			}
 		}
-		else
+		else if( Layer.Cmp_Name("PARAMETERS") )
 		{
-			pMap->Add_Graticule(pNode->Get_Child(i));
+			if( Layer.Cmp_Property("name", "GRATICULE") )
+			{
+				pMap->Add_Graticule(&Layer);
+			}
+
+			if( Layer.Cmp_Property("name", "BASEMAP") )
+			{
+				pMap->Add_BaseMap  (&Layer);
+			}
 		}
 	}
 
+	pMap->Get_Parameter("CRS_CHECK")->Set_Value(true);
+
 	//-----------------------------------------------------
 	if( Entry.Get_Child("PARAMETERS") && pMap->Get_Parameters()->Serialize(*Entry.Get_Child("PARAMETERS"), false) )
 	{
@@ -686,7 +934,12 @@ bool CWKSP_Project::_Save_Map(CSG_MetaData &Entry, const wxString &ProjectDir, C
 		return( false );
 	}
 
-	CSG_MetaData	*pEntry	= Entry.Add_Child(SG_T("MAP"));
+	CSG_MetaData	*pEntry	= Entry.Add_Child("MAP");
+
+	if( pMap->Get_Projection().is_Okay() )
+	{
+		pMap->Get_Projection().Save(*pEntry->Add_Child("PROJECTION"));
+	}
 
 	pEntry->Add_Child("XMIN", pMap->Get_Extent().Get_XMin());
 	pEntry->Add_Child("XMAX", pMap->Get_Extent().Get_XMax());
@@ -695,25 +948,42 @@ bool CWKSP_Project::_Save_Map(CSG_MetaData &Entry, const wxString &ProjectDir, C
 
 	pMap->Get_Parameters()->Serialize(*pEntry->Add_Child("PARAMETERS"), true);
 
-	pEntry	= pEntry->Add_Child("LAYERS");
+	CSG_MetaData	&Layers	= *pEntry->Add_Child("LAYERS");
 
 	for(int i=pMap->Get_Count()-1; i>=0; i--)
 	{
 		if( pMap->Get_Item(i)->Get_Type() == WKSP_ITEM_Map_Layer )
 		{
-			CSG_Data_Object	*pObject	= ((CWKSP_Map_Layer *)pMap->Get_Item(i))->Get_Layer()->Get_Object();
+			CWKSP_Map_Layer	*pLayer		= (CWKSP_Map_Layer *)pMap->Get_Item(i);
+			CSG_Data_Object	*pObject	= pLayer->Get_Layer()->Get_Object();
 
-			if( pObject && pObject->Get_File_Name() && wxFileExists(pObject->Get_File_Name()) )
+			if( pObject && pObject->Get_File_Name(false) && *pObject->Get_File_Name(false) )
 			{
-				wxString	s(Get_FilePath_Relative(ProjectDir, pObject->Get_File_Name()));
+				wxString	FileName(pObject->Get_File_Name(false));
 
-				pEntry->Add_Child("FILE", &s);
+				if( FileName.Find("PGSQL") == 0 )
+				{
+					pLayer->Save_Settings(
+						Layers.Add_Child("FILE", &FileName)
+					);
+				}
+				else if( wxFileExists(FileName) )
+				{
+					pLayer->Save_Settings(
+						Layers.Add_Child("FILE", SG_File_Get_Path_Relative(&ProjectDir, &FileName))
+					);
+				}
 			}
 		}
 
 		else if( pMap->Get_Item(i)->Get_Type() == WKSP_ITEM_Map_Graticule )
 		{
-			((CWKSP_Map_Graticule *)pMap->Get_Item(i))->Save(*pEntry);
+			((CWKSP_Map_Graticule *)pMap->Get_Item(i))->Save(Layers);
+		}
+
+		else if( pMap->Get_Item(i)->Get_Type() == WKSP_ITEM_Map_BaseMap )
+		{
+			((CWKSP_Map_BaseMap   *)pMap->Get_Item(i))->Save(Layers);
 		}
 	}
 
@@ -723,14 +993,12 @@ bool CWKSP_Project::_Save_Map(CSG_MetaData &Entry, const wxString &ProjectDir, C
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
 CWKSP_Base_Item * CWKSP_Project::_Get_byFileName(const wxString &FileName)
 {
-	return( g_pData->Get(SG_Get_Data_Manager().Find(&FileName)) );
+	return( g_pData->Get(SG_Get_Data_Manager().Find(&FileName, false)) );
 }
 
 
@@ -758,13 +1026,13 @@ bool CWKSP_Project::_Compatibility_Data(TSG_Data_Type Type, CSG_Parameters *pPar
 	{
 		CSG_Parameter	*pParameter;
 
-		if( Type == DATAOBJECT_TYPE_Grid )
+		if( Type == SG_DATAOBJECT_TYPE_Grid )
 		{
 			if( (pParameter = pParameters->Get_Parameter("COLORS_TYPE")) != NULL )
 			{
 				if( pParameter->asInt() == 3 )
-				{	// 0:Single >> 1:LUT >> 2:Discrete >> 3:Graduated >> 4:Shade >> 5:Overlay >> 6:RGB
-					pParameter->Set_Value(6);	// RGB moved to position 6
+				{	// 0:Single >> 1:LUT >> 2:Discrete >> 3:Graduated >> 4:RGB Overlay >> 5:RGB Composite >> 6:Shade
+					pParameter->Set_Value(5);	// RGB moved to position 5
 				}
 			}
 		}
@@ -815,7 +1083,7 @@ bool CWKSP_Project::_Compatibility_Load_Data(const wxString &FileName)
 	g_pSAGA_Frame->Freeze();
 
 	g_pData->Get_Menu_Files()->Set_Update(false);
-	while( _Compatibility_Load_Data(Stream, SG_File_Get_Path(FileName).w_str()) );
+	while( _Compatibility_Load_Data(Stream, SG_File_Get_Path(&FileName).w_str()) );
 	g_pData->Get_Menu_Files()->Set_Update(true);
 
 	//-------------------------------------------------
@@ -823,7 +1091,7 @@ bool CWKSP_Project::_Compatibility_Load_Data(const wxString &FileName)
 
 	if( !sLine.Cmp(MAP_ENTRIES_BEGIN) )
 	{
-		while( _Compatibility_Load_Map(Stream, SG_File_Get_Path(FileName).w_str()) );
+		while( _Compatibility_Load_Map(Stream, SG_File_Get_Path(&FileName).w_str()) );
 	}
 
 	switch( g_pData->Get_Parameter("PROJECT_MAP_ARRANGE")->asInt() )
@@ -898,6 +1166,7 @@ bool CWKSP_Project::_Compatibility_Load_Map(CSG_File &Stream, const wxString &Pr
 
 					if(	(pItem = _Get_byFileName(sLine.c_str())) != NULL
 					&&	(	pItem->Get_Type()	== WKSP_ITEM_Grid
+						||	pItem->Get_Type()	== WKSP_ITEM_Grids
 						||	pItem->Get_Type()	== WKSP_ITEM_TIN
 						||	pItem->Get_Type()	== WKSP_ITEM_PointCloud
 						||	pItem->Get_Type()	== WKSP_ITEM_Shapes) )
