@@ -1,6 +1,3 @@
-/**********************************************************
- * Version $Id: res_commands.cpp 2527 2015-06-30 15:38:35Z oconrad $
- *********************************************************/
 
 ///////////////////////////////////////////////////////////
 //                                                       //
@@ -24,7 +21,8 @@
 // Geoscientific Analyses'. SAGA is free software; you   //
 // can redistribute it and/or modify it under the terms  //
 // of the GNU General Public License as published by the //
-// Free Software Foundation; version 2 of the License.   //
+// Free Software Foundation, either version 2 of the     //
+// License, or (at your option) any later version.       //
 //                                                       //
 // SAGA is distributed in the hope that it will be       //
 // useful, but WITHOUT ANY WARRANTY; without even the    //
@@ -33,10 +31,8 @@
 // License for more details.                             //
 //                                                       //
 // You should have received a copy of the GNU General    //
-// Public License along with this program; if not,       //
-// write to the Free Software Foundation, Inc.,          //
-// 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, //
-// USA.                                                  //
+// Public License along with this program; if not, see   //
+// <http://www.gnu.org/licenses/>.                       //
 //                                                       //
 //-------------------------------------------------------//
 //                                                       //
@@ -51,15 +47,6 @@
 //                                                       //
 ///////////////////////////////////////////////////////////
 
-//---------------------------------------------------------
-
-
-///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
-///////////////////////////////////////////////////////////
-
 //---------------------------------------------------------
 #include <wx/event.h>
 #include <wx/menu.h>
@@ -93,10 +80,18 @@ wxString CMD_Get_Name(int Cmd_ID)
 	case ID_CMD_FRAME_TILE_HORZ:		return( _TL("Tile Horizontally") );
 	case ID_CMD_FRAME_TILE_VERT:		return( _TL("Tile Vertically") );
 	case ID_CMD_FRAME_ARRANGEICONS:		return( _TL("Arrange Icons") );
+	case ID_CMD_FRAME_UNSPLIT:			return( _TL("Unsplit") );
+	case ID_CMD_FRAME_SPLIT_LEFT:		return( _TL("Left") );
+	case ID_CMD_FRAME_SPLIT_RIGHT:		return( _TL("Right") );
+	case ID_CMD_FRAME_SPLIT_TOP:		return( _TL("Top") );
+	case ID_CMD_FRAME_SPLIT_BOTTOM:		return( _TL("Bottom") );
+	case ID_CMD_FRAME_SPLIT_ALL_HORZ:	return( _TL("Horizontally") );
+	case ID_CMD_FRAME_SPLIT_ALL_VERT:	return( _TL("Vertically") );
 	case ID_CMD_FRAME_NEXT:				return( _TL("Next") );
 	case ID_CMD_FRAME_PREVIOUS:			return( _TL("Previous") );
 	case ID_CMD_FRAME_CLOSE:			return( _TL("Close") );
 	case ID_CMD_FRAME_CLOSE_ALL:		return( _TL("Close All") );
+	case ID_CMD_FRAME_CLOSE_ALL_OTHERS:	return( _TL("Close All But This") );
 
 	case ID_CMD_FRAME_INFO_SHOW:		return( _TL("Show Message Window") );
 	case ID_CMD_FRAME_DATA_SOURCE_SHOW:	return( _TL("Show Data Source Window") );
@@ -113,9 +108,6 @@ wxString CMD_Get_Name(int Cmd_ID)
 	case ID_CMD_WKSP_ITEM_SETTINGS_COPY:return( _TL("Copy Settings from other Layer") );
 	case ID_CMD_WKSP_ITEM_SEARCH:		return( _TL("Search for...") );
 
-	case ID_CMD_WKSP_OPEN:				return( _TL("Open") );
-	case ID_CMD_WKSP_SAVE:				return( _TL("Save All") );
-
 	case ID_CMD_INFO_CLEAR:				return( _TL("Clear") );
 	case ID_CMD_INFO_COPY:				return( _TL("Copy") );
 	case ID_CMD_INFO_SELECTALL:			return( _TL("Select All") );
@@ -123,118 +115,134 @@ wxString CMD_Get_Name(int Cmd_ID)
 	case ID_CMD_DATASOURCE_REFRESH:		return( _TL("Refresh") );
 
 	case ID_CMD_DB_REFRESH:				return( _TL("Refresh") );
+	case ID_CMD_DB_SOURCE_CREATE:		return( _TL("New") );
+	case ID_CMD_DB_SOURCE_DROP:			return( _TL("Drop") );
 	case ID_CMD_DB_SOURCE_OPEN:			return( _TL("Open") );
 	case ID_CMD_DB_SOURCE_CLOSE:		return( _TL("Close") );
 	case ID_CMD_DB_SOURCE_CLOSE_ALL:	return( _TL("Close All") );
-	case ID_CMD_DB_SOURCE_DELETE:		return( _TL("Delete") );
+	case ID_CMD_DB_SOURCE_DELETE:		return( _TL("Remove") );
 	case ID_CMD_DB_TABLE_OPEN:			return( _TL("Load") );
 	case ID_CMD_DB_TABLE_DELETE:		return( _TL("Delete") );
 	case ID_CMD_DB_TABLE_RENAME:		return( _TL("Rename") );
 	case ID_CMD_DB_TABLE_INFO:			return( _TL("Field Description") );
 	case ID_CMD_DB_TABLE_FROM_QUERY:	return( _TL("Query") );
 
-	case ID_CMD_MODULES_OPEN:			return( _TL("Load Tool Library") );
-	case ID_CMD_MODULES_SEARCH:			return( _TL("Find and Run Tool") );
-	case ID_CMD_MODULES_SAVE_DOCS:		return( _TL("Create Tool Description Files") );
-	case ID_CMD_MODULES_SAVE_SCRIPT:		return( _TL("Save to Script File") );
-	case ID_CMD_MODULES_SAVE_TO_CLIPBOARD:	return( _TL("Save to Clipboard") );
-	case ID_CMD_MODULES_CHAIN_RELOAD:	return( _TL("Reload") );
-	case ID_CMD_MODULES_CHAIN_EDIT:		return( _TL("Edit") );
+	case ID_CMD_TOOL_OPEN:				return( _TL("Load Tool Library") );
+	case ID_CMD_TOOL_RELOAD:			return( _TL("Reload Standard Tool Libraries") );
+	case ID_CMD_TOOL_SEARCH:			return( _TL("Find and Run Tool") );
+	case ID_CMD_TOOL_SAVE_DOCS:			return( _TL("Create Tool Description Files") );
+	case ID_CMD_TOOL_SAVE_SCRIPT:		return( _TL("Save to Script File") );
+	case ID_CMD_TOOL_SAVE_TO_CLIPBOARD:	return( _TL("Copy to Clipboard") );
+	case ID_CMD_TOOL_CHAIN_RELOAD:		return( _TL("Reload") );
+	case ID_CMD_TOOL_CHAIN_EDIT:		return( _TL("Edit") );
 
-	case ID_CMD_DATA_PROJECT_NEW:		return( _TL("Close All") );
 	case ID_CMD_DATA_PROJECT_OPEN:		return( _TL("Load Project") );
 	case ID_CMD_DATA_PROJECT_OPEN_ADD:	return( _TL("Add Project") );
+	case ID_CMD_DATA_PROJECT_BROWSE:	return( _TL("Search for Projects") );		
+	case ID_CMD_DATA_PROJECT_CLOSE:		return( _TL("Close Project") );
 	case ID_CMD_DATA_PROJECT_SAVE:		return( _TL("Save Project") );
-	case ID_CMD_DATA_PROJECT_SAVE_AS:	return( _TL("Save Project As...") );
-	case ID_CMD_DATA_PROJECT_BROWSE:	return( _TL("Search for Projects...") );		
-	case ID_CMD_DATA_LEGEND_COPY:		return( _TL("Copy Legend to Clipboard...") );
+	case ID_CMD_DATA_PROJECT_SAVE_AS:	return( _TL("Save Project as...") );
+	case ID_CMD_DATA_PROJECT_COPY:		return( _TL("Copy Project to...") );
+	case ID_CMD_DATA_PROJECT_COPY_DB:	return( _TL("Copy Project to Database") );
+
+	case ID_CMD_DATA_LEGEND_COPY:		return( _TL("Copy Legend to Clipboard") );
 	case ID_CMD_DATA_LEGEND_SIZE_INC:	return( _TL("Increase Legend Size") );
 	case ID_CMD_DATA_LEGEND_SIZE_DEC:	return( _TL("Decrease Legend Size") );
+
 	case ID_CMD_DATA_HISTORY_CLEAR:		return( _TL("Clear History") );
 	case ID_CMD_DATA_HISTORY_TO_MODEL:	return( _TL("Save as Tool Chain") );
 	case ID_CMD_DATA_HISTORY_OPTIONS_COLLAPSE:	return( _TL("Collpase Options") );
 	case ID_CMD_DATA_HISTORY_OPTIONS_EXPAND:	return( _TL("Expand Options") );
-	case ID_CMD_DATA_PROJECTION:		return( _TL("Spatial Reference") );
 
+	case ID_CMD_DATA_OPEN:				return( _TL("Open") );
+	case ID_CMD_DATA_SAVE:				return( _TL("Save") );
+	case ID_CMD_DATA_SAVEAS:			return( _TL("Save as...") );
+	case ID_CMD_DATA_SAVETODB:			return( _TL("Save to Database...") );
+	case ID_CMD_DATA_RELOAD:			return( _TL("Reload") );
+	case ID_CMD_DATA_DEL_FILES:			return( _TL("Delete Associated Files") );
+	case ID_CMD_DATA_PROJECTION:		return( _TL("Spatial Reference") );
+	case ID_CMD_DATA_METADATA:			return( _TL("View Metadata") );
+	case ID_CMD_DATA_FORCE_UPDATE:		return( _TL("Force Update") );
+
+	case ID_CMD_GRID_OPEN:
 	case ID_CMD_GRIDS_OPEN:
 	case ID_CMD_SHAPES_OPEN:
-	case ID_CMD_TABLES_OPEN:
+	case ID_CMD_TABLE_OPEN:
 	case ID_CMD_TIN_OPEN:
 	case ID_CMD_POINTCLOUD_OPEN:		return( _TL("Load") );
 
-	case ID_CMD_GRIDS_SAVE:
-	case ID_CMD_SHAPES_SAVE:
-	case ID_CMD_TABLES_SAVE:
-	case ID_CMD_TIN_SAVE:
-	case ID_CMD_POINTCLOUD_SAVE:		return( _TL("Save") );
-
-	case ID_CMD_GRIDS_SAVEAS:
-	case ID_CMD_SHAPES_SAVEAS:
-	case ID_CMD_TABLES_SAVEAS:
-	case ID_CMD_TIN_SAVEAS:
-	case ID_CMD_POINTCLOUD_SAVEAS:		return( _TL("Save As...") );
-
+	case ID_CMD_GRID_SHOW:
 	case ID_CMD_GRIDS_SHOW:
 	case ID_CMD_SHAPES_SHOW:
 	case ID_CMD_TIN_SHOW:
 	case ID_CMD_POINTCLOUD_SHOW:		return( _TL("Add to Map") );
 
-	case ID_CMD_TABLES_SHOW:			return( _TL("Show") );
+	case ID_CMD_TABLE_SHOW:				return( _TL("Show") );
 
-	case ID_CMD_GRIDS_SET_LUT:
-	case ID_CMD_SHAPES_SET_LUT:			return( _TL("Create Lookup Table") );
+	case ID_CMD_GRID_SET_LUT:
+	case ID_CMD_SHAPES_SET_LUT:			return( _TL("Classify") );
 
-	case ID_CMD_GRIDS_HISTOGRAM:
+	case ID_CMD_DATA_CLASSIFY_IMPORT:	return( _TL("Import Classification") );
+
+	case ID_CMD_GRID_HISTOGRAM:
 	case ID_CMD_SHAPES_HISTOGRAM:		return( _TL("Histogram") );
 
+	case ID_CMD_GRID_SCATTERPLOT:
 	case ID_CMD_GRIDS_SCATTERPLOT:
-	case ID_CMD_TABLES_SCATTERPLOT:		return( _TL("Scatterplot") );
+	case ID_CMD_TABLE_SCATTERPLOT:		return( _TL("Scatterplot") );
 
-	case ID_CMD_TABLES_DIAGRAM:			return( _TL("Diagram") );
+	case ID_CMD_TABLE_DIAGRAM:			return( _TL("Diagram") );
 
-	case ID_CMD_GRIDS_SAVEAS_IMAGE:		return( _TL("Save As Image...") );
-	case ID_CMD_GRIDS_EQUALINTERVALS:	return( _TL("Create Normalised Classification") );
+	case ID_CMD_GRID_SAVEAS_IMAGE:		return( _TL("Save as Image...") );
+	case ID_CMD_GRID_CLIPBOARD_IMAGE:	return( _TL("Copy to Clipboard") );
+	case ID_CMD_GRID_EQUALINTERVALS:	return( _TL("Create Normalised Classification") );
+	case ID_CMD_GRID_SEL_CLEAR:			return( _TL("Clear Selection") );
+	case ID_CMD_GRID_SEL_DELETE:		return( _TL("Delete Selection") );
+
+	case ID_CMD_GRIDS_SAVEAS_IMAGE:		return( _TL("Save as Image...") );
 	case ID_CMD_GRIDS_FIT_MINMAX:		return( _TL("Histogram Stretch to Minimum/Maximum") );
 	case ID_CMD_GRIDS_FIT_STDDEV:		return( _TL("Histogram Stretch to Standard Deviation") );
 	case ID_CMD_GRIDS_FIT_PCTL  :		return( _TL("Histogram Stretch to Percentile") );
 	case ID_CMD_GRIDS_FIT_DIALOG:		return( _TL("Histogram Stretch Dialog") );
-	case ID_CMD_GRIDS_SEL_CLEAR:		return( _TL("Clear Selection") );
-	case ID_CMD_GRIDS_SEL_DELETE:		return( _TL("Delete Selection") );
 
-	case ID_CMD_SHAPES_EDIT_SHAPE:		return( _TL("Edit Selected Shape") );
+	case ID_CMD_SHAPES_SAVE_ATTRIBUTES:	return( _TL("Save Attributes as...") );
+	case ID_CMD_SHAPES_EDIT_SHAPE:		return( _TL("Edit Selection") );
 	case ID_CMD_SHAPES_EDIT_SEL_CLEAR:	return( _TL("Clear Selection") );
 	case ID_CMD_SHAPES_EDIT_SEL_INVERT:	return( _TL("Invert Selection") );
 	case ID_CMD_SHAPES_EDIT_ADD_SHAPE:	return( _TL("Add Shape") );
 	case ID_CMD_SHAPES_EDIT_ADD_PART:	return( _TL("Add Part") );
-	case ID_CMD_SHAPES_EDIT_DEL_SHAPE:	return( _TL("Delete Selected Shape(s)") );
+	case ID_CMD_SHAPES_EDIT_DEL_SHAPE:	return( _TL("Delete Selection") );
 	case ID_CMD_SHAPES_EDIT_DEL_PART:	return( _TL("Delete Selected Part") );
 	case ID_CMD_SHAPES_EDIT_DEL_POINT:	return( _TL("Delete Selected Point") );
 	case ID_CMD_SHAPES_EDIT_SPLIT:		return( _TL("Split with Line") );
 	case ID_CMD_SHAPES_EDIT_MERGE:		return( _TL("Merge Selection") );
 	case ID_CMD_SHAPES_EDIT_MOVE:		return( _TL("Move") );
 
-	case ID_CMD_POINTCLOUD_RANGE_MINMAX:		return( _TL("Set Range to Minmum/Maximum") );
+	case ID_CMD_POINTCLOUD_RANGE_MINMAX:		return( _TL("Set Range to Minimum/Maximum") );
 	case ID_CMD_POINTCLOUD_RANGE_STDDEV150:		return( _TL("Set Range to Standard Deviation (1.5)") );
 	case ID_CMD_POINTCLOUD_RANGE_STDDEV200:		return( _TL("Set Range to Standard Deviation (2.0)") );
 
 	case ID_CMD_MAPS_SHOW:						return( _TL("Show Map") );
 	case ID_CMD_MAPS_3D_SHOW:					return( _TL("Show 3D-View") );
 	case ID_CMD_MAPS_LAYOUT_SHOW:				return( _TL("Show Print Layout") );
-	case ID_CMD_MAPS_SAVE_IMAGE:				return( _TL("Save As Image...") );
-	case ID_CMD_MAPS_SAVE_IMAGE_ON_CHANGE:		return( _TL("Save As Image when changed") );
-	case ID_CMD_MAPS_SAVE_IMAGE_TO_MEMORY:		return( _TL("Save As Image to Workspace") );
-	case ID_CMD_MAPS_SAVE_TO_CLIPBOARD:			return( _TL("Copy Map to Clipboard [Ctrl+C]") );
-	case ID_CMD_MAPS_SAVE_TO_CLIPBOARD_LEGEND:	return( _TL("Copy Legend to Clipboard [Ctrl+L]") );
+	case ID_CMD_MAPS_SAVE_IMAGE:				return( _TL("Save as Image...") );
+	case ID_CMD_MAPS_SAVE_IMAGE_ON_CHANGE:		return( _TL("Save as Image when changed") );
+	case ID_CMD_MAPS_SAVE_IMAGE_TO_KMZ:			return( _TL("Save as KMZ...") );
+	case ID_CMD_MAPS_SAVE_IMAGE_TO_MEMORY:		return( _TL("Save as Image to Workspace") );
+	case ID_CMD_MAPS_SAVE_TO_CLIPBOARD:			return( _TL("Copy to Clipboard") );
+	case ID_CMD_MAPS_SAVE_TO_CLIPBOARD_LEGEND:	return( _TL("Copy Legend to Clipboard") );
 	case ID_CMD_MAPS_SYNCHRONIZE:				return( _TL("Synchronise Map Extents") );
 	case ID_CMD_MAPS_SCALEBAR:					return( _TL("Scale Bar") );
 	case ID_CMD_MAPS_PROJECTION:				return( _TL("Spatial Reference") );
 	case ID_CMD_MAPS_GRATICULE_ADD:				return( _TL("Add Graticule") );
+	case ID_CMD_MAPS_BASEMAP_ADD:				return( _TL("Add Base Map") );
 	case ID_CMD_MAPS_LAYER_SHOW:				return( _TL("Show Layer") );
 	case ID_CMD_MAPS_MOVE_TOP:					return( _TL("Move To Top") );
 	case ID_CMD_MAPS_MOVE_BOTTOM:				return( _TL("Move To Bottom") );
 	case ID_CMD_MAPS_MOVE_UP:					return( _TL("Move Up") );
 	case ID_CMD_MAPS_MOVE_DOWN:					return( _TL("Move Down") );
 	case ID_CMD_MAPS_GRID_FITCOLORS:			return( _TL("Adjust Histogram Stretch to Map Extent") );
+	case ID_CMD_MAPS_PROJECT:					return( _TL("On-The-Fly Projection") );
 
 	//-----------------------------------------------------
 	// Views...
@@ -245,6 +253,7 @@ wxString CMD_Get_Name(int Cmd_ID)
 	case ID_CMD_MAP_LAYOUT_SHOW:				return( CMD_Get_Name(ID_CMD_MAPS_LAYOUT_SHOW) );
 	case ID_CMD_MAP_SCALEBAR:					return( CMD_Get_Name(ID_CMD_MAPS_SCALEBAR) );
 	case ID_CMD_MAP_GRATICULE_ADD:				return( CMD_Get_Name(ID_CMD_MAPS_GRATICULE_ADD) );
+	case ID_CMD_MAP_BASEMAP_ADD:				return( CMD_Get_Name(ID_CMD_MAPS_BASEMAP_ADD) );
 	case ID_CMD_MAP_SAVE_IMAGE:					return( CMD_Get_Name(ID_CMD_MAPS_SAVE_IMAGE) );
 	case ID_CMD_MAP_SAVE_IMAGE_ON_CHANGE:		return( CMD_Get_Name(ID_CMD_MAPS_SAVE_IMAGE_ON_CHANGE) );
 	case ID_CMD_MAP_SYNCHRONIZE:				return( CMD_Get_Name(ID_CMD_MAPS_SYNCHRONIZE) );
@@ -261,6 +270,7 @@ wxString CMD_Get_Name(int Cmd_ID)
 	case ID_CMD_MAP_ZOOM_ACTIVE:				return( _TL("Zoom To Active Layer") );
 	case ID_CMD_MAP_ZOOM_SELECTION:				return( _TL("Zoom To Selection") );
 	case ID_CMD_MAP_ZOOM_EXTENT:				return( _TL("Zoom To Extent") );
+	case ID_CMD_MAP_CROSSHAIR:					return( _TL("Cross Hair") );
 
 	case ID_CMD_MAP3D_PARAMETERS:		return( _TL("Properties") );
 	case ID_CMD_MAP3D_ROTATE_X_LESS:	return( _TL("Up") );
@@ -277,21 +287,21 @@ wxString CMD_Get_Name(int Cmd_ID)
 	case ID_CMD_MAP3D_SHIFT_Z_MORE:		return( _TL("Backward") );
 	case ID_CMD_MAP3D_EXAGGERATE_LESS:	return( _TL("Decrease Exaggeration [F1]") );
 	case ID_CMD_MAP3D_EXAGGERATE_MORE:	return( _TL("Increase Exaggeration [F2]") );
-	case ID_CMD_MAP3D_CENTRAL_LESS:		return( _TL("Decrease Perspectivic Distance [F5]") );
-	case ID_CMD_MAP3D_CENTRAL_MORE:		return( _TL("Increase Perspectivic Distance [F6]") );
+	case ID_CMD_MAP3D_CENTRAL_LESS:		return( _TL("Decrease Perspectivic Distance [F9]") );
+	case ID_CMD_MAP3D_CENTRAL_MORE:		return( _TL("Increase Perspectivic Distance [F10]") );
 	case ID_CMD_MAP3D_CENTRAL:			return( _TL("Central Projection") );
 	case ID_CMD_MAP3D_STEREO_LESS:		return( _TL("Decrease Eye Distance") );
 	case ID_CMD_MAP3D_STEREO_MORE:		return( _TL("Increase Eye Distance") );
 	case ID_CMD_MAP3D_STEREO:			return( _TL("Anaglyph [S]") );
 	case ID_CMD_MAP3D_INTERPOLATED:		return( _TL("Interpolated Colors") );
-	case ID_CMD_MAP3D_SAVE:				return( _TL("Save As Image...") );
-	case ID_CMD_MAP3D_SEQ_POS_ADD:		return( _TL("Add Position [A]") );
-	case ID_CMD_MAP3D_SEQ_POS_DEL:		return( _TL("Delete Last Position [D]") );
+	case ID_CMD_MAP3D_SAVE:				return( _TL("Save as Image...") );
+	case ID_CMD_MAP3D_SEQ_POS_ADD:		return( _TL("Add Position [Ctrl + A]") );
+	case ID_CMD_MAP3D_SEQ_POS_DEL:		return( _TL("Delete Last Position [Ctrl + D]") );
 	case ID_CMD_MAP3D_SEQ_POS_DEL_ALL:	return( _TL("Delete All Positions") );
 	case ID_CMD_MAP3D_SEQ_POS_EDIT:		return( _TL("Edit Positions") );
-	case ID_CMD_MAP3D_SEQ_PLAY:			return( _TL("Play Once [P]") );
-	case ID_CMD_MAP3D_SEQ_PLAY_LOOP:	return( _TL("Play Loop [L]") );
-	case ID_CMD_MAP3D_SEQ_SAVE:			return( _TL("Play And Save As Images...") );
+	case ID_CMD_MAP3D_SEQ_PLAY:			return( _TL("Play Once [Ctrl + P]") );
+	case ID_CMD_MAP3D_SEQ_PLAY_LOOP:	return( _TL("Play Loop [Ctrl + L]") );
+	case ID_CMD_MAP3D_SEQ_SAVE:			return( _TL("Play And Save as Images...") );
 
 	case ID_CMD_LAYOUT_PRINT_SETUP:		return( _TL("Print Setup") );
 	case ID_CMD_LAYOUT_PAGE_SETUP:		return( _TL("Page Setup") );
@@ -304,33 +314,38 @@ wxString CMD_Get_Name(int Cmd_ID)
 	case ID_CMD_TABLE_FIELD_SORT:		return( _TL("Sort Fields") );
 	case ID_CMD_TABLE_FIELD_RENAME:		return( _TL("Rename Fields") );
 	case ID_CMD_TABLE_FIELD_TYPE:		return( _TL("Change Field Type") );
+	case ID_CMD_TABLE_FIELD_CALC:		return( _TL("Field Calculator") );
+	case ID_CMD_TABLE_FIELD_OPEN_APP:	return( _TL("Open with external application") );
+	case ID_CMD_TABLE_FIELD_OPEN_DATA:	return( _TL("Open as data set") );
 	case ID_CMD_TABLE_RECORD_ADD:		return( _TL("Add Record") );
 	case ID_CMD_TABLE_RECORD_INS:		return( _TL("Insert Record") );
-	case ID_CMD_TABLE_RECORD_DEL:		return( _TL("Delete Selected Record(s)") );
+	case ID_CMD_TABLE_RECORD_DEL:		return( _TL("Delete Selection") );
 	case ID_CMD_TABLE_RECORD_DEL_ALL:	return( _TL("Delete All Records") );
-	case ID_CMD_TABLE_SELECTION_TO_TOP:	return( _TL("Sort Selection to Top") );
-	case ID_CMD_TABLE_SELECTION_ONLY:	return( _TL("Only Show Selection") );
+	case ID_CMD_TABLE_SELECTION_CLEAR:	return( _TL("Clear Selection") );
+	case ID_CMD_TABLE_SELECTION_ONLY:	return( _TL("Show Selection") );
 	case ID_CMD_TABLE_AUTOSIZE_COLS:	return( _TL("Fit Column Sizes") );
 	case ID_CMD_TABLE_AUTOSIZE_ROWS:	return( _TL("Fit Row Sizes") );
+	case ID_CMD_TABLE_TO_CLIPBOARD:		return( _TL("Copy to Clipboard") );
 
 	case ID_CMD_DIAGRAM_PARAMETERS:		return( _TL("Properties") );
 	case ID_CMD_DIAGRAM_SIZE_FIT:		return( _TL("Fit Diagram to Window") );
 	case ID_CMD_DIAGRAM_SIZE_INC:		return( _TL("Increase Diagram Size") );
 	case ID_CMD_DIAGRAM_SIZE_DEC:		return( _TL("Decrease Diagram Size") );
+	case ID_CMD_DIAGRAM_TO_CLIPBOARD:	return( _TL("Copy to Clipboard") );
 
 	case ID_CMD_SCATTERPLOT_PARAMETERS:		return( _TL("Properties") );
-	case ID_CMD_SCATTERPLOT_OPTIONS:		return( _TL("Options") );
 	case ID_CMD_SCATTERPLOT_UPDATE:			return( _TL("Update Data") );
 	case ID_CMD_SCATTERPLOT_AS_TABLE:		return( _TL("Convert To Table") );
 	case ID_CMD_SCATTERPLOT_TO_CLIPBOARD:	return( _TL("Copy to Clipboard") );
 
 	case ID_CMD_HISTOGRAM_CUMULATIVE:		return( _TL("Cumulative") );
+	case ID_CMD_HISTOGRAM_CLASS_COUNT:		return( _TL("Change Number of Classes") );
 	case ID_CMD_HISTOGRAM_AS_TABLE:			return( _TL("Convert To Table") );
 	case ID_CMD_HISTOGRAM_TO_CLIPBOARD:		return( _TL("Copy to Clipboard") );
 	}
 
 	//-----------------------------------------------------
-	return( _TL("UNRECOGNISED COMMAND ID") );
+	return( wxString::Format("%s [%d]", _TL("unrecognized command identifier")) );
 }
 
 //---------------------------------------------------------
@@ -355,12 +370,13 @@ int CMD_Get_ImageID(int Cmd_ID)
 	case ID_CMD_FRAME_ACTIVE_SHOW:		return( ID_IMG_TB_ACTIVE );
 	case ID_CMD_FRAME_INFO_SHOW:		return( ID_IMG_TB_INFO );
 	case ID_CMD_FRAME_HELP:				return( ID_IMG_TB_HELP );
-	case ID_CMD_WKSP_OPEN:				return( ID_IMG_TB_OPEN );
-	case ID_CMD_WKSP_SAVE:				return( ID_IMG_TB_SAVE );
-	case ID_CMD_MODULES_OPEN:			return( ID_IMG_TB_OPEN_MODULE );
-	case ID_CMD_MODULES_SEARCH:			return( ID_IMG_NB_WKSP_MODULES );
-	case ID_CMD_TABLES_OPEN:			return( ID_IMG_TB_OPEN_TABLE );
+	case ID_CMD_DATA_OPEN:				return( ID_IMG_TB_OPEN );
+	case ID_CMD_DATA_PROJECT_SAVE:		return( ID_IMG_TB_SAVE );
+	case ID_CMD_TOOL_OPEN:				return( ID_IMG_TB_OPEN_TOOL );
+	case ID_CMD_TOOL_SEARCH:			return( ID_IMG_NB_WKSP_TOOLS );
+	case ID_CMD_TABLE_OPEN:				return( ID_IMG_TB_OPEN_TABLE );
 	case ID_CMD_SHAPES_OPEN:			return( ID_IMG_TB_OPEN_SHAPES );
+	case ID_CMD_GRID_OPEN:				return( ID_IMG_TB_OPEN_GRID );
 	case ID_CMD_GRIDS_OPEN:				return( ID_IMG_TB_OPEN_GRID );
 	case ID_CMD_TIN_OPEN:				return( ID_IMG_TB_OPEN_TIN );
 	case ID_CMD_POINTCLOUD_OPEN:		return( ID_IMG_TB_OPEN_POINTCLOUD );
@@ -375,17 +391,19 @@ int CMD_Get_ImageID(int Cmd_ID)
 	case ID_CMD_MAP_ZOOM_ACTIVE:		return( ID_IMG_TB_MAP_ZOOM_ACTIVE );
 	case ID_CMD_MAP_ZOOM_SELECTION:		return( ID_IMG_TB_MAP_ZOOM_SELECTION );
 	case ID_CMD_MAP_ZOOM_EXTENT:		return( ID_IMG_TB_MAP_ZOOM_SELECTION );
+	case ID_CMD_MAP_CROSSHAIR:			return( ID_IMG_TB_MAP_CROSSHAIR );
 	case ID_CMD_MAP_3D_SHOW:			return( ID_IMG_TB_MAP_3D_SHOW );
 	case ID_CMD_MAPS_3D_SHOW:			return( ID_IMG_TB_MAP_3D_SHOW );
 	case ID_CMD_MAP_LAYOUT_SHOW:		return( ID_IMG_TB_MAP_LAYOUT_SHOW );
 	case ID_CMD_MAPS_LAYOUT_SHOW:		return( ID_IMG_TB_MAP_LAYOUT_SHOW );
-	case ID_CMD_MAPS_SAVE_TO_CLIPBOARD:			break;
+	case ID_CMD_MAPS_SAVE_TO_CLIPBOARD:			return( ID_IMG_TB_CLIPBOARD );
 	case ID_CMD_MAPS_SAVE_TO_CLIPBOARD_LEGEND:	break;
 	case ID_CMD_MAPS_SAVE_IMAGE:				break;
 	case ID_CMD_MAP_NORTH_ARROW:		return( ID_IMG_TB_MAP_NORTH_ARROW );
 	case ID_CMD_MAP_SYNCHRONIZE:		return( ID_IMG_TB_MAP_SYNCHRONIZE );
 	case ID_CMD_MAP_SCALEBAR:			return( ID_IMG_TB_MAP_SCALEBAR );
 	case ID_CMD_MAP_GRATICULE_ADD:				break;
+	case ID_CMD_MAP_BASEMAP_ADD:				break;
 	case ID_CMD_MAP_LEGEND_VERTICAL:	return( ID_IMG_TB_MAP_LEGEND_VERTICAL );
 	case ID_CMD_MAP_LEGEND_HORIZONTAL:	return( ID_IMG_TB_MAP_LEGEND_HORIZONTAL );
 
@@ -412,9 +430,6 @@ int CMD_Get_ImageID(int Cmd_ID)
 	case ID_CMD_LAYOUT_PRINT:			return( ID_IMG_TB_LAYOUT_PRINT );
 	case ID_CMD_LAYOUT_PRINT_PREVIEW:	return( ID_IMG_TB_LAYOUT_PRINT_PREVIEW );
 
-	case ID_CMD_HISTOGRAM_CUMULATIVE:	return( ID_IMG_TB_HISTOGRAM_CUMULATIVE );
-	case ID_CMD_HISTOGRAM_AS_TABLE:		return( ID_IMG_TB_HISTOGRAM_AS_TABLE );
-
 	case ID_CMD_TABLE_FIELD_ADD:		return( ID_IMG_TB_TABLE_COL_ADD );
 	case ID_CMD_TABLE_FIELD_DEL:		return( ID_IMG_TB_TABLE_COL_DEL );
 	case ID_CMD_TABLE_RECORD_ADD:		return( ID_IMG_TB_TABLE_ROW_ADD );
@@ -422,9 +437,14 @@ int CMD_Get_ImageID(int Cmd_ID)
 	case ID_CMD_TABLE_RECORD_DEL:		return( ID_IMG_TB_TABLE_ROW_DEL );
 	case ID_CMD_TABLE_RECORD_DEL_ALL:	return( ID_IMG_TB_TABLE_ROW_DEL_ALL );
 
-	case ID_CMD_SCATTERPLOT_PARAMETERS:	return( ID_IMG_TB_SCATTERPLOT_PARAMETERS );
-	case ID_CMD_SCATTERPLOT_OPTIONS:	return( ID_IMG_TB_SCATTERPLOT_PARAMETERS );
-	case ID_CMD_SCATTERPLOT_UPDATE:		return( ID_IMG_DEFAULT );
+	case ID_CMD_HISTOGRAM_CUMULATIVE    :	return( ID_IMG_TB_HISTOGRAM_CUMULATIVE );
+	case ID_CMD_HISTOGRAM_AS_TABLE      :	return( ID_IMG_TB_HISTOGRAM_AS_TABLE );
+	case ID_CMD_HISTOGRAM_TO_CLIPBOARD  :	return( ID_IMG_TB_CLIPBOARD );
+
+	case ID_CMD_SCATTERPLOT_PARAMETERS  :	return( ID_IMG_TB_SCATTERPLOT_PARAMETERS );
+	case ID_CMD_SCATTERPLOT_UPDATE      :	return( ID_IMG_TB_UPDATE );
+	case ID_CMD_SCATTERPLOT_AS_TABLE    :	return( ID_IMG_TB_HISTOGRAM_AS_TABLE );
+	case ID_CMD_SCATTERPLOT_TO_CLIPBOARD:	return( ID_IMG_TB_CLIPBOARD );
 
 	case ID_CMD_DIAGRAM_PARAMETERS:		return( ID_IMG_TB_DIAGRAM_PARAMETERS );
 	case ID_CMD_DIAGRAM_SIZE_FIT:		break;
