@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: saga_frame.cpp 2482 2015-04-30 15:34:11Z oconrad $
+ * Version $Id$
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -24,7 +24,8 @@
 // Geoscientific Analyses'. SAGA is free software; you   //
 // can redistribute it and/or modify it under the terms  //
 // of the GNU General Public License as published by the //
-// Free Software Foundation; version 2 of the License.   //
+// Free Software Foundation, either version 2 of the     //
+// License, or (at your option) any later version.       //
 //                                                       //
 // SAGA is distributed in the hope that it will be       //
 // useful, but WITHOUT ANY WARRANTY; without even the    //
@@ -33,10 +34,8 @@
 // License for more details.                             //
 //                                                       //
 // You should have received a copy of the GNU General    //
-// Public License along with this program; if not,       //
-// write to the Free Software Foundation, Inc.,          //
-// 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, //
-// USA.                                                  //
+// Public License along with this program; if not, see   //
+// <http://www.gnu.org/licenses/>.                       //
 //                                                       //
 //-------------------------------------------------------//
 //                                                       //
@@ -71,6 +70,7 @@
 #include <wx/toolbar.h>
 #include <wx/tipdlg.h>
 #include <wx/aui/aui.h>
+#include <wx/display.h>
 
 #include <saga_api/saga_api.h>
 
@@ -92,8 +92,8 @@
 #include "active.h"
 #include "wksp.h"
 
-#include "wksp_module_manager.h"
-#include "wksp_module.h"
+#include "wksp_tool_manager.h"
+#include "wksp_tool.h"
 
 #include "wksp_data_manager.h"
 #include "wksp_data_menu_files.h"
@@ -138,10 +138,10 @@ public:
 	CSAGA_Frame_StatusBar(wxWindow *parent, wxWindowID id, long style = wxST_SIZEGRIP, const wxString& name = wxT("statusBar"))
 		: wxStatusBar(parent, id, style, name)
 	{
-		m_pProgressBar	= new wxGauge(this, ID_WND_PROGRESSBAR, 100);
+		m_pProgressBar	= new wxGauge(this, ID_WND_PROGRESSBAR, 100, wxDefaultPosition, wxDefaultSize, wxGA_HORIZONTAL|wxGA_SMOOTH);
 	}
 
-	void		On_Size		(wxSizeEvent     &event)
+	void		On_Size		(wxSizeEvent &event)
 	{
 		wxRect	r;
 
@@ -149,8 +149,6 @@ public:
 		{
 			m_pProgressBar->SetSize(r);
 		}
-
-		event.Skip();
 	}
 
 	wxGauge		*m_pProgressBar;
@@ -191,7 +189,7 @@ CSAGA_Frame	*g_pSAGA_Frame	= NULL;
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-BEGIN_EVENT_TABLE(CSAGA_Frame, wxMDIParentFrame)
+BEGIN_EVENT_TABLE(CSAGA_Frame, MDI_ParentFrame)
 	EVT_CLOSE			(CSAGA_Frame::On_Close)
 	EVT_SIZE			(CSAGA_Frame::On_Size)
 
@@ -208,6 +206,20 @@ BEGIN_EVENT_TABLE(CSAGA_Frame, wxMDIParentFrame)
 	EVT_UPDATE_UI		(ID_CMD_FRAME_TILE_VERT			, CSAGA_Frame::On_Frame_vTile_UI)
 	EVT_MENU			(ID_CMD_FRAME_ARRANGEICONS		, CSAGA_Frame::On_Frame_ArrangeIcons)
 	EVT_UPDATE_UI		(ID_CMD_FRAME_ARRANGEICONS		, CSAGA_Frame::On_Frame_ArrangeIcons_UI)
+	EVT_MENU			(ID_CMD_FRAME_UNSPLIT			, CSAGA_Frame::On_Frame_Unsplit)
+	EVT_UPDATE_UI		(ID_CMD_FRAME_UNSPLIT			, CSAGA_Frame::On_Frame_Unsplit_UI)
+	EVT_MENU			(ID_CMD_FRAME_SPLIT_LEFT		, CSAGA_Frame::On_Frame_Split)
+	EVT_UPDATE_UI		(ID_CMD_FRAME_SPLIT_LEFT		, CSAGA_Frame::On_Frame_Split_UI)
+	EVT_MENU			(ID_CMD_FRAME_SPLIT_RIGHT		, CSAGA_Frame::On_Frame_Split)
+	EVT_UPDATE_UI		(ID_CMD_FRAME_SPLIT_RIGHT		, CSAGA_Frame::On_Frame_Split_UI)
+	EVT_MENU			(ID_CMD_FRAME_SPLIT_TOP			, CSAGA_Frame::On_Frame_Split)
+	EVT_UPDATE_UI		(ID_CMD_FRAME_SPLIT_TOP			, CSAGA_Frame::On_Frame_Split_UI)
+	EVT_MENU			(ID_CMD_FRAME_SPLIT_BOTTOM		, CSAGA_Frame::On_Frame_Split)
+	EVT_UPDATE_UI		(ID_CMD_FRAME_SPLIT_BOTTOM		, CSAGA_Frame::On_Frame_Split_UI)
+	EVT_MENU			(ID_CMD_FRAME_SPLIT_ALL_HORZ	, CSAGA_Frame::On_Frame_Split)
+	EVT_UPDATE_UI		(ID_CMD_FRAME_SPLIT_ALL_HORZ	, CSAGA_Frame::On_Frame_Split_UI)
+	EVT_MENU			(ID_CMD_FRAME_SPLIT_ALL_VERT	, CSAGA_Frame::On_Frame_Split)
+	EVT_UPDATE_UI		(ID_CMD_FRAME_SPLIT_ALL_VERT	, CSAGA_Frame::On_Frame_Split_UI)
 	EVT_MENU			(ID_CMD_FRAME_NEXT				, CSAGA_Frame::On_Frame_Next)
 	EVT_UPDATE_UI		(ID_CMD_FRAME_NEXT				, CSAGA_Frame::On_Frame_Next_UI)
 	EVT_MENU			(ID_CMD_FRAME_PREVIOUS			, CSAGA_Frame::On_Frame_Previous)
@@ -228,8 +240,8 @@ BEGIN_EVENT_TABLE(CSAGA_Frame, wxMDIParentFrame)
 
 	EVT_MENU_RANGE		(ID_CMD_WKSP_FIRST				, ID_CMD_WKSP_LAST		, CSAGA_Frame::On_Command_Workspace)
 	EVT_UPDATE_UI_RANGE	(ID_CMD_WKSP_FIRST				, ID_CMD_WKSP_LAST		, CSAGA_Frame::On_Command_Workspace_UI)
-	EVT_MENU_RANGE		(ID_CMD_MODULE_FIRST			, ID_CMD_MODULE_LAST	, CSAGA_Frame::On_Command_Module)
-	EVT_UPDATE_UI_RANGE	(ID_CMD_MODULE_FIRST			, ID_CMD_MODULE_LAST	, CSAGA_Frame::On_Command_Module_UI)
+	EVT_MENU_RANGE		(ID_CMD_TOOL_MENU_FIRST			, ID_CMD_TOOL_MENU_LAST	, CSAGA_Frame::On_Command_Tool)
+	EVT_UPDATE_UI_RANGE	(ID_CMD_TOOL_MENU_FIRST			, ID_CMD_TOOL_MENU_LAST	, CSAGA_Frame::On_Command_Tool_UI)
 
 	EVT_MENU_RANGE		(ID_CMD_CHILD_FIRST				, ID_CMD_CHILD_LAST		, CSAGA_Frame::On_Command_Child)
 	EVT_UPDATE_UI_RANGE	(ID_CMD_MAP_FIRST				, ID_CMD_MAP_LAST		, CSAGA_Frame::On_Command_Child_UI)
@@ -243,9 +255,16 @@ END_EVENT_TABLE()
 //														 //
 ///////////////////////////////////////////////////////////
 
+//---------------------------------------------------------
+#ifdef MDI_TABBED
+#define MDI_PARENT_FRAME_STYLE	wxDEFAULT_FRAME_STYLE|wxNO_FULL_REPAINT_ON_RESIZE|wxFRAME_NO_WINDOW_MENU
+#else
+#define MDI_PARENT_FRAME_STYLE	wxDEFAULT_FRAME_STYLE|wxNO_FULL_REPAINT_ON_RESIZE|wxFRAME_NO_WINDOW_MENU|wxHSCROLL|wxVSCROLL
+#endif
+
 //---------------------------------------------------------
 CSAGA_Frame::CSAGA_Frame(void)
-	: wxMDIParentFrame(NULL, ID_WND_MAIN, SAGA_CAPTION, wxDefaultPosition, wxDefaultSize, wxDEFAULT_FRAME_STYLE|wxNO_FULL_REPAINT_ON_RESIZE|wxHSCROLL|wxVSCROLL|wxFRAME_NO_WINDOW_MENU)
+	: MDI_ParentFrame(NULL, ID_WND_MAIN, SAGA_CAPTION, wxDefaultPosition, wxDefaultSize, MDI_PARENT_FRAME_STYLE)
 {
 	//-----------------------------------------------------
 	g_pSAGA_Frame		= this;
@@ -291,72 +310,77 @@ CSAGA_Frame::CSAGA_Frame(void)
 	m_pLayout->SetFlags(m_pLayout->GetFlags() ^ wxAUI_MGR_TRANSPARENT_DRAG);
 //	m_pLayout->SetFlags(m_pLayout->GetFlags() ^ wxAUI_MGR_ALLOW_ACTIVE_PANE);
 
+	m_pLayout->GetPane(GetClientWindow()).Show().Center();
+
 	//-----------------------------------------------------
-	_Bar_Add(m_pINFO		= new CINFO       (this), 0, 0);	m_pINFO			->Add_Pages();
-	_Bar_Add(m_pWKSP		= new CWKSP       (this), 2, 1);	m_pWKSP			->Add_Pages();
-	_Bar_Add(m_pData_Source	= new CData_Source(this), 2, 1);	m_pData_Source	->Add_Pages();
-	_Bar_Add(m_pActive		= new CACTIVE     (this), 2, 0);	m_pActive		->Add_Pages();
+	_Bar_Add(m_pINFO        = new CINFO       (this), 0, 0); m_pINFO       ->Add_Pages();
+	_Bar_Add(m_pWKSP        = new CWKSP       (this), 2, 1); m_pWKSP       ->Add_Pages();
+	_Bar_Add(m_pData_Source = new CData_Source(this), 2, 1); m_pData_Source->Add_Pages();
+	_Bar_Add(m_pActive      = new CActive     (this), 2, 0); m_pActive     ->Add_Pages();
 
 	//-----------------------------------------------------
 	_Create_MenuBar();
 
 	//-----------------------------------------------------
-	m_pTB_Main			= 						  _Create_ToolBar();
-	m_pTB_Map			= CVIEW_Map				::_Create_ToolBar();
-	m_pTB_Map_3D		= CVIEW_Map_3D			::_Create_ToolBar();
-	m_pTB_Layout		= CVIEW_Layout			::_Create_ToolBar();
-	m_pTB_Table			= CVIEW_Table			::_Create_ToolBar();
-	m_pTB_Diagram		= CVIEW_Table_Diagram	::_Create_ToolBar();
-	m_pTB_Histogram		= CVIEW_Histogram		::_Create_ToolBar();
-	m_pTB_ScatterPlot	= CVIEW_ScatterPlot		::_Create_ToolBar();
-
-	//-----------------------------------------------------
-	m_pLayout->GetPane(GetClientWindow()).Show().Center();
-
 	wxString	s;
 
-	if( CONFIG_Read(wxT("/FL"), wxT("MANAGER"), s) )
+	if( CONFIG_Read("/FL", "MANAGER", s) )
 	{
-		m_pLayout->LoadPerspective(s);
+		m_pLayout->LoadPerspective(s, false);
 	}
 
+	Set_Pane_Caption(m_pINFO       , m_pINFO       ->GetName());	// captions might have been modified by perspective, so update again...
+	Set_Pane_Caption(m_pWKSP       , m_pWKSP       ->GetName());
+	Set_Pane_Caption(m_pData_Source, m_pData_Source->GetName());
+	Set_Pane_Caption(m_pActive     , m_pActive     ->GetName());
+
+	//-----------------------------------------------------
+	m_pTB_Main			=                      _Create_ToolBar();
+	m_pTB_Map			= CVIEW_Map          ::_Create_ToolBar();
+	m_pTB_Map_3D		= CVIEW_Map_3D       ::_Create_ToolBar();
+	m_pTB_Layout		= CVIEW_Layout       ::_Create_ToolBar();
+	m_pTB_Table			= CVIEW_Table        ::_Create_ToolBar();
+	m_pTB_Diagram		= CVIEW_Table_Diagram::_Create_ToolBar();
+	m_pTB_Histogram		= CVIEW_Histogram    ::_Create_ToolBar();
+	m_pTB_ScatterPlot	= CVIEW_ScatterPlot  ::_Create_ToolBar();
+
 	_Bar_Show(m_pTB_Main, true);
 
-	//-----------------------------------------------------
 	m_pLayout->Update();
 
-#if !defined(_SAGA_LINUX)
-	Show(true);
-#endif
-
-	int		x, y, dx, dy;
+	//-----------------------------------------------------
 	long	l;
+	wxRect	r, rDefault	= wxGetClientDisplayRect();
 
-	x	= CONFIG_Read(wxT("/FL"), wxT("X" ), l) ? l : -1;
-	y	= CONFIG_Read(wxT("/FL"), wxT("Y" ), l) ? l : -1;
-	dx	= CONFIG_Read(wxT("/FL"), wxT("DX"), l) ? l : 800;
-	dy	= CONFIG_Read(wxT("/FL"), wxT("DY"), l) ? l : 600;
+	rDefault.Deflate((int)(0.1 * rDefault.GetWidth()), (int)(0.1 * rDefault.GetHeight()));
 
-	SetSize(x, y, dx, dy);
+	r.SetLeft  (CONFIG_Read("/FL", "X" , l) ? l : rDefault.GetLeft  ());
+	r.SetTop   (CONFIG_Read("/FL", "Y" , l) ? l : rDefault.GetTop   ());
+	r.SetWidth (CONFIG_Read("/FL", "DX", l) ? l : rDefault.GetWidth ());
+	r.SetHeight(CONFIG_Read("/FL", "DY", l) ? l : rDefault.GetHeight());
 
-	if( !(CONFIG_Read(wxT("/FL"), wxT("STATE"), l) && l == 0) )
+	if( wxDisplay::GetFromPoint(r.GetTopLeft    ()) == wxNOT_FOUND
+	&&  wxDisplay::GetFromPoint(r.GetTopRight   ()) == wxNOT_FOUND
+	&&  wxDisplay::GetFromPoint(r.GetBottomLeft ()) == wxNOT_FOUND
+	&&  wxDisplay::GetFromPoint(r.GetBottomRight()) == wxNOT_FOUND	)
+	{
+		r	= rDefault;
+	}
+
+	SetSize(r);
+
+	if( !(CONFIG_Read("/FL", "STATE", l) && l == 0) )
 	{
 		Maximize();
 	}
 
-#if defined(_SAGA_LINUX)
+	//-----------------------------------------------------
 	Show(true);
-#endif
 
 	Update();
 
 	//-----------------------------------------------------
-	if( g_pSAGA->argc <= 1 && g_pData->Initialise() )
-	{
-		Refresh(false);
-	}
-
-	ProgressBar_Set_Position(0);
+	m_pData_Source->Autoconnect_DB();
 }
 
 //---------------------------------------------------------
@@ -370,25 +394,28 @@ CSAGA_Frame::~CSAGA_Frame(void)
 
 	if( IsMaximized() )
 	{
-		CONFIG_Write(wxT("/FL"), wxT("STATE"), (long)1);
+		CONFIG_Write("/FL", "STATE", (long)1);
 	}
 	else
 	{
-		CONFIG_Write(wxT("/FL"), wxT("STATE"), (long)0);
-		CONFIG_Write(wxT("/FL"), wxT("X"    ), (long)GetPosition().x);
-		CONFIG_Write(wxT("/FL"), wxT("Y"    ), (long)GetPosition().y);
-		CONFIG_Write(wxT("/FL"), wxT("DX"   ), (long)GetSize().x);
-		CONFIG_Write(wxT("/FL"), wxT("DY"   ), (long)GetSize().y);
+		CONFIG_Write("/FL", "STATE", (long)0);
+
+		CONFIG_Write("/FL", "X"    , (long)GetPosition().x);
+		CONFIG_Write("/FL", "Y"    , (long)GetPosition().y);
+		CONFIG_Write("/FL", "DX"   , (long)GetSize    ().x);
+		CONFIG_Write("/FL", "DY"   , (long)GetSize    ().y);
 	}
 
 	//-----------------------------------------------------
-	CONFIG_Write(wxT("/FL"), wxT("MANAGER"), m_pLayout->SavePerspective());
+	CONFIG_Write("/FL", "MANAGER", m_pLayout->SavePerspective());
 
 	m_pLayout->UnInit();
 
 	delete(m_pLayout);
 
 	//-----------------------------------------------------
+	On_Child_Activates(-1);
+
 	delete(m_pMN_Table);
 	delete(m_pMN_Diagram);
 	delete(m_pMN_Map);
@@ -420,15 +447,15 @@ void CSAGA_Frame::On_Close(wxCloseEvent &event)
 {
 	if( event.CanVeto() )
 	{
-		if( !g_pModule && DLG_Message_Confirm(ID_DLG_CLOSE) && g_pData->Close(true) )
+		if( !g_pTool && DLG_Message_Confirm(ID_DLG_CLOSE) && g_pData->Finalise() && g_pData->Close(true) )
 		{
-			g_pModules->Finalise();
+			g_pTools->Finalise();
 
 			Destroy();
 		}
 		else
 		{
-			if( g_pModule )
+			if( g_pTool )
 			{
 				DLG_Message_Show(_TL("Please stop tool execution before exiting SAGA."), _TL("Exit SAGA"));
 			}
@@ -438,7 +465,7 @@ void CSAGA_Frame::On_Close(wxCloseEvent &event)
 	}
 	else
 	{
-		g_pModules->Finalise();
+		g_pTools->Finalise();
 
 		g_pData->Close(true);
 
@@ -449,6 +476,13 @@ void CSAGA_Frame::On_Close(wxCloseEvent &event)
 //---------------------------------------------------------
 void CSAGA_Frame::On_Size(wxSizeEvent &event)
 {
+	if( wxDisplay::GetFromWindow(this) == wxNOT_FOUND )
+	{
+		wxRect	r	= wxGetClientDisplayRect();	r.Deflate((int)(0.1 * r.GetWidth()), (int)(0.1 * r.GetHeight()));
+
+		SetSize(r);
+	}
+
 	event.Skip();
 }
 
@@ -468,7 +502,7 @@ void CSAGA_Frame::On_Quit(wxCommandEvent &WXUNUSED(event))
 //---------------------------------------------------------
 void CSAGA_Frame::On_Help(wxCommandEvent &WXUNUSED(event))
 {
-	if( !Open_WebBrowser(SG_T("http://sourceforge.net/apps/trac/saga-gis/wiki/WikiStart")) )
+	if( !Open_WebBrowser("https://sourceforge.net/p/saga-gis/wiki/Home/") )
 	{
 	//	DLG_Message_Show(_TL("Online Help"), _TL("SAGA Help"));
 	}
@@ -495,10 +529,122 @@ void CSAGA_Frame::On_Tips(wxCommandEvent &WXUNUSED(event))
 //														 //
 ///////////////////////////////////////////////////////////
 
+//---------------------------------------------------------
+void CSAGA_Frame::On_Frame_Unsplit(wxCommandEvent &WXUNUSED(event))
+{
+#ifdef MDI_TABBED
+	int		i, n	= GetNotebook()->GetPageCount();
+
+	if( n > 1 )
+	{
+		GetNotebook()->Freeze();
+
+		class CPage { public: wxWindow *pPage; wxString Text; wxBitmap Bmp; };
+
+		CPage	*pPages	= new CPage[n];
+
+		for(i=n-1; i>=0; i--)
+		{
+			pPages[i].pPage	= GetNotebook()->GetPage      (i);
+			pPages[i].Text	= GetNotebook()->GetPageText  (i);
+			pPages[i].Bmp	= GetNotebook()->GetPageBitmap(i);
+
+			GetNotebook()->RemovePage(i);
+		}
+
+		for(i=0; i<n; i++)
+		{
+			GetNotebook()->AddPage(pPages[i].pPage, pPages[i].Text, false, pPages[i].Bmp);
+		}
+
+		delete[](pPages);
+
+		GetNotebook()->Thaw();
+	}
+#endif
+}
+
+void CSAGA_Frame::On_Frame_Unsplit_UI(wxUpdateUIEvent &event)
+{
+#ifdef MDI_TABBED
+	event.Enable(GetNotebook()->GetPageCount() > 1);
+#endif
+}
+
+//---------------------------------------------------------
+void CSAGA_Frame::On_Frame_Split(wxCommandEvent &event)
+{
+#ifdef MDI_TABBED
+	int		n	= GetNotebook()->GetPageCount();
+
+	if( n > 1 )
+	{
+		GetNotebook()->Freeze();
+
+		int	iActive	= GetNotebook()->GetSelection(); if( iActive < 0 ) iActive = 0;
+		int	nSqrt	= (int)sqrt((double)n);
+
+		switch( event.GetId() )
+		{
+		case ID_CMD_FRAME_SPLIT_LEFT  : GetNotebook()->Split(iActive, wxLEFT  ); break;
+		case ID_CMD_FRAME_SPLIT_RIGHT : GetNotebook()->Split(iActive, wxRIGHT ); break;
+		case ID_CMD_FRAME_SPLIT_TOP   : GetNotebook()->Split(iActive, wxTOP   ); break;
+		case ID_CMD_FRAME_SPLIT_BOTTOM: GetNotebook()->Split(iActive, wxBOTTOM); break;
+
+		case ID_CMD_FRAME_SPLIT_ALL_HORZ:
+			{
+				for(int i=n-1, iRow=0; i>0; i--, iRow++)
+				{
+					GetNotebook()->Split(i, wxLEFT);
+				}
+			}
+			break;
+
+		case ID_CMD_FRAME_SPLIT_ALL_VERT:
+			{
+				for(int i=n-1, iRow=0; i>0; i--, iRow++)
+				{
+					GetNotebook()->Split(i, wxBOTTOM);
+				}
+			}
+			break;
+		}
+
+		GetNotebook()->Thaw();
+	}
+#endif
+}
+
+void CSAGA_Frame::On_Frame_Split_UI(wxUpdateUIEvent &event)
+{
+#ifdef MDI_TABBED
+	event.Enable(GetNotebook()->GetPageCount() > 1);
+#endif
+}
+
+//---------------------------------------------------------
+void CSAGA_Frame::Tile(wxOrientation orient)
+{
+#ifndef MDI_TABBED
+	int		n	= _Get_MDI_Children_Count();
+
+	if( n == 1 && GetActiveChild() )
+	{
+		GetActiveChild()->Maximize();
+	}
+	else
+	{
+		MDI_ParentFrame::Tile(orient);
+	}
+#endif
+}
+
 //---------------------------------------------------------
 void CSAGA_Frame::On_Frame_Cascade(wxCommandEvent &WXUNUSED(event))
 {
+#ifndef MDI_TABBED
 	Cascade();
+#endif
 }
 
 void CSAGA_Frame::On_Frame_Cascade_UI(wxUpdateUIEvent &event)
@@ -578,10 +724,7 @@ void CSAGA_Frame::On_Frame_Close_UI(wxUpdateUIEvent &event)
 //---------------------------------------------------------
 void CSAGA_Frame::On_Frame_Close_All(wxCommandEvent &WXUNUSED(event))
 {
-	while( GetActiveChild() != NULL )
-	{
-		delete(GetActiveChild());
-	}
+	Close_Children();
 }
 
 void CSAGA_Frame::On_Frame_Close_All_UI(wxUpdateUIEvent &event)
@@ -659,20 +802,20 @@ void CSAGA_Frame::On_Command_Workspace_UI(wxUpdateUIEvent &event)
 }
 
 //---------------------------------------------------------
-void CSAGA_Frame::On_Command_Module(wxCommandEvent &event)
+void CSAGA_Frame::On_Command_Tool(wxCommandEvent &event)
 {
-	m_pWKSP->On_Command_Module(event);
+	m_pWKSP->On_Command_Tool(event);
 }
 
-void CSAGA_Frame::On_Command_Module_UI(wxUpdateUIEvent &event)
+void CSAGA_Frame::On_Command_Tool_UI(wxUpdateUIEvent &event)
 {
-	m_pWKSP->On_Command_UI_Module(event);
+	m_pWKSP->On_Command_UI_Tool(event);
 }
 
 //---------------------------------------------------------
 void CSAGA_Frame::On_Command_Child(wxCommandEvent &event)
 {
-	wxMDIChildFrame	*pChild;
+	MDI_ChildFrame	*pChild;
 
 	if( (pChild = GetActiveChild()) != NULL )
 	{
@@ -751,7 +894,7 @@ bool CSAGA_Frame::Process_Set_Okay(bool bOkay)
 {
 	StatusBar_Set_Text(_TL("ready"));
 
-	ProgressBar_Set_Position(-1);
+	m_pProgressBar->SetValue(0);
 
 	g_pSAGA->Process_Set_Okay(bOkay);
 
@@ -763,28 +906,26 @@ bool CSAGA_Frame::ProgressBar_Set_Position(int Position)
 {
 	if( Position < 0 )
 	{
-		m_pProgressBar->SetValue(0);
+		Position	= 0;
 	}
-	else
+	else if( Position > 100 )
 	{
-		if( Position > 100 )
-		{
-			Position	= 100;
-		}
-
-		if( m_pProgressBar->GetValue() != Position )
-		{
-			m_pProgressBar->SetValue(Position);
-		}
+		Position	= 100;
 	}
 
-	return( Process_Get_Okay(false) );
+	if( m_pProgressBar->GetValue() != Position )
+	{
+		m_pProgressBar->SetValue(Position);
+	//	m_pProgressBar->SetLabel(wxString::Format("%d%%", Position));
+	}
+
+	return( g_pSAGA->Process_Get_Okay() );
 }
 
 //---------------------------------------------------------
 bool CSAGA_Frame::ProgressBar_Set_Position(double Position, double Range)
 {
-	return( ProgressBar_Set_Position(Range != 0.0 ? (int)(100.0 * Position / Range) : 0) );
+	return( ProgressBar_Set_Position(Range > 0.0 ? (int)(0.5 + 100.0 * Position / Range) : 0) );
 }
 
 //---------------------------------------------------------
@@ -873,33 +1014,72 @@ wxWindow * CSAGA_Frame::Top_Window_Get(void)
 //														 //
 ///////////////////////////////////////////////////////////
 
+//---------------------------------------------------------
+void CSAGA_Frame::Close_Children(void)
+{
+	while( GetActiveChild() != NULL )
+	{
+		delete(GetActiveChild());
+	}
+}
+
+//---------------------------------------------------------
+int CSAGA_Frame::_Get_MDI_Children_Count(void)
+{
+#ifdef MDI_TABBED
+	return( GetNotebook()->GetPageCount() );
+#else
+	int		n	= 0;
+
+	for(wxWindowList::const_iterator Child=GetChildren().begin(); Child!=GetChildren().end(); Child++)
+	{
+		if( wxDynamicCast(*Child, wxMDIChildFrame) )
+		{
+			n++;
+		}
+	}
+
+	return( n );
+#endif
+}
+
 //---------------------------------------------------------
 void CSAGA_Frame::On_Child_Activates(int View_ID)
 {
+#ifdef MDI_TABBED
+	if( View_ID < 0 && GetNotebook()->GetPageCount() > 0 )	// child view closes, but it's not the last one
+#else
+	if( View_ID < 0 && _Get_MDI_Children_Count    () > 1 )	// child view closes, but it's not the last one
+#endif
+	{
+		return;	// nothing to do, another child will be activated next!
+	}
+
+	//-----------------------------------------------------
 	wxString		Title;
 	wxMenu			*pMenu		= NULL;
 	wxToolBarBase	*pToolBar	= NULL;
 
 	switch( View_ID )
 	{
-	case ID_VIEW_TABLE:			pToolBar = m_pTB_Table      ; pMenu	= m_pMN_Table      ; Title = _TL("Table"      ); break;
-	case ID_VIEW_TABLE_DIAGRAM:	pToolBar = m_pTB_Diagram    ; pMenu	= m_pMN_Diagram    ; Title = _TL("Diagram"    ); break;
-	case ID_VIEW_MAP:			pToolBar = m_pTB_Map        ; pMenu	= m_pMN_Map        ; Title = _TL("Map"        ); break;
-	case ID_VIEW_MAP_3D:		pToolBar = m_pTB_Map_3D     ; pMenu	= m_pMN_Map_3D     ; Title = _TL("3D View"    ); break;
-	case ID_VIEW_HISTOGRAM:		pToolBar = m_pTB_Histogram  ; pMenu	= m_pMN_Histogram  ; Title = _TL("Histogram"  ); break;
-	case ID_VIEW_SCATTERPLOT:	pToolBar = m_pTB_ScatterPlot; pMenu	= m_pMN_ScatterPlot; Title = _TL("Scatterplot"); break;
-	case ID_VIEW_LAYOUT:		pToolBar = m_pTB_Layout     ; pMenu	= m_pMN_Layout     ; Title = _TL("Layout"     ); break;
+	case ID_VIEW_TABLE        : pToolBar = m_pTB_Table      ; pMenu	= m_pMN_Table      ; Title = _TL("Table"      ); break;
+	case ID_VIEW_TABLE_DIAGRAM: pToolBar = m_pTB_Diagram    ; pMenu	= m_pMN_Diagram    ; Title = _TL("Diagram"    ); break;
+	case ID_VIEW_MAP          : pToolBar = m_pTB_Map        ; pMenu	= m_pMN_Map        ; Title = _TL("Map"        ); break;
+	case ID_VIEW_MAP_3D       : pToolBar = m_pTB_Map_3D     ; pMenu	= m_pMN_Map_3D     ; Title = _TL("3D View"    ); break;
+	case ID_VIEW_HISTOGRAM    : pToolBar = m_pTB_Histogram  ; pMenu	= m_pMN_Histogram  ; Title = _TL("Histogram"  ); break;
+	case ID_VIEW_SCATTERPLOT  : pToolBar = m_pTB_ScatterPlot; pMenu	= m_pMN_ScatterPlot; Title = _TL("Scatterplot"); break;
+	case ID_VIEW_LAYOUT       : pToolBar = m_pTB_Layout     ; pMenu	= m_pMN_Layout     ; Title = _TL("Layout"     ); break;
 	}
 
 	//-----------------------------------------------------
-	_Bar_Show(m_pTB_Main		, true);
-	_Bar_Show(m_pTB_Table		, pToolBar == m_pTB_Table);
-	_Bar_Show(m_pTB_Diagram		, pToolBar == m_pTB_Diagram);
-	_Bar_Show(m_pTB_Map			, pToolBar == m_pTB_Map);
-	_Bar_Show(m_pTB_Map_3D		, pToolBar == m_pTB_Map_3D);
-	_Bar_Show(m_pTB_Histogram	, pToolBar == m_pTB_Histogram);
-	_Bar_Show(m_pTB_ScatterPlot	, pToolBar == m_pTB_ScatterPlot);
-	_Bar_Show(m_pTB_Layout		, pToolBar == m_pTB_Layout);
+	_Bar_Show(m_pTB_Main       , true                         );
+	_Bar_Show(m_pTB_Table      , pToolBar == m_pTB_Table      );
+	_Bar_Show(m_pTB_Diagram    , pToolBar == m_pTB_Diagram    );
+	_Bar_Show(m_pTB_Map        , pToolBar == m_pTB_Map        );
+	_Bar_Show(m_pTB_Map_3D     , pToolBar == m_pTB_Map_3D     );
+	_Bar_Show(m_pTB_Histogram  , pToolBar == m_pTB_Histogram  );
+	_Bar_Show(m_pTB_ScatterPlot, pToolBar == m_pTB_ScatterPlot);
+	_Bar_Show(m_pTB_Layout     , pToolBar == m_pTB_Layout     );
 
 	//-----------------------------------------------------
 	wxMenuBar	*pMenuBar	= GetMenuBar();
@@ -948,7 +1128,20 @@ wxMenuBar * CSAGA_Frame::_Create_MenuBar(void)
 	CMD_Menu_Add_Item(pMenu_Window,  true, ID_CMD_FRAME_DATA_SOURCE_SHOW);
 	CMD_Menu_Add_Item(pMenu_Window,  true, ID_CMD_FRAME_INFO_SHOW);
 
-#ifdef __WXMSW__
+#if defined(MDI_TABBED)
+	wxMenu	*pMenu_Split	= new wxMenu;
+	CMD_Menu_Add_Item(pMenu_Split, false, ID_CMD_FRAME_SPLIT_LEFT);
+	CMD_Menu_Add_Item(pMenu_Split, false, ID_CMD_FRAME_SPLIT_RIGHT);
+	CMD_Menu_Add_Item(pMenu_Split, false, ID_CMD_FRAME_SPLIT_TOP);
+	CMD_Menu_Add_Item(pMenu_Split, false, ID_CMD_FRAME_SPLIT_BOTTOM);
+	pMenu_Split->AppendSeparator();
+	CMD_Menu_Add_Item(pMenu_Split, false, ID_CMD_FRAME_SPLIT_ALL_HORZ);
+	CMD_Menu_Add_Item(pMenu_Split, false, ID_CMD_FRAME_SPLIT_ALL_VERT);
+
+	pMenu_Window->AppendSeparator();
+	pMenu_Window->AppendSubMenu(pMenu_Split, _TL("Split"));
+	CMD_Menu_Add_Item(pMenu_Window, false, ID_CMD_FRAME_UNSPLIT);
+#elif defined(__WXMSW__)
 	pMenu_Window->AppendSeparator();
 	CMD_Menu_Add_Item(pMenu_Window, false, ID_CMD_FRAME_CASCADE);
 	CMD_Menu_Add_Item(pMenu_Window, false, ID_CMD_FRAME_TILE_HORZ);
@@ -972,10 +1165,10 @@ wxMenuBar * CSAGA_Frame::_Create_MenuBar(void)
 	//-----------------------------------------------------
 	wxMenuBar	*pMenuBar	= new wxMenuBar;
 
-	pMenuBar->Append(g_pData   ->Get_Menu_Files  ()->Get_Menu(), _TL("File"         ));	// 0
-	pMenuBar->Append(g_pModules->Get_Menu_Modules()            , _TL("Geoprocessing"));	// 1
-	pMenuBar->Append(pMenu_Window                              , _TL("Window"       ));	// 2
-	pMenuBar->Append(pMenu_Help                                , _TL("?"            ));	// 3
+	pMenuBar->Append(g_pData ->Get_Menu_Files()->Get_Menu(), _TL("File"         ));	// 0
+	pMenuBar->Append(g_pTools->Get_Menu_Tools()            , _TL("Geoprocessing"));	// 1
+	pMenuBar->Append(pMenu_Window                          , _TL("Window"       ));	// 2
+	pMenuBar->Append(pMenu_Help                            , _TL("?"            ));	// 3
 
 	SetMenuBar(pMenuBar);
 
@@ -1004,23 +1197,22 @@ void CSAGA_Frame::_Bar_Add(wxWindow *pWindow, int Position, int Row)
 {
 	wxAuiPaneInfo	Pane;
 
-	Pane.Name			(pWindow->GetName());
-	Pane.Caption		(pWindow->GetName());
-	Pane.MinSize		(100, 100);
-	Pane.BestSize		(400, 400);
-	Pane.FloatingSize	(400, 400);
-	Pane.Position		(0);
-	Pane.Layer			(Row);
-	Pane.Row			(Row);
+	Pane.Name        (wxString::Format("PANE_%d", pWindow->GetId()));
+	Pane.Caption     (pWindow->GetName());
+	Pane.MinSize     (100, 100);
+	Pane.BestSize    (400, 400);
+	Pane.FloatingSize(400, 400);
+	Pane.Position    (0);
+	Pane.Layer       (Row);
+	Pane.Row         (Row);
 
 	switch( Position )
 	{
-	default:
-	case 0:	Pane.Bottom();	break;
-	case 1:	Pane.Right ();	break;
-	case 2:	Pane.Left  ();	break;
-	case 3:	Pane.Top   ();	break;
-	case 4:	Pane.Center();	break;
+	default:	Pane.Bottom();	break;
+	case  1:	Pane.Right ();	break;
+	case  2:	Pane.Left  ();	break;
+	case  3:	Pane.Top   ();	break;
+	case  4:	Pane.Center();	break;
 	}
 
 	m_pLayout->AddPane(pWindow, Pane);
@@ -1062,15 +1254,14 @@ void CSAGA_Frame::_Bar_Show(wxWindow *pWindow, bool bShow)
 //														 //
 ///////////////////////////////////////////////////////////
 
-//---------------------------------------------------------
-#define TOOLBAR_SIZE_IMG		16
-
 //---------------------------------------------------------
 wxToolBarBase * CSAGA_Frame::TB_Create(int ID)
 {
+	int	Size	= g_pTools->Get_Parameter("LOOK_TB_SIZE")->asInt();
+
 	wxToolBar	*pToolBar	= new wxToolBar(this, ID, wxDefaultPosition, wxDefaultSize, wxTB_HORIZONTAL|wxTB_FLAT|wxTB_NODIVIDER);
 
-	pToolBar->SetToolBitmapSize(wxSize(TOOLBAR_SIZE_IMG, TOOLBAR_SIZE_IMG));
+	pToolBar->SetToolBitmapSize(wxSize(Size, Size));
 
 	return( pToolBar );
 }
@@ -1081,14 +1272,14 @@ void CSAGA_Frame::TB_Add(wxToolBarBase *pToolBar, const wxString &Name)
 	pToolBar->Realize();
 
 	m_pLayout->AddPane(pToolBar, wxAuiPaneInfo()
-		.Name			(Name)
-		.Caption		(Name)
-		.ToolbarPane	()
-		.Top			()
-		.LeftDockable	(false)
-		.RightDockable	(false)
-		.Hide			()
-		.BestSize		(pToolBar->GetBestSize())
+		.Name         (wxString::Format("PANE_%d", pToolBar->GetId()))
+		.Caption      (Name)
+		.ToolbarPane  ()
+		.Top          ()
+		.LeftDockable (false)
+		.RightDockable(false)
+		.Hide         ()
+		.BestSize     (pToolBar->GetBestSize())
 	);
 }
 
@@ -1096,9 +1287,9 @@ void CSAGA_Frame::TB_Add(wxToolBarBase *pToolBar, const wxString &Name)
 void CSAGA_Frame::TB_Add_Item(wxToolBarBase *pToolBar, bool bCheck, int Cmd_ID)
 {
 	if( bCheck )
-		((wxToolBar *)pToolBar)->AddTool(Cmd_ID, CMD_Get_Name(Cmd_ID), IMG_Get_Bitmap(CMD_Get_ImageID(Cmd_ID), TOOLBAR_SIZE_IMG), CMD_Get_Help(Cmd_ID), wxITEM_CHECK);
+		((wxToolBar *)pToolBar)->AddTool(Cmd_ID, CMD_Get_Name(Cmd_ID), IMG_Get_Bitmap(CMD_Get_ImageID(Cmd_ID), pToolBar->GetToolBitmapSize()), CMD_Get_Help(Cmd_ID), wxITEM_CHECK);
 	else
-		((wxToolBar *)pToolBar)->AddTool(Cmd_ID, CMD_Get_Name(Cmd_ID), IMG_Get_Bitmap(CMD_Get_ImageID(Cmd_ID), TOOLBAR_SIZE_IMG), CMD_Get_Help(Cmd_ID));
+		((wxToolBar *)pToolBar)->AddTool(Cmd_ID, CMD_Get_Name(Cmd_ID), IMG_Get_Bitmap(CMD_Get_ImageID(Cmd_ID), pToolBar->GetToolBitmapSize()), CMD_Get_Help(Cmd_ID));
 }
 
 //---------------------------------------------------------
@@ -1112,15 +1303,15 @@ wxToolBarBase * CSAGA_Frame::_Create_ToolBar(void)
 {
 	wxToolBarBase	*pToolBar	= TB_Create(ID_TB_MAIN);
 
-	CMD_ToolBar_Add_Item(pToolBar, false, ID_CMD_WKSP_OPEN);
-	CMD_ToolBar_Add_Item(pToolBar, false, ID_CMD_WKSP_SAVE);
+	CMD_ToolBar_Add_Item(pToolBar, false, ID_CMD_DATA_OPEN);
+	CMD_ToolBar_Add_Item(pToolBar, false, ID_CMD_DATA_PROJECT_SAVE);
 	CMD_ToolBar_Add_Separator(pToolBar);
 	CMD_ToolBar_Add_Item(pToolBar, true , ID_CMD_FRAME_WKSP_SHOW);
 	CMD_ToolBar_Add_Item(pToolBar, true , ID_CMD_FRAME_ACTIVE_SHOW);
 	CMD_ToolBar_Add_Item(pToolBar, true , ID_CMD_FRAME_DATA_SOURCE_SHOW);
 	CMD_ToolBar_Add_Item(pToolBar, true , ID_CMD_FRAME_INFO_SHOW);
 	CMD_ToolBar_Add_Separator(pToolBar);
-	CMD_ToolBar_Add_Item(pToolBar, false, ID_CMD_MODULES_SEARCH);
+	CMD_ToolBar_Add_Item(pToolBar, false, ID_CMD_TOOL_SEARCH);
 	CMD_ToolBar_Add_Separator(pToolBar);
 	CMD_ToolBar_Add_Item(pToolBar, false, ID_CMD_FRAME_HELP);
 
