@@ -1,6 +1,3 @@
-/**********************************************************
- * Version $Id: view_map_control.cpp 2482 2015-04-30 15:34:11Z oconrad $
- *********************************************************/
 
 ///////////////////////////////////////////////////////////
 //                                                       //
@@ -24,7 +21,8 @@
 // Geoscientific Analyses'. SAGA is free software; you   //
 // can redistribute it and/or modify it under the terms  //
 // of the GNU General Public License as published by the //
-// Free Software Foundation; version 2 of the License.   //
+// Free Software Foundation, either version 2 of the     //
+// License, or (at your option) any later version.       //
 //                                                       //
 // SAGA is distributed in the hope that it will be       //
 // useful, but WITHOUT ANY WARRANTY; without even the    //
@@ -33,10 +31,8 @@
 // License for more details.                             //
 //                                                       //
 // You should have received a copy of the GNU General    //
-// Public License along with this program; if not,       //
-// write to the Free Software Foundation, Inc.,          //
-// 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, //
-// USA.                                                  //
+// Public License along with this program; if not, see   //
+// <http://www.gnu.org/licenses/>.                       //
 //                                                       //
 //-------------------------------------------------------//
 //                                                       //
@@ -51,15 +47,6 @@
 //                                                       //
 ///////////////////////////////////////////////////////////
 
-//---------------------------------------------------------
-
-
-///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
-///////////////////////////////////////////////////////////
-
 //---------------------------------------------------------
 #include <wx/window.h>
 #include <wx/image.h>
@@ -71,7 +58,7 @@
 #include "dc_helper.h"
 
 #include "wksp_map.h"
-#include "wksp_module.h"
+#include "wksp_tool.h"
 #include "wksp_grid.h"
 #include "wksp_shapes.h"
 
@@ -86,12 +73,13 @@
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-#define GET_KEYS(e)	((e.LeftIsDown()   ? MODULE_INTERACTIVE_KEY_LEFT   : 0)\
-					|(e.MiddleIsDown() ? MODULE_INTERACTIVE_KEY_MIDDLE : 0)\
-					|(e.RightIsDown()  ? MODULE_INTERACTIVE_KEY_RIGHT  : 0)\
-					|(e.ShiftDown()    ? MODULE_INTERACTIVE_KEY_SHIFT  : 0)\
-					|(e.AltDown()      ? MODULE_INTERACTIVE_KEY_ALT    : 0)\
-					|(e.ControlDown()  ? MODULE_INTERACTIVE_KEY_CTRL   : 0))
+#define GET_KEYS(e)	\
+	((e.LeftIsDown  () ? TOOL_INTERACTIVE_KEY_LEFT   : 0)\
+	|(e.MiddleIsDown() ? TOOL_INTERACTIVE_KEY_MIDDLE : 0)\
+	|(e.RightIsDown () ? TOOL_INTERACTIVE_KEY_RIGHT  : 0)\
+	|(e.ShiftDown   () ? TOOL_INTERACTIVE_KEY_SHIFT  : 0)\
+	|(e.AltDown     () ? TOOL_INTERACTIVE_KEY_ALT    : 0)\
+	|(e.ControlDown () ? TOOL_INTERACTIVE_KEY_CTRL   : 0))
 
 //---------------------------------------------------------
 #define STANDARD_ZOOM_FACTOR	50
@@ -99,8 +87,6 @@
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -125,8 +111,6 @@ END_EVENT_TABLE()
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -141,7 +125,9 @@ CVIEW_Map_Control::CVIEW_Map_Control(CVIEW_Map *pParent, CWKSP_Map *pMap)
 	m_Mode		= -1;
 	Set_Mode(MAP_MODE_ZOOM);
 
-	m_Drag_Mode	= MODULE_INTERACTIVE_DRAG_NONE;
+	m_Drag_Mode	= TOOL_INTERACTIVE_DRAG_NONE;
+
+	m_CrossHair.x = m_CrossHair.y = -1;
 }
 
 //---------------------------------------------------------
@@ -153,8 +139,6 @@ CVIEW_Map_Control::~CVIEW_Map_Control(void)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -200,7 +184,7 @@ bool CVIEW_Map_Control::Set_Mode(int Mode)
 }
 
 //---------------------------------------------------------
-inline void CVIEW_Map_Control::_Set_StatusBar(CSG_Point ptWorld)
+inline void CVIEW_Map_Control::_Set_StatusBar(const TSG_Point &Point)
 {
 	static bool	bBuisy	= false;
 
@@ -208,62 +192,37 @@ inline void CVIEW_Map_Control::_Set_StatusBar(CSG_Point ptWorld)
 	{
 		bBuisy	= true;
 
-		CSG_Module	*pProjector	= NULL;
-
-		if( m_pMap->Get_Parameter("GCS_POSITION")->asBool() && m_pMap->Get_Projection().is_Okay() && (pProjector = SG_Get_Module_Library_Manager().Get_Module("pj_proj4", 2)) != NULL )	// Coordinate Transformation (Shapes)
+		if( m_pMap->Get_Parameter("GCS_POSITION")->asBool() && m_pMap->Get_Projection().is_Okay() )
 		{
-			if( pProjector->is_Executing() )
+			CSG_Projection	GCS; GCS.Set_GCS_WGS84(); TSG_Point Position = Point;
+
+			if( SG_Get_Projected(m_pMap->Get_Projection(), GCS, Position) )
 			{
-				pProjector	= NULL;
-			}
-			else
-			{
-				SG_UI_Progress_Lock(true);
-				SG_UI_Msg_Lock     (true);
-
-				CSG_Shapes	prj(SHAPE_TYPE_Point), gcs(SHAPE_TYPE_Point); prj.Add_Shape()->Add_Point(ptWorld); prj.Get_Projection().Assign(m_pMap->Get_Projection());
-
-				pProjector->Settings_Push(NULL);
-
-				if( pProjector->Set_Parameter("CRS_PROJ4", SG_T("+proj=longlat +ellps=WGS84 +datum=WGS84"))
-				&&  pProjector->Set_Parameter("SOURCE"   , &prj)
-				&&  pProjector->Set_Parameter("TARGET"   , &gcs)
-				&&  pProjector->Execute() )
-				{
-					ptWorld	= gcs.Get_Shape(0)->Get_Point(0);
-
-					STATUSBAR_Set_Text(wxString::Format("X %s", SG_Double_To_Degree(ptWorld.Get_X()).c_str()), STATUSBAR_VIEW_X);
-					STATUSBAR_Set_Text(wxString::Format("Y %s", SG_Double_To_Degree(ptWorld.Get_Y()).c_str()), STATUSBAR_VIEW_Y);
-
-					pProjector->Settings_Pop();
-				}
-				else
-				{
-					pProjector->Settings_Pop();		pProjector	= NULL;
-				}
-
-				SG_UI_Progress_Lock(false);
-				SG_UI_Msg_Lock     (false);
+				STATUSBAR_Set_Text(wxString::Format("X %s", SG_Double_To_Degree(Position.x).c_str()), STATUSBAR_VIEW_X);
+				STATUSBAR_Set_Text(wxString::Format("Y %s", SG_Double_To_Degree(Position.y).c_str()), STATUSBAR_VIEW_Y);
 			}
 		}
-
-		if( !pProjector )
+		else
 		{
-			STATUSBAR_Set_Text(wxString::Format("X %f", ptWorld.Get_X()), STATUSBAR_VIEW_X);
-			STATUSBAR_Set_Text(wxString::Format("Y %f", ptWorld.Get_Y()), STATUSBAR_VIEW_Y);
+			STATUSBAR_Set_Text(wxString::Format("X %f", Point.x), STATUSBAR_VIEW_X);
+			STATUSBAR_Set_Text(wxString::Format("Y %f", Point.y), STATUSBAR_VIEW_Y);
 		}
 
 		if( m_Mode == MAP_MODE_DISTANCE )
 		{
-			STATUSBAR_Set_Text(wxString::Format(wxT("D %f"), m_Distance + m_Distance_Move), STATUSBAR_VIEW_Z);
+			STATUSBAR_Set_Text(wxString::Format("D %f", m_Distance + m_Distance_Move), STATUSBAR_VIEW_Z);
 		}
 		else if( Get_Active_Layer() )
 		{
-			STATUSBAR_Set_Text(wxString::Format(wxT("Z %s"), Get_Active_Layer()->Get_Value(ptWorld, _Get_World(2.0)).c_str()), STATUSBAR_VIEW_Z);
+			TSG_Point	p(Point);
+
+			SG_Get_Projected(m_pMap->Get_Projection(), Get_Active_Layer()->Get_Object()->Get_Projection(), p);
+
+			STATUSBAR_Set_Text(wxString::Format("Z %s", Get_Active_Layer()->Get_Value(p, _Get_Client2World(2.)).c_str()), STATUSBAR_VIEW_Z);
 		}
 		else
 		{
-			STATUSBAR_Set_Text(wxT("Z"), STATUSBAR_VIEW_Z);
+			STATUSBAR_Set_Text("Z", STATUSBAR_VIEW_Z);
 		}
 
 		bBuisy	= false;
@@ -273,54 +232,97 @@ inline void CVIEW_Map_Control::_Set_StatusBar(CSG_Point ptWorld)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-inline wxRect CVIEW_Map_Control::_Get_Client(void)
+inline wxPoint CVIEW_Map_Control::_Get_World2Client(const TSG_Point &Point)
 {
-	return( wxRect(wxPoint(0, 0), GetClientSize()) );
-}
-
-//---------------------------------------------------------
-inline wxPoint CVIEW_Map_Control::_Get_Client(TSG_Point Point)
-{
-	wxRect		rClient(_Get_Client());
+	wxRect		rClient(GetClientSize());
 	CSG_Rect	rWorld(m_pMap->Get_World(rClient));
-	double		World2DC	= (double)rClient.GetWidth() / rWorld.Get_XRange();
+	double		World2Client	= (double)rClient.GetWidth() / rWorld.Get_XRange();
 
-	return( wxPoint(          (int)(0.5 + (Point.x - rWorld.Get_XMin()) * World2DC),
-		rClient.GetHeight() - (int)(0.5 + (Point.y - rWorld.Get_YMin()) * World2DC)
+	return( wxPoint(          (int)(0.5 + (Point.x - rWorld.Get_XMin()) * World2Client),
+		rClient.GetHeight() - (int)(0.5 + (Point.y - rWorld.Get_YMin()) * World2Client)
 	));
 }
 
 //---------------------------------------------------------
-inline CSG_Point CVIEW_Map_Control::_Get_World(wxPoint ptClient)
+inline CSG_Point CVIEW_Map_Control::_Get_Client2World(const wxPoint &Point, bool bToActive)
 {
-	return( m_pMap->Get_World(_Get_Client(), ptClient) );
+	TSG_Point	pWorld	= m_pMap->Get_World(wxRect(GetClientSize()), Point);
+
+	if( bToActive && m_pMap->Find_Active() )
+	{
+		SG_Get_Projected(m_pMap->Get_Projection(), Get_Active_Layer()->Get_Object()->Get_Projection(), pWorld);
+	}
+
+	return( pWorld );
 }
 
 //---------------------------------------------------------
-inline double CVIEW_Map_Control::_Get_World(double xClient)
+inline double CVIEW_Map_Control::_Get_Client2World(double Length, bool bToActive)
 {
-	wxRect		rClient(_Get_Client());
-	CSG_Rect	rWorld(m_pMap->Get_World(rClient));
+	wxRect	rClient(GetClientSize());	CSG_Rect	rWorld(m_pMap->Get_World(rClient));
 
-	return( xClient * rWorld.Get_XRange() / (double)rClient.GetWidth() );
+	if( bToActive && m_pMap->Find_Active() )
+	{
+		SG_Get_Projected(m_pMap->Get_Projection(), Get_Active_Layer()->Get_Object()->Get_Projection(), rWorld.m_rect);
+	}
+
+	return( Length * rWorld.Get_XRange() / (double)rClient.GetWidth() );
 }
 
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+void CVIEW_Map_Control::Set_CrossHair(const TSG_Point &Point)
+{
+	_Draw_CrossHair(m_CrossHair);
+	_Draw_CrossHair(m_CrossHair = _Get_World2Client(Point));
+}
+
+//---------------------------------------------------------
+void CVIEW_Map_Control::Set_CrossHair_Off(void)
+{
+	_Draw_CrossHair(m_CrossHair);
+
+	m_CrossHair.x = m_CrossHair.y = -1;
+}
+
+//---------------------------------------------------------
+bool CVIEW_Map_Control::_Draw_CrossHair(const wxPoint &Point)
+{
+	wxRect	r(GetClientSize());
+
+	if( r.Contains(Point) )
+	{
+		wxClientDC	dc(this);
+
+		dc.SetLogicalFunction(wxINVERT);
+
+	//	dc.CrossHair(Point);
+
+		dc.DrawLine(Point.x, r.GetTop(), Point.x, r.GetBottom());
+		dc.DrawLine(r.GetLeft(), Point.y, r.GetRight(), Point.y);
+
+		return( true );
+	}
+
+	return( false );
+}
+
+
+///////////////////////////////////////////////////////////
 //														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-void CVIEW_Map_Control::_Draw_Inverse(wxPoint ptA, wxPoint ptB)
+void CVIEW_Map_Control::_Draw_Inverse(const wxPoint &A, const wxPoint &B)
 {
-	if( m_Drag_Mode != MODULE_INTERACTIVE_DRAG_NONE )
+	if( m_Drag_Mode != TOOL_INTERACTIVE_DRAG_NONE )
 	{
 		wxClientDC	dc(this);
 
@@ -328,79 +330,52 @@ void CVIEW_Map_Control::_Draw_Inverse(wxPoint ptA, wxPoint ptB)
 
 		switch( m_Drag_Mode )
 		{
-		case MODULE_INTERACTIVE_DRAG_LINE:
-			dc.DrawLine			(ptA.x, ptA.y, ptB.x, ptB.y);
+		case TOOL_INTERACTIVE_DRAG_LINE:
+			dc.DrawLine     (A.x, A.y, B.x, B.y);
 			break;
 
-		case MODULE_INTERACTIVE_DRAG_BOX:
-			dc.DrawRectangle	(ptA.x, ptA.y, ptB.x - ptA.x, ptB.y - ptA.y);
+		case TOOL_INTERACTIVE_DRAG_BOX:
+			dc.DrawRectangle(A.x, A.y, B.x - A.x, B.y - A.y);
 			break;
 
-		case MODULE_INTERACTIVE_DRAG_CIRCLE:
-			dc.DrawCircle		(ptA.x, ptA.y, (int)SG_Get_Distance(ptA.x, ptA.y, ptB.x, ptB.y));
+		case TOOL_INTERACTIVE_DRAG_CIRCLE:
+			dc.DrawCircle   (A.x, A.y, (int)SG_Get_Distance(A.x, A.y, B.x, B.y));
 			break;
 		}
 	}
 }
 
 //---------------------------------------------------------
-void CVIEW_Map_Control::_Draw_Inverse(wxPoint ptA, wxPoint ptB_Old, wxPoint ptB_New)
+void CVIEW_Map_Control::_Draw_Inverse(const wxPoint &A, const wxPoint &B_Old, const wxPoint &B_New)
 {
-	if( m_Drag_Mode != MODULE_INTERACTIVE_DRAG_NONE )
-	{
-		wxClientDC	dc(this);
-
-		dc.SetLogicalFunction(wxINVERT);
-
-		switch( m_Drag_Mode )
-		{
-		case MODULE_INTERACTIVE_DRAG_LINE:
-			dc.DrawLine			(ptA.x, ptA.y, ptB_Old.x, ptB_Old.y);
-			dc.DrawLine			(ptA.x, ptA.y, ptB_New.x, ptB_New.y);
-			break;
-
-		case MODULE_INTERACTIVE_DRAG_BOX:
-			dc.DrawRectangle	(ptA.x, ptA.y, ptB_Old.x - ptA.x, ptB_Old.y - ptA.y);
-			dc.DrawRectangle	(ptA.x, ptA.y, ptB_New.x - ptA.x, ptB_New.y - ptA.y);
-			break;
-
-		case MODULE_INTERACTIVE_DRAG_CIRCLE:
-			dc.DrawCircle		(ptA.x, ptA.y, (int)SG_Get_Distance(ptA.x, ptA.y, ptB_Old.x, ptB_Old.y));
-			dc.DrawCircle		(ptA.x, ptA.y, (int)SG_Get_Distance(ptA.x, ptA.y, ptB_New.x, ptB_New.y));
-			break;
-		}
-	}
+	_Draw_Inverse(A, B_Old);
+	_Draw_Inverse(A, B_New);
 }
 
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-bool CVIEW_Map_Control::_Zoom(wxPoint A, wxPoint B)
+bool CVIEW_Map_Control::_Zoom(const wxPoint &A, const wxPoint &B)
 {
-	CSG_Rect	rWorld;
-
 	if( A.x == B.x && A.y == B.y )
 	{
-		return( _Zoom(_Get_World(A), true) );
+		return( _Zoom(_Get_Client2World(A), true) );
 	}
 
-	rWorld.Assign(_Get_World(A), _Get_World(B));
-
-	m_pMap->Set_Extent(rWorld);
+	m_pMap->Set_Extent(CSG_Rect(_Get_Client2World(A), _Get_Client2World(B)));
 
 	return( true );
 }
 
-bool CVIEW_Map_Control::_Zoom(CSG_Point ptCenter, bool bZoomIn)
+//---------------------------------------------------------
+bool CVIEW_Map_Control::_Zoom(const CSG_Point &Center, bool bZoomIn)
 {
 	CSG_Rect	rWorld(m_pMap->Get_Extent());
 
-	rWorld.Move(ptCenter - rWorld.Get_Center());
+	rWorld.Move(Center - rWorld.Get_Center());
 
 	if( bZoomIn )
 	{
@@ -419,12 +394,10 @@ bool CVIEW_Map_Control::_Zoom(CSG_Point ptCenter, bool bZoomIn)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-bool CVIEW_Map_Control::_Pan(wxPoint A, wxPoint B)
+bool CVIEW_Map_Control::_Pan(const wxPoint &A, const wxPoint &B)
 {
 	if( A.x != B.x || A.y != B.y )
 	{
@@ -441,14 +414,14 @@ bool CVIEW_Map_Control::_Pan(wxPoint A, wxPoint B)
 }
 
 //---------------------------------------------------------
-bool CVIEW_Map_Control::_Move(wxPoint &A, wxPoint B)
+bool CVIEW_Map_Control::_Move(wxPoint &A, const wxPoint &B)
 {
 	if( A.x != B.x || A.y != B.y )
 	{
 		CSG_Rect	rWorld;
 
 		rWorld.Assign(m_pMap->Get_Extent());
-		rWorld.Move(_Get_World(A) - _Get_World(B));
+		rWorld.Move(_Get_Client2World(A) - _Get_Client2World(B));
 		m_pMap->Set_Extent(rWorld);
 
 		A	= B;
@@ -459,16 +432,17 @@ bool CVIEW_Map_Control::_Move(wxPoint &A, wxPoint B)
 	return( false );
 }
 
-bool CVIEW_Map_Control::_Move(wxPoint ptMove)
+//---------------------------------------------------------
+bool CVIEW_Map_Control::_Move(const wxPoint &Move)
 {
-	return( _Move(ptMove, wxPoint(0, 0)) );
+	wxPoint	A(Move), B(0, 0);
+
+	return( _Move(A, B) );
 }
 
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -483,11 +457,11 @@ void CVIEW_Map_Control::_Distance_Reset(void)
 }
 
 //---------------------------------------------------------
-void CVIEW_Map_Control::_Distance_Add(wxPoint Point)
+void CVIEW_Map_Control::_Distance_Add(const wxPoint &Point)
 {
 	int		n	= m_Distance_Pts.Get_Count();
 
-	m_Distance_Pts.Add(_Get_World(Point));
+	m_Distance_Pts.Add(_Get_Client2World(Point));
 
 	if( n > 0 )
 	{
@@ -497,6 +471,7 @@ void CVIEW_Map_Control::_Distance_Add(wxPoint Point)
 	m_Distance_Move	= 0.0;
 
 	wxClientDC	dc(this);
+	
 	_Distance_Draw(dc);
 }
 
@@ -511,24 +486,24 @@ void CVIEW_Map_Control::_Distance_Draw(wxDC &dc)
 		wxPoint	A, B;
 
 		dc.SetPen(wxPen(*wxWHITE, 4));
-		for(i=1, A=_Get_Client(m_Distance_Pts[0]); i<n; i++)
+		for(i=1, A=_Get_World2Client(m_Distance_Pts[0]); i<n; i++)
 		{
 			B	= A;
-			A	= _Get_Client(m_Distance_Pts[i]);
+			A	= _Get_World2Client(m_Distance_Pts[i]);
 			dc.DrawLine(A.x, A.y, B.x, B.y);
 		}
 
 		dc.SetPen(wxPen(*wxBLACK, 2));
-		for(i=1, A=_Get_Client(m_Distance_Pts[0]); i<n; i++)
+		for(i=1, A=_Get_World2Client(m_Distance_Pts[0]); i<n; i++)
 		{
 			B	= A;
-			A	= _Get_Client(m_Distance_Pts[i]);
+			A	= _Get_World2Client(m_Distance_Pts[i]);
 			dc.DrawLine(A.x, A.y, B.x, B.y);
 		}
 
 		dc.SetPen(wxNullPen);
 		dc.SetLogicalFunction(wxINVERT);
-		A	= _Get_Client(m_Distance_Pts[n - 1]);
+		A	= _Get_World2Client(m_Distance_Pts[n - 1]);
 		dc.DrawLine(A.x, A.y, m_Mouse_Move.x, m_Mouse_Move.y);
 	}
 }
@@ -536,8 +511,6 @@ void CVIEW_Map_Control::_Distance_Draw(wxDC &dc)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -550,13 +523,15 @@ void CVIEW_Map_Control::On_Paint(wxPaintEvent &event)
 		dc.DrawBitmap(m_Bitmap, 0, 0, false);
 
 		_Distance_Draw(dc);
+
+		m_CrossHair.x = m_CrossHair.y = -1;
 	}
 }
 
 //---------------------------------------------------------
 void CVIEW_Map_Control::On_Size(wxSizeEvent &event)
 {
-	wxRect	r(_Get_Client());
+	wxRect	r(GetClientSize());
 
 	if( !m_Bitmap.Ok() || m_Bitmap.GetWidth() != r.GetWidth() || m_Bitmap.GetHeight() != r.GetHeight() )
 	{
@@ -574,7 +549,7 @@ void CVIEW_Map_Control::Refresh_Map(void)
 		wxMemoryDC	dc;
 
 		dc.SelectObject(m_Bitmap);
-		m_pMap->Draw_Map(dc, 1.0, _Get_Client(), true);
+		m_pMap->Draw_Map(dc, 1.0, GetClientSize());
 		dc.SelectObject(wxNullBitmap);
 
 		m_pParent->Ruler_Refresh();
@@ -586,14 +561,12 @@ void CVIEW_Map_Control::Refresh_Map(void)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
 void CVIEW_Map_Control::On_Key_Down(wxKeyEvent &event)
 {
-	if( m_pMap->Find_Layer(Get_Active_Layer()) && Get_Active_Layer()->Edit_On_Key_Down(event.GetKeyCode()) )
+	if( m_pMap->Find_Active(true) && Get_Active_Layer()->Edit_On_Key_Down(event.GetKeyCode()) )
 	{
 		return;
 	}
@@ -607,11 +580,11 @@ void CVIEW_Map_Control::On_Key_Down(wxKeyEvent &event)
 	case WXK_PAGEUP:
 		_Zoom(m_pMap->Get_Extent().Get_Center(), true);
 		break;
-		
+
 	case WXK_PAGEDOWN:
 		_Zoom(m_pMap->Get_Extent().Get_Center(), false);
 		break;
-		
+
 	case WXK_LEFT:
 		_Move(wxPoint(-10,   0));
 		break;
@@ -654,68 +627,83 @@ void CVIEW_Map_Control::On_Key_Down(wxKeyEvent &event)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
 void CVIEW_Map_Control::On_Mouse_LDown(wxMouseEvent &event)
 {
-	bool	bCaptureMouse	= true;
-
 	m_Mouse_Down	= m_Mouse_Move	= event.GetPosition();
 
+	//-----------------------------------------------------
+	if( m_Mode != MAP_MODE_SELECT )	// clipboard copy ?
+	{
+		if( event.ControlDown() )
+		{
+			m_pMap->SaveAs_Image_Clipboard(false);
+
+			return;
+		}
+
+		if( event.AltDown() )
+		{
+			m_pMap->SaveAs_Image_Clipboard(GetClientSize().x, GetClientSize().y, -1);
+
+			return;
+		}
+	}
+
+	//-----------------------------------------------------
+	bool	bCaptureMouse	= true;
+
 	switch( m_Mode )
 	{
 	//-----------------------------------------------------
 	case MAP_MODE_SELECT:
-		if( g_pModule && g_pModule->is_Interactive() )
+		if( g_pTool && g_pTool->is_Interactive() )
 		{
-			m_Drag_Mode		= ((CSG_Module_Interactive *)g_pModule->Get_Module())->Get_Drag_Mode();
-			bCaptureMouse	= !g_pModule->Execute(_Get_World(event.GetPosition()), MODULE_INTERACTIVE_LDOWN, GET_KEYS(event));
+			m_Drag_Mode		= ((CSG_Tool_Interactive *)g_pTool->Get_Tool())->Get_Drag_Mode();
+			bCaptureMouse	= !g_pTool->Execute(_Get_Client2World(event.GetPosition()), TOOL_INTERACTIVE_LDOWN, GET_KEYS(event));
 		}
-		else if( m_pMap->Find_Layer(Get_Active_Layer()) )
+		else if( m_pMap->Find_Active(false) )
 		{
 			switch(	Get_Active_Layer()->Get_Type() )
 			{
-			default:
-				m_Drag_Mode		= MODULE_INTERACTIVE_DRAG_NONE;
-				break;
-
-			case WKSP_ITEM_Grid:
-				m_Drag_Mode		= MODULE_INTERACTIVE_DRAG_BOX;
-				break;
-
-			case WKSP_ITEM_Shapes:
-				m_Drag_Mode		= ((CWKSP_Shapes *)Get_Active_Layer())->is_Editing()
-								? MODULE_INTERACTIVE_DRAG_NONE
-								: MODULE_INTERACTIVE_DRAG_BOX;
-				break;
+			default                  : m_Drag_Mode = TOOL_INTERACTIVE_DRAG_NONE; break;
+			case WKSP_ITEM_Grid      :
+			case WKSP_ITEM_PointCloud: m_Drag_Mode = TOOL_INTERACTIVE_DRAG_BOX ; break;
+			case WKSP_ITEM_Shapes    : m_Drag_Mode = ((CWKSP_Shapes *)Get_Active_Layer())->is_Editing()
+												   ? TOOL_INTERACTIVE_DRAG_NONE
+												   : TOOL_INTERACTIVE_DRAG_BOX ; break;
 			}
 
-			Get_Active_Layer()->Edit_On_Mouse_Down(_Get_World(event.GetPosition()), _Get_World(1.0), GET_KEYS(event));
+			if( m_pMap->Find_Active(true) )
+			{
+				Get_Active_Layer()->Edit_On_Mouse_Down(
+					_Get_Client2World(event.GetPosition()), _Get_Client2World(1.), GET_KEYS(event)
+				);
+			}
 		}
 		break;
 
 	//-----------------------------------------------------
 	case MAP_MODE_DISTANCE:
-		m_Drag_Mode		= MODULE_INTERACTIVE_DRAG_NONE;
+		m_Drag_Mode		= TOOL_INTERACTIVE_DRAG_NONE;
 		break;
 
 	//-----------------------------------------------------
 	case MAP_MODE_ZOOM:
-		m_Drag_Mode		= MODULE_INTERACTIVE_DRAG_BOX;
+		m_Drag_Mode		= TOOL_INTERACTIVE_DRAG_BOX;
 		break;
 
 	//-----------------------------------------------------
 	case MAP_MODE_PAN:
-		m_Drag_Mode		= MODULE_INTERACTIVE_DRAG_NONE;
+		m_Drag_Mode		= TOOL_INTERACTIVE_DRAG_NONE;
 		Set_Mode(MAP_MODE_PAN_DOWN);
 		break;
 
 	//-----------------------------------------------------
 	case MAP_MODE_PAN_DOWN:
-		m_Drag_Mode		= MODULE_INTERACTIVE_DRAG_NONE;
+		m_Drag_Mode		= TOOL_INTERACTIVE_DRAG_NONE;
 		break;
 	}
 
@@ -736,20 +724,36 @@ void CVIEW_Map_Control::On_Mouse_LUp(wxMouseEvent &event)
 		ReleaseMouse();
 	}
 
+	//-----------------------------------------------------
+	if( m_Mode != MAP_MODE_SELECT && (event.ControlDown() || event.AltDown()) )	// clipboard copy ?
+	{
+		return;
+	}
+
+	//-----------------------------------------------------
 	_Draw_Inverse(m_Mouse_Down, event.GetPosition());
-	m_Drag_Mode	= MODULE_INTERACTIVE_DRAG_NONE;
+	m_Drag_Mode	= TOOL_INTERACTIVE_DRAG_NONE;
 
 	switch( m_Mode )
 	{
 	//-----------------------------------------------------
 	case MAP_MODE_SELECT:
-		if( g_pModule )
+		if( g_pTool )
 		{
-			g_pModule->Execute(_Get_World(event.GetPosition()), MODULE_INTERACTIVE_LUP, GET_KEYS(event));
+			g_pTool->Execute(_Get_Client2World(event.GetPosition()), TOOL_INTERACTIVE_LUP, GET_KEYS(event));
 		}
-		else if( m_pMap->Find_Layer(Get_Active_Layer()) )
+		else if( m_pMap->Find_Active(true) )
 		{
-			Get_Active_Layer()->Edit_On_Mouse_Up(_Get_World(event.GetPosition()), _Get_World(1.0), GET_KEYS(event)|MODULE_INTERACTIVE_KEY_LEFT);
+			Get_Active_Layer()->Edit_On_Mouse_Up(
+				_Get_Client2World(event.GetPosition()), _Get_Client2World(1.), GET_KEYS(event)|TOOL_INTERACTIVE_KEY_LEFT
+			);
+		}
+		else if( m_pMap->Find_Active() )	// on-the-fly projected layer !
+		{
+			double	d	= _Get_Client2World(1., true);
+
+			Get_Active_Layer()->Edit_On_Mouse_Down(_Get_Client2World(m_Mouse_Down       , true), d, GET_KEYS(event));
+			Get_Active_Layer()->Edit_On_Mouse_Up  (_Get_Client2World(event.GetPosition(), true), d, GET_KEYS(event)|TOOL_INTERACTIVE_KEY_LEFT);
 		}
 		break;
 
@@ -782,26 +786,14 @@ void CVIEW_Map_Control::On_Mouse_LDClick(wxMouseEvent &event)
 	{
 	//-----------------------------------------------------
 	case MAP_MODE_SELECT:
-		if( g_pModule )
+		if( g_pTool )
 		{
-			g_pModule->Execute(_Get_World(event.GetPosition()), MODULE_INTERACTIVE_LDCLICK, GET_KEYS(event));
+			g_pTool->Execute(_Get_Client2World(event.GetPosition()), TOOL_INTERACTIVE_LDCLICK, GET_KEYS(event));
 		}
 		break;
 
 	//-----------------------------------------------------
-	case MAP_MODE_DISTANCE:
-		break;
-
-	//-----------------------------------------------------
-	case MAP_MODE_ZOOM:
-		break;
-
-	//-----------------------------------------------------
-	case MAP_MODE_PAN:
-		break;
-
-	//-----------------------------------------------------
-	case MAP_MODE_PAN_DOWN:
+	default:
 		break;
 	}
 }
@@ -809,47 +801,36 @@ void CVIEW_Map_Control::On_Mouse_LDClick(wxMouseEvent &event)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
 void CVIEW_Map_Control::On_Mouse_RDown(wxMouseEvent &event)
 {
+	m_pParent->Activate();
+
 	m_Mouse_Down	= m_Mouse_Move	= event.GetPosition();
 
 	_Draw_Inverse(m_Mouse_Down, event.GetPosition());
-	m_Drag_Mode	= MODULE_INTERACTIVE_DRAG_NONE;
+	m_Drag_Mode	= TOOL_INTERACTIVE_DRAG_NONE;
 
 	switch( m_Mode )
 	{
 	//-----------------------------------------------------
 	case MAP_MODE_SELECT:
-		if( g_pModule )
+		if( g_pTool )
 		{
-			g_pModule->Execute(_Get_World(event.GetPosition()), MODULE_INTERACTIVE_RDOWN, GET_KEYS(event));
+			g_pTool->Execute(_Get_Client2World(event.GetPosition()), TOOL_INTERACTIVE_RDOWN, GET_KEYS(event));
 		}
-		else if( m_pMap->Find_Layer(Get_Active_Layer()) )
+		else if( m_pMap->Find_Active(true) )
 		{
-			Get_Active_Layer()->Edit_On_Mouse_Down(_Get_World(event.GetPosition()), _Get_World(1.0), GET_KEYS(event));
+			Get_Active_Layer()->Edit_On_Mouse_Down(
+				_Get_Client2World(event.GetPosition()), _Get_Client2World(1.), GET_KEYS(event)
+			);
 		}
 		break;
 
 	//-----------------------------------------------------
-	case MAP_MODE_DISTANCE:
-		_Distance_Reset();
-		break;
-
-	//-----------------------------------------------------
-	case MAP_MODE_ZOOM:
-		break;
-
-	//-----------------------------------------------------
-	case MAP_MODE_PAN:
-		break;
-
-	//-----------------------------------------------------
-	case MAP_MODE_PAN_DOWN:
+	default:
 		break;
 	}
 }
@@ -859,44 +840,58 @@ void CVIEW_Map_Control::On_Mouse_RUp(wxMouseEvent &event)
 {
 	wxMenu	*pMenu	= NULL;
 
-	m_Drag_Mode		= MODULE_INTERACTIVE_DRAG_NONE;
+	m_Drag_Mode		= TOOL_INTERACTIVE_DRAG_NONE;
 
 	switch( m_Mode )
 	{
 	//-----------------------------------------------------
 	case MAP_MODE_SELECT:
-		if( g_pModule )
+		if( g_pTool )
 		{
-			g_pModule->Execute(_Get_World(event.GetPosition()), MODULE_INTERACTIVE_RUP, GET_KEYS(event));
+			g_pTool->Execute(_Get_Client2World(event.GetPosition()), TOOL_INTERACTIVE_RUP, GET_KEYS(event));
 		}
-		else if( m_pMap->Find_Layer(Get_Active_Layer()) )
+		else if( m_pMap->Find_Active(true) && !Get_Active_Layer()->Edit_On_Mouse_Up(
+			_Get_Client2World(event.GetPosition()), _Get_Client2World(1.), GET_KEYS(event)|TOOL_INTERACTIVE_KEY_RIGHT) )
 		{
-			if(	!Get_Active_Layer()->Edit_On_Mouse_Up(_Get_World(event.GetPosition()), _Get_World(1.0), GET_KEYS(event)|MODULE_INTERACTIVE_KEY_RIGHT)
-			&&	(pMenu = Get_Active_Layer()->Edit_Get_Menu()) != NULL )
-			{
-				PopupMenu(pMenu, event.GetPosition());
-
-				delete(pMenu);
-			}
+			pMenu	= Get_Active_Layer()->Edit_Get_Menu();
 		}
 		break;
 
 	//-----------------------------------------------------
 	case MAP_MODE_DISTANCE:
+		if( event.ControlDown() )	// context menu
+		{
+			pMenu	= m_pParent->_Create_Menu();
+		}
+		else	// reset
+		{
+			_Distance_Reset();
+		}
 		break;
 
-	//-----------------------------------------------------
 	case MAP_MODE_ZOOM:
-		_Zoom(_Get_World(event.GetPosition()), false);
+		if( event.ControlDown() )	// context menu
+		{
+			pMenu	= m_pParent->_Create_Menu();
+		}
+		else	// zoom out
+		{
+			_Zoom(_Get_Client2World(event.GetPosition()), false);
+		}
 		break;
 
 	//-----------------------------------------------------
-	case MAP_MODE_PAN:
+	default:
+		pMenu	= m_pParent->_Create_Menu();
 		break;
+	}
 
 	//-----------------------------------------------------
-	case MAP_MODE_PAN_DOWN:
-		break;
+	if( pMenu != NULL )
+	{
+		PopupMenu(pMenu, event.GetPosition());
+
+		delete(pMenu);
 	}
 }
 
@@ -907,9 +902,9 @@ void CVIEW_Map_Control::On_Mouse_RDClick(wxMouseEvent &event)
 	{
 	//-----------------------------------------------------
 	case MAP_MODE_SELECT:
-		if( g_pModule )
+		if( g_pTool )
 		{
-			g_pModule->Execute(_Get_World(event.GetPosition()), MODULE_INTERACTIVE_RDCLICK, GET_KEYS(event));
+			g_pTool->Execute(_Get_Client2World(event.GetPosition()), TOOL_INTERACTIVE_RDCLICK, GET_KEYS(event));
 		}
 		break;
 
@@ -934,8 +929,6 @@ void CVIEW_Map_Control::On_Mouse_RDClick(wxMouseEvent &event)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -949,9 +942,9 @@ void CVIEW_Map_Control::On_Mouse_MDown(wxMouseEvent &event)
 	{
 	//-----------------------------------------------------
 	case MAP_MODE_SELECT:
-		if( g_pModule )
+		if( g_pTool )
 		{
-			g_pModule->Execute(_Get_World(event.GetPosition()), MODULE_INTERACTIVE_MDOWN, GET_KEYS(event));
+			g_pTool->Execute(_Get_Client2World(event.GetPosition()), TOOL_INTERACTIVE_MDOWN, GET_KEYS(event));
 		}
 		break;
 
@@ -961,7 +954,7 @@ void CVIEW_Map_Control::On_Mouse_MDown(wxMouseEvent &event)
 
 	//-----------------------------------------------------
 	case MAP_MODE_ZOOM:
-		m_Drag_Mode		= MODULE_INTERACTIVE_DRAG_NONE;
+		m_Drag_Mode		= TOOL_INTERACTIVE_DRAG_NONE;
 		Set_Mode(MAP_MODE_PAN_DOWN);
 		break;
 
@@ -971,7 +964,7 @@ void CVIEW_Map_Control::On_Mouse_MDown(wxMouseEvent &event)
 
 	//-----------------------------------------------------
 	case MAP_MODE_PAN_DOWN:
-		m_Drag_Mode		= MODULE_INTERACTIVE_DRAG_NONE;
+		m_Drag_Mode		= TOOL_INTERACTIVE_DRAG_NONE;
 		break;
 	}
 
@@ -993,7 +986,7 @@ void CVIEW_Map_Control::On_Mouse_MUp(wxMouseEvent &event)
 	}
 
 	_Draw_Inverse(m_Mouse_Down, event.GetPosition());
-	m_Drag_Mode	= MODULE_INTERACTIVE_DRAG_NONE;
+	m_Drag_Mode	= TOOL_INTERACTIVE_DRAG_NONE;
 
 	switch( m_Mode )
 	{
@@ -1024,39 +1017,31 @@ void CVIEW_Map_Control::On_Mouse_MUp(wxMouseEvent &event)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
 void CVIEW_Map_Control::On_Mouse_Motion(wxMouseEvent &event)
 {
-	if( m_Mode != MAP_MODE_PAN_DOWN )
-	{
-		_Set_StatusBar(_Get_World(event.GetPosition()));
-
-		m_pParent->Ruler_Set_Position(event.GetPosition().x, event.GetPosition().y);
-	}
+	wxPoint	Point	= event.GetPosition();
 
 	switch( m_Mode )
 	{
 	//-----------------------------------------------------
 	case MAP_MODE_SELECT:
-		if( g_pModule )
+		if( g_pTool )
 		{
-			TSG_Module_Interactive_Mode	iMode
-				= event.LeftIsDown()	? MODULE_INTERACTIVE_MOVE_LDOWN
-				: event.MiddleIsDown()	? MODULE_INTERACTIVE_MOVE_MDOWN
-				: event.RightIsDown()	? MODULE_INTERACTIVE_MOVE_RDOWN
-										: MODULE_INTERACTIVE_MOVE;
+			TSG_Tool_Interactive_Mode	iMode
+				= event.LeftIsDown  () ? TOOL_INTERACTIVE_MOVE_LDOWN
+				: event.MiddleIsDown() ? TOOL_INTERACTIVE_MOVE_MDOWN
+				: event.RightIsDown () ? TOOL_INTERACTIVE_MOVE_RDOWN : TOOL_INTERACTIVE_MOVE;
 
-			g_pModule->Execute(_Get_World(event.GetPosition()), iMode, GET_KEYS(event));
+			g_pTool->Execute(_Get_Client2World(Point), iMode, GET_KEYS(event));
 		}
-		else if( m_pMap->Find_Layer(Get_Active_Layer()) )
+		else if( m_pMap->Find_Active(true) )
 		{
 			Get_Active_Layer()->Edit_On_Mouse_Move(
-				this, m_pMap->Get_World(_Get_Client()),
-				event.GetPosition(), m_Mouse_Move,
+				this, m_pMap->Get_World(GetClientSize()),
+				Point, m_Mouse_Move,
 				GET_KEYS(event)
 			);
 		}
@@ -1068,11 +1053,11 @@ void CVIEW_Map_Control::On_Mouse_Motion(wxMouseEvent &event)
 		{
 			int			n	= m_Distance_Pts.Get_Count();
 			wxClientDC	dc(this);
-			wxPoint		Last(_Get_Client(m_Distance_Pts[n - 1]));
+			wxPoint		Last(_Get_World2Client(m_Distance_Pts[n - 1]));
 			dc.SetLogicalFunction(wxINVERT);
-			dc.DrawLine(Last.x, Last.y, m_Mouse_Move       .x, m_Mouse_Move       .y);
-			dc.DrawLine(Last.x, Last.y, event.GetPosition().x, event.GetPosition().y);
-			m_Distance_Move	= SG_Get_Distance(m_Distance_Pts[n - 1], _Get_World(event.GetPosition()));
+			dc.DrawLine(Last.x, Last.y, m_Mouse_Move.x, m_Mouse_Move.y);
+			dc.DrawLine(Last.x, Last.y,        Point.x,        Point.y);
+			m_Distance_Move	= SG_Get_Distance(m_Distance_Pts[n - 1], _Get_Client2World(Point));
 		}
 		break;
 
@@ -1086,23 +1071,30 @@ void CVIEW_Map_Control::On_Mouse_Motion(wxMouseEvent &event)
 
 	//-----------------------------------------------------
 	case MAP_MODE_PAN_DOWN:
-	//	_Move(m_Mouse_Down, event.GetPosition());
-		_Pan(m_Mouse_Down, event.GetPosition());
+	//	_Move(m_Mouse_Down, Point);
+		_Pan(m_Mouse_Down, Point);
 		break;
 	}
 
 	//-----------------------------------------------------
-	_Draw_Inverse(m_Mouse_Down, m_Mouse_Move, event.GetPosition());
+	if( m_Mode != MAP_MODE_PAN_DOWN )
+	{
+		m_pParent->Ruler_Set_Position(Point.x, Point.y);
+
+		m_pMap->Set_Mouse_Position(_Get_Client2World(Point));
+
+		_Set_StatusBar(_Get_Client2World(Point));
+	}
 
 	//-----------------------------------------------------
-	m_Mouse_Move	= event.GetPosition();
+	_Draw_Inverse(m_Mouse_Down, m_Mouse_Move, Point);
+
+	m_Mouse_Move	= Point;
 }
 
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1110,24 +1102,23 @@ void CVIEW_Map_Control::On_Mouse_Wheel(wxMouseEvent &event)
 {
 	if( event.GetWheelRotation() < 0 )
 	{
-		_Zoom(_Get_World(event.GetPosition()), false);
+		_Zoom(_Get_Client2World(event.GetPosition()), false);
 	}
 	else if( event.GetWheelRotation() > 0 )
 	{
-		_Zoom(_Get_World(event.GetPosition()), true);
+		_Zoom(_Get_Client2World(event.GetPosition()), true);
 	}
 }
 
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
 void CVIEW_Map_Control::On_Mouse_Lost(wxMouseCaptureLostEvent &event)
 {
+	Set_CrossHair_Off();
 }
 
 
