@@ -1,6 +1,3 @@
-/**********************************************************
- * Version $Id: wksp_layer.cpp 2208 2014-08-27 16:00:51Z oconrad $
- *********************************************************/
 
 ///////////////////////////////////////////////////////////
 //                                                       //
@@ -24,7 +21,8 @@
 // Geoscientific Analyses'. SAGA is free software; you   //
 // can redistribute it and/or modify it under the terms  //
 // of the GNU General Public License as published by the //
-// Free Software Foundation; version 2 of the License.   //
+// Free Software Foundation, either version 2 of the     //
+// License, or (at your option) any later version.       //
 //                                                       //
 // SAGA is distributed in the hope that it will be       //
 // useful, but WITHOUT ANY WARRANTY; without even the    //
@@ -33,10 +31,8 @@
 // License for more details.                             //
 //                                                       //
 // You should have received a copy of the GNU General    //
-// Public License along with this program; if not,       //
-// write to the Free Software Foundation, Inc.,          //
-// 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, //
-// USA.                                                  //
+// Public License along with this program; if not, see   //
+// <http://www.gnu.org/licenses/>.                       //
 //                                                       //
 //-------------------------------------------------------//
 //                                                       //
@@ -51,19 +47,12 @@
 //                                                       //
 ///////////////////////////////////////////////////////////
 
-//---------------------------------------------------------
-
-
-///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
-///////////////////////////////////////////////////////////
-
 //---------------------------------------------------------
 #include "res_commands.h"
 #include "res_dialogs.h"
 
+#include "saga_frame.h"
+
 #include "helper.h"
 
 #include "active.h"
@@ -78,6 +67,7 @@
 #include "wksp_map.h"
 #include "wksp_map_layer.h"
 
+#include "wksp_data_manager.h"
 #include "wksp_data_layers.h"
 
 #include "wksp_layer.h"
@@ -122,14 +112,12 @@ static int	s_Def_Layer_Colours[DEF_LAYER_COLOUR_COUNT]	=
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
 CWKSP_Layer *	Get_Active_Layer(void)
 {
-	return( g_pACTIVE ? g_pACTIVE->Get_Active_Layer() : NULL );
+	return( g_pActive ? g_pActive->Get_Active_Layer() : NULL );
 }
 
 
@@ -154,17 +142,19 @@ CWKSP_Layer::CWKSP_Layer(CSG_Data_Object *pObject)
 //---------------------------------------------------------
 CWKSP_Layer::~CWKSP_Layer(void)
 {
+	g_pSAGA_Frame->Freeze();
+
 	if( g_pMaps     )	{	g_pMaps->Del(this);	}
 
 	if( m_pClassify )	{	delete(m_pClassify);	}
 	if( m_pLegend   )	{	delete(m_pLegend  );	}
+
+	g_pSAGA_Frame->Thaw();
 }
 
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -177,7 +167,7 @@ bool CWKSP_Layer::On_Command(int Cmd_ID)
 
 	case ID_CMD_WKSP_ITEM_RETURN:
 	case ID_CMD_SHAPES_SHOW:
-	case ID_CMD_GRIDS_SHOW:
+	case ID_CMD_GRID_SHOW:
 	case ID_CMD_TIN_SHOW:
 	case ID_CMD_POINTCLOUD_SHOW:
 		g_pMaps->Add(this);
@@ -186,6 +176,11 @@ bool CWKSP_Layer::On_Command(int Cmd_ID)
 	case ID_CMD_DATA_PROJECTION:
 		_Set_Projection();
 		break;
+
+	case ID_CMD_DATA_FORCE_UPDATE:
+		m_pObject->Update(true);
+		DataObject_Changed();
+		break;
 	}
 
 	return( true );
@@ -200,8 +195,6 @@ bool CWKSP_Layer::On_Command_UI(wxUpdateUIEvent &event)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -212,205 +205,540 @@ void CWKSP_Layer::On_Create_Parameters(void)
 	//-----------------------------------------------------
 	// Nodes...
 
-	m_Parameters.Add_Node(NULL, "NODE_DISPLAY"  , _TL("Display"  ), _TL(""));
-	m_Parameters.Add_Node(NULL, "NODE_COLORS"   , _TL("Colors"   ), _TL(""));
-	m_Parameters.Add_Node(NULL, "NODE_SIZE"     , _TL("Size"     ), _TL(""));
-	m_Parameters.Add_Node(NULL, "NODE_LABEL"    , _TL("Labels"   ), _TL(""));
-	m_Parameters.Add_Node(NULL, "NODE_SELECTION", _TL("Selection"), _TL(""));
-	m_Parameters.Add_Node(NULL, "NODE_EDIT"     , _TL("Edit"     ), _TL(""));
-
+	m_Parameters.Add_Node("", "NODE_DISPLAY"  , _TL("Display"  ), _TL(""));
+	m_Parameters.Add_Node("", "NODE_COLORS"   , _TL("Colors"   ), _TL(""));
+	m_Parameters.Add_Node("", "NODE_SIZE"     , _TL("Size"     ), _TL(""));
+	m_Parameters.Add_Node("", "NODE_LABEL"    , _TL("Labels"   ), _TL(""));
+	m_Parameters.Add_Node("", "NODE_SELECTION", _TL("Selection"), _TL(""));
+	m_Parameters.Add_Node("", "NODE_EDIT"     , _TL("Edit"     ), _TL(""));
 
 	//-----------------------------------------------------
 	// General...
 
-	m_Parameters.Add_Value(
-		m_Parameters("NODE_GENERAL"), "LEGEND_SHOW"   , _TL("Show Legend"),
+	m_Parameters.Add_Bool("NODE_GENERAL",
+		"LEGEND_SHOW"   , _TL("Show Legend"),
 		_TL(""),
-		PARAMETER_TYPE_Bool, true
+		true
 	);
 
-	m_Parameters.Add_Choice(
-		m_Parameters("LEGEND_SHOW" ), "LEGEND_STYLE"  , _TL("Style"),
+	m_Parameters.Add_Choice("LEGEND_SHOW",
+		"LEGEND_STYLE"	, _TL("Style"),
 		_TL(""),
-		CSG_String::Format(SG_T("%s|%s|"),
+		CSG_String::Format("%s|%s",
 			_TL("vertical"),
 			_TL("horizontal")
 		), 0
 	);
 
-
 	//-----------------------------------------------------
 	// Display...
 
-	m_Parameters.Add_Value(
-		m_Parameters("NODE_DISPLAY"), "DISPLAY_TRANSPARENCY", _TL("Transparency [%]"),
+	m_Parameters.Add_Double("NODE_DISPLAY",
+		"DISPLAY_TRANSPARENCY", _TL("Transparency [%]"),
 		_TL(""),
-		PARAMETER_TYPE_Double, 0.0, 0.0, true, 100.0, true
+		0.0, 0.0, true, 100.0, true
 	);
 
-	m_Parameters.Add_Value(
-		m_Parameters("NODE_DISPLAY"), "SHOW_ALWAYS"         , _TL("Show at all scales"),
+	m_Parameters.Add_Bool("NODE_DISPLAY",
+		"SHOW_ALWAYS"	, _TL("Show at all scales"),
 		_TL(""),
-		PARAMETER_TYPE_Bool, true
+		true
 	);
 
-	m_Parameters.Add_Range(
-		m_Parameters("SHOW_ALWAYS" ), "SHOW_RANGE"			, _TL("Scale Range"),
+	m_Parameters.Add_Range("SHOW_ALWAYS",
+		"SHOW_RANGE"	, _TL("Scale Range"),
 		_TL("only show within scale range; values are given as extent measured in map units"),
 		100.0, 1000.0, 0.0, true
 	);
 
+	//-----------------------------------------------------
+	if( m_pObject->Get_ObjectType() == SG_DATAOBJECT_TYPE_Shapes
+	||  m_pObject->Get_ObjectType() == SG_DATAOBJECT_TYPE_PointCloud
+	||  m_pObject->Get_ObjectType() == SG_DATAOBJECT_TYPE_TIN )
+	{
+		m_Parameters.Add_Choice("NODE_DISPLAY",
+			"TABLE_FLT_STYLE"		, _TL("Floating Point Numbers"),
+			_TL("Specify floating point decimal precision in table and similar views."),
+			CSG_String::Format("%s|%s|%s",
+				_TL("system default"),
+				_TL("maximum number of significant decimals"),
+				_TL("fix number of decimals")
+			), g_pData->Get_Parameter("TABLE_FLT_STYLE")->asInt()
+		);
+
+		m_Parameters.Add_Int("TABLE_FLT_STYLE",
+			"TABLE_FLT_DECIMALS"	, _TL("Decimals"),
+			_TL(""),
+			g_pData->Get_Parameter("TABLE_FLT_DECIMALS")->asInt(), 0, true
+		);
+	}
 
 	//-----------------------------------------------------
 	// Classification...
 
-	m_Parameters.Add_Choice(
-		m_Parameters("NODE_COLORS" ), "COLORS_TYPE"			, _TL("Type"),
-		_TL(""),
-		CSG_String::Format(SG_T("%s|%s|%s|%s|"),
+	if( m_pObject->Get_ObjectType() == SG_DATAOBJECT_TYPE_Shapes
+	||  m_pObject->Get_ObjectType() == SG_DATAOBJECT_TYPE_TIN )
+	{
+		m_Parameters.Add_Choice("NODE_COLORS", "COLORS_TYPE", _TL("Type"), _TL(""), CSG_String::Format("%s|%s|%s|%s",
 			_TL("Single Symbol"   ),	// CLASSIFY_UNIQUE
-			_TL("Lookup Table"    ),	// CLASSIFY_LUT
+			_TL("Classified"      ),	// CLASSIFY_LUT
 			_TL("Discrete Colors" ),	// CLASSIFY_METRIC
 			_TL("Graduated Colors") 	// CLASSIFY_GRADUATED
-		), 0
-	);
+		), 0);
+	}
+	else if( m_pObject->Get_ObjectType() == SG_DATAOBJECT_TYPE_PointCloud )
+	{
+		m_Parameters.Add_Choice("NODE_COLORS", "COLORS_TYPE", _TL("Type"), _TL(""), CSG_String::Format("%s|%s|%s|%s|%s",
+			_TL("Single Symbol"   ),	// CLASSIFY_UNIQUE
+			_TL("Classified"      ),	// CLASSIFY_LUT
+			_TL("Discrete Colors" ),	// CLASSIFY_METRIC
+			_TL("Graduated Colors"),	// CLASSIFY_GRADUATED
+			_TL("RGB Coded Values")		// CLASSIFY_OVERLAY !!!
+		), 0);
+	}
+	else if( m_pObject->Get_ObjectType() == SG_DATAOBJECT_TYPE_Grids )
+	{
+		m_Parameters.Add_Choice("NODE_COLORS", "COLORS_TYPE", _TL("Type"), _TL(""), CSG_String::Format("%s|%s|%s|%s|%s",
+			_TL("Single Symbol"   ),	// CLASSIFY_UNIQUE
+			_TL("Classified"      ),	// CLASSIFY_LUT
+			_TL("Discrete Colors" ),	// CLASSIFY_METRIC
+			_TL("Graduated Colors"),	// CLASSIFY_GRADUATED
+			_TL("RGB Composite"   )		// CLASSIFY_OVERLAY
+		), 4);
 
+		m_Parameters.Add_Choice("NODE_COLORS", "BAND"       , _TL("Band"      ), _TL(""), _TL("<default>"));
+		m_Parameters.Add_Choice("NODE_COLORS", "OVERLAY_FIT", _TL("Statistics"), _TL(""), CSG_String::Format("%s|%s", _TL("all bands"), _TL("each band")), 1);
+		m_Parameters.Add_Choice("NODE_COLORS", "BAND_R"     , _TL("Red"       ), _TL(""), "");
+		m_Parameters.Add_Choice("NODE_COLORS", "BAND_G"     , _TL("Green"     ), _TL(""), "");
+		m_Parameters.Add_Choice("NODE_COLORS", "BAND_B"     , _TL("Blue"      ), _TL(""), "");
+	}
+	else if( m_pObject->Get_ObjectType() == SG_DATAOBJECT_TYPE_Grid )
+	{
+		m_Parameters.Add_Choice("NODE_COLORS", "COLORS_TYPE", _TL("Type"), _TL(""), CSG_String::Format("%s|%s|%s|%s|%s|%s|%s",
+			_TL("Single Symbol"   ),	// CLASSIFY_UNIQUE
+			_TL("Classified"      ),	// CLASSIFY_LUT
+			_TL("Discrete Colors" ),	// CLASSIFY_METRIC
+			_TL("Graduated Colors"),	// CLASSIFY_GRADUATED
+			_TL("RGB Composite"   ),	// CLASSIFY_OVERLAY
+			_TL("RGB Coded Values"),	// CLASSIFY_RGB
+			_TL("Shade"           )		// CLASSIFY_SHADE
+		), 3);
+	}
 
 	//-----------------------------------------------------
-	// Classification: Unique Value...
-
-	m_Parameters.Add_Node(
-		m_Parameters("NODE_COLORS"   ), "NODE_UNISYMBOL"		, _TL("Single Symbol"),
-		_TL("")
-	);
+	// Classification: Single Symbol...
 
 	static	BYTE	s_Def_Layer_Colour	= 0;
 
-	m_Parameters.Add_Value(
-		m_Parameters("NODE_UNISYMBOL"), "UNISYMBOL_COLOR"		, _TL("Color"),
-		_TL(""),
-		PARAMETER_TYPE_Color, s_Def_Layer_Colours[s_Def_Layer_Colour++ % DEF_LAYER_COLOUR_COUNT]
-	//	PARAMETER_TYPE_Color, SG_GET_RGB(Get_Random(128, 250), Get_Random(128, 200), Get_Random(128, 200))
+	m_Parameters.Add_Node("NODE_COLORS",
+		"NODE_UNISYMBOL"	, _TL("Single Symbol"),
+		_TL("")
 	);
 
+	m_Parameters.Add_Color("NODE_UNISYMBOL",
+		"UNISYMBOL_COLOR"	, _TL("Color"),
+		_TL(""),
+		s_Def_Layer_Colours[s_Def_Layer_Colour++ % DEF_LAYER_COLOUR_COUNT]
+	);
 
 	//-----------------------------------------------------
-	// Classification: Lookup Table...
+	// Classification: Classified...
 
-	m_Parameters.Add_Node(
-		m_Parameters("NODE_COLORS")		, "NODE_LUT"			, _TL("Lookup Table"),
+	m_Parameters.Add_Node("NODE_COLORS",
+		"NODE_LUT"			, _TL("Classified"),
 		_TL("")
 	);
 
-	CSG_Table	LUT;
-	LUT.Add_Field(_TL("COLOR")			, SG_DATATYPE_Color);
-	LUT.Add_Field(_TL("NAME")			, SG_DATATYPE_String);
-	LUT.Add_Field(_TL("DESCRIPTION")	, SG_DATATYPE_String);
-	LUT.Add_Field(_TL("MINIMUM")		, SG_DATATYPE_Double);
-	LUT.Add_Field(_TL("MAXIMUM")		, SG_DATATYPE_Double);
+	if( m_pObject->Get_ObjectType() == SG_DATAOBJECT_TYPE_Shapes
+	||  m_pObject->Get_ObjectType() == SG_DATAOBJECT_TYPE_PointCloud
+	||  m_pObject->Get_ObjectType() == SG_DATAOBJECT_TYPE_TIN )
+	{
+		m_Parameters.Add_Choice("NODE_LUT", "LUT_ATTRIB", _TL("Attribute"), _TL(""), _TL("<default>"));
+	}
 
-	m_Parameters.Add_FixedTable(
-		m_Parameters("NODE_LUT")		, "LUT"					, _TL("Table"),
-		_TL(""),
-		&LUT
-	);
+	CSG_Table	*pLUT	= m_Parameters.Add_FixedTable("NODE_LUT",
+		"LUT"				, _TL("Table"),
+		_TL("")
+	)->asTable();
 
+	pLUT->Add_Field(_TL("Color"      ), SG_DATATYPE_Color );
+	pLUT->Add_Field(_TL("Name"       ), SG_DATATYPE_String);
+	pLUT->Add_Field(_TL("Description"), SG_DATATYPE_String);
+	pLUT->Add_Field(_TL("Minimum"    ), SG_DATATYPE_Double);
+	pLUT->Add_Field(_TL("Maximum"    ), SG_DATATYPE_Double);
+
+	m_pClassify->Initialise(this, m_Parameters("LUT")->asTable(), g_pData->Get_Parameter("COLORS_DEFAULT")->asColors());
 
 	//-----------------------------------------------------
-	// Classification: Metric...
+	// Classification: Colors...
 
-	m_Parameters.Add_Node(
-		m_Parameters("NODE_COLORS")		, "NODE_METRIC"			, _TL("Scaling"),
+	ColorsParms_Add();
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+void CWKSP_Layer::ColorsParms_Add(void)
+{
+	m_Parameters.Add_Node("NODE_COLORS",
+		"NODE_METRIC"		, _TL("Histogram Stretch"),
 		_TL("")
 	);
 
-	m_Parameters.Add_Colors(
-		m_Parameters("NODE_METRIC")		, "METRIC_COLORS"		, _TL("Colors"),
-		_TL("")
-	);
+	if( m_pObject->Get_ObjectType() == SG_DATAOBJECT_TYPE_Shapes )
+	{
+		m_Parameters.Add_Choice("NODE_METRIC", "METRIC_ATTRIB", _TL("Attribute"), _TL(""), _TL("<default>"));
+		m_Parameters.Add_Choice("NODE_METRIC", "METRIC_NORMAL", _TL("Normalize"), _TL(""), _TL("<default>"));
+	}
+	else if( m_pObject->Get_ObjectType() == SG_DATAOBJECT_TYPE_PointCloud
+		||   m_pObject->Get_ObjectType() == SG_DATAOBJECT_TYPE_TIN )
+	{
+		m_Parameters.Add_Choice("NODE_METRIC", "METRIC_ATTRIB", _TL("Attribute"), _TL(""), _TL("<default>"));
+	}
 
-	m_Parameters.Add_Range(
-		m_Parameters("NODE_METRIC")		, "METRIC_ZRANGE"		, _TL("Value Range"),
-		_TL("")
-	);
-
-	m_Parameters.Add_Choice(
-		m_Parameters("NODE_METRIC")		, "METRIC_SCALE_MODE"	, _TL("Mode"),
+	//-----------------------------------------------------
+	m_Parameters.Add_Colors("NODE_METRIC",
+		"METRIC_COLORS"	, _TL("Colors"),
 		_TL(""),
-		CSG_String::Format(SG_T("%s|%s|%s|"),
-			_TL("Linear"),
-			_TL("Logarithmic (up)"),
-			_TL("Logarithmic (down)")
+		g_pData->Get_Parameter("COLORS_DEFAULT")->asColors()
+	);
+
+	m_Parameters.Add_Range("NODE_METRIC",
+		"METRIC_ZRANGE"		, _TL("Range"),
+		_TL("")
+	);
+
+	if( m_pObject->Get_ObjectType() == SG_DATAOBJECT_TYPE_Grids )
+	{
+		m_Parameters.Add_Range("NODE_METRIC", "METRIC_ZRANGE_R", _TL("Red"  ), _TL(""));
+		m_Parameters.Add_Range("NODE_METRIC", "METRIC_ZRANGE_G", _TL("Green"), _TL(""));
+		m_Parameters.Add_Range("NODE_METRIC", "METRIC_ZRANGE_B", _TL("Blue" ), _TL(""));
+	}
+
+	//-----------------------------------------------------
+	if( m_pObject->Get_ObjectType() == SG_DATAOBJECT_TYPE_Grid
+	||  m_pObject->Get_ObjectType() == SG_DATAOBJECT_TYPE_Grids )
+	{
+		m_Parameters.Add_Bool("", "STRETCH_UPDATE", "update flag, for internal use only", "", true)->Set_Enabled(false);
+
+		m_Parameters.Add_Choice("NODE_METRIC",
+			"STRETCH_DEFAULT"	, _TL("Adjustment"),
+			_TL("Specify how to adjust histogram stretch."),
+			CSG_String::Format("%s|%s|%s|%s",
+				_TL("Linear"),
+				_TL("Standard Deviation"),
+				_TL("Percentile"),
+				_TL("Manual")
+			), g_pData->Get_Parameter("GRID_STRETCH_DEFAULT")->asInt()
+		);
+
+		m_Parameters.Add_Double("STRETCH_DEFAULT",
+			"STRETCH_LINEAR"	, _TL("Linear Percent Stretch"),
+			_TL("Linear percent stretch allows you to trim extreme values from both ends of the histogram using the percentage specified here."),
+			5.0, 0.0, true, 50.0, true
+		);
+
+		m_Parameters.Add_Double("STRETCH_DEFAULT",
+			"STRETCH_STDDEV"	, _TL("Standard Deviation"),
+			_TL(""),
+			2.0, 0.0, true
+		);
+
+		m_Parameters.Add_Bool("STRETCH_STDDEV",
+			"STRETCH_INRANGE"	, _TL("Keep in Range"),
+			_TL("Prevents that minimum or maximum stretch value fall outside the data value range."),
+			true
+		);
+
+		m_Parameters.Add_Double("STRETCH_DEFAULT",
+			"STRETCH_PCTL"		, _TL("Percentile"),
+			_TL(""),
+			2.0, 0.0, true, 50.0, true
+		);
+	}
+
+	//-----------------------------------------------------
+	m_Parameters.Add_Choice("NODE_METRIC",
+		"METRIC_SCALE_MODE"	, _TL("Scaling"),
+		_TL(""),
+		CSG_String::Format("%s|%s|%s",
+			_TL("linear intervals"),
+			_TL("increasing geometrical intervals"),
+			_TL("decreasing geometrical intervals")
 		), 0
 	);
 
-	m_Parameters.Add_Value(
-		m_Parameters("NODE_METRIC")		, "METRIC_SCALE_LOG"	, _TL("Logarithmic Stretch Factor"),
+	m_Parameters.Add_Double("METRIC_SCALE_MODE",
+		"METRIC_SCALE_LOG"	, _TL("Geometrical Interval Factor"),
 		_TL(""),
-		PARAMETER_TYPE_Double, 1.0
+		10.0
 	);
+}
+
+//---------------------------------------------------------
+void CWKSP_Layer::ColorsParms_On_Changed(CSG_Parameters *pParameters, CSG_Parameter *pParameter, int Flags)
+{
+	//-----------------------------------------------------
+	if( Flags & PARAMETER_CHECK_VALUES )
+	{
+		if( pParameter->Cmp_Identifier("METRIC_ZRANGE"  )
+		||  pParameter->Cmp_Identifier("METRIC_ZRANGE_R")
+		||  pParameter->Cmp_Identifier("METRIC_ZRANGE_G")
+		||  pParameter->Cmp_Identifier("METRIC_ZRANGE_B") )
+		{
+			pParameters->Set_Parameter("STRETCH_DEFAULT", 3);	// manual
+		}
+	}
 
 	//-----------------------------------------------------
-	m_pClassify->Initialise(this, m_Parameters("LUT")->asTable(), m_Parameters("METRIC_COLORS")->asColors());
+	if( Flags & PARAMETER_CHECK_ENABLE )
+	{
+		if(	pParameter->Cmp_Identifier("METRIC_SCALE_MODE") )
+		{
+			pParameters->Set_Enabled("METRIC_SCALE_LOG", pParameter->asInt() != 0);
+		}
+
+		CSG_Parameter	*pStretch	= (*pParameters)("STRETCH_DEFAULT");
+
+		if(	pStretch )
+		{
+			pParameters->Set_Enabled("STRETCH_LINEAR", pStretch->asInt() == 0);
+			pParameters->Set_Enabled("STRETCH_STDDEV", pStretch->asInt() == 1);
+			pParameters->Set_Enabled("STRETCH_PCTL"  , pStretch->asInt() == 2);
+		}
+	}
 }
 
+//---------------------------------------------------------
+bool CWKSP_Layer::ColorsParms_Adjust(CSG_Parameters &Parameters, CSG_Data_Object *pObject, const CSG_String &Suffix)
+{
+	if( !pObject )
+	{
+		pObject	= Get_Object();
+
+		if( pObject->Get_ObjectType() == SG_DATAOBJECT_TYPE_Grids && Parameters("COLORS_TYPE")->asInt() != CLASSIFY_OVERLAY )
+		{
+			int	i	= Parameters("BAND") ? Parameters("BAND")->asInt() : -1;
+
+			if( i < 0 || i >= ((CSG_Grids *)pObject)->Get_NZ() || !(pObject = ((CSG_Grids *)pObject)->Get_Grid_Ptr(i)) )
+			{
+				return( false );
+			}
+		}
+	}
+
+	int	Field	= -1;
+
+	if( pObject->Get_ObjectType() == SG_DATAOBJECT_TYPE_Shapes
+	||  pObject->Get_ObjectType() == SG_DATAOBJECT_TYPE_PointCloud
+	||  pObject->Get_ObjectType() == SG_DATAOBJECT_TYPE_TIN )
+	{
+		if( !Parameters("METRIC_ATTRIB") || (Field = Parameters("METRIC_ATTRIB")->asInt()) < 0 || Field >= ((CSG_Shapes *)pObject)->Get_Field_Count() )
+		{
+			return( false );
+		}
+
+		// currently no support for histogram stretch options with shapes...
+		Parameters.Set_Parameter("METRIC_ZRANGE" + Suffix + ".MIN", ((CSG_Shapes *)pObject)->Get_Minimum(Field));
+		Parameters.Set_Parameter("METRIC_ZRANGE" + Suffix + ".MAX", ((CSG_Shapes *)pObject)->Get_Maximum(Field));
+
+		return( true );
+	}
+
+	double	Minimum, Maximum;
+
+	switch( Parameters("STRETCH_DEFAULT")->asInt() )
+	{
+	case 0: {
+		//-------------------------------------------------
+		switch( pObject->Get_ObjectType() )
+		{
+		case SG_DATAOBJECT_TYPE_Grid:
+			Minimum	= ((CSG_Grid   *)pObject)->Get_Min();
+			Maximum	= ((CSG_Grid   *)pObject)->Get_Max();
+			break;
+
+		case SG_DATAOBJECT_TYPE_Grids:
+			Minimum	= ((CSG_Grids  *)pObject)->Get_Min();
+			Maximum	= ((CSG_Grids  *)pObject)->Get_Max();
+			break;
+
+		case SG_DATAOBJECT_TYPE_Shapes:
+		case SG_DATAOBJECT_TYPE_PointCloud:
+		case SG_DATAOBJECT_TYPE_TIN:
+			Minimum	= ((CSG_Shapes *)pObject)->Get_Minimum(Field);
+			Maximum	= ((CSG_Shapes *)pObject)->Get_Maximum(Field);
+			break;
+
+		default:
+			return( false );
+		}
+
+		double	d	= Parameters("STRETCH_DEFAULT")->asInt() ? 0.0
+					: Parameters("STRETCH_LINEAR")->asDouble() * 0.01 * (Maximum - Minimum);
+
+		Minimum	+= d;
+		Maximum	-= d;
+
+		break;	}
+
+	//-----------------------------------------------------
+	case  1: {
+		double	d	= Parameters("STRETCH_STDDEV")->asDouble(), Mean;
+
+		switch( pObject->Get_ObjectType() )
+		{
+		case SG_DATAOBJECT_TYPE_Grid:
+			Minimum	= ((CSG_Grid   *)pObject)->Get_Min   ();
+			Maximum	= ((CSG_Grid   *)pObject)->Get_Max   ();
+			Mean	= ((CSG_Grid   *)pObject)->Get_Mean  ();
+			d	   *= ((CSG_Grid   *)pObject)->Get_StdDev();
+			break;
+
+		case SG_DATAOBJECT_TYPE_Grids:
+			Minimum	= ((CSG_Grids  *)pObject)->Get_Min   ();
+			Maximum	= ((CSG_Grids  *)pObject)->Get_Max   ();
+			Mean	= ((CSG_Grids  *)pObject)->Get_Mean  ();
+			d	   *= ((CSG_Grids  *)pObject)->Get_StdDev();
+			break;
+
+		case SG_DATAOBJECT_TYPE_Shapes:
+		case SG_DATAOBJECT_TYPE_PointCloud:
+		case SG_DATAOBJECT_TYPE_TIN:
+			Minimum	= ((CSG_Shapes *)pObject)->Get_Minimum(Field);
+			Maximum	= ((CSG_Shapes *)pObject)->Get_Maximum(Field);
+			Maximum	= ((CSG_Shapes *)pObject)->Get_Mean   (Field);
+			d	   *= ((CSG_Shapes *)pObject)->Get_StdDev (Field);
+			break;
+
+		default:
+			return( false );
+		}
+
+		if( !Parameters("STRETCH_INRANGE")->asBool() || Minimum < Mean - d ) Minimum = Mean - d;
+		if( !Parameters("STRETCH_INRANGE")->asBool() || Maximum > Mean + d ) Maximum = Mean + d;
+
+		break;	}
+
+	//-----------------------------------------------------
+	case  2: {
+		double	d	= Parameters("STRETCH_PCTL")->asDouble();
+
+		switch( pObject->Get_ObjectType() )
+		{
+		case SG_DATAOBJECT_TYPE_Grid:
+			Minimum	= ((CSG_Grid  *)pObject)->Get_Percentile(      d);
+			Maximum	= ((CSG_Grid  *)pObject)->Get_Percentile(100 - d);
+			break;
+
+		case SG_DATAOBJECT_TYPE_Grids:
+			Minimum	= ((CSG_Grids *)pObject)->Get_Percentile(      d);
+			Maximum	= ((CSG_Grids *)pObject)->Get_Percentile(100 - d);
+			break;
+
+		default:
+			return( false );
+		}
+		break;	}
+
+	//-----------------------------------------------------
+	default:
+		return( false );
+	}
+
+	Parameters.Set_Parameter("METRIC_ZRANGE" + Suffix + ".MIN", Minimum);
+	Parameters.Set_Parameter("METRIC_ZRANGE" + Suffix + ".MAX", Maximum);
+
+	return( true );
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+void CWKSP_Layer::On_DataObject_Changed(void)
+{
+	ColorsParms_Adjust(m_Parameters);
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
 //---------------------------------------------------------
 int CWKSP_Layer::On_Parameter_Changed(CSG_Parameters *pParameters, CSG_Parameter *pParameter, int Flags)
 {
+	//-----------------------------------------------------
+	if( Flags & PARAMETER_CHECK_VALUES )
+	{
+		if( pParameter->Cmp_Identifier("STRETCH_DEFAULT")
+		||  pParameter->Cmp_Identifier("STRETCH_LINEAR" )
+		||  pParameter->Cmp_Identifier("STRETCH_STDDEV" )
+		||  pParameter->Cmp_Identifier("STRETCH_INRANGE")
+		||  pParameter->Cmp_Identifier("STRETCH_PCTL"   )
+		||  pParameter->Cmp_Identifier("BAND"           ) )
+		{
+			ColorsParms_Adjust(*pParameters);
+		}
+	}
+
+	//-----------------------------------------------------
 	if( Flags & PARAMETER_CHECK_ENABLE )
 	{
-		if(	!SG_STR_CMP(pParameter->Get_Identifier(), SG_T("LEGEND_SHOW"))
-		||	!SG_STR_CMP(pParameter->Get_Identifier(), SG_T("COLORS_TYPE")) )
+		if(	pParameter->Cmp_Identifier("LEGEND_SHOW")
+		||	pParameter->Cmp_Identifier("COLORS_TYPE") )
 		{
-			pParameters->Get_Parameter("LEGEND_STYLE")->Set_Enabled(
-				pParameters->Get_Parameter("LEGEND_SHOW")->asBool()
-			&&	pParameters->Get_Parameter("COLORS_TYPE")->asInt() == CLASSIFY_GRADUATED
+			pParameters->Set_Enabled("LEGEND_STYLE",
+					(*pParameters)("LEGEND_SHOW")->asBool()
+				&&	(*pParameters)("COLORS_TYPE")->asInt() == CLASSIFY_GRADUATED
 			);
 		}
 
-		if(	!SG_STR_CMP(pParameter->Get_Identifier(), SG_T("SHOW_ALWAYS")) )
+		if(	pParameter->Cmp_Identifier("SHOW_ALWAYS") )
 		{
-			pParameters->Get_Parameter("SHOW_RANGE")->Set_Enabled(pParameter->asBool() == false);
+			pParameters->Set_Enabled("SHOW_RANGE", pParameter->asBool() == false);
 		}
 
-		if(	!SG_STR_CMP(pParameter->Get_Identifier(), SG_T("COLORS_TYPE")) )
+		if( pParameter->Cmp_Identifier("TABLE_FLT_STYLE") )
+		{
+			pParameters->Set_Enabled("TABLE_FLT_DECIMALS", pParameter->asInt() > 0);
+		}
+
+		if(	pParameter->Cmp_Identifier("COLORS_TYPE") )
 		{
 			int		Value	= pParameter->asInt();
 
-			pParameters->Get_Parameter("NODE_UNISYMBOL")->Set_Enabled(Value == CLASSIFY_UNIQUE);
-			pParameters->Get_Parameter("NODE_LUT"      )->Set_Enabled(Value == CLASSIFY_LUT);
-			pParameters->Get_Parameter("NODE_METRIC"   )->Set_Enabled(Value != CLASSIFY_UNIQUE && Value != CLASSIFY_LUT && Value != CLASSIFY_RGB);
-
-			pParameters->Get_Parameter("METRIC_COLORS" )->Set_Enabled(Value == CLASSIFY_METRIC || Value == CLASSIFY_GRADUATED);
-		}
-
-		if(	!SG_STR_CMP(pParameter->Get_Identifier(), SG_T("METRIC_SCALE_MODE")) )
-		{
-			pParameters->Get_Parameter("METRIC_SCALE_LOG")->Set_Enabled(pParameter->asInt() != 0);
+			pParameters->Set_Enabled("METRIC_COLORS" , Value == CLASSIFY_METRIC || Value == CLASSIFY_GRADUATED);
+			pParameters->Set_Enabled("NODE_UNISYMBOL", Value == CLASSIFY_UNIQUE);
+			pParameters->Set_Enabled("NODE_LUT"      , Value == CLASSIFY_LUT);
+		
+			if( m_pObject->Get_ObjectType() != SG_DATAOBJECT_TYPE_Grids )
+			{
+				pParameters->Set_Enabled("NODE_METRIC", Value != CLASSIFY_UNIQUE && Value != CLASSIFY_LUT && Value != CLASSIFY_RGB);
+			}
 		}
 	}
 
+	//-----------------------------------------------------
+	ColorsParms_On_Changed(pParameters, pParameter, Flags);
+
 	return( CWKSP_Data_Item::On_Parameter_Changed(pParameters, pParameter, Flags) );
 }
 
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
 void CWKSP_Layer::On_Parameters_Changed(void)
 {
-	m_pObject->Set_Name       (m_Parameters("OBJECT_NAME")->asString());
-	m_pObject->Set_Description(m_Parameters("OBJECT_DESC")->asString());
-
-	m_pObject->Set_NoData_Value_Range(
-		m_Parameters("GENERAL_NODATA")->asRange()->Get_LoVal(),
-		m_Parameters("GENERAL_NODATA")->asRange()->Get_HiVal()
-	);
-
 	//-----------------------------------------------------
 	m_pClassify->Initialise(this, m_Parameters("LUT")->asTable(), m_Parameters("METRIC_COLORS")->asColors());
 
@@ -419,10 +747,10 @@ void CWKSP_Layer::On_Parameters_Changed(void)
 	m_pClassify->Set_Unique_Color(m_Parameters("UNISYMBOL_COLOR")->asInt());
 
 	m_pClassify->Set_Metric(
-		m_Parameters("METRIC_SCALE_MODE")->asInt(),
-		m_Parameters("METRIC_SCALE_LOG")->asDouble(),
-		m_Parameters("METRIC_ZRANGE")->asRange()->Get_LoVal(),
-		m_Parameters("METRIC_ZRANGE")->asRange()->Get_HiVal()
+		m_Parameters("METRIC_SCALE_MODE")->asInt   (),
+		m_Parameters("METRIC_SCALE_LOG" )->asDouble(),
+		m_Parameters("METRIC_ZRANGE.MIN")->asDouble(),
+		m_Parameters("METRIC_ZRANGE.MAX")->asDouble()
 	);
 
 	m_pLegend->Set_Orientation(m_Parameters("LEGEND_STYLE")->asInt() == LEGEND_VERTICAL ? LEGEND_VERTICAL : LEGEND_HORIZONTAL);
@@ -434,8 +762,6 @@ void CWKSP_Layer::On_Parameters_Changed(void)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -460,7 +786,7 @@ bool CWKSP_Layer::_Set_Thumbnail(bool bRefresh)
 		wxRect			r(0, 0, m_Thumbnail.GetWidth(), m_Thumbnail.GetHeight());
 		CWKSP_Map_DC	dc_Map(Get_Extent(), r, 1.0, SG_GET_RGB(255, 255, 255));
 
-		Draw(dc_Map, false);
+		Draw(dc_Map, LAYER_DRAW_FLAG_NOEDITS|LAYER_DRAW_FLAG_NOLABELS|LAYER_DRAW_FLAG_THUMBNAIL);
 
 		dc.SelectObject(m_Thumbnail);
 		dc.SetBackground(*wxWHITE_BRUSH);
@@ -484,8 +810,6 @@ bool CWKSP_Layer::_Set_Thumbnail(bool bRefresh)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -495,16 +819,19 @@ CSG_Rect CWKSP_Layer::Get_Extent(void)
 	{
 		switch( m_pObject->Get_ObjectType() )
 		{
-		case DATAOBJECT_TYPE_Grid:
+		case SG_DATAOBJECT_TYPE_Grid:
 			return( ((CSG_Grid       *)m_pObject)->Get_Extent(true) );
 
-		case DATAOBJECT_TYPE_Shapes:
+		case SG_DATAOBJECT_TYPE_Grids:
+			return( ((CSG_Grids      *)m_pObject)->Get_Extent(true) );
+
+		case SG_DATAOBJECT_TYPE_Shapes:
 			return( ((CSG_Shapes     *)m_pObject)->Get_Extent() );
 
-		case DATAOBJECT_TYPE_TIN:
+		case SG_DATAOBJECT_TYPE_TIN:
 			return( ((CSG_TIN        *)m_pObject)->Get_Extent() );
 
-		case DATAOBJECT_TYPE_PointCloud:
+		case SG_DATAOBJECT_TYPE_PointCloud:
 			return( ((CSG_PointCloud *)m_pObject)->Get_Extent() );
 
 		default:
@@ -518,38 +845,53 @@ CSG_Rect CWKSP_Layer::Get_Extent(void)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
 void CWKSP_Layer::_Set_Projection(void)
 {
-	CSG_Module	*pModule	= SG_Get_Module_Library_Manager().Get_Module(SG_T("pj_proj4"), 15);	// CCRS_Picker
+	CSG_Tool	*pTool	= SG_Get_Tool_Library_Manager().Create_Tool("pj_proj4", 15);	// CCRS_Picker
 
-	if(	pModule && Get_Object() )
+	CSG_Projection	Projection(Get_Object()->Get_Projection());
+
+	if(	pTool
+	&&  pTool->Set_Parameter("CRS_EPSG"     , Projection.Get_Authority_ID())
+	&&  pTool->Set_Parameter("CRS_EPSG_AUTH", Projection.Get_Authority   ())
+	&&  pTool->Set_Parameter("CRS_PROJ4"    , Projection.Get_Proj4       ())
+	&&	pTool->On_Before_Execution() && DLG_Parameters(pTool->Get_Parameters()) )
 	{
-		CSG_Parameters	P; P.Assign(pModule->Get_Parameters());
+		Projection.Destroy();
 
-		if( pModule->Get_Parameters()->Set_Parameter("CRS_PROJ4", Get_Object()->Get_Projection().Get_Proj4())
-		&&	pModule->On_Before_Execution() && DLG_Parameters(pModule->Get_Parameters())
-		&&  pModule->Execute() )
+		if( pTool->Get_Parameter("CRS_EPSG")->asInt() > 0 )
 		{
-			Get_Object()->Get_Projection().Create(pModule->Get_Parameters()->Get_Parameter("CRS_PROJ4")->asString(), SG_PROJ_FMT_Proj4);
+			Projection.Create(
+				pTool->Get_Parameter("CRS_EPSG"     )->asInt   (),
+				pTool->Get_Parameter("CRS_EPSG_AUTH")->asString()
+			);
+		}
+		
+		if( !Projection.is_Okay() )
+		{
+			Projection.Create(
+				pTool->Get_Parameter("CRS_PROJ4")->asString(), SG_PROJ_FMT_Proj4
+			);
+		}
+
+		if( Projection.is_Okay() && !Projection.is_Equal(Get_Object()->Get_Projection()) )
+		{
+			Get_Object()->Get_Projection().Create(Projection);
 			Get_Object()->Set_Modified();
 
 			DataObject_Changed();
 		}
-
-		pModule->Get_Parameters()->Assign_Values(&P);
 	}
+
+	SG_Get_Tool_Library_Manager().Delete_Tool(pTool);
 }
 
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -572,29 +914,33 @@ bool CWKSP_Layer::Get_Colors(CSG_Colors *pColors)
 //---------------------------------------------------------
 bool CWKSP_Layer::Set_Colors(CSG_Colors *pColors)
 {
-	if( m_pClassify->Get_Metric_Colors() && pColors )
+	if( m_pClassify->Get_Metric_Colors() && pColors && m_pClassify->Get_Metric_Colors()->Assign(pColors) )
 	{
-		return( m_pClassify->Get_Metric_Colors()->Assign(pColors) && DataObject_Changed() );
+		Parameters_Changed();
+
+		return( true );
 	}
 
 	return( false );
 }
 
 //---------------------------------------------------------
-bool CWKSP_Layer::Set_Color_Range(double zMin, double zMax)
+bool CWKSP_Layer::Set_Color_Range(double Minimum, double Maximum)
 {
-	CSG_Parameters	Parameters;
+	m_Parameters.Set_Parameter("STRETCH_DEFAULT"  , 3);	// manual
+	m_Parameters.Set_Parameter("METRIC_ZRANGE.MIN", Minimum < Maximum ? Minimum : Maximum);
+	m_Parameters.Set_Parameter("METRIC_ZRANGE.MAX", Minimum < Maximum ? Maximum : Minimum);
 
-	Parameters.Add_Range(NULL, "METRIC_ZRANGE"	, _TL(""), _TL(""), zMin, zMax);
+	Parameters_Changed();
 
-	return( DataObject_Changed(&Parameters) );
+	return( true );
+
+//	return( DataObject_Changed() );
 }
 
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -604,17 +950,22 @@ bool CWKSP_Layer::do_Legend(void)
 }
 
 //---------------------------------------------------------
-bool CWKSP_Layer::do_Show(CSG_Rect const &rMap)
+bool CWKSP_Layer::do_Show(CSG_Rect const &Map_Extent, bool bIntersects)
 {
-	double				d;
-	CSG_Parameter_Range	*pRange;
+	if( bIntersects && !Map_Extent.Intersects(Get_Extent()) )
+	{
+		return( false );
+	}
 
 	if( !m_Parameters("SHOW_ALWAYS")->asBool() )
 	{
-		pRange	= m_Parameters("SHOW_RANGE")->asRange();
-		d		= rMap.Get_XRange() > rMap.Get_YRange() ? rMap.Get_XRange() : rMap.Get_YRange();
+		double	d	= Map_Extent.Get_XRange() > Map_Extent.Get_YRange()
+			? Map_Extent.Get_XRange()
+			: Map_Extent.Get_YRange();
 
-		return( pRange->Get_LoVal() <= d && d <= pRange->Get_HiVal() );
+		return( m_Parameters("SHOW_RANGE.MIN")->asDouble() <= d
+			&&  m_Parameters("SHOW_RANGE.MAX")->asDouble() >= d 
+		);
 	}
 
 	return( true );
@@ -623,21 +974,35 @@ bool CWKSP_Layer::do_Show(CSG_Rect const &rMap)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-void CWKSP_Layer::Draw(CWKSP_Map_DC &dc_Map, bool bEdit)
+bool CWKSP_Layer::Draw(CWKSP_Map_DC &dc_Map, int Flags, CSG_Data_Object *pObject)
 {
-	On_Draw(dc_Map, bEdit);
+	if( pObject && pObject->is_Valid() && pObject->Get_ObjectType() == m_pObject->Get_ObjectType() )
+	{
+		CSG_Data_Object	*pOriginal	= m_pObject;
+
+		m_pObject	= pObject;
+
+		On_Draw(dc_Map, Flags);
+
+		m_pObject	= pOriginal;
+	}
+	else
+	{
+		On_Draw(dc_Map, Flags);
+	}
+
+	dc_Map.dc.SetBrush(wxNullBrush);
+	dc_Map.dc.SetPen  (wxNullPen  );
+
+	return( true );
 }
 
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -713,7 +1078,7 @@ void CWKSP_Layer::On_Update_Views(bool bAll)
 }
 
 //---------------------------------------------------------
-bool CWKSP_Layer::View_Closes(wxMDIChildFrame *pView)
+bool CWKSP_Layer::View_Closes(MDI_ChildFrame *pView)
 {
 	if( pView == m_pHistogram )
 	{
@@ -726,8 +1091,6 @@ bool CWKSP_Layer::View_Closes(wxMDIChildFrame *pView)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -753,8 +1116,6 @@ void CWKSP_Layer::Histogram_Toggle(void)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 
