@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: wksp_layer_classify.cpp 2241 2014-09-22 11:04:27Z oconrad $
+ * Version $Id$
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -24,7 +24,8 @@
 // Geoscientific Analyses'. SAGA is free software; you   //
 // can redistribute it and/or modify it under the terms  //
 // of the GNU General Public License as published by the //
-// Free Software Foundation; version 2 of the License.   //
+// Free Software Foundation, either version 2 of the     //
+// License, or (at your option) any later version.       //
 //                                                       //
 // SAGA is distributed in the hope that it will be       //
 // useful, but WITHOUT ANY WARRANTY; without even the    //
@@ -33,10 +34,8 @@
 // License for more details.                             //
 //                                                       //
 // You should have received a copy of the GNU General    //
-// Public License along with this program; if not,       //
-// write to the Free Software Foundation, Inc.,          //
-// 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, //
-// USA.                                                  //
+// Public License along with this program; if not, see   //
+// <http://www.gnu.org/licenses/>.                       //
 //                                                       //
 //-------------------------------------------------------//
 //                                                       //
@@ -66,10 +65,131 @@
 #include "wksp_layer.h"
 #include "wksp_layer_classify.h"
 #include "wksp_grid.h"
+#include "wksp_grids.h"
 #include "wksp_shapes.h"
 #include "wksp_pointcloud.h"
 
 
+///////////////////////////////////////////////////////////
+//														 //
+//														 //
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+CSG_Scaler::CSG_Scaler(void)
+{
+	Create(0., 0.);
+}
+
+//-----------------------------------------------------
+CSG_Scaler::CSG_Scaler(double Minimum, double Maximum, double Interval)
+{
+	Create(Minimum, Maximum, Interval);
+}
+
+//-----------------------------------------------------
+bool CSG_Scaler::Create(double Minimum, double Maximum, double Interval)
+{
+	return( Set_Interval(Interval) && Set_Range(Minimum, Maximum) );
+}
+
+//-----------------------------------------------------
+bool CSG_Scaler::Set_Range(double Minimum, double Maximum)
+{
+	m_Minimum	= Minimum < Maximum ? Minimum : Maximum;
+	m_Range		= Minimum < Maximum ? Maximum - Minimum : Minimum - Maximum;
+
+	return( m_Range > 0. );
+}
+
+bool CSG_Scaler::Set_Interval(double Interval)
+{
+	m_Interval	= Interval < 0. ? DECREASING : Interval > 0. ? INCREASING : LINEAR;
+	m_LogRange	= fabs(Interval);
+
+	return( true );
+}
+
+//-----------------------------------------------------
+bool CSG_Scaler::Set_Linear(CSG_Table *pTable, int Field, double Interval, double Percent)
+{
+	double	Minimum	= pTable->Get_Minimum(Field) + 0.01 * Percent;
+	double	Maximum	= pTable->Get_Maximum(Field) - 0.01 * Percent;
+
+	return( Create(Minimum, Maximum, Interval) );
+}
+
+bool CSG_Scaler::Set_StdDev(CSG_Table *pTable, int Field, double Interval, double StdDev, bool bKeepInRange)
+{
+	double	Minimum	= pTable->Get_Mean(Field) - StdDev * pTable->Get_StdDev(Field); if( bKeepInRange && Minimum < pTable->Get_Minimum(Field) ) Minimum = pTable->Get_Minimum(Field);
+	double	Maximum	= pTable->Get_Mean(Field) + StdDev * pTable->Get_StdDev(Field); if( bKeepInRange && Maximum > pTable->Get_Maximum(Field) ) Maximum = pTable->Get_Maximum(Field);
+
+	return( Create(Minimum, Maximum, Interval) );
+}
+
+bool CSG_Scaler::Set_Percentile(CSG_Table *pTable, int Field, double Interval, double Percentile)
+{
+	return( false );
+
+	//	double	Minimum	= pTable->Get_Percentile(Field,        Percentile);
+	//	double	Maximum	= pTable->Get_Percentile(Field, 100. - Percentile);
+
+	//	return( Create(Minimum, Maximum, Interval) );
+}
+
+
+//-----------------------------------------------------
+bool CSG_Scaler::Set_Linear(CSG_Grid *pGrid, double Interval, double Percent)
+{
+	double	Minimum	= pGrid->Get_Min() + 0.01 * Percent;
+	double	Maximum	= pGrid->Get_Max() - 0.01 * Percent;
+
+	return( Create(Minimum, Maximum, Interval) );
+}
+
+bool CSG_Scaler::Set_StdDev(CSG_Grid *pGrid, double Interval, double StdDev, bool bKeepInRange)
+{
+	double	Minimum	= pGrid->Get_Mean() - StdDev * pGrid->Get_StdDev(); if( bKeepInRange && Minimum < pGrid->Get_Min() ) Minimum = pGrid->Get_Min();
+	double	Maximum	= pGrid->Get_Mean() + StdDev * pGrid->Get_StdDev(); if( bKeepInRange && Maximum > pGrid->Get_Max() ) Maximum = pGrid->Get_Max();
+
+	return( Create(Minimum, Maximum, Interval) );
+}
+
+bool CSG_Scaler::Set_Percentile(CSG_Grid *pGrid, double Interval, double Percentile)
+{
+	double	Minimum	= pGrid->Get_Percentile(       Percentile);
+	double	Maximum	= pGrid->Get_Percentile(100. - Percentile);
+
+	return( Create(Minimum, Maximum, Interval) );
+}
+
+//-----------------------------------------------------
+bool CSG_Scaler::Set_Linear(CSG_Grids *pGrids, double Interval, double Percent)
+{
+	double	Minimum	= pGrids->Get_Min() + 0.01 * Percent;
+	double	Maximum	= pGrids->Get_Max() - 0.01 * Percent;
+
+	return( Create(Minimum, Maximum, Interval) );
+}
+
+bool CSG_Scaler::Set_StdDev(CSG_Grids *pGrids, double Interval, double StdDev, bool bKeepInRange)
+{
+	double	Minimum	= pGrids->Get_Mean() - StdDev * pGrids->Get_StdDev(); if( bKeepInRange && Minimum < pGrids->Get_Min() ) Minimum = pGrids->Get_Min();
+	double	Maximum	= pGrids->Get_Mean() + StdDev * pGrids->Get_StdDev(); if( bKeepInRange && Maximum > pGrids->Get_Max() ) Maximum = pGrids->Get_Max();
+
+	return( Create(Minimum, Maximum, Interval) );
+}
+
+bool CSG_Scaler::Set_Percentile(CSG_Grids *pGrids, double Interval, double Percentile)
+{
+	double	Minimum	= pGrids->Get_Percentile(       Percentile);
+	double	Maximum	= pGrids->Get_Percentile(100. - Percentile);
+
+	return( Create(Minimum, Maximum, Interval) );
+}
+
+
 ///////////////////////////////////////////////////////////
 //														 //
 //														 //
@@ -87,22 +207,11 @@ CWKSP_Layer_Classify::CWKSP_Layer_Classify(void)
 	m_pLayer		= NULL;
 	m_pColors		= NULL;
 	m_pLUT			= NULL;
-
-	m_HST_Count		= NULL;
 }
 
 //---------------------------------------------------------
 CWKSP_Layer_Classify::~CWKSP_Layer_Classify(void)
-{
-	if( m_HST_Count )
-	{
-		SG_Free(m_HST_Count);
-		m_HST_Count	= NULL;
-
-		SG_Free(m_HST_Cumul);
-		m_HST_Cumul	= NULL;
-	}
-}
+{}
 
 
 ///////////////////////////////////////////////////////////
@@ -158,9 +267,6 @@ double CWKSP_Layer_Classify::Get_Class_Value_Minimum(int iClass)
 {
 	switch( m_Mode )
 	{
-	default:
-		break;
-
 	case CLASSIFY_LUT:
 		if( iClass >= 0 && iClass < m_pLUT->Get_Record_Count() )
 		{
@@ -177,6 +283,8 @@ double CWKSP_Layer_Classify::Get_Class_Value_Minimum(int iClass)
 			return( Get_RelativeToMetric(iClass / (double)Get_Class_Count()) );
 		}
 		break;
+
+	default: break;
 	}
 
 	return( m_zMin );
@@ -186,9 +294,6 @@ double CWKSP_Layer_Classify::Get_Class_Value_Maximum(int iClass)
 {
 	switch( m_Mode )
 	{
-	default:
-		break;
-
 	case CLASSIFY_LUT:
 		if( iClass >= 0 && iClass < m_pLUT->Get_Record_Count() )
 		{
@@ -205,6 +310,8 @@ double CWKSP_Layer_Classify::Get_Class_Value_Maximum(int iClass)
 			return( Get_RelativeToMetric((1.0 + iClass) / (double)Get_Class_Count()) );
 		}
 		break;
+
+	default: break;
 	}
 
 	return( m_zMin + m_zRange );
@@ -214,9 +321,6 @@ double CWKSP_Layer_Classify::Get_Class_Value_Center(int iClass)
 {
 	switch( m_Mode )
 	{
-	default:
-		break;
-
 	case CLASSIFY_LUT:
 		if( iClass >= 0 && iClass < m_pLUT->Get_Record_Count() )
 		{
@@ -233,6 +337,8 @@ double CWKSP_Layer_Classify::Get_Class_Value_Center(int iClass)
 			return( Get_RelativeToMetric((0.5 + iClass) / (double)Get_Class_Count()) );
 		}
 		break;
+
+	default: break;
 	}
 
 	return( m_zMin + 0.5 * m_zRange );
@@ -245,9 +351,6 @@ wxString CWKSP_Layer_Classify::Get_Class_Name(int iClass)
 
 	switch( m_Mode )
 	{
-	default:
-		break;
-
 	case CLASSIFY_LUT:
 		if( iClass >= 0 && iClass < m_pLUT->Get_Record_Count() )
 		{
@@ -262,6 +365,8 @@ wxString CWKSP_Layer_Classify::Get_Class_Name(int iClass)
 		s	= SG_Get_String(Get_Class_Value_Minimum(iClass), -2) + SG_T(" < ")
 			+ SG_Get_String(Get_Class_Value_Maximum(iClass), -2);
 		break;
+
+	default: break;
 	}
 
 	return( s.c_str() );
@@ -301,12 +406,25 @@ void CWKSP_Layer_Classify::Set_Unique_Color(int Color)
 //														 //
 ///////////////////////////////////////////////////////////
 
+//---------------------------------------------------------
+bool CWKSP_Layer_Classify::Set_Class_Count(int Count)
+{
+	if( Count > 0 && Count != m_Count )
+	{
+		m_Count	= Count;
+
+		return( Histogram_Update() );
+	}
+
+	return( false );
+}
+
 //---------------------------------------------------------
 void CWKSP_Layer_Classify::Set_Metric(int Mode, double LogFactor, double zMin, double zMax)
 {
 	m_zMode		= Mode;
-	m_zMin		= zMin;
-	m_zRange	= zMax - zMin;
+	m_zMin		= zMin < zMax ? zMin : zMax;
+	m_zRange	= zMin < zMax ? (zMax - zMin) : (zMin - zMax);
 	m_zLogRange	= LogFactor;
 	m_zLogMax	= log(1.0 + m_zLogRange);
 
@@ -461,49 +579,48 @@ int CWKSP_Layer_Classify::_LUT_Get_Class(const CSG_String &Value)
 //---------------------------------------------------------
 void CWKSP_Layer_Classify::Metric2EqualElements(void)
 {
-	int				x, y, iClass, nClasses;
-	double			dClass, zA, zB;
-	CSG_Table_Record	*pRecord;
-	CSG_Grid			*pGrid;
-
-	if( m_pLayer->Get_Type() == WKSP_ITEM_Grid && (nClasses = m_pColors->Get_Count()) > 1 )
+	if( m_pLayer->Get_Type() == WKSP_ITEM_Grid && m_pColors->Get_Count() > 1 )
 	{
 		m_pLUT->Del_Records();
 
-		pGrid	= ((CWKSP_Grid *)m_pLayer)->Get_Grid();
-		dClass	= (double)pGrid->Get_NCells() / (double)(nClasses);
+		CSG_Grid	*pGrid	= ((CWKSP_Grid *)m_pLayer)->Get_Grid();
+
+		double	zA, zB, dClass	= (double)pGrid->Get_NCells() / (double)(m_pColors->Get_Count());
 
 		if( !pGrid->Set_Index() )
 		{
 			return;
 		}
 
+		int		x, y;
+
 		pGrid->Get_Sorted(0, x, y, false, false);
 		zA		= pGrid->asDouble(x, y);
 
-		for(iClass=0; iClass<nClasses-1; iClass++)
+		for(int iClass=0; iClass<m_pColors->Get_Count()-1; iClass++)
 		{
 			pGrid->Get_Sorted((sLong)(dClass * (iClass + 1.0)), x, y, false, false);
 			zB		= zA;
 			zA		= pGrid->asDouble(x, y);
 
-			pRecord	= m_pLUT->Add_Record();
-			pRecord->Set_Value(LUT_COLOR		, m_pColors->Get_Color(iClass));
-			pRecord->Set_Value(LUT_TITLE		, CSG_String::Format(SG_T(">=%f"), zB));
-			pRecord->Set_Value(LUT_DESCRIPTION	, CSG_String::Format(SG_T("%f <-> %f"), zB, zA));
-			pRecord->Set_Value(LUT_MIN			, zB);
-			pRecord->Set_Value(LUT_MAX			, zA);
+			CSG_Table_Record	*pClass	= m_pLUT->Add_Record();
+			pClass->Set_Value(LUT_COLOR      , m_pColors->Get_Color(iClass));
+			pClass->Set_Value(LUT_TITLE      , CSG_String::Format(">=%f"     , zB    ));
+			pClass->Set_Value(LUT_DESCRIPTION, CSG_String::Format("%f <-> %f", zB, zA));
+			pClass->Set_Value(LUT_MIN        , zB);
+			pClass->Set_Value(LUT_MAX        , zA);
 		}
 
 		pGrid->Get_Sorted(pGrid->Get_NCells() - 1, x, y, false, false);
 		zB		= zA;
 		zA		= pGrid->asDouble(x, y);
-		pRecord	= m_pLUT->Add_Record();
-		pRecord->Set_Value(LUT_COLOR		, m_pColors->Get_Color(iClass));
-		pRecord->Set_Value(LUT_TITLE		, CSG_String::Format(SG_T(">=%f"), zB));
-		pRecord->Set_Value(LUT_DESCRIPTION	, CSG_String::Format(SG_T("%f <-> %f"), zB, zA));
-		pRecord->Set_Value(LUT_MIN			, zB);
-		pRecord->Set_Value(LUT_MAX			, zA);
+
+		CSG_Table_Record	*pClass	= m_pLUT->Add_Record();
+		pClass->Set_Value(LUT_COLOR      , m_pColors->Get_Color(m_pColors->Get_Count() - 1));
+		pClass->Set_Value(LUT_TITLE      , CSG_String::Format(">=%f"     , zB    ));
+		pClass->Set_Value(LUT_DESCRIPTION, CSG_String::Format("%f <-> %f", zB, zA));
+		pClass->Set_Value(LUT_MIN        , zB);
+		pClass->Set_Value(LUT_MAX        , zA);
 
 		Set_Mode(CLASSIFY_LUT);
 		m_pLayer->Update_Views();
@@ -520,57 +637,55 @@ void CWKSP_Layer_Classify::Metric2EqualElements(void)
 //---------------------------------------------------------
 bool CWKSP_Layer_Classify::Histogram_Update(void)
 {
-	int		i;
-
-	//-----------------------------------------------------
-	if( m_HST_Count )
+	if( Get_Class_Count() < 1 )
 	{
-		SG_Free(m_HST_Count);
-		SG_Free(m_HST_Cumul);
-		m_HST_Count	= NULL;
-		m_HST_Cumul	= NULL;
+		m_Histogram.Destroy();
+
+		return( false );
 	}
 
 	//-----------------------------------------------------
-	if( Get_Class_Count() > 0 )
+	STATUSBAR_Set_Text(_TL("Build Histogram..."));
+
+	m_Histogram.Create(Get_Class_Count(), 0, Get_Class_Count() - 1);
+
+	switch( m_pLayer->Get_Type() )
 	{
-		STATUSBAR_Set_Text(_TL("Build Histogram..."));
+	case WKSP_ITEM_Grid:
+		_Histogram_Update(((CWKSP_Grid  *)m_pLayer)->Get_Grid ());
+		break;
 
-		m_HST_Count	= (int *)SG_Calloc(Get_Class_Count(), sizeof(int));
-		m_HST_Cumul	= (int *)SG_Calloc(Get_Class_Count(), sizeof(int));
-
-		switch( m_pLayer->Get_Type() )
+	case WKSP_ITEM_Grids:
+		if( m_Mode == CLASSIFY_OVERLAY )
 		{
-		default:
-			break;
-
-		case WKSP_ITEM_Grid:
-			_Histogram_Update(((CWKSP_Grid   *)m_pLayer)->Get_Grid());
-			break;
-
-		case WKSP_ITEM_Shapes:
-			_Histogram_Update(((CWKSP_Shapes *)m_pLayer)->Get_Shapes(), ((CWKSP_Shapes *)m_pLayer)->Get_Color_Field());
-			break;
-
-		case WKSP_ITEM_PointCloud:
-			_Histogram_Update(((CWKSP_PointCloud *)m_pLayer)->Get_PointCloud(), ((CWKSP_PointCloud *)m_pLayer)->Get_Color_Field());
-			break;
+			_Histogram_Update(((CWKSP_Grids *)m_pLayer)->Get_Grids());
 		}
-
-		PROCESS_Set_Okay();
-
-		//-------------------------------------------------
-		for(i=0, m_HST_Maximum=0, m_HST_Total=0; i<Get_Class_Count(); i++)
+		else
 		{
-			m_HST_Cumul[i]	= (m_HST_Total += m_HST_Count[i]);
-
-			if( m_HST_Count[i] > m_HST_Maximum )
-			{
-				m_HST_Maximum	= m_HST_Count[i];
-			}
+			_Histogram_Update(((CWKSP_Grids *)m_pLayer)->Get_Grid ());
 		}
+		break;
+
+	case WKSP_ITEM_Shapes:
+		_Histogram_Update(((CWKSP_Shapes *)m_pLayer)->Get_Shapes(),
+			((CWKSP_Shapes *)m_pLayer)->Get_Field_Value (),
+			((CWKSP_Shapes *)m_pLayer)->Get_Field_Normal()
+		);
+		break;
+
+	case WKSP_ITEM_PointCloud:
+		_Histogram_Update(((CWKSP_PointCloud *)m_pLayer)->Get_PointCloud(),
+			((CWKSP_PointCloud *)m_pLayer)->Get_Field_Value()
+		);
+		break;
+
+	default: break;
 	}
 
+	m_Histogram.Update();
+
+	PROCESS_Set_Okay();
+
 	//-----------------------------------------------------
 	return( true );
 }
@@ -578,46 +693,107 @@ bool CWKSP_Layer_Classify::Histogram_Update(void)
 //---------------------------------------------------------
 bool CWKSP_Layer_Classify::_Histogram_Update(CSG_Grid *pGrid)
 {
-	for(int y=0; y<pGrid->Get_NY() && PROGRESSBAR_Set_Position(y, pGrid->Get_NY()); y++)
+	if( pGrid->Get_Max_Samples() > 0 && pGrid->Get_Max_Samples() < pGrid->Get_NCells() )
 	{
-		for(int x=0; x<pGrid->Get_NX(); x++)
+		double	d	= (double)pGrid->Get_NCells() / (double)pGrid->Get_Max_Samples();
+
+		for(double i=0; i<(double)pGrid->Get_NCells() && PROGRESSBAR_Set_Position(i, (double)pGrid->Get_NCells()); i+=d)
 		{
-			if( !pGrid->is_NoData(x, y) )
+			if( !pGrid->is_NoData((sLong)i) )
 			{
-				int		Class	= Get_Class(pGrid->asDouble(x, y));
-				
-				if( Class >= 0 && Class < Get_Class_Count() )
-				{
-					m_HST_Count[Class]++;
-				}
+				m_Histogram	+= Get_Class(pGrid->asDouble((sLong)i));
 			}
 		}
+
+		if( m_Histogram.Update() && m_Histogram.Get_Element_Count() < pGrid->Get_Max_Samples() )	// any no-data cells ?
+		{
+			d	*= (double)m_Histogram.Get_Element_Count() / (double)pGrid->Get_Max_Samples();
+		}
+
+		m_Histogram.Scale_Element_Count(d);
+
+		return( true );
+	}
+
+	for(sLong i=0; i<pGrid->Get_NCells() && PROGRESSBAR_Set_Position((double)i, (double)pGrid->Get_NCells()); i++)
+	{
+		if( !pGrid->is_NoData(i) )
+		{
+			m_Histogram	+= Get_Class(pGrid->asDouble(i));
+		}
 	}
 
 	return( true );
 }
 
 //---------------------------------------------------------
-bool CWKSP_Layer_Classify::_Histogram_Update(CSG_Shapes *pShapes, int Attribute)
+bool CWKSP_Layer_Classify::_Histogram_Update(CSG_Grids *pGrids)
 {
-	if( Attribute >= 0 && Attribute < pShapes->Get_Field_Count() )
+	if( pGrids->Get_Max_Samples() > 0 && pGrids->Get_Max_Samples() < pGrids->Get_NCells() )
 	{
-		for(int i=0; i<pShapes->Get_Count() && PROGRESSBAR_Set_Position(i, pShapes->Get_Count()); i++)
+		double	d	= (double)pGrids->Get_NCells() / (double)pGrids->Get_Max_Samples();
+
+		for(double i=0; i<(double)pGrids->Get_NCells() && PROGRESSBAR_Set_Position(i, (double)pGrids->Get_NCells()); i+=d)
 		{
-			int		Class	= SG_Data_Type_is_Numeric(m_pLUT->Get_Field_Type(LUT_MIN))
-				? Get_Class(pShapes->Get_Record(i)->asDouble(Attribute))
-				: Get_Class(pShapes->Get_Record(i)->asString(Attribute));
-			
-			if( Class >= 0 && Class < Get_Class_Count() )
+			if( !pGrids->is_NoData((sLong)i) )
 			{
-				m_HST_Count[Class]++;
+				m_Histogram	+= Get_Class(pGrids->asDouble((sLong)i));
 			}
 		}
 
+		if( m_Histogram.Update() && m_Histogram.Get_Element_Count() < pGrids->Get_Max_Samples() )	// any no-data cells ?
+		{
+			d	*= (double)m_Histogram.Get_Element_Count() / (double)pGrids->Get_Max_Samples();
+		}
+
+		m_Histogram.Scale_Element_Count(d);
+
 		return( true );
 	}
 
-	return( false );
+	for(sLong i=0; i<pGrids->Get_NCells() && PROGRESSBAR_Set_Position((double)i, (double)pGrids->Get_NCells()); i++)
+	{
+		if( !pGrids->is_NoData(i) )
+		{
+			m_Histogram	+= Get_Class(pGrids->asDouble(i));
+		}
+	}
+
+	return( true );
+}
+
+//---------------------------------------------------------
+bool CWKSP_Layer_Classify::_Histogram_Update(CSG_Shapes *pShapes, int Attribute, int Normalize)
+{
+	if( Attribute < 0 || Attribute >= pShapes->Get_Field_Count() )
+	{
+		return( false );
+	}
+
+	for(int i=0; i<pShapes->Get_Count() && PROGRESSBAR_Set_Position(i, pShapes->Get_Count()); i++)
+	{
+		CSG_Shape	*pShape	= pShapes->Get_Shape(i);
+
+		if( m_Mode == CLASSIFY_LUT )
+		{
+			m_Histogram	+= SG_Data_Type_is_Numeric(m_pLUT->Get_Field_Type(LUT_MIN))
+				? Get_Class(pShape->asDouble(Attribute))
+				: Get_Class(pShape->asString(Attribute));
+		}
+		else if( !pShape->is_NoData(Attribute) )
+		{
+			if( Normalize < 0 )
+			{
+				m_Histogram	+= Get_Class(pShape->asDouble(Attribute));
+			}
+			else if( !pShape->is_NoData(Normalize) && pShape->asDouble(Normalize) )
+			{
+				m_Histogram	+= Get_Class(pShape->asDouble(Attribute) / pShape->asDouble(Normalize));
+			}
+		}
+	}
+
+	return( true );
 }
 
 
